/***********************************************************/
/*                                                         */
/* AccMapRenderer.cpp                                      */
/*                                                         */
/* created 24.05.2003                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "AccMapRenderer.h"
#include "TerrainLine.h"

#include "IMath.h"

// www.infinitecode.com 3d library
namespace I3D
{

AccMapRenderer::AccMapRenderer(void)
	: Lines(0)
{
}

AccMapRenderer::~AccMapRenderer(void)
{
	if(Lines)
		delete[] Lines;
}

void AccMapRenderer::CreateLines()
{
	Lines = new TerrainLine[AccMapRenderer::Specs.TI->GetAccResX() * AccMapRenderer::Specs.TI->GetAccResZ() * 4];
	for(int x = 0; x < AccMapRenderer::Specs.TI->GetAccResX(); x++)
		for(int z = 0; z < AccMapRenderer::Specs.TI->GetAccResZ(); z++)
		{
			int Index = 4 * (x + z * AccMapRenderer::Specs.TI->GetAccResX());
			unsigned int Color = 0xFF000000;
			float Delta = absf(AccMapRenderer::Specs.TI->GetAccNormed(((float)x / AccMapRenderer::Specs.TI->GetAccResX() - 0.5f) * AccMapRenderer::Specs.TI->GetSize()
				, ((float)z / AccMapRenderer::Specs.TI->GetAccResZ() - 0.5f) * AccMapRenderer::Specs.TI->GetSize()));
			float Max = maxf(Delta, 1.0f - Delta);
			int Green = (int)(0xFF * (1.0f - Delta) / Max);
			int Red = (int)(0xFF * Delta / Max);
			Color += 0x100 * Green;
			Color += 0x10000 * Red;
			// 1
			TerrainLine::TerrainLineSpecs LSpecs;
			LSpecs.DS = AccMapRenderer::Specs.DS;
			LSpecs.TI = AccMapRenderer::Specs.TI;
			LSpecs.TLR = AccMapRenderer::Specs.TLR;
			LSpecs.Color = Color;
			LSpecs.Pos[0] = IMATH::Vector2DXZ(((float)(x + 0.4f) / AccMapRenderer::Specs.TI->GetAccResX() - 0.5f) * AccMapRenderer::Specs.TI->GetSize(),
				((float)(z - 0.4f) / AccMapRenderer::Specs.TI->GetAccResZ() - 0.5f) * AccMapRenderer::Specs.TI->GetSize());
			LSpecs.Pos[1] = IMATH::Vector2DXZ(((float)(x + 0.4f) / AccMapRenderer::Specs.TI->GetAccResX() - 0.5f) * AccMapRenderer::Specs.TI->GetSize(),
				((float)(z + 0.4f) / AccMapRenderer::Specs.TI->GetAccResZ() - 0.5f) * AccMapRenderer::Specs.TI->GetSize());
			Lines[Index].Create(&LSpecs);
			// 2
			LSpecs.Pos[0] = IMATH::Vector2DXZ(((float)(x - 0.4f) / AccMapRenderer::Specs.TI->GetAccResX() - 0.5f) * AccMapRenderer::Specs.TI->GetSize(),
				((float)(z - 0.4f) / AccMapRenderer::Specs.TI->GetAccResZ() - 0.5f) * AccMapRenderer::Specs.TI->GetSize());
			LSpecs.Pos[1] = IMATH::Vector2DXZ(((float)(x - 0.4f) / AccMapRenderer::Specs.TI->GetAccResX() - 0.5f) * AccMapRenderer::Specs.TI->GetSize(),
				((float)(z + 0.4f) / AccMapRenderer::Specs.TI->GetAccResZ() - 0.5f) * AccMapRenderer::Specs.TI->GetSize());
			Lines[Index + 1].Create(&LSpecs);
			// 3
			LSpecs.Pos[0] = IMATH::Vector2DXZ(((float)(x + 0.4f) / AccMapRenderer::Specs.TI->GetAccResX() - 0.5f) * AccMapRenderer::Specs.TI->GetSize(),
				((float)(z - 0.4f) / AccMapRenderer::Specs.TI->GetAccResZ() - 0.5f) * AccMapRenderer::Specs.TI->GetSize());
			LSpecs.Pos[1] = IMATH::Vector2DXZ(((float)(x - 0.4f) / AccMapRenderer::Specs.TI->GetAccResX() - 0.5f) * AccMapRenderer::Specs.TI->GetSize(),
				((float)(z - 0.4f) / AccMapRenderer::Specs.TI->GetAccResZ() - 0.5f) * AccMapRenderer::Specs.TI->GetSize());
			Lines[Index + 2].Create(&LSpecs);
			// 4
			LSpecs.Pos[0] = IMATH::Vector2DXZ(((float)(x + 0.4f) / AccMapRenderer::Specs.TI->GetAccResX() - 0.5f) * AccMapRenderer::Specs.TI->GetSize(),
				((float)(z + 0.4f) / AccMapRenderer::Specs.TI->GetAccResZ() - 0.5f) * AccMapRenderer::Specs.TI->GetSize());
			LSpecs.Pos[1] = IMATH::Vector2DXZ(((float)(x - 0.4f) / AccMapRenderer::Specs.TI->GetAccResX() - 0.5f) * AccMapRenderer::Specs.TI->GetSize(),
				((float)(z + 0.4f) / AccMapRenderer::Specs.TI->GetAccResZ() - 0.5f) * AccMapRenderer::Specs.TI->GetSize());
			Lines[Index + 3].Create(&LSpecs);
		}
}

void AccMapRenderer::Create(AccMapRenderer::AMRSpecs *Specs)
{
	AccMapRenderer::Specs = *Specs;
	if(Lines)
		delete[] Lines;
	Lines = 0;
	CreateLines();
}

void AccMapRenderer::Render(D3DStruct *DS)
{
}

}
