/***********************************************************/
/*                                                         */
/* Airfield.h                                              */
/*                                                         */
/* created 29.12.2003                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef AIRFIELD_H
#define AIRFIELD_H

#include "MathStructures.h"
#include "TextureManager.h"
#include "D3DStruct.h"

#include <d3d9.h>

// www.infinitecode.com game library
namespace IGAME
{

class Airfield
{
public:
	class AirfieldSpecs
	{
	public:
		char Name[1024];
		IMATH::Vector2DXZ Pos;
	};

private:
	class OwnerData
	{
	public:
		bool Captured;
		float Points;

		OwnerData()
			:Captured(false), Points(200.0f) {}
	};

	static LPDIRECT3DTEXTURE9 Texture;
	AirfieldSpecs Specs;
	int Owner;
	float LastPlane;
	ISL::DynamicArray<OwnerData> Data;

public:
	Airfield()
		:Owner(-1), LastPlane(-1000) {}

	Airfield(const AirfieldSpecs &Specs)
	{
		Airfield::Specs = Specs;
	}

	static LPDIRECT3DTEXTURE9 GetTexture(I3D::D3DStruct *DS)
	{
		if(!Texture)
			Texture = I3D::TextureManager::Load(DS->D3DDevice, "Textures\\Airfield.bmp", 0, I3D::TM_FLAG_ALPHACC0);
		return Texture;
	}

	const char *GetName() const
	{
		return Specs.Name;
	}

	int GetOwner() const
	{
		return Owner;
	}

	float GetPoints(int Owner)
	{
		return Data[Owner].Points;
	}

	void SetPoints(int Owner, float Points)
	{
		Data[Owner].Points = Points;
	}

	void SetName(const char *Value)
	{
		strcpy(Specs.Name, Value);
	}

	void SetPosX(float Value)
	{
		Specs.Pos.x = Value;
	}

	void SetPosZ(float Value)
	{
		Specs.Pos.z = Value;
	}

	float GetLastPlane()
	{
		return LastPlane;
	}

	void SetLastPlane(float Value)
	{
		LastPlane = Value;
	}

	const IMATH::Vector2DXZ &GetPos() const
	{
		return Specs.Pos;
	}

	const IMATH::Vector2DXZ GetLandingPos() const
	{
		return Specs.Pos + IMATH::Vector2DXZ(112.0f, -22.5f);
	}

	void SetOwner(int Owner);

	int GetSizeX()
	{
		return 256;
	}

	int GetSizeZ()
	{
		return 86;
	}
};

}

#endif
