/***********************************************************/
/*                                                         */
/* Brush.cpp                                               */
/*                                                         */
/* created 17.03.2003                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "Brush.h"
#include "RenderableModel.h"
#include "IDType.h"
#include "IDParticleFX.h"
#include "Text.h"
#include "Waterlevel.h"

#include <stdio.h>

// www.infinitecode.com game library
namespace IGAME
{

void Brush::Create(Brush::BrushSpecs *Specs)
{
	Brush::Specs = *Specs;
	memset(Area, 0, 4 * sizeof(Lines[0]));
	Type = BRUSH_TYPE_NONE;
	Owner = 1;
	Angle = 0;
	AreaEx = false;
	LastSpawn = Brush::Specs.DS->Sp->GameTime;
	for(int i = 0; i < 4; i++)
	{
		if(!Lines[i])
			Lines[i] = new I3D::TerrainLine();
		if(!AreaLines[i])
			AreaLines[i] = new I3D::TerrainLine();
		/*I3D::TerrainLine::TerrainLineSpecs TLS;
		TLS.TI = Brush::Specs.Wrld->GetTerrainInterface();
		TLS.TLR = Brush::Specs.Wrld->GetTerrainLineRenderer();
		TLS.DS = Brush::Specs.DS;
		Lines[i]->Create(&TLS);*/
	}
}

void Brush::ClearArea()
{
	// destroy old lines
	PreLines.Delete(AreaPreLines[0]);
	delete AreaPreLines[0];
	PreLines.Delete(AreaPreLines[1]);
	delete AreaPreLines[1];
	PreLines.Delete(AreaPreLines[2]);
	delete AreaPreLines[2];
	PreLines.Delete(AreaPreLines[3]);
	delete AreaPreLines[3];
	memset(Area, 0, 4 * sizeof(Lines[0]));
	RemoveMarkers();
}

void Brush::RemoveMarkers(void)
{
	for(int i = 0; i < 4; i++)
		Lines[i]->Destroy();
}

void Brush::SetTank(int ID)
{
	Type = BRUSH_TYPE_IDOBJECT;
	TypeID = IDTYPE_TANK;
	Brush::ID = ID;
	Sequence = 0;
	MaxSeq = 0;
	RemoveMarkers();
}

void Brush::SetWay(int ID)
{
	Type = BRUSH_TYPE_IDOBJECT;
	TypeID = IDTYPE_WAY;
	Brush::ID = ID;
	Sequence = 1;
	MaxSeq = 1;
	RemoveMarkers();
}

void Brush::SetArea()
{
	Type = BRUSH_TYPE_IDOBJECT;
	TypeID = IDTYPE_AREA;
	Sequence = 1;
	MaxSeq = 1;
	RemoveMarkers();
}

void Brush::SetAirfield()
{
	Type = BRUSH_TYPE_IDOBJECT;
	TypeID = IDTYPE_AIRFIELD;
	Sequence = 0;
	MaxSeq = 0;
	RemoveMarkers();
}

void Brush::SetDecal(int ID)
{
	Type = BRUSH_TYPE_IDOBJECT;
	TypeID = IDTYPE_DECAL;
	Brush::ID = ID;
	Sequence = 0;
	MaxSeq = 0;
	RemoveMarkers();
}

void Brush::SetTerrain(int ID)
{
	Type = BRUSH_TYPE_IDOBJECT;
	TypeID = IDTYPE_TERRAIN;
	Brush::ID = ID;
	Sequence = 2;
	MaxSeq = 2;
	RemoveMarkers();
}

void Brush::LoadingScreen(const char *Str)
{
	Specs.DS->D3DDevice->Clear(0, NULL, D3DCLEAR_ZBUFFER | D3DCLEAR_TARGET, 0x00000000, 1.0f, 0);
	D3DMATRIX Matrix = {1,0,0,0,  0,1,0,0,  0,0,1,0,  0,0,0,1};
	// world and view matrix
	Specs.DS->D3DDevice->SetTransform(D3DTS_WORLD, &Matrix);
	Specs.DS->D3DDevice->SetTransform(D3DTS_VIEW, &Matrix);
	Specs.DS->D3DDevice->SetTransform(D3DTS_PROJECTION, &Matrix);
	// loading text
	I3D::Text::DrawString(Specs.DS, Str, -0.05f * 0.5f * strlen(Str), 0, 0.05f, 0.05f);
	// display
	Specs.DS->D3DDevice->Present(NULL, NULL, NULL, NULL);
}

void Brush::Render(I3D::D3DStruct *DS)
{
	if(Type == BRUSH_TYPE_IDOBJECT && TypeID == IDTYPE_TANK)
	{
		IMATH::Vector2D Pos = Specs.Mouse->GetPos();
		IMATH::Vector2DXZ XZPos = Specs.Wrld->GetSurfaceXZ(Pos);
		I3D::RenderableModel RM;
		RM.Mdl = I3D::Model::GetModel(Specs.DS, Specs.TLib->Tanks[ID].Model);
		RM.SetPos(IMATH::Vector(XZPos.x, Specs.TLib->Tanks[ID].IsShip ? WATERLEVEL : Specs.Wrld->GetSurfaceY(XZPos.x, XZPos.z), XZPos.z));
		IMATH::Vector Normal = Specs.Wrld->GetSurfaceNormal(RM.GetPos().x, RM.GetPos().z);
		if(Specs.TLib->Tanks[ID].IsShip || !Specs.TLib->Tanks[ID].CanMove)
			Normal = IMATH::Vector(0, 1, 0);
		((I3D::Matrix *)RM.Mtx)->SetTangentMatrix(RM.GetPos(), Angle, Normal);
		RM.Render(Specs.DS);
		RM.Texture = 0;
		if(Specs.TLib->Tanks[ID].HasTurret)
		{
			RM.Mdl = I3D::Model::GetModel(Specs.DS, Specs.TLib->Tanks[ID].TurretModel);
			RM.Render(Specs.DS);
		}
	}
	// Airfield
	if(Type == BRUSH_TYPE_IDOBJECT && TypeID == IDTYPE_AIRFIELD)
	{
		SpawnInterface::SpawnInterfaceSpecs SS;
		SS.Type = IDTYPE_PARTICLEFX;
		SS.ID = IDPFX_SMALLRISINGSMOKE;
		// mouse pos on surface
		IMATH::Vector2D MPos = Specs.Mouse->GetPos();
		IMATH::Vector2DXZ XZPos = Specs.Wrld->GetSurfaceXZ(MPos);
		// spawn pfx at mouse pos
		SS.Pos = IMATH::Vector(XZPos.x - 128, Specs.Wrld->GetSurfaceY(XZPos.x - 128, XZPos.z - 32), XZPos.z - 32);
		Specs.Wrld->GetSpawnInterface()->Spawn(&SS);
		// spawn pfx at mouse pos
		SS.Pos = IMATH::Vector(XZPos.x + 128, Specs.Wrld->GetSurfaceY(XZPos.x + 128, XZPos.z - 32), XZPos.z - 32);
		Specs.Wrld->GetSpawnInterface()->Spawn(&SS);
		// spawn pfx at mouse pos
		SS.Pos = IMATH::Vector(XZPos.x + 128, Specs.Wrld->GetSurfaceY(XZPos.x + 128, XZPos.z + 32), XZPos.z + 32);
		Specs.Wrld->GetSpawnInterface()->Spawn(&SS);
		// spawn pfx at mouse pos
		SS.Pos = IMATH::Vector(XZPos.x - 128, Specs.Wrld->GetSurfaceY(XZPos.x - 128, XZPos.z + 32), XZPos.z + 32);
		Specs.Wrld->GetSpawnInterface()->Spawn(&SS);
	}
	// Decal
	if(Type == BRUSH_TYPE_IDOBJECT && TypeID == IDTYPE_DECAL)
	{
		SpawnInterface::SpawnInterfaceSpecs SS;
		SS.Type = IDTYPE_PARTICLEFX;
		SS.ID = IDPFX_SMALLRISINGSMOKE;
		// mouse pos on surface
		IMATH::Vector2D MPos = Specs.Mouse->GetPos();
		IMATH::Vector2DXZ XZPos = Specs.Wrld->GetSurfaceXZ(MPos);
		// spawn pfx at mouse pos
		SS.Pos = IMATH::Vector(XZPos.x + Specs.DLib->Decals[ID].Bounds[0].x, Specs.Wrld->GetSurfaceY(XZPos.x + Specs.DLib->Decals[ID].Bounds[0].x, XZPos.z + Specs.DLib->Decals[ID].Bounds[0].z), XZPos.z + Specs.DLib->Decals[ID].Bounds[0].z);
		Specs.Wrld->GetSpawnInterface()->Spawn(&SS);
		// spawn pfx at mouse pos
		SS.Pos = IMATH::Vector(XZPos.x + Specs.DLib->Decals[ID].Bounds[1].x, Specs.Wrld->GetSurfaceY(XZPos.x + Specs.DLib->Decals[ID].Bounds[1].x, XZPos.z + Specs.DLib->Decals[ID].Bounds[1].z), XZPos.z + Specs.DLib->Decals[ID].Bounds[1].z);
		Specs.Wrld->GetSpawnInterface()->Spawn(&SS);
		// spawn pfx at mouse pos
		SS.Pos = IMATH::Vector(XZPos.x + Specs.DLib->Decals[ID].Bounds[2].x, Specs.Wrld->GetSurfaceY(XZPos.x + Specs.DLib->Decals[ID].Bounds[2].x, XZPos.z + Specs.DLib->Decals[ID].Bounds[2].z), XZPos.z + Specs.DLib->Decals[ID].Bounds[2].z);
		Specs.Wrld->GetSpawnInterface()->Spawn(&SS);
		// spawn pfx at mouse pos
		SS.Pos = IMATH::Vector(XZPos.x + Specs.DLib->Decals[ID].Bounds[3].x, Specs.Wrld->GetSurfaceY(XZPos.x + Specs.DLib->Decals[ID].Bounds[3].x, XZPos.z + Specs.DLib->Decals[ID].Bounds[3].z), XZPos.z + Specs.DLib->Decals[ID].Bounds[3].z);
		Specs.Wrld->GetSpawnInterface()->Spawn(&SS);
		// spawn pfx at mouse pos
		SS.Pos = IMATH::Vector(XZPos.x, Specs.Wrld->GetSurfaceY(XZPos.x, XZPos.z), XZPos.z);
		Specs.Wrld->GetSpawnInterface()->Spawn(&SS);
	}
	int State = 0;
	try
	{
		State = 1;
		if(Sequence != MaxSeq)
		{
			State = 2;
			if(DS->Sp->GameTime - LastSpawn > 0.05f)
			{
				State = 3;
				LastSpawn = DS->Sp->GameTime;
				SpawnInterface::SpawnInterfaceSpecs SS;
				SS.Type = IDTYPE_PARTICLEFX;
				SS.ID = IDPFX_SMALLRISINGSMOKE;
				// mouse pos on surface
				IMATH::Vector2D MPos = Specs.Mouse->GetPos();
				IMATH::Vector2DXZ XZPos = Specs.Wrld->GetSurfaceXZ(MPos);
				State = 4;
				if(Type == BRUSH_TYPE_IDOBJECT && TypeID == IDTYPE_AREA)
				{
					State = 5;
					I3D::TerrainLine::TerrainLineSpecs TLS;
					TLS.Pos[0] = XZPos;
					TLS.Pos[1] = IMATH::Vector2DXZ(XZPos.x, Pos[Sequence].z);
					State = 6;
					TLS.TI = Specs.Wrld->GetTerrainInterface();
					TLS.TLR = Specs.Wrld->GetTerrainLineRenderer();
					State = 7;
					TLS.DS = Specs.DS;
					AreaLines[0]->Create(&TLS);
					TLS.Pos[0] = IMATH::Vector2DXZ(XZPos.x, Pos[Sequence].z);
					TLS.Pos[1] = Pos[Sequence];
					AreaLines[1]->Create(&TLS);
					TLS.Pos[0] = Pos[Sequence];
					TLS.Pos[1] = IMATH::Vector2DXZ(Pos[Sequence].x, XZPos.z);
					AreaLines[2]->Create(&TLS);
					TLS.Pos[0] = IMATH::Vector2DXZ(Pos[Sequence].x, XZPos.z);
					TLS.Pos[1] = XZPos;
					State = 8;
					AreaLines[3]->Create(&TLS);
					State = 9;
				}
				else
				{
					State = 10;
					for(int i = Sequence + 1; i < MaxSeq; i++)
					{
						// smoke emitters at Pos[]
						/*SS.Pos = IMATH::Vector(Pos[i].x, Specs.Wrld->GetSurfaceY(Pos[i].x, Pos[i].z), Pos[i].z);
						Specs.Wrld->GetSpawnInterface()->Spawn(&SS);*/
						// line from Pos[i - 1] to Pos[i]
						State = 11;
						I3D::TerrainLine::TerrainLineSpecs TLS;
						TLS.Pos[0] = Pos[i - 1];
						TLS.Pos[1] = Pos[i];
						TLS.TI = Specs.Wrld->GetTerrainInterface();
						TLS.TLR = Specs.Wrld->GetTerrainLineRenderer();
						TLS.DS = Specs.DS;
						State = 12;
						Lines[i]->Create(&TLS);
						State = 13;
					}
					State = 14;
					// line from XZPos to Pos[Sequence]
					if(Sequence != MaxSeq)
					{
						State = 15;
						I3D::TerrainLine::TerrainLineSpecs TLS;
						TLS.Pos[0] = XZPos;
						TLS.Pos[1] = Pos[Sequence];
						TLS.TI = Specs.Wrld->GetTerrainInterface();
						TLS.TLR = Specs.Wrld->GetTerrainLineRenderer();
						TLS.DS = Specs.DS;
						State = 16;
						Lines[Sequence]->Create(&TLS);
						State = 17;
					}
					State = 18;
					// line from XZPos to MaxSeq
					if(MaxSeq > 1 && Sequence <= MaxSeq - 2)
					{
						State = 19;
						I3D::TerrainLine::TerrainLineSpecs TLS;
						TLS.Pos[0] = XZPos;
						TLS.Pos[1] = Pos[MaxSeq - 1];
						TLS.TI = Specs.Wrld->GetTerrainInterface();
						TLS.TLR = Specs.Wrld->GetTerrainLineRenderer();
						TLS.DS = Specs.DS;
						State = 20;
						Lines[MaxSeq]->Create(&TLS);
						State = 21;
					}
				}
				State = 22;
				// spawn pfx at mouse pos
				SS.Pos = IMATH::Vector(XZPos.x, Specs.Wrld->GetSurfaceY(XZPos.x, XZPos.z), XZPos.z);
				State = 23;
				Specs.Wrld->GetSpawnInterface()->Spawn(&SS);
				State = 24;
			}
			State = 25;
		}
		State = 26;
	}
	catch(char *s)
	{
		throw s;
	}
	catch(...)
	{
		char Str[1024];
		sprintf(Str, "Brush::Render#Seq#%d", State);
		throw Str;
	}
}

void Brush::Compile()
{
	for(int i = 0; i < 4; ++i)
		AreaPreLines[i] = 0;
	for(int i = 0; i < PreLines.Count(); ++i)
		delete PreLines[i];
	PreLines.Clear();
	for(int i = 0; i < NewTDS.Count(); ++i)
		Specs.Wrld->AddTerrainNoHMap(&NewTDS[i]);
	for(int i = 0; i < NewWS.Count(); ++i)
		Specs.Wrld->AddWayNoHMap(&NewWS[i]);
	NewTDS.Clear();
	NewWS.Clear();
	Specs.Wrld->CreateHMap(Specs.MLib->Missions[Specs.Mission]->Seed);
}

bool Brush::Click(void)
{
	if(Type == BRUSH_TYPE_IDOBJECT)
	{
		IMATH::Vector2D MPos = Specs.Mouse->GetPos();
		IMATH::Vector2DXZ XZPos = Specs.Wrld->GetSurfaceXZ(MPos);
		if(Sequence)
		{
			Pos[--Sequence] = XZPos;
			return true;
		}
		else
		{
			switch(TypeID)
			{
			case IDTYPE_TERRAIN:
				{
					Sequence = MaxSeq;
					I3D::TerrainDsc::TerrainDscSpecs TS;
					TS.ID = ID;
					TS.Pos[0] = Pos[0];
					TS.Pos[1] = Pos[1];
					TS.Pos[2] = XZPos;
					sprintf(TS.Name, "MissionEditor Terrain I:%d ID:%d", TS.ID, Specs.NextID++);
					Specs.MLib->Missions[Specs.Mission]->AddTerrainDsc(&TS);
					Specs.MLib->Missions[Specs.Mission]->NextID = Specs.NextID;
					/*LoadingScreen("Adding Terrain...");
					Specs.Wrld->AddTerrain(&TS);*/
					NewTDS.Add(TS);
					PreLines.Add(&(*(new I3D::TerrainLine) = *Lines[0]));
					PreLines.Add(&(*(new I3D::TerrainLine) = *Lines[1]));
					PreLines.Add(&(*(new I3D::TerrainLine) = *Lines[2]));
					SetTerrain(ID);
					return true;
				}
			case IDTYPE_WAY:
				{
					Sequence = MaxSeq;
					I3D::Way::WaySpecs WS;
					WS.ID = ID;
					WS.Pos1 = Pos[0];
					WS.Pos2 = XZPos;
					sprintf(WS.Name, "MissionEditor Way I:%d ID:%d", WS.ID, Specs.NextID++);
					Specs.MLib->Missions[Specs.Mission]->AddWay(&WS);
					Specs.MLib->Missions[Specs.Mission]->NextID = Specs.NextID;
					/*LoadingScreen("Adding Way...");
					Specs.Wrld->AddWay(&WS);*/
					NewWS.Add(WS);
					PreLines.Add(&(*(new I3D::TerrainLine) = *Lines[0]));
					SetWay(ID);
					return true;
				}
			case IDTYPE_AIRFIELD:
				{
					Airfield::AirfieldSpecs Specs;
					sprintf(Specs.Name, "MissionEditor Airfield %d", Brush::Specs.NextID++);
					Specs.Pos = XZPos;
					Airfield *AF = new Airfield(Specs);
					Brush::Specs.MLib->Missions[Brush::Specs.Mission]->AddAirfield(AF);
					Brush::Specs.MLib->Missions[Brush::Specs.Mission]->NextID = Brush::Specs.NextID;
					LoadingScreen("Adding Airfield...");
					Brush::Specs.Wrld->AddAirfield(AF);
					SetAirfield();
					return true;
				}
			case IDTYPE_DECAL:
				{
					Decal::DecalSpecs Specs;
					sprintf(Specs.Name, "MissionEditor Decal %d", Brush::Specs.NextID++);
					Specs.Pos = XZPos;
					//Specs.DLib = Brush::Specs.DLib;
					Specs.ID = ID;
					Decal *D = new Decal(Specs);
					Brush::Specs.MLib->Missions[Brush::Specs.Mission]->AddDecal(D);
					Brush::Specs.MLib->Missions[Brush::Specs.Mission]->NextID = Brush::Specs.NextID;
					LoadingScreen("Adding Decal...");
					Brush::Specs.Wrld->AddDecal(D);
					SetDecal(ID);
					return true;
				}
			case IDTYPE_AREA:
				{
					Sequence = MaxSeq;
					AreaEx = true;
					Area[0] = Pos[0];
					Area[1] = XZPos;
					if(Area[0].x > Area[1].x)
					{
						float swap = Area[0].x;
						Area[0].x = Area[1].x;
						Area[1].x = swap;
					}
					if(Area[0].z > Area[1].z)
					{
						float swap = Area[0].x;
						Area[0].x = Area[1].x;
						Area[1].x = swap;
					}
					I3D::TerrainLine::TerrainLineSpecs TLS;
					TLS.Pos[0] = Area[0];
					TLS.Pos[1] = IMATH::Vector2DXZ(Area[0].x, Area[1].z);
					TLS.TI = Specs.Wrld->GetTerrainInterface();
					TLS.TLR = Specs.Wrld->GetTerrainLineRenderer();
					TLS.DS = Specs.DS;
					TLS.Color = 0xFF00FFFF;
					// destroy old lines
					PreLines.Delete(AreaPreLines[0]);
					delete AreaPreLines[0];
					PreLines.Delete(AreaPreLines[1]);
					delete AreaPreLines[1];
					PreLines.Delete(AreaPreLines[2]);
					delete AreaPreLines[2];
					PreLines.Delete(AreaPreLines[3]);
					delete AreaPreLines[3];
					// create new lines
					AreaLines[0]->Create(&TLS);
					TLS.Pos[0] = IMATH::Vector2DXZ(Area[0].x, Area[1].z);
					TLS.Pos[1] = Area[1];
					AreaLines[1]->Create(&TLS);
					TLS.Pos[0] = Area[1];
					TLS.Pos[1] = IMATH::Vector2DXZ(Area[1].x, Area[0].z);
					AreaLines[2]->Create(&TLS);
					TLS.Pos[0] = IMATH::Vector2DXZ(Area[1].x, Area[0].z);
					TLS.Pos[1] = Area[0];
					AreaLines[3]->Create(&TLS);
					PreLines.Add(&(*(AreaPreLines[0] = new I3D::TerrainLine) = *AreaLines[0]));
					PreLines.Add(&(*(AreaPreLines[1] = new I3D::TerrainLine) = *AreaLines[1]));
					PreLines.Add(&(*(AreaPreLines[2] = new I3D::TerrainLine) = *AreaLines[2]));
					PreLines.Add(&(*(AreaPreLines[3] = new I3D::TerrainLine) = *AreaLines[3]));
					SetArea();
					return true;
				}
			default:
				// Tank
				SpawnInterface::SpawnInterfaceSpecs SS;
				memset(&SS, 0, sizeof(SS));
				SS.Type = TypeID;
				SS.ID = ID;
				SS.Owner = Owner;
				SS.Angle = Angle;
				SS.Pos = IMATH::Vector(XZPos.x, Specs.TLib->Tanks[ID].IsShip ? WATERLEVEL : Specs.Wrld->GetSurfaceY(XZPos.x, XZPos.z), XZPos.z);
				sprintf(SS.Name, "MissionEditor Object O:%d T:%d I:%d ID:%d", SS.Owner, SS.Type, SS.ID, Specs.NextID++);
				Specs.Wrld->GetSpawnInterface()->Spawn(&SS);
				MissionLib::Mission::MissionObjectSpecs MOS;
				MOS.Type = TypeID;
				MOS.ID = ID;
				MOS.Owner = Owner;
				MOS.Pos = XZPos;
				MOS.Angle = Angle;
				sprintf(MOS.Name, SS.Name);
				Specs.MLib->Missions[Specs.Mission]->AddObject(&MOS);
				Specs.MLib->Missions[Specs.Mission]->NextID = Specs.NextID;
				SetTank(ID);
				return true;
			}
		}
	}
	return false;
}

void Brush::RightClick(void)
{
	Type = BRUSH_TYPE_NONE;
	RemoveMarkers();
	Sequence = MaxSeq;
}

}
