/***********************************************************/
/*                                                         */
/* Brush.h                                                 */
/*                                                         */
/* created 17.03.2003                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef BRUSH_H
#define BRUSH_H

#include "IDType.h"
#include "D3DStruct.h"
//#include "RenderableModel.h"
#include "TankLib.h"
#include "MouseCursor.h"
#include "World.h"
#include "MathStructures.h"
#include "MissionLib.h"
#include "DecalLib.h"
#include "Terrain.h"
#include "Way.h"

// www.infinitecode.com game library
namespace IGAME
{

class Brush
{
public:
	struct BrushSpecs
	{
		I3D::D3DStruct *DS;
		TankLib *TLib;
		DecalLib *DLib;
		MissionLib *MLib;
		int Mission;
		IGAME::World *Wrld;
		I3D::MouseCursor *Mouse;
		int NextID;
	};

private:

	enum
	{
		BRUSH_TYPE_NONE = 0,
		BRUSH_TYPE_IDOBJECT = 1,
		BRUSH_TYPE_TERRAIN = 2,
		BRUSH_TYPE_WAY = 3,
	};
	BrushSpecs Specs;
	int Type, TypeID, ID, Sequence, MaxSeq;
	int Owner;
	float Angle;
	bool AreaEx;
	IMATH::Vector2DXZ Pos[2], Area[2];
	I3D::TerrainLine *Lines[4], *AreaLines[4], *AreaPreLines[4];
	ISL::DynamicArray<I3D::TerrainLine *> PreLines;
	float LastSpawn;
	ISL::DynamicArray<I3D::TerrainDsc::TerrainDscSpecs> NewTDS;
	ISL::DynamicArray<I3D::Way::WaySpecs> NewWS;

	void LoadingScreen(const char *Str);
	void RemoveMarkers(void);

public:
	//I3D::RenderableModel RM;

	Brush(void)
	{
		memset(Lines, 0, 4 * sizeof(Lines[0]));
		memset(Area, 0, 4 * sizeof(Lines[0]));
		memset(AreaLines, 0, 4 * sizeof(AreaLines[0]));
		memset(AreaPreLines, 0, 4 * sizeof(AreaPreLines[0]));
	}

	void SetNextID(int ID)
	{
		Specs.NextID = ID;
	}

	void SetOwner(int Owner)
	{
		Brush::Owner = Owner;
	}

	void SetAngle(float Value)
	{
		Angle = Value;
	}

	float GetAngle() const
	{
		return Angle;
	}

	const IMATH::Vector2DXZ *GetArea()
	{
		return Area;
	}

	int GetNextID(void)
	{
		return Specs.NextID;
	}

	int GetOwner()
	{
		return Owner;
	}

	void SetMission(int Mission)
	{
		Specs.Mission = Mission;
	}

	void Create(BrushSpecs *Specs);
	void SetTank(int ID);
	void SetWay(int ID);
	void SetArea();
	void ClearArea();
	void SetAirfield();
	void SetDecal(int ID);
	void SetTerrain(int ID);
	void Render(I3D::D3DStruct *DS);
	bool Click(void);
	void RightClick(void);
	void Compile();
};

}

#endif
