/***********************************************************/
/*                                                         */
/* Button.cpp                                              */
/*                                                         */
/* created 01.03.2004                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "Button.h"
#include "TextureManager.h"
#include "Text.h"

// www.infinitecode.com 3d library
namespace I3D
{

void Button::Create(const Button::Specs &Sp)
{
	Button::Sp = Sp;
	if(!Sp.Texture)
		Button::Sp.Texture = TextureManager::GetTexture(Sp.DS, "Textures\\Buttons.bmp", 0, TM_FLAG_ALPHACC0);
	int IndexX = Sp.Index % Sp.TilesX;
	int IndexY = Sp.Index / Sp.TilesX;
	float DeltaX = 1.0f / Sp.TilesX;
	float DeltaY = 1.0f / Sp.TilesY;
	float x = (float)IndexX * DeltaX;
	float y = (float)IndexY * DeltaY;
	RS.AddVertex(Vertex(Vector(0, 0, 0), Vector(0, 0, -1), Vector2D(x, y + DeltaY)));
	RS.AddVertex(Vertex(Vector(1, 0, 0), Vector(0, 0, -1), Vector2D(x + DeltaX, y + DeltaY)));
	RS.AddVertex(Vertex(Vector(1, 1, 0), Vector(0, 0, -1), Vector2D(x + DeltaX, y)));
	RS.AddVertex(Vertex(Vector(0, 1, 0), Vector(0, 0, -1), Vector2D(x, y)));
	RS.CreateVBuf(Sp.DS->D3DDevice);
	RS.AddIndex(0);
	RS.AddIndex(1);
	RS.AddIndex(2);
	RS.AddIndex(0);
	RS.AddIndex(2);
	RS.AddIndex(3);
	RS.CreateIBuf(Sp.DS->D3DDevice);
}

void Button::Init3D(D3DStruct *DS, bool recurse)
{
	Button::Sp.Texture = TextureManager::GetTexture(Sp.DS, "Textures\\Buttons.bmp", 0, TM_FLAG_ALPHACC0);
	RS.CreateVBuf(Sp.DS->D3DDevice);
	RS.CreateIBuf(Sp.DS->D3DDevice);
}

void Button::Render(D3DStruct *DS)
{
	// prepare
	D3DMATRIX IM = {1,0,0,0,  0,1,0,0,  0,0,1,0,  0,0,0,1};
	DS->D3DDevice->SetTransform(D3DTS_VIEW, &IM);
	DS->D3DDevice->SetTransform(D3DTS_PROJECTION, &IM);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
	DS->D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATER);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHAREF, 0x10);
	DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
	// render
	DS->D3DDevice->SetTexture(0, Sp.Texture);
	Matrix M;
	M.SetScaleTranslationMatrix(IMATH::Vector(Sp.Size.x, Sp.Size.y, 0), IMATH::Vector(Sp.Pos.x, Sp.Pos.y, 0));
	DS->D3DDevice->SetTransform(D3DTS_WORLD, &M);
	RS.RenderIndex(DS->D3DDevice);
	if(*Sp.Text)
		Text::DrawString(DS, Sp.Text, Sp.Pos.x + 0.2f * Sp.Size.x, Sp.Pos.y + 0.8f * Sp.Size.y, 0.6f * Sp.Size.y, 0.6f * Sp.Size.y);
	// reset
	DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, TRUE);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, FALSE);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
	DS->D3DDevice->SetTransform(D3DTS_VIEW, &DS->ViewMtx);
	DS->D3DDevice->SetTransform(D3DTS_PROJECTION, &DS->ProjMtx);
}

bool Button::Click(const IMATH::Vector2D &Pos)
{
	return Pos.x > Sp.Pos.x && Pos.x < Sp.Pos.x + Sp.Size.x
		&& Pos.y > Sp.Pos.y && Pos.y < Sp.Pos.y + Sp.Size.y;
}

}
