/***********************************************************/
/*                                                         */
/* COLOR.cpp                                               */
/*                                                         */
/* created 23.03.2003                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef COLOR_H
#define COLOR_H

// www.infinitecode.com 3d library
namespace I3D
{

class Color
{
public:
	enum
	{
		ALPHA = 0,
		RED = 1,
		GREEN = 2,
		BLUE = 3
	};

	class Color_int4
	{
	public:
		Color_int4(void)
		{
			memset(Data, 4, sizeof(int));
		}

		Color_int4(int a, int r, int g, int b)
		{
			Data[ALPHA] = a;
			Data[RED] = r;
			Data[GREEN] = g;
			Data[BLUE] = b;
			if(Data[ALPHA] < 0)
				Data[ALPHA] = 0;
			if(Data[ALPHA] > 0xFF)
				Data[ALPHA] = 0xFF;
			if(Data[RED] < 0)
				Data[RED] = 0;
			if(Data[RED] > 0xFF)
				Data[RED] = 0xFF;
			if(Data[GREEN] < 0)
				Data[GREEN] = 0;
			if(Data[GREEN] > 0xFF)
				Data[GREEN] = 0xFF;
			if(Data[BLUE] < 0)
				Data[BLUE] = 0;
			if(Data[BLUE] > 0xFF)
				Data[BLUE] = 0xFF;
		}

		int Data[4];
	};

	static Color_int4 ToColor_int4(unsigned int Color)
	{
		Color_int4 i4;
		i4.Data[ALPHA] = Color >> 24;
		i4.Data[RED] = (Color >> 16) & 0xFF;
		i4.Data[GREEN] = (Color >> 8) & 0xFF;
		i4.Data[BLUE] = Color & 0xFF;
		return i4;
	}
	static unsigned int ToColor_uint(Color_int4 Color)
	{
		if(Color.Data[ALPHA] < 0)
			Color.Data[ALPHA] = 0;
		if(Color.Data[ALPHA] > 0xFF)
			Color.Data[ALPHA] = 0xFF;
		if(Color.Data[RED] < 0)
			Color.Data[RED] = 0;
		if(Color.Data[RED] > 0xFF)
			Color.Data[RED] = 0xFF;
		if(Color.Data[GREEN] < 0)
			Color.Data[GREEN] = 0;
		if(Color.Data[GREEN] > 0xFF)
			Color.Data[GREEN] = 0xFF;
		if(Color.Data[BLUE] < 0)
			Color.Data[BLUE] = 0;
		if(Color.Data[BLUE] > 0xFF)
			Color.Data[BLUE] = 0xFF;
		return (Color.Data[ALPHA] << 24) + (Color.Data[RED] << 16) + (Color.Data[GREEN] << 8) + Color.Data[BLUE];
	}

	static unsigned int Brightness(unsigned int Color, int Amount)
	{
		Color_int4 i4 = ToColor_int4(Color);
		i4.Data[ALPHA] += Amount;
		i4.Data[RED] += Amount;
		i4.Data[GREEN] += Amount;
		i4.Data[BLUE] += Amount;
		return ToColor_uint(i4);
	}

	static unsigned int Blend(unsigned int Color1, unsigned int Color2, float Alpha)
	{
		Color_int4 i4 = ToColor_int4(Color1);
		Color_int4 i4_2 = ToColor_int4(Color2);
		i4.Data[ALPHA] = (int)(Alpha * i4.Data[ALPHA] + (1 - Alpha) * i4_2.Data[ALPHA]);
		i4.Data[RED] = (int)(Alpha * i4.Data[RED] + (1 - Alpha) * i4_2.Data[RED]);
		i4.Data[GREEN] = (int)(Alpha * i4.Data[GREEN] + (1 - Alpha) * i4_2.Data[GREEN]);
		i4.Data[BLUE] = (int)(Alpha * i4.Data[BLUE] + (1 - Alpha) * i4_2.Data[BLUE]);
		return ToColor_uint(i4);
	}
};

}

#endif
