/***********************************************************/
/*                                                         */
/* D3DWindow.cpp                                           */
/*                                                         */
/* created 13.07.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "D3DWindow.h"

// Debug
#include "MessageWindow.h"
#include "LogFile.h"

#include <stdio.h>
#include <stdlib.h>
#include <sstream>

// www.infinitecode.com 3d library
namespace I3D
{

// lookup table for HWND and D3DWindow
ISL::HashTable<HWND, D3DWindow *, 10, 10> D3DWindow::HWndTable;

D3DWindow::D3DWindow(void)
:HWnd(NULL)
{
	ITL::LogFile::Clear(DBG_D3D);
}

D3DWindow::~D3DWindow(void)
{
	if(HWnd) Destroy();
}

// called by windows to send window messages
LRESULT CALLBACK D3DW_WndProc(HWND HWnd, UINT Message, WPARAM WParam, LPARAM LParam)
{
	D3DWindow *Wnd = D3DWindow::HWndTable.GetValue(HWnd);
	if(Wnd) return Wnd->OnWindowMessage(Message, WParam, LParam);
	return DefWindowProc(HWnd, Message, WParam, LParam);
}

void D3DWindow::Create(int WindowStyle, char *Caption, int dx, int dy)
{
	ITL::LogFile::Msg(DBG_D3D, "D3DWindow::Create:");
	DWCreateWindow(WindowStyle, Caption, dx, dy);
	DWCreateDirect3D();
}

void D3DWindow::DWCreateWindow(int WindowStyle, char *Caption, int dx, int dy)
{
	ITL::LogFile::Msg(DBG_D3D, "D3DWindow::DWCreateWindow:");
	strcpy(WndClassName, "I3D::D3DWindow");
	D3DWindow::Instance = Instance;
	D3DWindow::Pos.WindowStyle = WindowStyle;
	// create Window class
	WinCl.hInstance = Instance;
	WinCl.lpszClassName = WndClassName;
	WinCl.lpfnWndProc = D3DW_WndProc;
	WinCl.style = 0;//CS_NOCLOSE;
	WinCl.hIcon = (HICON)LoadImage(Instance, "Tank4.ico", IMAGE_ICON, 0, 0, LR_LOADFROMFILE);
	WinCl.hCursor = 0;//LoadCursor(NULL, IDC_ARROW);
	WinCl.lpszMenuName = NULL;
	WinCl.cbClsExtra = 0;
	WinCl.cbWndExtra = 0;
	WinCl.hbrBackground = NULL;//(HBRUSH) GetStockObject(LTGRAY_BRUSH);
	// set client size
	Pos.Clientdx = dx;
	Pos.Clientdy = dy;
	DesiredPos = Pos;
	// create a window
	if(RegisterClass(&WinCl))
	{
		HMODULE Module = GetModuleHandle(NULL);
		HWnd = CreateWindowEx(
			/*WS_EX_TOPMOST*/0, WndClassName, Caption, WS_OVERLAPPEDWINDOW, 0, 0, Pos.Clientdx, Pos.Clientdy,
			HWND_DESKTOP, NULL, Module, NULL);
		if(!HWnd)
			throw("D3DWindow::DWCreateWindow: CreateWindowEx failed");
		ShowWindow(HWnd, WindowStyle);
	}
	else
	{
		throw D3DWException("D3DWindow::DWCreateWindow: Can't register window class");
	}
	// get client size
	RECT rect;
	GetClientRect(HWnd, &rect);
	Pos.Clientdx = rect.right - rect.left;
	Pos.Clientdy = rect.bottom - rect.top;
	DesiredPos = Pos;
	// add the window to the hwnd-d3dwindow lookup table
	HWndTable.Add(HWnd, this);
}

void D3DWindow::Destroy(void)
{
	ITL::LogFile::Msg(DBG_D3D, "D3DWindow::Destroy:");
	DWDestroyDirect3D();
	DWDestroyWindow();
}

void D3DWindow::DWDestroyWindow(void)
{
	ITL::LogFile::Msg(DBG_D3D, "D3DWindow::DWDestroyWindow:");
	HWND OldHWnd = HWnd;
	DestroyWindow(HWnd);
	HWnd = NULL;
	HWndTable.DeleteKey(OldHWnd);
	UnregisterClass(WndClassName, Instance);
}

void D3DWindow::ResetDirect3D(void)
{
	int State = 0;
	try
	{
		/*if(DesiredPos.WindowStyle != SW_SHOWMAXIMIZED)
		{
			ITL::LogFile::Msg(DBG_D3D, "ResetDirect3D: DWDestroyDirect3D");
			DWDestroyDirect3D();
			ITL::LogFile::Msg(DBG_D3D, "ResetDirect3D: DWCreateDirect3D");
			DWCreateDirect3D();
			Pos = DesiredPos;
			D3DDevice->SetRenderState(D3DRS_AMBIENT, 0xFFFFFFFF);
			ITL::LogFile::Msg(DBG_D3D, "ResetDirect3D: SendEvent");
			SendEvent(D3DWEvent(D3DW_EVENT_RESET));
			ITL::LogFile::Msg(DBG_D3D, "ResetDirect3D: done");
			return;
		}*/
		/*DWDestroyDirect3D();
		return;*/

		State = 1;
		ITL::LogFile::Msg(DBG_D3D, "D3DWindow::ResetDirect3D:");

		// create a d3d device
		D3DPRESENT_PARAMETERS D3DPP = GetParams(DesiredPos.WindowStyle == SW_SHOWMAXIMIZED);

		/*D3DPP.BackBufferCount = 1;
		D3DPP.hDeviceWindow  = HWnd;
		if(DesiredPos.WindowStyle == SW_SHOWMAXIMIZED && GetFocus() == HWnd)
		{
			// fullscreen
			ITL::LogFile::Msg(DBG_D3D, "ResetDirect3D: mode: fullscreen");
			D3DPP.Windowed = false;
			D3DPP.BackBufferWidth = Mode.Width;
			D3DPP.BackBufferHeight = Mode.Height;
			D3DPP.BackBufferFormat = Mode.Format;
			D3DPP.FullScreen_RefreshRateInHz = Mode.RefreshRate;
 		}
		else
		{
			// windowed
			ITL::LogFile::Msg(DBG_D3D, "ResetDirect3D: mode: windowed");
			D3DDISPLAYMODE D3DDM;
			memset(&D3DDM, 0, sizeof(D3DDM));
			State = 2;
			if(D3D->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &D3DDM) != D3D_OK)
			{
				ITL::LogFile::Msg(DBG_D3D, "ResetDirect3D: Couldn't get displaymode (D3D->GetAdapterDisplayMode)");
				throw("ResetDirect3D: Couldn't get displaymode (D3D->GetAdapterDisplayMode)");
			}
			State = 3;
			D3DPP.Windowed = true;

			D3DPP.BackBufferFormat = D3DDM.Format;
			D3DPP.BackBufferWidth = DesiredPos.Clientdx;
			D3DPP.BackBufferHeight = DesiredPos.Clientdy;
			D3DPP.FullScreen_RefreshRateInHz = D3DPRESENT_RATE_DEFAULT;
		}
		D3DPP.AutoDepthStencilFormat = D3DFMT_D16; // TODO: 32 bit?*/
		static bool b = true;
		State = 4;
		{
			std::ostringstream oss;
			oss << "D3DWindow::ResetDirect3D: new Backbuffer Size: (" << D3DPP.BackBufferWidth << ", " << D3DPP.BackBufferHeight << ")";
			ITL::LogFile::Msg(DBG_D3D, oss.str().c_str());
		}
		//Sleep(500);
		HRESULT ResetResult;
		if(D3DDevice) // only if there is already a device
			ResetResult = D3DDevice->Reset(&D3DPP);
		State = 5;
		// ignore errors
		if(!D3DDevice || ResetResult != D3D_OK)
		{
			if(D3DDevice) // only if there is already a device
			{
				switch(ResetResult)
				{
				case D3DERR_INVALIDCALL:
					ITL::LogFile::Msg(DBG_D3D, "ResetDirect3D: D3DDevice->Reset failed (D3DERR_INVALIDCALL)");
					break;
				case D3DERR_OUTOFVIDEOMEMORY:
					ITL::LogFile::Msg(DBG_D3D, "ResetDirect3D: D3DDevice->Reset failed (D3DERR_OUTOFVIDEOMEMORY)");
					break;
				case E_OUTOFMEMORY:
					ITL::LogFile::Msg(DBG_D3D, "ResetDirect3D: D3DDevice->Reset failed (E_OUTOFMEMORY)");
					break;
				default:
					ITL::LogFile::Msg(DBG_D3D, "ResetDirect3D: D3DDevice->Reset failed (unknown reason)");
					break;
				}
				ITL::LogFile::Msg(DBG_D3D, "ResetDirect3D: D3DDevice->Reset failed, trying manual restart");
				State = 50;
				DWDestroyDirect3D();
			}
			State = 52;
			DWCreateDirect3D();
			State = 6;
			if(D3DDevice)
			{
				HRESULT CoopResult = D3DDevice->TestCooperativeLevel();
				State = 7;
				switch(CoopResult)
				{
				case D3DERR_DEVICENOTRESET:
					ITL::LogFile::Msg(DBG_D3D, "ResetDirect3D: D3DDevice->TestCooperativeLevel returned D3DERR_DEVICENOTRESET");
					break;
				case D3DERR_DEVICELOST:
					ITL::LogFile::Msg(DBG_D3D, "ResetDirect3D: D3DDevice->TestCooperativeLevel returned D3DERR_DEVICELOST");
					break;
				default:
					ITL::LogFile::Msg(DBG_D3D, "ResetDirect3D: D3DDevice->TestCooperativeLevel succeded");
					break;
				}
			}
			// send message
			//if(!CoopResult)
			SendEvent(D3DWEvent(D3DW_EVENT_RESET));
			State = 8;
			/*switch(ResetResult)
			{
			case D3DERR_INVALIDCALL:
				ITL::LogFile::Msg(DBG_D3D, "ResetDirect3D: D3DDevice->Reset failed (D3DERR_INVALIDCALL)");
				throw("ResetDirect3D: D3DDevice->Reset failed (D3DERR_INVALIDCALL)");
			case D3DERR_OUTOFVIDEOMEMORY:
				ITL::LogFile::Msg(DBG_D3D, "ResetDirect3D: D3DDevice->Reset failed (D3DERR_OUTOFVIDEOMEMORY)");
				throw("ResetDirect3D: D3DDevice->Reset failed (D3DERR_OUTOFVIDEOMEMORY)");
			case E_OUTOFMEMORY:
				ITL::LogFile::Msg(DBG_D3D, "ResetDirect3D: D3DDevice->Reset failed (E_OUTOFMEMORY)");
				throw("ResetDirect3D: D3DDevice->Reset failed (E_OUTOFMEMORY)");
			default:
				;//throw D3DWException("ResetDirect3D: D3DDevice->Reset failed");
			}
			*/
			State = 9;
		}
		else
		{
			// Reset was successful
			State = 10;
			Pos = DesiredPos;
			State = 11;
			ITL::LogFile::Msg(DBG_D3D, "ResetDirect3D: Reset successful");
			SendEvent(D3DWEvent(D3DW_EVENT_RESET));
		}
		State = 12;
	}
	catch(char *s)
	{
		throw(s);
	}
	catch(...)
	{
		char Str[1024];
		sprintf(Str, "D3DWindow::ResetDirect3D#%d", State);
		throw(Str);
	}
}

LRESULT CALLBACK D3DWindow::OnWindowMessage(UINT Message, WPARAM WParam, LPARAM LParam)
{
	switch (Message){
		case WM_CREATE:
			break;
		case WM_DESTROY:
			PostQuitMessage(0);
			break;
		/*case WM_ACTIVATE:
			if(!WParam)
			{
				DesiredPos.WindowStyle = SW_SHOWNORMAL;
				ResetDirect3D();
			}
			break;*/
		case WM_SIZE:
/*			switch(WParam)
			{
			case SIZEFULLSCREEN:
				// maximized*/
			//DesiredPos.WindowStyle = (WParam == SIZEFULLSCREEN) ? SW_SHOWMAXIMIZED : SW_SHOWNORMAL;
			if(WParam == SIZEFULLSCREEN)
				ITL::LogFile::Msg(DBG_D3D, "D3DWindow::OnWindowMessage: WM_SIZE with fullscreen");
			else
				ITL::LogFile::Msg(DBG_D3D, "D3DWindow::OnWindowMessage: WM_SIZE, not fullscreen");
/*				break;
			case SIZENORMAL:
				DesiredPos.WindowStyle = SW_SHOWNORMAL;
				break;
			case SIZEICONIC:
				DesiredPos.WindowStyle = SW_SHOWNORMAL;
				break;
			}*/
			// resize
			DesiredPos.Clientdx = LParam & 0xFFFF;
			DesiredPos.Clientdy = LParam >> 16;
			{
				std::ostringstream oss;
				oss << "new Size: (" << DesiredPos.Clientdx << ", " << DesiredPos.Clientdy << ")";
				ITL::LogFile::Msg(DBG_D3D, oss.str().c_str());
			}
			//if(DesiredPos.WindowStyle != Pos.WindowStyle || !Pos.WindowStyle)
			ITL::LogFile::Msg(DBG_D3D, "D3DWindow::OnWindowMessage: calling ResetDirect3D");
			//if(WParam == SIZEFULLSCREEN)
			ResetDirect3D();
			//SendEvent(D3DWEvent(D3DW_EVENT_WINDOWSEVENT, Message, WParam, LParam));
			break;
		case WM_SETFOCUS:
			//ResetDirect3D();
			break;
	}
	SendEvent(D3DWEvent(D3DW_EVENT_WINDOWSEVENT, Message, WParam, LParam));
	return DefWindowProc(HWnd, Message, WParam, LParam);
}

void D3DWindow::SetFullscreen(bool Value)
{
	if(Value != (DesiredPos.WindowStyle == SW_SHOWMAXIMIZED))
		ToggleFullscreen();
}

void D3DWindow::ToggleFullscreen()
{
	if(DesiredPos.WindowStyle == SW_SHOWMAXIMIZED)
	{
		//DWDestroyDirect3D();
		DesiredPos.WindowStyle = SW_NORMAL;
		ITL::LogFile::Msg(DBG_D3D, "D3DWindow::ToggleFullscreen: calling ShowWindow(SW_NORMAL)");
		ShowWindow(HWnd, DesiredPos.WindowStyle);
		/*DWORD style = GetWindowLong(HWnd, GWL_STYLE);
		style |= WS_OVERLAPPEDWINDOW;
		SetWindowLong(HWnd, GWL_STYLE, style);
		UINT flags = SWP_NOACTIVATE | SWP_NOOWNERZORDER | SWP_NOSENDCHANGING |
                     SWP_NOZORDER | SWP_FRAMECHANGED;*/
		//DWCreateDirect3D();
		SetWindowPos(HWnd, HWND_TOP, 0, 0, 800, 600, 0);
	}
	else
	{
		DesiredPos.WindowStyle = SW_SHOWMAXIMIZED;
		ITL::LogFile::Msg(DBG_D3D, "D3DWindow::ToggleFullscreen: calling ShowWindow(SW_SHOWMAXIMIZED)");
		ShowWindow(HWnd, DesiredPos.WindowStyle);
	}
	//ITL::LogFile::Msg(DBG_D3D, "D3DWindow::ToggleFullscreen: calling ResetDirect3D");
	/*RECT Rect;
	GetClientRect(HWnd, &Rect);
	DesiredPos.Clientdx = Rect.right - Rect.left;
	DesiredPos.Clientdy = Rect.bottom - Rect.top;*/
	ResetDirect3D();
}

D3DPRESENT_PARAMETERS D3DWindow::GetParams(bool Fullscreen)
{
	D3DPRESENT_PARAMETERS D3DPP;
	// set D3DPP
	memset(&D3DPP, 0, sizeof(D3DPP));
	D3DPP.BackBufferCount = 1;
	D3DPP.AutoDepthStencilFormat = D3DFMT_D16;
	D3DPP.EnableAutoDepthStencil = true; // z buffer
	D3DPP.SwapEffect = D3DSWAPEFFECT_DISCARD;
	D3DPP.FullScreen_RefreshRateInHz = D3DPRESENT_INTERVAL_DEFAULT;
	if(Fullscreen)
	{
		// fullscreen
		ITL::LogFile::Msg(DBG_D3D, "mode: fullscreen");
		D3DPP.Windowed = false;
		D3DPP.BackBufferWidth = Mode.Width;
		D3DPP.BackBufferHeight = Mode.Height;
		D3DPP.BackBufferFormat = Mode.Format;
		D3DPP.FullScreen_RefreshRateInHz = Mode.RefreshRate;
		D3DPP.hDeviceWindow = 0;
	}
	else
	{
		// windowed
		ITL::LogFile::Msg(DBG_D3D, "mode: windowed");
		D3DDISPLAYMODE D3DDM;
		memset(&D3DDM, 0, sizeof(D3DDM));
		if(D3D->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &D3DDM) != D3D_OK)
		{
			ITL::LogFile::Msg(DBG_D3D, "DWCreateDirect3D: Couldn't get displaymode (D3D->GetAdapterDisplayMode)");
			throw D3DWException("DWCreateDirect3D: Couldn't get displaymode (D3D->GetAdapterDisplayMode)");
		}
		D3DPP.Windowed = true;
		D3DPP.BackBufferFormat = D3DDM.Format;
		D3DPP.BackBufferWidth = DesiredPos.Clientdx;
		D3DPP.BackBufferHeight = DesiredPos.Clientdy;
		D3DPP.FullScreen_RefreshRateInHz = D3DPRESENT_RATE_DEFAULT;
		D3DPP.hDeviceWindow = HWnd;
	}
	return D3DPP;
}

void D3DWindow::DWCreateDirect3D(void)
{
	int State = 0;
	try
	{
		// create a d3d object
		ITL::LogFile::Msg(DBG_D3D, "D3DWindow::DWCreateDirect3D:");
		D3D = Direct3DCreate9(D3D_SDK_VERSION);
		if(!D3D)
		{
			ITL::LogFile::Msg(DBG_D3D, "DWCreateDirect3D: Direct3DCreate8 returned nullpointer");
			throw D3DWException("DWCreateDirect3D: Direct3DCreate8 returned nullpointer");
		}

		// enumerate display modes
		D3DDISPLAYMODE Enum;
		Mode.Width = 0;
		char mstr[1024];
		for(int i = 0; i < (int)D3D->GetAdapterModeCount(D3DADAPTER_DEFAULT, D3DFMT_X8R8G8B8); i++)
		{
			D3D->EnumAdapterModes(D3DADAPTER_DEFAULT, D3DFMT_X8R8G8B8, i, &Enum);
			if(Enum.Width == 640 && Enum.Height == 480
				&& (Enum.Format == D3DFMT_R5G6B5 || Enum.Format == D3DFMT_A8R8G8B8 || Enum.Format == D3DFMT_X8R8G8B8))
			{
	/*					if(Enum.Width > Mode.Width) Mode = Enum;
						if(Enum.Width < CompMode.Width) CompMode = Enum;
						if(Enum.Width == Mode.Width
							&& (Enum.Format == D3DFMT_A8R8G8B8 || Enum.Format == D3DFMT_X8R8G8B8) && (Mode.Format == D3DFMT_R5G6B5)) Mode = Enum;
						if(Enum.Width == CompMode.Width
							&& (Enum.Format == D3DFMT_R5G6B5) && (CompMode.Format == D3DFMT_A8R8G8B8 || CompMode.Format == D3DFMT_X8R8G8B8)) CompMode = Enum;
						if(Enum.Width == Mode.Width
							&& Enum.Format == Mode.Format
							&& Enum.RefreshRate > Mode.RefreshRate)
							Mode = Enum;
						if(Enum.Width == CompMode.Width
							&& Enum.Format == CompMode.Format
							&& Enum.RefreshRate < CompMode.RefreshRate)
							CompMode = Enum;*/
				Mode = Enum;
				sprintf(mstr, "enum display mode (ok): %dx%d, %d bit, %d Hz", Enum.Width, Enum.Height, (Enum.Format == D3DFMT_R5G6B5)?16:32, Enum.RefreshRate);
				ITL::LogFile::Msg(DBG_D3D, mstr);
			}
			else
			{
				sprintf(mstr, "enum display mode (rejected): %dx%d, %d, %d Hz", Enum.Width, Enum.Height, Enum.Format, Enum.RefreshRate);
				ITL::LogFile::Msg(DBG_D3D, mstr);
			}
		}
		if(Mode.Width == 0)
		{
			ITL::LogFile::Msg(DBG_D3D, "DWCreateDirect3D: Display Mode Enumeration returned no usable display modes");
			throw D3DWException("DWCreateDirect3D: Display Mode Enumeration returned no usable display modes");
		}
		sprintf(mstr, "selected display mode: %dx%d, %d bit, %d Hz", Mode.Width, Mode.Height, (Mode.Format == D3DFMT_R5G6B5)?16:32, Mode.RefreshRate);
		ITL::LogFile::Msg(DBG_D3D, mstr);

		// create a d3d device
		D3DPRESENT_PARAMETERS D3DPP = GetParams(DesiredPos.WindowStyle == SW_SHOWMAXIMIZED);
		sprintf(mstr, "mode parameters: %s", D3DPP.Windowed ? "windowed" : "fullscreen");
		ITL::LogFile::Msg(DBG_D3D, mstr);

		State = 20;
		// create device
		if(D3D->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, HWnd, D3DCREATE_HARDWARE_VERTEXPROCESSING, &D3DPP, &D3DDevice) == D3D_OK)
		{
			State = 21;
			ITL::LogFile::Msg(DBG_D3D, "T&L supported. Using hardware vertex processing");
			ITL::LogFile::Msg(DBG_D3D, "CreateDevice successful");
		}
		else if(D3D->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, HWnd, D3DCREATE_MIXED_VERTEXPROCESSING, &D3DPP, &D3DDevice) == D3D_OK)
		{
			State = 22;
			ITL::LogFile::Msg(DBG_D3D, "T&L partially supported. Using mixed vertex processing");
			ITL::LogFile::Msg(DBG_D3D, "CreateDevice successful");
		}
		else if(D3D->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, HWnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING, &D3DPP, &D3DDevice) == D3D_OK)
		{
			State = 23;
			ITL::LogFile::Msg(DBG_D3D, "T&L not supported. Using software vertex processing");
			ITL::LogFile::Msg(DBG_D3D, "CreateDevice successful");
		}
		else
		{
			State = 24;
			ITL::LogFile::Msg(DBG_D3D, "DWCreateDirect3D: Can't create device");
			D3DDevice = 0;
			State = 25;
			//throw D3DWException("DWCreateDirect3D: Can't create device");
		}
		State = 25;
		Pos = DesiredPos;
		State = 26;
	}
	catch(char *s)
	{
		throw(s);
	}
	catch(...)
	{
		char Str[1024];
		sprintf(Str, "D3DWindow::ResetDirect3D#%d", State);
		throw(Str);
	}
}

void D3DWindow::DWDestroyDirect3D(void)
{
	ITL::LogFile::Msg(DBG_D3D, "D3DWindow::DWDestroyDirect3D:");
	RELEASE(D3DDevice);
	RELEASE(D3D);
	ITL::LogFile::Msg(DBG_D3D, "D3DWindow::DWDestroyDirect3D successful");
}

}
