/***********************************************************/
/*                                                         */
/* DSSound.cpp                                             */
/*                                                         */
/* created 08.05.2003                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "DSSound.h"

#include <stdio.h>

// www.infinitecode.com audio library
namespace IAU
{

DSSound::~DSSound(void)
{
	Destroy();
}

void DSSound::CheckEof(ifstream *f, const char *Filename)
{
	if(f->eof())
	{
		char Str[1024];
		sprintf("DSSound::Create: unexpected end of file in %s", Filename);
		throw(Str);
	}
}

void DSSound::Create(LPDIRECTSOUND8 DS, const char *Filename, int MaxInstances)
{
	if(!DS)
	{
		Valid = false;
		return;
	}
	DSSound::MaxInstances = MaxInstances;
	CurrentBuffer = 0;
	// create buffers
	Buffers = new LPDIRECTSOUNDBUFFER[MaxInstances];

	strcpy(DSSound::Filename, Filename);
	// open file
	ifstream f(Filename, ios::binary);
	f.seekg(0, ios::end);
	//int Filesize = f.tellg();
	int DataSize;

	unsigned short USData;
	unsigned UData;

	// file format
	WAVEFORMATEX WF;
	memset(&WF, 0, sizeof(WF));

	// find blocks
	f.seekg(0, ios::beg);
	char fmt[4];
	// check for riff
	f.read((char *)fmt, 4);
	if(strnicmp(fmt, "riff", 4))
	{
		char Str[1024];
		sprintf(Str, "DSSound::Create: can't find riff block in %s", Filename);
		throw(Str);
	}
	CheckEof(&f, Filename);
	// find fmt
	f.seekg(12, ios::beg);
	f.read((char *)fmt, 4);
	while(strnicmp(fmt, "fmt ", 4))
	{
		int BlockSize;
		f.read((char *)&BlockSize, 4);
		f.seekg((int)f.tellg() + BlockSize, ios::beg);
		CheckEof(&f, Filename);
		f.read((char *)fmt, 4);
	}
	// skip fmt block size
	int BlockSize;
	f.read((char *)&BlockSize, 4);
	int pos = (int)f.tellg();
	//f.seekg((int)f.tellg() + 4, ios::beg);
	
	// read format
	// format tag
	f.read((char *)&USData, 2);
	WF.wFormatTag = USData;
	// channels
	f.read((char *)&USData, 2);
	WF.nChannels = USData;
	// samples
	f.read((char *)&UData, 4);
	WF.nSamplesPerSec = UData;
	// bytes per sec
	f.read((char *)&UData, 4);
	WF.nAvgBytesPerSec = UData;
	// block align
	f.read((char *)&USData, 2);
	WF.nBlockAlign = USData;
	// bits per sample
	f.read((char *)&USData, 2);
	WF.wBitsPerSample = USData;
	// jump to next block
	f.seekg(pos + BlockSize, ios::beg);

	// find data
	f.read((char *)fmt, 4);
	while(strnicmp(fmt, "data", 4))
	{
		int BlockSize;
		f.read((char *)&BlockSize, 4);
		f.seekg((int)f.tellg() + BlockSize, ios::beg);
		CheckEof(&f, Filename);
		f.read((char *)fmt, 4);
	}
	// read data block size
	f.read((char *)&UData, 4);
	DataSize = UData;

	// buffer specs
	DSBUFFERDESC BD;
	memset(&BD, 0, sizeof(BD));
	BD.dwSize = sizeof(BD);
	BD.dwFlags = DSBCAPS_CTRLPAN | DSBCAPS_CTRLVOLUME; // | DSBCAPS_CTRLFREQUENCY;
	BD.dwBufferBytes = DataSize;
	BD.lpwfxFormat = &WF;
	// create buffer
	if(DS->CreateSoundBuffer(&BD, &Buffers[0], 0) == DS_OK)
	{
		// load data
		unsigned char *Data;
		int Size;
		if(Buffers[0]->Lock(0, 0, (void **)&Data, (LPDWORD)&Size, 0, 0, DSBLOCK_ENTIREBUFFER) != DS_OK)
			throw("DSSound::Create Locking of sound buffer failed");
		// copy
		f.read((char *)Data, min(DataSize, Size));
		Buffers[0]->Unlock(Data, Size, 0, 0);

		if(MaxInstances > 1)
		{
			for(int i = 1; i < MaxInstances; i++)
				DS->DuplicateSoundBuffer(Buffers[0], &Buffers[i]);
		}
		Valid = true;
	}
	else
		Valid = false;
}

void DSSound::Destroy(void)
{
	for(int i = 0; i < MaxInstances; i++)
		RELEASE(Buffers[i]);
	delete[] Buffers;
	Buffers = 0;
}

void DSSound::Play(float Direction)
{
	if(!Valid)
		return;
	for(int i = 0; i < MaxInstances; i++)
	{
		const int Index = (i + CurrentBuffer) % MaxInstances;
		unsigned Status;
		Buffers[Index]->GetStatus((LPDWORD)&Status);
		if(!(Status & DSBSTATUS_PLAYING))
		{
			switch(Buffers[Index]->Play(0, 0, 0))
			{
			case DSERR_BUFFERLOST:
				throw("DSSound::Play: DSERR_BUFFERLOST");
			case DSERR_INVALIDCALL:
				throw("DSSound::Play: DSERR_INVALIDCALL");
			case DSERR_INVALIDPARAM:
				throw("DSSound::Play: DSERR_INVALIDPARAM");
			case DSERR_PRIOLEVELNEEDED:
				throw("DSSound::Play: DSERR_PRIOLEVELNEEDED");
			}
			Buffers[Index]->SetPan((long)(Direction * 1000));
			Buffers[Index]->SetVolume(max(DSBVOLUME_MAX - 1000, DSBVOLUME_MIN));
			CurrentBuffer = Index + 1;
			break;
		}
	}
}

}
