/***********************************************************/
/*                                                         */
/* DShow.cpp                                               */
/*                                                         */
/* created 17.07.2004                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "DShow.h"

// www.infinitecode.com audio library
namespace IAU
{

DShow::DShow()
{
	if(CoInitialize(NULL) != S_OK)
		throw("DShow::DShow: CoInitialize failed");
    // Create the filter graph manager and query for interfaces.
    if(CoCreateInstance(CLSID_FilterGraph, NULL, CLSCTX_INPROC_SERVER, IID_IGraphBuilder, (void **)&pGraph) != S_OK)
		throw("DShow::DShow: CoCreateInstance failed");
    if(pGraph->QueryInterface(IID_IMediaControl, (void **)&pMediaControl) != S_OK)
		throw("DShow::DShow: QueryInterface IID_IMediaControl failed");
    if(pGraph->QueryInterface(IID_IMediaEvent, (void **)&pEvent) != S_OK)
		throw("DShow::DShow: QueryInterface IID_IMediaEventEx failed");
}

void DShow::Enqueue(const char *Filename)
{
	// build the graph
	unsigned short *Str = new unsigned short[strlen(Filename) + 1];
	MultiByteToWideChar(CP_ACP, 0, Filename, -1, (LPWSTR)Str, strlen(Filename) + 1);
    //pGraph->RenderFile(Str, NULL);
    if(pGraph->RenderFile((LPWSTR)Str, 0) != S_OK)
		throw("DShow::Play: RenderFile failed");
	delete[] Str;
    // Run the graph.
	pMediaControl->Run();
    /*if(pMediaControl->Run() != S_OK)
		throw("DShow::Play: Run failed");*/
/*	long evCode;
	pEvent->WaitForCompletion(INFINITE, &evCode);*/
}

void DShow::Play(const char *Filename)
{
	Stop();
	Enqueue(Filename);
}

void DShow::Stop()
{
	pMediaControl->Stop();
	// stop dshow
    pMediaControl->Release();
    pEvent->Release();
    pGraph->Release();
	// restart dshow
    if(CoCreateInstance(CLSID_FilterGraph, NULL, CLSCTX_INPROC_SERVER, IID_IGraphBuilder, (void **)&pGraph) != S_OK)
		throw("DShow::DShow: CoCreateInstance failed");
	if(pGraph->QueryInterface(IID_IMediaControl, (void **)&pMediaControl) != S_OK)
			throw("DShow::DShow: QueryInterface IID_IMediaControl failed");
    if(pGraph->QueryInterface(IID_IMediaEvent, (void **)&pEvent) != S_OK)
		throw("DShow::DShow: QueryInterface IID_IMediaEventEx failed");
}

DShow::~DShow()
{
	// clean up
    pMediaControl->Release();
    pEvent->Release();
    pGraph->Release();
    CoUninitialize();
}

}
