/***********************************************************/
/*                                                         */
/* DataStructures.h                                        */
/*                                                         */
/* created 20.07.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef DATASTRUCTURES_H
#define DATASTRUCTURES_H

#include "MathStructures.h"

// www.infinitecode.com 3d library
namespace I3D
{

// members
enum Indices
{
	BB_LEFT		= 0,
	BB_RIGHT	= 1,
	BB_BOTTOM	= 2,
	BB_TOP		= 3,
	BB_NEAR		= 4,
	BB_FAR		= 5
};

class BoundingBox // 3D: XYZ
{
public:

	float Data[6];

	BoundingBox(void) {}

	BoundingBox(float Left, float Right, float Bottom, float Top, float Near, float Far)
	{
		Data[BB_LEFT] = Left;
		Data[BB_RIGHT] = Right;
		Data[BB_BOTTOM] = Bottom;
		Data[BB_TOP] = Top;
		Data[BB_NEAR] = Near;
		Data[BB_FAR] = Far;
	}

	bool IsIn(const IMATH::Vector2DXZ *Pos)
	{
		return (Pos->x >= Data[BB_LEFT] && Pos->x <= Data[BB_RIGHT] && Pos->z >= Data[BB_NEAR] && Pos->z <= Data[BB_FAR]);
	}

	bool IsIn(const IMATH::Vector2D *Pos)
	{
		// works for both top-left and bottom-left origin
		return (   Pos->x >= Data[BB_LEFT] && Pos->x <= Data[BB_RIGHT]
		&&      (  (Pos->y >= Data[BB_BOTTOM] && Pos->y <= Data[BB_TOP])
		         ||(Pos->y <= Data[BB_BOTTOM] && Pos->y >= Data[BB_TOP])));
	}

	IMATH::Vector2D GetCenterXY()
	{
		return IMATH::Vector2D(0.5f * (Data[BB_LEFT] + Data[BB_RIGHT]), 0.5f * (Data[BB_TOP] + Data[BB_BOTTOM]));
	}
};

class BoundingBox2D // 2D: XY
{
public:

	union
	{
		struct
		{
			// XY
			float Left, Right, Bottom, Top;
		};
		struct
		{
			// XZ
			float Left, Right, Near, Far;
		};
		float Data[4];
	};

	BoundingBox2D(void) {}

	BoundingBox2D(float Left, float Right, float Bottom, float Top)
	{
		Data[BB_LEFT] = Left;
		Data[BB_RIGHT] = Right;
		Data[BB_BOTTOM] = Bottom;
		Data[BB_TOP] = Top;
	}

	bool IsIn(const IMATH::Vector2D *Pos)
	{
		// works for both top-left and bottom-left origin
		return (   Pos->x >= Data[BB_LEFT] && Pos->x <= Data[BB_RIGHT]
		&&      (  (Pos->y >= Data[BB_BOTTOM] && Pos->y <= Data[BB_TOP])
		         ||(Pos->y <= Data[BB_BOTTOM] && Pos->y >= Data[BB_TOP])));
	}
};

}

#endif
