/***********************************************************/
/*                                                         */
/* Decal.h                                                 */
/*                                                         */
/* created 11.01.2004                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef DECAL_H
#define DECAL_H

#include "MathStructures.h"
#include "TextureManager.h"
#include "D3DStruct.h"
#include "DecalLib.h"

#include <d3d9.h>

// www.infinitecode.com game library
namespace IGAME
{

class Decal
{
public:
	class DecalSpecs
	{
	public:
		char Name[1024];
		IMATH::Vector2DXZ Pos;
		int ID;
		//DecalLib *DLib;
	};

private:
	LPDIRECT3DTEXTURE9 Texture;
	DecalSpecs Specs;
	static DecalLib *DLib;
	DecalLib::Decal *Lib;

public:
	Decal(): Texture(0), Lib(0) {}

	Decal(const DecalSpecs &Specs)
		: Texture(0)
	{
		Decal::Specs = Specs;
		if(!DLib)
		{
			DLib = new DecalLib;
			DLib->Create();
		}
		Lib = &DLib->Decals[Specs.ID];
	}

	LPDIRECT3DTEXTURE9 GetTexture(I3D::D3DStruct *DS)
	{
		if(!Texture)
		{
			char fname[1024] = "Textures\\";
			strcat(fname, Lib->Texture);
			Texture = I3D::TextureManager::Load(DS->D3DDevice, fname, 0, I3D::TM_FLAG_ALPHACC0);
		}
		return Texture;
	}

	const char *GetName()
	{
		return Specs.Name;
	}

	int GetID()
	{
		return Specs.ID;
	}

	void SetID(int ID)
	{
		Specs.ID = ID;
		Lib = &DLib->Decals[Specs.ID];
	}

	void SetName(const char *Value)
	{
		strcpy(Specs.Name, Value);
	}

	void SetPosX(float Value)
	{
		Specs.Pos.x = Value;
	}

	void SetPosZ(float Value)
	{
		Specs.Pos.z = Value;
	}

	const IMATH::Vector2DXZ &GetPos() const
	{
		return Specs.Pos;
	}

	float GetSizeX() const
	{
		float max_x = Lib->Bounds[0].x;
		float min_x = Lib->Bounds[0].x;
		for(int i = 1; i < 3; ++i)
		{
			if(Lib->Bounds[i].x < min_x)
				min_x = Lib->Bounds[i].x;
			if(Lib->Bounds[i].x > max_x)
				max_x = Lib->Bounds[i].x;
		}
		return max_x - min_x;
	}

	float GetSizeZ() const
	{
		float max_z = Lib->Bounds[0].z;
		float min_z = Lib->Bounds[0].z;
		for(int i = 1; i < 3; ++i)
		{
			if(Lib->Bounds[i].z < min_z)
				min_z = Lib->Bounds[i].z;
			if(Lib->Bounds[i].z > max_z)
				max_z = Lib->Bounds[i].z;
		}
		return max_z - min_z;
	}

	float GetCenterX() const
	{
		return Lib->Center.x;
	}

	float GetCenterZ() const
	{
		return Lib->Center.z;
	}

	bool IsStreet()
	{
		return true;
	}
};

}

#endif
