/***********************************************************/
/*                                                         */
/* DialogWindow.cpp                                        */
/*                                                         */
/* created 04.07.2004                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "DialogWindow.h"

#include <cstdio>

// www.infinitecode.com tool library
namespace ITL
{

ISL::HashTable<HWND, DialogWindow *> DialogWindow::Map;

void DialogWindow::AddItem(const Item &It)
{
	Items.Add(It);
}

void DialogWindow::Run(HINSTANCE hInstance, HWND Parent, DialogWindow::Handler *Hnd)
{
	if(DialogWindow::Hnd)
		throw("DialogWindow::Run: an instance of the dialogbox is already running");
	DialogWindow::Hnd = Hnd;
	AddListener(Hnd);
	Hnd->SetLinked(true);
	DLGTEMPLATE *DT = GetData();
	HWND Wnd = CreateDialogIndirect(hInstance, DT, Parent, DlgProc);
	Map.Add(Wnd, this);
	/*char t[1024];
	sprintf(t, "ret: %d, last: %d", Ret, GetLastError());
	MessageBox(0, t, "", MB_OK);*/
}

void DialogWindow::RunModal(HINSTANCE hInstance, HWND Parent, DialogWindow::Handler *Hnd)
{
	if(DialogWindow::Hnd)
		throw("DialogWindow::Run: an instance of the dialogbox is already running");
	DialogWindow::Hnd = Hnd;
	AddListener(Hnd);
	Hnd->SetLinked(true);
	DLGTEMPLATE *DT = GetData();
	int Ret = DialogBoxIndirectParam(hInstance, DT, Parent, DlgProc, (LPARAM)this);
	/*char t[1024];
	sprintf(t, "ret: %d, last: %d", Ret, GetLastError());
	MessageBox(0, t, "", MB_OK);*/
}

void DialogWindow::Stop()
{
	DestroyWindow(Map.GetKey(this));
}

DLGTEMPLATE *DialogWindow::GetData(void)
{
	if(Data)
		GlobalFree(Data);
	// TODO: calculate memory sizes correctly
	HGLOBAL Handle = GlobalAlloc(GMEM_ZEROINIT, 2048 + Items.Count() * 2048);
	Data = (unsigned short *)GlobalLock(Handle);
	unsigned short *Ptr = Data;
	Sp.DlgTmp.cdit = Items.Count();

	Ptr = DWordAlign(Ptr);
	DLGTEMPLATE *DataPtr = (DLGTEMPLATE *)Ptr;
	memcpy(Ptr, &Sp.DlgTmp, sizeof(Sp.DlgTmp));
	Ptr = (unsigned short *)((char *)Ptr + sizeof(Sp.DlgTmp));
	if(*Sp.Menu)
	{
		Ptr += 0*1 + MultiByteToWideChar (CP_ACP, 0, Sp.Menu, -1, (LPWSTR)Ptr, strlen(Sp.Menu) + 1);
	}
	else
		*Ptr++ = 0;
	if(*Sp.WClass)
	{
		Ptr += 0*1 + MultiByteToWideChar (CP_ACP, 0, Sp.WClass, -1, (LPWSTR)Ptr, strlen(Sp.WClass) + 1);
	}
	else
		*Ptr++ = 0;
	if(*Sp.Title)
	{
		Ptr += 0*1 + MultiByteToWideChar (CP_ACP, 0, Sp.Title, -1, (LPWSTR)Ptr, strlen(Sp.Title) + 1);
	}
	else
		*Ptr++ = 0;
	// Items
	for(int i = 0; i < Items.Count(); ++i)
	{
		Ptr = DWordAlign(Ptr);
		memcpy(Ptr, &Items[i].Tmp, sizeof(DLGITEMTEMPLATE));
		Ptr = (unsigned short *)((char *)Ptr + sizeof(DLGITEMTEMPLATE));
		if(Items[i].BWClass)
		{
			*Ptr++ = 0xFFFF;
			*Ptr++ = Items[i].BWClass;
		}
		else
			*Ptr++ = 0;
		if(*Items[i].Title)
		{
			Ptr += 0*1 + MultiByteToWideChar (CP_ACP, 0, Items[i].Title, -1, (LPWSTR)Ptr, strlen(Items[i].Title) + 1);
		}
		else
			*Ptr++ = 0;
		if(*(Ptr - 1) == 0 && *(Ptr - 2) == 0)
			MessageBox(0, "something is wrong", "", MB_OK);
		//Ptr = DWordAlign(Ptr);
		if(Items[i].CData)
		{
			memcpy(Ptr, Items[i].CreationData, Items[i].CreationDataLength);
			Ptr = (unsigned short *)((char *)Ptr + Items[i].CreationDataLength);
		}
		else
			*Ptr++ = 0;
	}
	GlobalUnlock(Handle);
	return DataPtr;
}

int __stdcall DialogWindow::DlgProc(HWND hDlg, unsigned Msg, unsigned WParam, long LParam)
{
	DialogWindow *Wnd = Map.GetValue(hDlg);
	switch(Msg)
	{
	case WM_INITDIALOG:
		if(LParam)
			Map.Add(hDlg, (DialogWindow *)LParam);
		return TRUE;
	case WM_CLOSE:
		Wnd->Hnd->SetLinked(false);
		Wnd->DeleteListener(Wnd->Hnd);
		Wnd->Hnd = 0;
		EndDialog(hDlg, 0);
		return TRUE;
		break;
	case WM_COMMAND:
		DlgWndEvent Ev;
		Ev.Dlg = hDlg;
		Ev.id = WParam;
		Wnd->SendEvent(Ev);
		return TRUE;
	}
	return FALSE;
}

}
