/***********************************************************/
/*                                                         */
/* DialogWindow.h                                          */
/*                                                         */
/* created 04.07.2004                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef DIALOGWINDOW_H
#define DIALOGWINDOW_H

#include "DynamicArray.h"
#include "EventSender.h"
#include "HashTable.h"

#include <windows.h>

// www.infinitecode.com tool library
namespace ITL
{

	struct DlgWndEvent
	{
		HWND Dlg;
		int id;
	};

	class DialogWindow: IEV::EventSender<DlgWndEvent>
	{
	public:
		enum
		{
			BUTTON = 0x0080,
			EDIT = 0x0081,
			STATIC = 0x0082,
			LISTBOX = 0x0083,
			SCROLLBAR = 0x0084,
			COMBOBOX = 0x0085
		};

		struct Specs
		{
			DLGTEMPLATE DlgTmp;
			// TODO: calculate memory sizes correctly
			char Menu[1024], WClass[1024], Title[1024];

			Specs()
			{
				memset(&DlgTmp, 0, sizeof(DlgTmp));
				*Menu = 0;
				*WClass = 0;
				*Title = 0;
			}
		};

		struct Item
		{
			DLGITEMTEMPLATE Tmp;
			// TODO: calculate memory sizes correctly
			char Title[2048];
			unsigned short BWClass;
			bool CData;
			void *CreationData;
			int CreationDataLength;

			Item()
			{
				memset(&Tmp, 0, sizeof(Tmp));
				*Title = 0;
				CData = false;
				BWClass = STATIC;
			}
		};

		// interface
		class Handler: public IEV::EventListener<DlgWndEvent>
		{
			bool Linked;

		public:
			virtual void OnEvent(DlgWndEvent Ev) {}
			void SetLinked(bool Value) { Linked = Value; }
			bool GetLinked() { return Linked; }
		};

	private:
		Specs Sp;
		ISL::DynamicArray<Item> Items;
		unsigned short *Data;
		static ISL::HashTable<HWND, DialogWindow *> Map;
		Handler *Hnd;

		static int __stdcall DlgProc(HWND hDlg, unsigned Msg, unsigned WParam, long LParam);

		unsigned short *WordAlign(unsigned short *Ptr)
		{
			unsigned Ptr2;

			Ptr2 = (unsigned)Ptr;
			Ptr2 += 1;
			Ptr2 >>= 1;
			Ptr2 <<= 1;
			return (unsigned short *)Ptr2;
		}

		unsigned short *DWordAlign(unsigned short *Ptr)
		{
			unsigned Ptr2;

			Ptr2 = (unsigned)Ptr;
			Ptr2 += 3;
			Ptr2 >>= 2;
			Ptr2 <<= 2;
			return (unsigned short *)Ptr2;
		}

		DLGTEMPLATE *GetData(void);

	public:
		DialogWindow(const Specs &Sp)
		{
			DialogWindow::Sp = Sp;
			Data = 0;
			Hnd = 0;
		}

		bool GetRunning()
		{
			if(Hnd)
				return Hnd->GetLinked();
			else
				return false;
		}

		void SetSize(int x, int y)
		{
			Sp.DlgTmp.cx = x;
			Sp.DlgTmp.cy = y;
		}

		void RunModal(HINSTANCE hInstance, HWND Parent, Handler *Hnd = 0);
		void Run(HINSTANCE hInstance, HWND Parent, Handler *Hnd = 0);
		void Stop();
		void AddItem(const Item &It);
	};

}

#endif
