/***********************************************************/
/*                                                         */
/* DynamicArray.h                                          */
/*                                                         */
/* created 13.07.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef DYNAMICARRAY_H
#define DYNAMICARRAY_H

#include <stdlib.h>
#include <string.h>

#include <sstream>

// www.infinitecode.com Standard Data Structure Library
namespace ISL
{

// A dynamic array with automatic memory allocation
template <typename Type, int StartSize = 0, int Increment = 4>
class DynamicArray
{
	Type *Array;
	int Size, MemSize;

public:
	DynamicArray(void)
	:MemSize(StartSize)
	{
		Size = 0;
//		Array = (Type *)malloc(StartSize * sizeof(Type));
		Array = new Type[StartSize];
		//memset(Array, 0, StartSize * sizeof(Type));
	}

	~DynamicArray(void)
	{
		delete[] Array;
	}

	// copy functions
	DynamicArray(const DynamicArray& DA)
	{
		Array = 0;
		*this = DA;
	}

	// copy the array
	DynamicArray& operator = (const DynamicArray& DA)
	{
		Size = DA.Size;
		MemSize = DA.MemSize;
		// TODO: remove memory leak
		if(Array) delete[] Array;
		Array = new Type[MemSize];//(Type *)malloc(MemSize * sizeof(Type));
		//memset(Array, 0, MemSize * sizeof(Type));
		for(int i = 0; i < Size; i++)
		{
			Array[i] = DA.Array[i];
		}
		return *this;
	}

	bool operator == (const DynamicArray& DA)
	{
		if(Size != DA.Size)
			return false;
		for(int i = 0; i < Size; i++)
		{
			if(!(Array[i] == DA.Array[i]))
				return false;
		}
		return true;
	}

	// access the members (in constant time)
	Type& operator [](int Index)
	{
		if(Index < Size)
			return Array[Index];
		else
		{
			Size = Index + 1;
			if(Index < MemSize)
			{
				return Array[Index];
			}
			else
				return ReSize(Index);
		}
	}

	bool IsIn(const Type &Element)
	{
		for(int i = 0; i < Size; i++)
			if(Array[i] == Element)
				return true;
		return false;
	}

	Type *GetPtr(void)
	{
		return Array;
	}

	Type& ReSize(int Index)
	{
		if(Index >= MemSize)
		{
			//int NewSize = ((Index / Increment) + 1) * Increment;
			int NewSize = MemSize? (2 * Index) : (Index + 1);
			Type *NewArray = new Type[NewSize];
			for(int i = 0; i < MemSize; i++)
				NewArray[i] = Array[i];
			delete[] Array;
			Array = NewArray;
			//Array = (Type *)realloc(Array, NewSize * sizeof(Type));
			//memset(&Array[MemSize], 0, (NewSize - MemSize) * sizeof(Type));
			MemSize = NewSize;
		}
		return Array[Index];
	}

	int Count(void) const
	{
		return Size;
	}

	void Add(const Type &Value)
	{
		int Size = Count();
		/*try
		{*/
		operator[](Size) = Value;
		/*}
		catch(...)
		{
			std::ostringstream os;
			os << "Add: " << Size << "/" << MemSize;
			throw os.str().c_str();
		}*/
	}

	Type& Last(void)
	{
		return Array[Size - 1];
	}

	void SetCount(int Size)
	{
		ReSize(Size - 1);
		this->Size = Size;
	}

	void Clear(void)
	{
		Size = 0;
		MemSize = 0;
		delete[] Array;
		Array = new Type[Size];
	}

	void ClearPtr(void)
	{
		for(int i = 0; i < Size; i++)
			delete Array[i];
		Size = 0;
		MemSize = 0;
		delete[] Array;
		Array = new Type[Size];
	}

	void ClearElements(void)
	{
		Size = 0;
	}

	void ClearPtrElements(void)
	{
		for(int i = 0; i < Size; i++)
			delete Array[i];
		Size = 0;
	}

	bool Delete(const Type &Var)
	{
		for(int i = 0; i < Size; i++)
		{
			if(Array[i] == Var)
			{
				if(i != Size - 1)
				{
					Array[i] = Array[Size - 1];
				}
				SetCount(Size - 1);
				return true;
			}
		}
		return false;
	}

	// foo: dummy to distinguish Delete functions for <int>
	bool Delete(int Index, int foo = 0)
	{
		if(Index >= 0 && Index < Size)
		{
			if(Index != Size - 1)
			{
				Array[Index] = Array[Size - 1];
			}
			SetCount(Size - 1);
			return true;
		}
		return false;
	}

	// foo: dummy to distinguish Delete functions for <int>
	bool DeleteKeepOrder(int Index, int foo = 0)
	{
		if(Index >= 0 && Index < Size)
		{
			if(Index != Size - 1)
				for(int i = Index + 1; i < Size; i++)
					Array[i - 1] = Array[i];
			SetCount(Size - 1);
			return true;
		}
		return false;
	}

	bool DeleteKeepOrder(const Type &Var)
	{
		for(int Index = 0; Index < Size; Index++)
			if(Array[Index] == Var)
			{
				if(Index != Size - 1)
					for(int i = Index + 1; i < Size; i++)
						Array[i - 1] = Array[i];
				SetCount(Size - 1);
				return true;
			}
		return false;
	}

	/*void DeleteAllPtr(void)
	{
		for(int i = 0; i < MemSize; i++)
			if(Array[i]) delete Array[i];
	}*/

};

}

#endif
