/***********************************************************/
/*                                                         */
/* Forest.cpp                                              */
/*                                                         */
/* created 16.07.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "Forest.h"

//#include "Debugger.h"

// www.infinitecode.com 3d library
namespace I3D
{

void Forest::Create(D3DStruct *DS, BoundingBox2D Area, NURBS *Surface)
{
	TSList[0].Texture = TextureManager::Load(DS->D3DDevice, "Textures\\Tree.bmp", NULL, TM_FALPHACC0 | TM_FACCBILOPT | TM_FMIPALPHA60);
	// create trees
	for(int i = 0; i < FOREST_TREETEXTURES; i++)
	{
		for(int j = 0; j < FOREST_TREES / FOREST_TREETEXTURES; j++)
		{
			Vector Pos;
			// 0x3FFF = 16383
			float OORand = 1/8192.0f;
			Pos.x = (((rand() & 0x3FFF) - 8192) * OORand) * 0.5f * (Area.Data[BB_RIGHT] - Area.Data[BB_LEFT]);
			Pos.z = (((rand() & 0x3FFF) - 8192) * OORand) * 0.5f * (Area.Data[BB_BOTTOM] - Area.Data[BB_TOP]);
			Pos.y = Surface->GetY(Pos.x, Pos.z);
			Tree NewTree;
			NewTree.Create(Pos);
			TSList[i].Trees[TSList[i].Trees.Count()] = NewTree;
		}
		TSList[i].RS.ClearVertex();
			TSList[i].Trees[0].CreateRenderStruct(&TSList[i].RS);
		TSList[i].RS.CreateVBuf(DS->D3DDevice);
	}
	TreeModel.Create(DS, "Models\\Tree.t4m");
}

void Forest::CreateView(D3DStruct *DS)
{
	for(int i = 0; i < FOREST_TREETEXTURES; i++)
	{
		TSList[i].RS.ClearIndex();
		/*for(int j = 0; j < TSList[i].Trees.GetSize(); j++)
		{*/
			TSList[i].Trees[0].CreateIndices(&TSList[i].RS);
		//}
		TSList[i].RS.CreateIBuf(DS->D3DDevice);
	}
}

void Forest::Render(D3DStruct *DS)
{
	Renderable::Render(DS);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
	DS->D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATER);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHAREF, 0x60);
	DS->D3DDevice->SetRenderState(D3DRS_CULLMODE,D3DCULL_NONE);
/*	DS->D3DDevice->SetTextureStageState( 0, D3DTSS_ADDRESSU, D3DTADDRESS_CLAMP);
	DS->D3DDevice->SetTextureStageState( 0, D3DTSS_ADDRESSV, D3DTADDRESS_CLAMP);*/
	for(int i = 0; i < FOREST_TREETEXTURES; i++)
	{
		DS->D3DDevice->SetTexture(0, TSList[i].Texture);
		//TSList[i].RS.RenderIndex(DS->D3DDevice);
		//if(TSList[i].RS.SetupIndex(DS->D3DDevice))
		{
			D3DMATRIX Mtx = {1,0,0,0,  0,1,0,0, 0,0,1,0,  0,0,0,1};
			for(int j = 0; j < TSList[i].Trees.Count(); j++)
			{
				Mtx.m[3][0] = TSList[i].Trees[j].Pos.x;
				Mtx.m[3][1] = TSList[i].Trees[j].Pos.y;
				Mtx.m[3][2] = TSList[i].Trees[j].Pos.z;
				DS->D3DDevice->SetTransform(D3DTS_WORLD, &Mtx);
				//ITL::Debugger::Msg("test.txt", TSList[i].RS.GetIndexNumber());
				//DS->D3DDevice->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, TSList[i].RS.GetVertexNumber(), 0, TSList[i].RS.GetIndexNumber() / 3);
				TreeModel.Render(DS);
			}
		}
	}
	D3DMATRIX Mtx = {1,0,0,0,  0,1,0,0, 0,0,1,0,  0,0,0,1};
	DS->D3DDevice->SetTransform(D3DTS_WORLD, &Mtx);
	DS->D3DDevice->SetRenderState(D3DRS_CULLMODE,D3DCULL_CW);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, FALSE);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
/*	DS->D3DDevice->SetTextureStageState( 0, D3DTSS_ADDRESSU, D3DTADDRESS_WRAP);
	DS->D3DDevice->SetTextureStageState( 0, D3DTSS_ADDRESSV, D3DTADDRESS_WRAP);*/
}

}
