/***********************************************************/
/*                                                         */
/* Forest.h                                                */
/*                                                         */
/* created 25.07.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef FOREST_H
#define FOREST_H

#include "Renderable.h"
#include "Tree.h"
#include "D3DStruct.h"
#include "DataStructures.h"
#include "NURBS.h"
#include "Model.h"

// www.infinitecode.com 3d library
namespace I3D
{

#define FOREST_TREETEXTURES	1 // must be a divisor of FOREST_TREES
#define FOREST_TREES		(2048 * FOREST_TREETEXTURES) // must be dividable by FOREST_TREETEXTURES

struct TreeStruct
{
	ISL::DynamicArray<Tree, 16, 16> Trees;
	LPDIRECT3DTEXTURE8 Texture;
	RenderStruct<Vertex, I3D_FVFVERTEX, 256, 256> RS;
};

class Forest: public Renderable
{
	ISL::DynamicArray<TreeStruct, 1, 1> TSList;
	Model TreeModel;

protected:
	// from Renderable:
	virtual void CreateView(D3DStruct *DS);

public:
	void Create(D3DStruct *DS, BoundingBox2D Area, NURBS *Surface);

	virtual void Render(D3DStruct *DS);
};

}

#endif
