/***********************************************************/
/*                                                         */
/* Game.h                                                  */
/*                                                         */
/* created 13.07.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef GAME_H
#define GAME_H

#include "D3DWindow.h"
#include "D3DStruct.h"
#include "World.h"
#include "EventListener.h"
#include "DynamicArray.h"
#include "MouseCursor.h"
#include "Speed.h"
#include "PhysicsStruct.h"
#include "MissionLib.h"
#include "TankLib.h"
#include "ProjectileLib.h"
#include "Menu.h"
#include "CommandInterface.h"
#include "Profiler.h"
#include "StoryRenderer.h"
#include "Credits.h"
#include "SoundManager.h"
#include "DataInterface.h"
#include "Button.h"
#include "DShow.h"

// Tank 4
namespace T4
{

enum
{
	// flags
	GAME_D3DFLAG_WIREFRAME = 1,
};

class Game: public IEV::EventListener<I3D::D3DWEvent>, public IEV::EventListener<const char *>
{
	enum
	{
		// flags
		FLAG_DEBUG = 1
	};

	class CICommand: public I3D::CommandInterface
	{
	public:
		void Create(Game *Game)
		{
			CICommand::Game = Game;
		}

		Game *Game;
		virtual void Command(const char *Command);
	} ICommand;

	class CIData: public DataInterface
	{
		Game *Game;

	public:
		void Create(T4::Game *Game)
		{
			CIData::Game = Game;
		}

		virtual const char *GetData(const char *Str);
	} IData;

	friend CICommand;

	enum
	{
		GAME_STATE_MAINMENU = 0,
		GAME_STATE_LOADING = 1,
		GAME_STATE_ACTIVE = 2,
		GAME_STATE_DONE = 3,
		GAME_STATE_CREDITS = 4
	};

	I3D::D3DWindow Wnd;
	I3D::D3DStruct DS;
	// the game world
	IGAME::World Wrld;
	int D3DFlags, Flags;
	I3D::MouseCursor Mouse;
	ITL::Speed Sp;
	bool AsyncReady;
	IGAME::MissionLib MLib;
	int Mission, State;
	IGAME::MissionLib::StoryItem *StoryItem;
	float StoryTime, MissionStartTime;
	I3D::Menu Menu;
	ITL::Profiler Prf;
	IGAME::StoryRenderer StoryRnd;
	I3D::Credits Credits;
	IAU::SoundManager SndMgr;
	LPDIRECT3DTEXTURE9 MouseTextures[4];
	I3D::Button MaximizeButton;
	bool FlagFullscreen, Reinitialize;
	IAU::DShow Music;

	ISL::DynamicArray<I3D::D3DWEvent, 0, 4> Events;

	virtual void OnEvent(I3D::D3DWEvent Data);
	void EventHandler(void);
	void KeyboardHandler(char c);
	void MouseHandler(void);
	void Scrolling(void);
	void Init3D(bool recurse);
	void Create3D(void);
	void LoadingScreen(const char *Str);
	void LoadCredits(void);
	void LoadMission(void);
	void LoadState(int State);
	void Game::LoadStory(const char *Name);
	void ClearScreen(void);
	void Set3DMatrices(void);
	void DisplayFPS(void);
	bool GoalComplete(IGAME::MissionLib::Goal *Goal);
	void CheckEvents(void);
	void CheckStory(void);
	void CheckAirfields(void);
	bool GetIngame(void) const;
	void DebugScreenRender(I3D::D3DStruct *DS);

public:
	Game(void)
		:AsyncReady(false), D3DFlags(0), Flags(0) {}

	void Create(int WindowStyle);

	~Game(void)
	{
		ShowCursor(true);
		AsyncReady = false;
	}

	void ToggleFullscreen()
	{
		FlagFullscreen = true;
	}

	void SetFullscreen(bool Value)
	{
		FlagFullscreen = Value;
		Wnd.SetFullscreen(Value);
	}

	HWND GetHWnd()
	{
		return Wnd.GetHWnd();
	}

	virtual void OnEvent(const char *Event)
	{
		LoadingScreen(Event);
	}

	char *DebugInfo();
	// this is called continuously while the application is running
	void MainLoop(void);
};

}

#endif
