/***********************************************************/
/*                                                         */
/* HashTable.h                                             */
/*                                                         */
/* created 13.07.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef HASHTABLE_H
#define HASHTABLE_H

#include "DynamicArray.h"

// www.infinitecode.com Standard Data Structure Library
namespace ISL
{

template <class KeyType, class ValueType, int StartSize = 8, int Increment = 8>
class HashTable
{

	template <class KeyType, class ValueType>
	class HashTableEntry
	{
	public:	
		KeyType Key;
		ValueType Value;

		HashTableEntry(void)
		{
		}

		// copy functions
		HashTableEntry(HashTableEntry& HT)
		{
			*this = HT;
		}

		HashTableEntry& operator=(HashTableEntry& HT)
		{
			Key = HT.Key;
			Value = HT.Value;
			return *this;
		}
	};

	class TableEntry: public HashTableEntry<KeyType, ValueType> {};
	class TableType: public DynamicArray<TableEntry, StartSize, Increment> {};

	TableType Table;

public:

	HashTable(void)
	{
	}

	// copy functions
	HashTable(HashTable& HT)
	{
		*this = HT;
	}

	void operator=(HashTable& HT)
	{
		Table = HT.Table;
	}

	void Clear(void)
	{
		Table.Clear();
	}

	void Add(KeyType Key, ValueType Value)
	{
		for(int i = 0; i < Table.Count(); i++)
		{
			if(Table[i].Key == Key)
			{
				Table[i].Value = Value;
				return;
			}
		}
		// else
		int LastElement = Table.Count();
		Table[LastElement].Key = Key;
		Table[LastElement].Value = Value;
	}

	void DeleteKey(KeyType Key)
	{
		for(int i = 0; i < Table.Count(); i++)
		{
			if(Table[i].Key == Key)
			{
				if(i != Table.Count() - 1)
				{
					Table[i] = Table[Table.Count() - 1];
				}
				Table.SetCount(Table.Count());
				return;
			}
		}
	}

	ValueType GetValue(KeyType Key)
	{
		for(int i = 0; i < Table.Count(); i++)
		{
			if(Table[i].Key == Key)
			{
				return Table[i].Value;
			}
		}
		return 0;
	}

	ValueType *GetValuePtr(int Num)
	{
		return &Table[Num].Value;
	}

	ValueType *GetValuePtr(KeyType Key)
	{
		for(int i = 0; i < Table.Count(); i++)
		{
			if(Table[i].Key == Key)
			{
				return &Table[i].Value;
			}
		}
		return 0;
	}

	KeyType GetKey(int Num)
	{
		return Table[Num].Key;
	}

	KeyType GetKey(ValueType Value)
	{
		for(int i = 0; i < Table.Count(); i++)
		{
			if(Table[i].Value == Value)
			{
				return Table[i].Key;
			}
		}
		return 0;
	}

	KeyType *GetKeyPtr(ValueType Value)
	{
		for(int i = 0; i < Table.Count(); i++)
		{
			if(Table[i].Value == Value)
			{
				return &Table[i].Key;
			}
		}
		return 0;
	}

	// return the number of keys
	int CountKeys(void)
	{
		return Table.Count();
	}

};

}

#endif
