/***********************************************************/
/*                                                         */
/* IniLoader.cpp                                           */
/*                                                         */
/* created 05.08.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "IniLoader.h"

#include <iostream>
#include <fstream>
using namespace std;

namespace ITL
{

IniLoader::Group::Group(void)
:Name(NULL)
{
}

IniLoader::Group::Group(const char *Name)
{
	Create(Name);
}

void IniLoader::Group::Create(const char *Name)
{
	Group::Name = new char[strlen(Name) + 1];
	strcpy(Group::Name, Name);
}

IniLoader::Group::~Group(void)
{
	delete[] Name;
}

void IniLoader::Group::Add(const char *Key, const char *Value)
{
	Table.Add(Key, Value);
}

//#include <stdio.h>

void IniLoader::Decode(char *Str)
{
	char *it = Str;
	while(*it)
	{
		if(*it == '\\')
		{
			if(it[1] == '\\')
			{
				// reduce string
				char *it2 = it + 1;
				while(*it2)
				{
					*it2 = it2[1];
					it2++;
				}
			}
			else if (it[1] && it[2])
			{
				if(it[1] >= '0' && it[1] < '9')
					*it = (it[1] - '0') << 4;
				else if(it[1] >= 'a' && it[1] < 'f')
					*it = (it[1] - 'a' + 10) << 4;
				else if(it[1] >= 'A' && it[1] < 'F')
					*it = (it[1] - 'A' + 10) << 4;
				else
				{
					++it;
					continue;
				}
				if(it[2] >= '0' && it[2] < '9')
					*it += it[2] - '0';
				else if(it[2] >= 'a' && it[2] < 'f')
					*it += it[2] - 'a' + 10;
				else if(it[2] >= 'A' && it[2] < 'F')
					*it += it[2] - 'A' + 10;
				else
				{
					++it;
					continue;
				}
				// reduce string
				char *it2;
				for(int i = 0; i < 2; ++i)
				{
					it2 = it + 1;
					while(*it2)
					{
						*it2 = it2[1];
						it2++;
					}
				}
			}
		}
		++it;
	}
}

IniLoader::IniLoader(const char *FileName)
{
	using namespace std;
	ifstream in(FileName);
//	FILE *inf = fopen(FileName, "rt");
	Group *CurrentGroup = NULL;
	char Str[1024];
	if(in.fail())
	{
		char str[1024];
		sprintf(str, "IniLoader::IniLoader: couldn't open file %s", FileName);
		throw(str);
	}
	while(!in.eof())
//	while(!feof(inf))
	{
		in.getline(Str, 1024, '\n');
		//fgets(Str, 1024, inf);
		// sort out inline comments
		char *s = Str;
		while(*s && *s != ';')
			s++;
		*s = 0;
		switch(Str[0])
		{
		case '[':
			{
				// new group
				char *s = Str;
				while(*s && *s != ']')
					s++;
				if(*s == ']')
				{
					*s = 0;
					s = Str + 1;
					CurrentGroup = NULL;
					for(int i = 0; i < Groups.Count(); i++)
					{
						if(!strcmpi(Groups[i].Name, s))
							CurrentGroup = &Groups[i];
					}
					if(!CurrentGroup)
					{
						Groups[Groups.Count()] = ITL::IniLoader::Group(s);
						CurrentGroup = &Groups[Groups.Count() - 1];
					}
				}
				//else ignore
				break;
			}
		case ';':
			// comment
			cout << "comment: " << Str << endl;
			break;
		case 0:
			// empty line
			break;
		default:
			// line
			char *Value = Str;
			while(*Value && *Value != '=')
				Value++;
			if(*Value == '=')
			{
				*Value = 0;
				Value++;
				if(!CurrentGroup)
				{
					Groups[Groups.Count()] = ITL::IniLoader::Group("");
					CurrentGroup = &Groups[Groups.Count() - 1];
				}
//				char *NewKey, *NewValue;
				char *Key = Str;
				while((*Key == ' ' || *Key == '\t') && *Key) Key++;
				while(Key[strlen(Key) - 1] == ' ' || Key[strlen(Key) - 1] == '\t') Key[strlen(Key) - 1] = 0;
				while((*Value == ' ' || *Value == '\t') && *Value) Value++;
				bool Quotation = (*Value == '\"');
				if(Quotation)
					Value++;
				while(*Value && (Value[strlen(Value) - 1] == ' ' || Value[strlen(Value) - 1] == '\t')) Value[strlen(Value) - 1] = 0;
				if(Quotation && *Value && Value[strlen(Value) - 1] != '\"')
				{
					// read more lines
					char *Txt = new char[strlen(Value) + 1 + 1024];
					strcpy(Txt, Value);
					while(Quotation && !in.eof())
					{
						char *NewString = new char[strlen(Txt) + 1026];
						strcpy(NewString, Txt);
						delete[] Txt;
						Txt = NewString;
						strcat(Txt, "\n");
						char *NewText = Txt + strlen(Txt);
						in.getline(NewText, 1024, '\n');
						char *it = NewText;
						while(*it != '\"' && *it)
							it++;
						if(*it == '\"')
						{
							*it = 0;
							Decode(Txt);
							CurrentGroup->Add(Str, Txt);
							Quotation = false; // break;
						}
					}
					delete[] Txt;
				}
				else
				{
					if(Quotation && Value[strlen(Value) - 1] == '\"')
						Value[strlen(Value) - 1] = 0;
					Decode(Value);
					CurrentGroup->Add(Str, Value);
				}
			}
			// else ignore
			break;
		}
	}
	//fclose(inf);
}

const char *IniLoader::GetValue(const char *Group, const char *Key)
{
	for(int i = 0; i < Groups.Count(); i++)
	{
		if(!strcmpi(Groups[i].Name, Group))
		{
			ISL::String *Str = Groups[i].Table.GetValuePtr(ISL::String(Key));
			if(Str)
				return Str->GetChars();
			else
				return NULL;
		}
	}
	return 0;
}

const char *IniLoader::GetValue(const char *Group, int Key)
{
	for(int i = 0; i < Groups.Count(); i++)
	{
		if(!strcmpi(Groups[i].Name, Group))
		{
			ISL::String *Str = Groups[i].Table.GetValuePtr(Key);
			if(Str)
				return Str->GetChars();
			else
				return NULL;
		}
	}
	return 0;
}

const char *IniLoader::GetValue(int Group, const char *Key)
{
	if(Group >= 0 && Group < Groups.Count())
	{
		ISL::String *Str = Groups[Group].Table.GetValuePtr(ISL::String(Key));
		if(Str)
			return Str->GetChars();
		else
			return NULL;
	}
	return 0;
}

void IniLoader::Add(const char *Group, const char *Key, const char *Value)
{
	for(int i = 0; i < Groups.Count(); i++)
	{
		if(!strcmpi(Groups[i].Name, Group))
		{
			Groups[i].Add(Key, Value);
		}
	}
}

void IniLoader::Add(int Group, const char *Key, const char *Value)
{
	if(Group >= 0 && Group < Groups.Count())
	{
		Groups[Group].Add(Key, Value);
	}
}

const char *IniLoader::GetValue(int Group, int Num)
{
	if(Group >= 0 && Group < Groups.Count())
	{
		ISL::String *Str = Groups[Group].Table.GetValuePtr(Num);
		if(Str)
			return Str->GetChars();
		else
			return 0;
	}
	else
		return 0;
}

const char *IniLoader::GetKey(const char *Group, int Num)
{
	for(int i = 0; i < Groups.Count(); i++)
	{
		if(!strcmpi(Groups[i].Name, Group))
		{
			const char *Ret = Groups[i].Table.GetKeyPtr(Num)->GetChars();
			return Ret;
		}
	}
	return 0;
}

const char *IniLoader::GetKey(int Group, int Num)
{
	if(Group >= 0 && Group < Groups.Count())
		return Groups[Group].Table.GetKeyPtr(Num)->GetChars();
	else
		return 0;
}

const char *IniLoader::GetGroupName(int Group)
{
	if(Group >= 0 && Group < Groups.Count())
		return Groups[Group].Name;
	else
		return 0;	
}

int IniLoader::CountGroups(void)
{
	return Groups.Count();
}

int IniLoader::CountKeys(const char *Group)
{
	for(int i = 0; i < Groups.Count(); i++)
	{
		if(!strcmpi(Groups[i].Name, Group))
		return Groups[i].Table.CountKeys();
	}
	return 0;	
}

int IniLoader::CountKeys(int Group)
{
	if(Group >= 0 && Group < Groups.Count())
		return Groups[Group].Table.CountKeys();
	else
		return 0;	
}

}
