/***********************************************************/
/*                                                         */
/* IniLoader.h                                             */
/*                                                         */
/* created 05.08.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef INILOADER_H
#define INILOADER_H

#include "DynamicArray.h"
#include "MultiHashTable.h"
#include "ISLString.h"

// www.infinitecode.com tool library
namespace ITL
{

class IniLoader
{
	class Group
	{
		void Create(const char *Name);
	public:
		char *Name;
		ISL::MultiHashTable<ISL::String, ISL::String, 0, 1, 0, 1> Table;

		Group(void);
		Group(const char *Name);
		~Group(void);

		// copy functions
		Group(Group& Gr)
		{
			*this = Gr;
		}

		Group &operator = (Group& Gr)
		{
			Create(Gr.Name);
			Table = Gr.Table;
			return *this;
		}

		void Add(const char *Key, const char *Value);
	};

	ISL::DynamicArray<Group, 0, 1> Groups;

	void IniLoader::Decode(char *Str);

public:
	IniLoader(const char *FileName);
	IniLoader(void) {}

	const char *GetValue(const char *Group, const char *Key);
	const char *GetValue(int Group, const char *Key);
	const char *IniLoader::GetValue(const char *Group, int Key);
	const char *GetValue(int Group, int Num);
	const char *GetKey(const char *Group, int Num);
	const char *GetKey(int Group, int Num);
	const char *GetGroupName(int Group);
	void Add(const char *Group, const char *Key, const char *Value);
	void Add(int Group, const char *Key, const char *Value);

	int CountGroups(void);
	int CountKeys(const char *Group);
	int CountKeys(int Group);
};

}

#endif
