/***********************************************************/
/*                                                         */
/* Input.cpp                                               */
/*                                                         */
/* created 19.08.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "Input.h"

// www.infinitecode.com input library
namespace IIN
{

void Input::Create(HWND Wnd)
{
	HMODULE Module = GetModuleHandle(NULL);
	DirectInput8Create(Module, DIRECTINPUT_VERSION, IID_IDirectInput8A, (void **)&DI, NULL);
	if(DI->CreateDevice(GUID_SysMouse, &DIDevice, NULL) != DI_OK)
		throw "Input::Create: DirectInput CreateDevice failed";
	DIDevice->SetDataFormat(&c_dfDIMouse2);
	int ret = DIDevice->SetCooperativeLevel(Wnd, DISCL_NONEXCLUSIVE | DISCL_BACKGROUND);
	DIPROPDWORD Prop;
	Prop.diph.dwSize = sizeof(Prop);
	Prop.diph.dwHeaderSize = sizeof(Prop.diph);
	Prop.diph.dwObj = 0;
	Prop.diph.dwHow = DIPH_DEVICE;
	Prop.dwData = DIPROPAXISMODE_ABS;
	DIDevice->SetProperty(DIPROP_AXISMODE, &Prop.diph);
	Prop.dwData = 16;
	DIDevice->SetProperty(DIPROP_BUFFERSIZE, &Prop.diph);
	DIDevice->Acquire();
	DIMOUSESTATE2 State;
	DIDevice->GetDeviceState(sizeof(State), &State);
	WheelOrigin = State.lZ;
}

void Input::Update(void)
{
	GetCursorPos(&Pos);
	DIDEVICEOBJECTDATA Data[16];
	unsigned long Number;
	int Ret = DI_BUFFEROVERFLOW;
	do
	{
		Number = 16;
		Ret = DIDevice->GetDeviceData(sizeof(DIDEVICEOBJECTDATA), Data, &Number, 0);
		if(Number)
		{
			for(int i = 0; i < (int)Number; i++)
			{
				MouseEvent Ev;
				if(Data[i].dwData & 0x80)
					Ev.Code = INPUT_DOWN;
				else
					Ev.Code = INPUT_UP;
				Ev.x = Pos.x;
				Ev.y = Pos.y;
				switch(Data[i].dwOfs)
				{
				case DIMOFS_BUTTON0: // 0..7
					Ev.Button = 0;
					Events[Events.Count()] = Ev;
					break;
				case DIMOFS_BUTTON1: // 0..7
					Ev.Button = 1;
					Events[Events.Count()] = Ev;
					break;
				case DIMOFS_BUTTON2: // 0..7
					Ev.Button = 2;
					Events[Events.Count()] = Ev;
					break;
				case DIMOFS_BUTTON3: // 0..7
					Ev.Button = 3;
					Events[Events.Count()] = Ev;
					break;
				case DIMOFS_Z: // mouse wheel
					Wheel = Data[i].dwData - WheelOrigin;
					break;
				}
			}
		}
	} while(Ret == DI_BUFFEROVERFLOW);
}

bool Input::GetEvent(Input::MouseEvent *Event)
{
	if(Events.Count())
	{
		MouseEvent Ret = Events[0];
		for(int i = 0; i < Events.Count() - 1; i++)
			Events[i] = Events[i + 1];
		Events.SetCount(Events.Count() - 1);
		*Event = Ret;
		return true;
	}
	else
		return false;
}

Input::~Input(void)
{
	RELEASE(DIDevice);
	RELEASE(DI);
}

}
