/***********************************************************/
/*                                                         */
/* ManagedObject.h                                         */
/*                                                         */
/* created 18.02.2003                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef MANAGEDOBJECT_H
#define MANAGEDOBJECT_H

#include "MathStructures.h"
#include "DynamicArray.h"

// www.infinitecode.com 3d library
namespace I3D
{

// class for objects that are managed by data structures like
// bsp trees, quadtrees, octrees or any other subdivision
// structures or space managing structures
class ManagedObject
{
protected:
	bool Visible;

public:
	typedef ISL::DynamicArray<ManagedObject *> MgdObjArray;

	void *Node;

	ManagedObject(void): Node(0), Visible(0) {}

	virtual void *GetBase(void) const = 0;
	virtual void *GetType(void) const = 0;
	virtual void SetPos(const IMATH::Vector &Value) = 0;
	virtual const IMATH::Vector &GetPos(void) const = 0;

	virtual void SetVisible(bool Visible)
	{
		ManagedObject::Visible = Visible;
	}


	virtual bool GetVisible(void) const
	{
		return Visible;
	}
};

}

#endif
