/***********************************************************/
/*                                                         */
/* Menu.cpp                                                */
/*                                                         */
/* created 10.03.2003                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "Menu.h"
#include "IniLoader.h"
#include "TextureManager.h"
#include "Text.h"

#include <stdio.h>

// www.infinitecode.com 3d library
namespace I3D
{

void Menu::Create(Menu::MenuSpecs *Specs)
{
	Menu::Specs = *Specs;
	// set default values
	Columns = 32;
	Rows = 32;
	if(Submenu)
		delete Submenu;
	Submenu = 0;
	CloseOnClick = false;
	*CommandOnClick = 0;
	Items.Clear();
	// menu position
	Pos = IMATH::Vector2D(-0.6f, -0.6f);
	Size = IMATH::Vector2D(1.2f, 1.2f);
	Spacing = IMATH::Vector2D(1.0f, 1.0f);
	// read ini file
	char Filename[1024];
	sprintf(Filename, "%s\\%s", Menu::Specs.Directory, Menu::Specs.MenuFile);
	ITL::IniLoader Ini(Filename);
	// properties
	for(int j = 0; j < Ini.CountKeys("Properties"); j++)
	{
		const char *Key = Ini.GetKey("Properties", j);
		const char *Value = Ini.GetValue("Properties", j);
		if(!strcmp(Key, "CloseOnClick"))
			CloseOnClick = !stricmp(Value, "true");
		if(!strcmp(Key, "CommandOnClick"))
			strcpy(CommandOnClick, Value);
		if(!strcmp(Key, "Columns"))
			Columns = (float)atof(Value);
		if(!strcmp(Key, "Rows"))
			Rows = (float)atof(Value);
		if(!strcmp(Key, "PosX"))
			Pos.x = (float)atof(Value);
		if(!strcmp(Key, "PosY"))
			Pos.y = (float)atof(Value);
		if(!strcmp(Key, "SizeX"))
			Size.x = (float)atof(Value);
		if(!strcmp(Key, "SizeY"))
			Size.y = (float)atof(Value);
		if(!strcmp(Key, "SpacingX"))
			Spacing.x = (float)atof(Value);
		if(!strcmp(Key, "SpacingY"))
			Spacing.y = (float)atof(Value);
	}			
	// init item positioning
	const float Width = Size.x / 32.0f;
	const float Height = Size.y / 32.0f;
	const float xSpacing = Spacing.x * Height;
	const float ySpacing = Spacing.y * Height;
	const float LineSpacing = 2.0f * Height;
	IMATH::Vector2D ItemPos = IMATH::Vector2D(Pos.x + xSpacing, Pos.y + Size.y - ySpacing);
	for(int i = 0; i < Ini.CountGroups(); i++)
	{
		const char *Group = Ini.GetGroupName(i);
		if(stricmp(Group, "Properties"))
		{
			// item
			MenuItem Item;
			for(int j = 0; j < Ini.CountKeys(i); j++)
			{
				const char *Key = Ini.GetKey(i, j);
				const char *Value = Ini.GetValue(i, j);
				if(!strcmp(Key, "Text"))
				{
					if(*Value == '<' && Value[strlen(Value) - 1] == '>')
					{
						char *Str = new char[strlen(Value) + 1];
						strcpy(Str, Value + 1);
						Str[strlen(Str) - 1] = 0;
						strcpy(Item.Text, Menu::Specs.DataIf->GetData(Str));
						delete[] Str;
					}
					else
						strcpy(Item.Text, Value);
					for(unsigned i = 0; i < strlen(Item.Text); ++i)
						if(Item.Text[i] == '\\' && Item.Text[i + 1] == 'n')
						{
							Item.Text[i] = '\n';
							char *s = Item.Text + i + 1;
							while(*s = *(s + 1))
								++s;
						}
				}
				if(!strcmp(Key, "FontSize"))
					Item.FontSize = (float)atof(Value);
				if(!strcmp(Key, "Type"))
				{
					if(!stricmp(Value, "Text"))
						Item.Type = MenuItem::MITEM_TYPE_TEXT;
					if(!stricmp(Value, "Button"))
						Item.Type = MenuItem::MITEM_TYPE_BUTTON;
				}
				if(!strcmp(Key, "CommandOnClick"))
					strcpy(Item.CommandOnClick, Value);
				if(!strcmp(Key, "SubmenuOnClick"))
					strcpy(Item.SubmenuOnClick, Value);
				if(!strcmp(Key, "CloseOnClick"))
					Item.CloseOnClick = !stricmp(Value, "true");
				if(!strcmp(Key, "Center"))
					Item.Center = !stricmp(Value, "true");
			}
			// item position
			const float Width = Size.x / Columns * Item.FontSize;
			const float Height = Size.y / Rows * Item.FontSize;
			const float LineSpacing = Height;
			float SizeFactor = 1.0f;
			if(Item.Type == MenuItem::MITEM_TYPE_BUTTON)
				SizeFactor = 2.0f;
			float TextLength;
			if(Ini.GetValue(i, "Text"))
				TextLength = (float)strlen(Ini.GetValue(i, "Text"));
			else
				TextLength = 10;

			if(Item.Type == MenuItem::MITEM_TYPE_BUTTON)
			{
				Item.Size.x = Width * (TextLength + 1);
				Item.TextOffset = IMATH::Vector2D(0.5f * Width, 0.5f * Height);
			}
			else
			{
				Item.Size.x = Width * TextLength;
				Item.TextOffset = IMATH::Vector2D(0, 0);
			}
			Item.Size.y = SizeFactor * Height;
			if(Item.Center)
				Item.Pos.x = -Item.Size.x /2.0f;
			else
				Item.Pos.x = ItemPos.x;
			Item.Pos.y = ItemPos.y - Item.Size.y;
			Item.LetterSize.x = Width;
			Item.LetterSize.y = Height;
			int Lines = 1;
			if(Item.Text)
			{
				const char *it = Item.Text;
				while(*it)
				{
					if(*it == '\n')
						Lines++;
					++it;
				}
			}
			// adjust ItemPos for next item
			ItemPos.y -= SizeFactor * Height * Lines + LineSpacing;
			if(Ini.GetValue(i, "Text"))
				Items.Add(Item);
		}
	}
	RS.ClearVertex();
	RS.ClearIndex();
	// Background
	Background = TextureManager::GetTexture(Menu::Specs.DS, "Textures\\Mainmenu1.bmp", "Textures\\Mainmenu1_Alpha.bmp", TM_FLAG_ALPHACCFF00FF);
	Button = TextureManager::GetTexture(Menu::Specs.DS, "Textures\\MenuButton.bmp", 0, TM_FLAG_ALPHACCFF00FF);
	RS.AddVertex(Vertex(Vector(0, 0, 0), Vector(0, 0, -1), Vector2D(0, 1)));
	RS.AddVertex(Vertex(Vector(1, 0, 0), Vector(0, 0, -1), Vector2D(1, 1)));
	RS.AddVertex(Vertex(Vector(1, 1, 0), Vector(0, 0, -1), Vector2D(1, 0)));
	RS.AddVertex(Vertex(Vector(0, 1, 0), Vector(0, 0, -1), Vector2D(0, 0)));
	// Button
	/*RS.AddVertex(Vertex(Vector(0, 0, 0), Vector(0, 0, -1), Vector2D(0, 0.25f)));
	RS.AddVertex(Vertex(Vector(1, 0, 0), Vector(0, 0, -1), Vector2D(1, 0.25f)));
	RS.AddVertex(Vertex(Vector(1, 1, 0), Vector(0, 0, -1), Vector2D(1, 0)));
	RS.AddVertex(Vertex(Vector(0, 1, 0), Vector(0, 0, -1), Vector2D(0, 0)));*/
	RS.CreateVBuf(Menu::Specs.DS->D3DDevice);
	RS.AddIndex(0);
	RS.AddIndex(1);
	RS.AddIndex(2);
	RS.AddIndex(0);
	RS.AddIndex(2);
	RS.AddIndex(3);
	RS.CreateIBuf(Menu::Specs.DS->D3DDevice);
}

void Menu::Init3D(D3DStruct *DS, bool recurse)
{
	Background = TextureManager::GetTexture(DS, "Textures\\Mainmenu1.bmp", "Textures\\Mainmenu1_Alpha.bmp", TM_FLAG_ALPHACCFF00FF);
	Button = TextureManager::GetTexture(DS, "Textures\\MenuButton.bmp", 0, TM_FLAG_ALPHACCFF00FF);
	RS.CreateVBuf(DS->D3DDevice);
	RS.CreateIBuf(DS->D3DDevice);
	if(recurse && Submenu)
		Submenu->Init3D(DS, true);
}

bool Menu::Click(Vector2D Pos)
{
	if(Submenu && !Submenu->GetVisible())
	{
		delete Submenu;
		Submenu = 0;
	}
	if(Submenu)
		return Submenu->Click(Pos);
	else
	{
		bool Processed = false;
		if(Specs.Visible && Pos.x > Menu::Pos.x && Pos.x < Menu::Pos.x + Size.x
			&& Pos.y > Menu::Pos.y && Pos.y < Menu::Pos.y + Size.y)
		{
			bool ItemClicked = false;
			for(int i = 0; i < Items.Count(); i++)
			{
				if(Pos.x > Items[i].Pos.x && Pos.x < Items[i].Pos.x + Items[i].Size.x
					&& Pos.y > Items[i].Pos.y && Pos.y < Items[i].Pos.y + Items[i].Size.y)
				{
					// handle item click
					if(*Items[i].SubmenuOnClick)
					{
						if(!Submenu)
						{
							Submenu = new Menu();
							I3D::Menu::MenuSpecs MSpecs;
							MSpecs.DS = Specs.DS;
							MSpecs.Directory = "Menus";
							MSpecs.MenuFile = Items[i].SubmenuOnClick;
							MSpecs.Visible = true;
							MSpecs.CommandInterface = Specs.CommandInterface;
							//MSpecs.Visible = true;
							Submenu->Create(&MSpecs);
						}
					}
					if(Items[i].CloseOnClick)
						Specs.Visible = false;
					ItemClicked = true;
					Processed = true;
					if(*Items[i].CommandOnClick && Specs.CommandInterface)
					{
						Specs.CommandInterface->Command(Items[i].CommandOnClick); // can display a different menu and/or kill Menu::this!
						return true;
					}
				}
			}
			if(!ItemClicked)
			{
				if(CloseOnClick)
				{
					Specs.Visible = false;
					Processed = true;
				}
				// clicked on background
				if(*CommandOnClick && Specs.CommandInterface)
				{
					Specs.CommandInterface->Command(CommandOnClick); // can display a different menu and/or kill Menu::this!
					return true;
				}
			}
		}
		return Processed;
	}
}

void Menu::Render(D3DStruct *DS)
{
	if(Specs.Visible)
	{
		Matrix Mtx, WMtx;
		Mtx.SetIdentityMatrix();
		WMtx.SetScaleTranslationMatrix(Vector(Size.x, Size.y, 0), Vector(Pos.x, Pos.y, 0));
		DS->D3DDevice->SetTransform(D3DTS_VIEW, &Mtx);
		DS->D3DDevice->SetTransform(D3DTS_PROJECTION, &Mtx);
		DS->D3DDevice->SetTransform(D3DTS_WORLD, &WMtx);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
		DS->D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
		DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATER);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHAREF, 0x10);
		DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
		DS->D3DDevice->SetTexture(0, Background);
		// Background
		RS.RenderIndex(DS->D3DDevice, 0, 4, 0, 6);
		// Text
		for(int i = 0; i < Items.Count(); i++)
		{
			if(Items[i].Type == MenuItem::MITEM_TYPE_BUTTON)
			{
				DS->D3DDevice->SetTexture(0, Button);
				WMtx.SetTranslationMatrix(Vector(Items[i].Pos.x, Items[i].Pos.y, 0));
				DS->D3DDevice->SetTransform(D3DTS_VIEW, &Mtx);
				DS->D3DDevice->SetTransform(D3DTS_PROJECTION, &Mtx);
				DS->D3DDevice->SetTransform(D3DTS_WORLD, &WMtx);
				DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
				//WMtx.SetScaleTranslationMatrix(Vector(Items[i].Size.x, Items[i].Size.y, 0), Vector(Items[i].Pos.x, Items[i].Pos.y, 0));
				RenderStruct<Vertex, I3D_FVFVERTEX, 16, 16> RS;

				// plain texture display
				RS.AddVertex(Vertex(Vector(0, 0, 0), Vector(0, 0, -1), Vector2D(0, 1)));
				RS.AddVertex(Vertex(Vector(Items[i].Size.x, 0, 0), Vector(0, 0, -1), Vector2D(1, 1)));
				RS.AddVertex(Vertex(Vector(Items[i].Size.x, Items[i].Size.y, 0), Vector(0, 0, -1), Vector2D(1, 0)));
				RS.AddVertex(Vertex(Vector(Items[i].Size.x, Items[i].Size.y, 0), Vector(0, 0, -1), Vector2D(1, 0)));
				RS.AddVertex(Vertex(Vector(0, Items[i].Size.y, 0), Vector(0, 0, -1), Vector2D(0, 0)));
				RS.AddVertex(Vertex(Vector(0, 0, 0), Vector(0, 0, -1), Vector2D(0, 1)));

				// edge/area mapping
				/*#define B_EDGE_X (0.25f * Items[i].Size.x)
				#define B_EDGE_Y (0.25f * Items[i].Size.x)
				#define B_EDGE_TEX_X 0.25f
				#define B_EDGE_TEX_Y 0.25f
				// left bottom
				float Edgex = min(B_EDGE_X, Items[i].Size.x / 2.0f);
				float Edgey = min(B_EDGE_Y, Items[i].Size.y / 2.0f);
				RS.AddVertex(Vertex(Vector(0, 0, 0), Vector(0, 0, -1), Vector2D(0, 1)));
				RS.AddVertex(Vertex(Vector(Edgex, 0, 0), Vector(0, 0, -1), Vector2D(B_EDGE_TEX_X, 1)));
				RS.AddVertex(Vertex(Vector(Edgex, Edgey, 0), Vector(0, 0, -1), Vector2D(B_EDGE_TEX_X, 1 - B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(Edgex, Edgey, 0), Vector(0, 0, -1), Vector2D(B_EDGE_TEX_X, 1 - B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(0, Edgey, 0), Vector(0, 0, -1), Vector2D(0, 1 - B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(0, 0, 0), Vector(0, 0, -1), Vector2D(0, 1)));
				// left top
				Edgex = min(B_EDGE_X, Items[i].Size.x / 2.0f);
				Edgey = max(Items[i].Size.y - B_EDGE_Y, Items[i].Size.y / 2.0f);
				RS.AddVertex(Vertex(Vector(0, Items[i].Size.y, 0), Vector(0, 0, -1), Vector2D(0, 0)));
				RS.AddVertex(Vertex(Vector(0, Edgey, 0), Vector(0, 0, -1), Vector2D(0, B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(Edgex, Edgey, 0), Vector(0, 0, -1), Vector2D(B_EDGE_TEX_X, B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(Edgex, Edgey, 0), Vector(0, 0, -1), Vector2D(B_EDGE_TEX_X, B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(Edgex, Items[i].Size.y, 0), Vector(0, 0, -1), Vector2D(B_EDGE_TEX_X, 0)));
				RS.AddVertex(Vertex(Vector(0, Items[i].Size.y, 0), Vector(0, 0, -1), Vector2D(0, 0)));
				// right bottom
				Edgex = max(Items[i].Size.x - B_EDGE_X, Items[i].Size.x / 2.0f);
				Edgey = min(B_EDGE_Y, Items[i].Size.y / 2.0f);
				RS.AddVertex(Vertex(Vector(Items[i].Size.x, 0, 0), Vector(0, 0, -1), Vector2D(1, 1)));
				RS.AddVertex(Vertex(Vector(Items[i].Size.x, Edgey, 0), Vector(0, 0, -1), Vector2D(1, 1 - B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(Edgex, Edgey, 0), Vector(0, 0, -1), Vector2D(1 - B_EDGE_TEX_X, 1 - B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(Edgex, Edgey, 0), Vector(0, 0, -1), Vector2D(1 - B_EDGE_TEX_X, 1 - B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(Edgex, 0, 0), Vector(0, 0, -1), Vector2D(1 - B_EDGE_TEX_X, 1)));
				RS.AddVertex(Vertex(Vector(Items[i].Size.x, 0, 0), Vector(0, 0, -1), Vector2D(1, 1)));
				// right top
				Edgex = max(Items[i].Size.x - B_EDGE_X, Items[i].Size.x / 2.0f);
				Edgey = max(Items[i].Size.y - B_EDGE_Y, Items[i].Size.y / 2.0f);
				RS.AddVertex(Vertex(Vector(Items[i].Size.x, Items[i].Size.y, 0), Vector(0, 0, -1), Vector2D(1, 0)));
				RS.AddVertex(Vertex(Vector(Edgex, Items[i].Size.y, 0), Vector(0, 0, -1), Vector2D(1 - B_EDGE_TEX_X, 0)));
				RS.AddVertex(Vertex(Vector(Edgex, Edgey, 0), Vector(0, 0, -1), Vector2D(1 - B_EDGE_TEX_X, B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(Edgex, Edgey, 0), Vector(0, 0, -1), Vector2D(1 - B_EDGE_TEX_X, B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(Items[i].Size.x, Edgey, 0), Vector(0, 0, -1), Vector2D(1, B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(Items[i].Size.x, Items[i].Size.y, 0), Vector(0, 0, -1), Vector2D(1, 0)));
				// top
				float Edgex1 = min(B_EDGE_X, Items[i].Size.x / 2.0f);
				float Edgex2 = max(Items[i].Size.x - B_EDGE_X, Items[i].Size.x / 2.0f);
				Edgey = max(Items[i].Size.y - B_EDGE_Y, Items[i].Size.y / 2.0f);
				RS.AddVertex(Vertex(Vector(Edgex1, Items[i].Size.y, 0), Vector(0, 0, -1), Vector2D(B_EDGE_TEX_X, 0)));
				RS.AddVertex(Vertex(Vector(Edgex1, Edgey, 0), Vector(0, 0, -1), Vector2D(B_EDGE_TEX_X, B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(Edgex2, Edgey, 0), Vector(0, 0, -1), Vector2D(1 - B_EDGE_TEX_X, B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(Edgex2, Edgey, 0), Vector(0, 0, -1), Vector2D(1 - B_EDGE_TEX_X, B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(Edgex2, Items[i].Size.y, 0), Vector(0, 0, -1), Vector2D(1 - B_EDGE_TEX_X, 0)));
				RS.AddVertex(Vertex(Vector(Edgex1, Items[i].Size.y, 0), Vector(0, 0, -1), Vector2D(B_EDGE_TEX_X, 0)));
				// bottom
				Edgex1 = min(B_EDGE_X, Items[i].Size.x / 2.0f);
				Edgex2 = max(Items[i].Size.x - B_EDGE_X, Items[i].Size.x / 2.0f);
				Edgey = min(B_EDGE_Y, Items[i].Size.y / 2.0f);
				RS.AddVertex(Vertex(Vector(Edgex1, 0, 0), Vector(0, 0, -1), Vector2D(B_EDGE_TEX_X, 1)));
				RS.AddVertex(Vertex(Vector(Edgex2, 0, 0), Vector(0, 0, -1), Vector2D(1 - B_EDGE_TEX_X, 1)));
				RS.AddVertex(Vertex(Vector(Edgex2, Edgey, 0), Vector(0, 0, -1), Vector2D(1 - B_EDGE_TEX_X, 1 - B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(Edgex2, Edgey, 0), Vector(0, 0, -1), Vector2D(1 - B_EDGE_TEX_X, 1 - B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(Edgex1, Edgey, 0), Vector(0, 0, -1), Vector2D(B_EDGE_TEX_X, 1 - B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(Edgex1, 0, 0), Vector(0, 0, -1), Vector2D(B_EDGE_TEX_X, 1)));
				// left
				Edgex = min(B_EDGE_X, Items[i].Size.x / 2.0f);
				float Edgey1 = min(B_EDGE_Y, Items[i].Size.y / 2.0f);
				float Edgey2 = max(Items[i].Size.y - B_EDGE_Y, Items[i].Size.y / 2.0f);
				RS.AddVertex(Vertex(Vector(0, Edgey1, 0), Vector(0, 0, -1), Vector2D(0, B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(Edgex, Edgey1, 0), Vector(0, 0, -1), Vector2D(B_EDGE_TEX_X, B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(Edgex, Edgey2, 0), Vector(0, 0, -1), Vector2D(B_EDGE_TEX_X, 1 - B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(Edgex, Edgey2, 0), Vector(0, 0, -1), Vector2D(B_EDGE_TEX_X, 1 - B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(0, Edgey2, 0), Vector(0, 0, -1), Vector2D(0, 1 - B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(0, Edgey1, 0), Vector(0, 0, -1), Vector2D(0, B_EDGE_TEX_Y)));
				// right
				Edgex = max(Items[i].Size.x - B_EDGE_X, Items[i].Size.x / 2.0f);
				Edgey1 = min(B_EDGE_Y, Items[i].Size.y / 2.0f);
				Edgey2 = max(Items[i].Size.y - B_EDGE_Y, Items[i].Size.y / 2.0f);
				RS.AddVertex(Vertex(Vector(Items[i].Size.x, Edgey1, 0), Vector(0, 0, -1), Vector2D(1, B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(Items[i].Size.x, Edgey2, 0), Vector(0, 0, -1), Vector2D(1, 1 - B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(Edgex, Edgey2, 0), Vector(0, 0, -1), Vector2D(1 - B_EDGE_TEX_X, 1 - B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(Edgex, Edgey2, 0), Vector(0, 0, -1), Vector2D(1 - B_EDGE_TEX_X, 1 - B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(Edgex, Edgey1, 0), Vector(0, 0, -1), Vector2D(1 - B_EDGE_TEX_X, B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(Items[i].Size.x, Edgey1, 0), Vector(0, 0, -1), Vector2D(1, B_EDGE_TEX_Y)));
				// center
				Edgex1 = min(B_EDGE_X, Items[i].Size.x / 2.0f);
				Edgex2 = max(Items[i].Size.x - B_EDGE_X, Items[i].Size.x / 2.0f);
				Edgey1 = min(B_EDGE_Y, Items[i].Size.y / 2.0f);
				Edgey2 = max(Items[i].Size.y - B_EDGE_Y, Items[i].Size.y / 2.0f);
				RS.AddVertex(Vertex(Vector(Edgex1, Edgey1, 0), Vector(0, 0, -1), Vector2D(B_EDGE_TEX_X, B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(Edgex2, Edgey1, 0), Vector(0, 0, -1), Vector2D(1 - B_EDGE_TEX_X, B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(Edgex2, Edgey2, 0), Vector(0, 0, -1), Vector2D(1 - B_EDGE_TEX_X, 1 - B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(Edgex2, Edgey2, 0), Vector(0, 0, -1), Vector2D(1 - B_EDGE_TEX_X, 1 - B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(Edgex1, Edgey2, 0), Vector(0, 0, -1), Vector2D(B_EDGE_TEX_X, 1 - B_EDGE_TEX_Y)));
				RS.AddVertex(Vertex(Vector(Edgex1, Edgey1, 0), Vector(0, 0, -1), Vector2D(B_EDGE_TEX_X, B_EDGE_TEX_Y)));*/

				DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
				DS->D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
				DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
				DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE);
				DS->D3DDevice->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATER);
				DS->D3DDevice->SetRenderState(D3DRS_ALPHAREF, 0x10);
				DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, FALSE);

				RS.CreateVBuf(DS->D3DDevice);
				RS.RenderVertex(DS->D3DDevice);
				//RS.RenderIndex(DS->D3DDevice, 4, 4, 0, 6);
			}
			Text::DrawString(DS, Items[i].Text,
				Items[i].Pos.x + Items[i].TextOffset.x,
				Items[i].Pos.y + Items[i].LetterSize.y + Items[i].TextOffset.y,
				Items[i].LetterSize.x,
				Items[i].LetterSize.y, 0.5f);
		}
		DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, TRUE);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, FALSE);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
		DS->D3DDevice->SetTransform(D3DTS_VIEW, &DS->ProjMtx);
		DS->D3DDevice->SetTransform(D3DTS_PROJECTION, &DS->ViewMtx);
		// submenus
		if(Submenu)
			Submenu->Render(DS);
	}
}

}
