/***********************************************************/
/*                                                         */
/* Menu.h                                                  */
/*                                                         */
/* created 10.03.2003                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef MENU_H
#define MENU_H

#include "D3DStruct.h"
#include "RenderStruct.h"
#include "CommandInterface.h"
#include "DataInterface.h"

// www.infinitecode.com 3d library
namespace I3D
{

class Menu
{
public:
	struct MenuSpecs
	{
		D3DStruct *DS;
		char *Directory;
		char *MenuFile;
		bool Visible;
		CommandInterface *CommandInterface;
		DataInterface *DataIf;
	};

private:
	MenuSpecs Specs;
	LPDIRECT3DTEXTURE9 Background, Button;
	RenderStruct<Vertex, I3D_FVFVERTEX, 16, 16> RS;
	bool CloseOnClick;
	char CommandOnClick[1024];
	float Columns, Rows;
	Menu *Submenu;

	class MenuItem
	{
	public:
		MenuItem(void) : FontSize(1.0f), CloseOnClick(false)
		{
			*CommandOnClick = 0;
			*SubmenuOnClick = 0;
		}

		// item types
		enum
		{
			MITEM_TYPE_TEXT = 1,
			MITEM_TYPE_BUTTON = 2
		};

		char Text[1024], CommandOnClick[1024], SubmenuOnClick[1024];
		IMATH::Vector2D Pos, Size, LetterSize, TextOffset;
		float FontSize;
		int Type;
		bool CloseOnClick, Center;
	};

	ISL::DynamicArray<MenuItem, 0, 1> Items;
	IMATH::Vector2D Pos, Size, Spacing;

public:

	Menu(void): Submenu(0) {}

	void Create(MenuSpecs *Specs);
	void Render(D3DStruct *DS);
	void Init3D(D3DStruct *DS, bool recurse);
	bool Click(Vector2D Pos);

	void SetVisible(bool Value)
	{
		Specs.Visible = Value;
	}

	bool GetVisible(void)
	{
		return Specs.Visible;
	}

};

}

#endif
