/***********************************************************/
/*                                                         */
/* Minimap.cpp                                             */
/*                                                         */
/* created 16.01.2004                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "Minimap.h"
#include "Waterlevel.h"
#include "TextureManager.h"

#ifndef PI
#define PI 3.1415926535897932f
#endif

// www.infinitecode.com game library
namespace IGAME
{

void Minimap::Create(Specs &Sp)
{
	int State = 0;
	try
	{
		State = 1;
		Minimap::Sp = Sp;
		Destroy();
		State = 2;
		ViewingPos = IMATH::Vector2DXZ(0, 0);
		ViewingVector = IMATH::Vector2DXZ(0, 0);
		State = 3;
		RS.ClearVertex();
		RS.ClearIndex();
		State = 4;
		using namespace I3D;
		RS.AddVertex(Vertex(Vector(0, 0, 0), Vector(0, 0, -1), Vector2D(0, 1)));
		RS.AddVertex(Vertex(Vector(1, 0, 0), Vector(0, 0, -1), Vector2D(1, 1)));
		RS.AddVertex(Vertex(Vector(1, 1, 0), Vector(0, 0, -1), Vector2D(1, 0)));
		RS.AddVertex(Vertex(Vector(0, 1, 0), Vector(0, 0, -1), Vector2D(0, 0)));
		RS.AddIndex(0);
		RS.AddIndex(1);
		RS.AddIndex(2);
		RS.AddIndex(0);
		RS.AddIndex(2);
		RS.AddIndex(3);
		Init3D(Sp.DS, false);
		State = 5;
	}
	catch(char *s)
	{
		throw(s);
	}
	catch(...)
	{
		char Str[1024];
		sprintf(Str, "Minimap::Create#%i", State);
		throw(Str);
	}
}

void Minimap::ProcessSurface()
{
	// terrain
	// get Texture width and height
	D3DSURFACE_DESC LDesc;
	if(SurfaceTexture->GetLevelDesc(0, &LDesc) != D3D_OK)
		throw("Minimap::ProcessSurface: Can't get description of Texture level 0");
	D3DLOCKED_RECT LRect;
	memset(&LRect, 0, sizeof(LRect));
	if(SurfaceTexture->LockRect( 0, &LRect, NULL, 0 ) != D3D_OK)
		throw("Minimap::ProcessSurface can't lock texture");
	// get buffer
	unsigned int *Buf=(unsigned int*)LRect.pBits;
	for(int x = 0; x < (int)LDesc.Width; ++x)
	{
		for(int z = 0; z < (int)LDesc.Height; ++z)
		{
			//const WaterAlpha = 0xe0;
			// get old pixel
			unsigned int Color;// = Buf[x + LDesc.Width * z];
			// process pixel
			IMATH::Vector2DXZ Pos(-(x - (int)LDesc.Width / 2) * Sp.Surface->GetSize() / LDesc.Width, (z - (int)LDesc.Height / 2) * Sp.Surface->GetSize() / LDesc.Height);
			if(Sp.Surface->GetY(Pos.x, Pos.z) < WATERLEVEL)
				Color = 0x000020ff;
			else
				Color = 0;
			// set new pixel
			if(x >= 0 && x < (int)LDesc.Width && z >= 0 && z < (int)LDesc.Height)
			{
				unsigned int Alpha = Color >> 24;
				unsigned int Red = (Color >> 16) & 0xff;
				unsigned int Green = (Color >> 8) & 0xff;
				unsigned int Blue = Color & 0xff;
				unsigned int OldRed = (Buf[x + LDesc.Width * z] >> 16) & 0xff;
				unsigned int OldGreen = (Buf[x + LDesc.Width * z] >> 8) & 0xff;
				unsigned int OldBlue = Buf[x + LDesc.Width * z] & 0xff;
				// Alpha
				Buf[x + LDesc.Width * z] = MinimapAlpha << 24;
				// Red
				Buf[x + LDesc.Width * z] |= min(Red + OldRed, 0xff) << 16;
				// Green
				Buf[x + LDesc.Width * z] |= min(Green + OldGreen, 0xff) << 8;
				// Blue
				Buf[x + LDesc.Width * z] |= min(Blue + OldBlue, 0xff);
			}
		}
	}
	// unlock
	if(SurfaceTexture->UnlockRect(0) != D3D_OK)
		throw("Minimap::ProcessSurface can't unlock texture");
	// mipmaps
	//I3D::TextureManager::CreateMipMaps(Texture, 0);
}

void Minimap::ProcessTexture()
{
	int State = 0;
	try
	{
		State = 1;
		// terrain
		// get Texture width and height
		D3DSURFACE_DESC LDesc;
		if(Texture->GetLevelDesc(0, &LDesc) != D3D_OK)
			throw("Minimap::ProcessTexture: Can't get description of Texture level 0");
		D3DLOCKED_RECT LRect;
		memset(&LRect, 0, sizeof(LRect));
		State = 2;
		if(Texture->LockRect( 0, &LRect, NULL, 0 ) != D3D_OK)
			throw("Minimap::ProcessTexture can't lock texture");
		// get buffer
		unsigned *Buf=(unsigned int*)LRect.pBits;
		static float Max, Max2;
		static int Index, Index2;
		IMATH::Vector2DXZ Pos;
		unsigned *OwnerMap = new unsigned[LDesc.Width * LDesc.Height];
		State = 3;
		/*for(int x = 0; x < (int)LDesc.Width; ++x)
		{
			Max = -1;
			Max2 = -1;
			State = 4;
			for(int z = 0; z < (int)LDesc.Height; ++z)
			{
				// get old pixel
				unsigned int Color = Buf[x + LDesc.Width * z];
				// voronoi map of ownership
				if(x & 3)
					OwnerMap[x + LDesc.Width * z] = OwnerMap[x - 1 + LDesc.Width * z];
				else if(z & 3)
					OwnerMap[x + LDesc.Width * z] = OwnerMap[x + LDesc.Width * (z - 1)];
				else
				{
					{
						State = 5;
						Max = -1;
						Max2 = -1;
						Index = -1;
						Index2 = -1;
						Pos = IMATH::Vector2DXZ(-(x - (int)LDesc.Width / 2) * Sp.Surface->GetSize() / LDesc.Width, (z - (int)LDesc.Height / 2) * Sp.Surface->GetSize() / LDesc.Height);
						for(int i = 0; i < Objects.Count(); ++i)
							if((Objects[i]->GetXZPos() - Pos).Length() < Max || Max == -1)
							{
								Max2 = Max;
								Index2 = Index;
								Max = (Objects[i]->GetXZPos() - Pos).Length();
								Index = i;
							}
						State = 6;
					}
					State = 7;
					if(Index != -1)
					{
						if(Objects[Index]->GetOwner() == 2)
							OwnerMap[x + LDesc.Width * z] = 0x00000070;
							//Color = (Color & 0xffffff00) | min((Color & 0xff) + 0x70, 0xff);
						else if(Objects[Index]->GetOwner() == 3)
							OwnerMap[x + LDesc.Width * z] = 0x00400000;
							//Color = (Color & 0xff00ffff) | (min(((Color >> 16) & 0xff) + 0x40, 0xff) << 16);
						else
							OwnerMap[x + LDesc.Width * z] = 0;
					}
					else
						OwnerMap[x + LDesc.Width * z] = 0;
					State = 8;
				}
				State = 9;
				Color = (min(((Color >> 24) & 0xff) + ((OwnerMap[x + LDesc.Width * z] >> 24) & 0xff), 0xff) << 24)
					| (min(((Color >> 16) & 0xff) + ((OwnerMap[x + LDesc.Width * z] >> 16) & 0xff), 0xff) << 16)
					| (min(((Color >> 8) & 0xff) + ((OwnerMap[x + LDesc.Width * z] >> 8) & 0xff), 0xff) << 8)
					| min((Color & 0xff) + (OwnerMap[x + LDesc.Width * z] & 0xff), 0xff);
				State = 10;
				// set new pixel
				if(x >= 0 && x < (int)LDesc.Width && z >= 0 && z < (int)LDesc.Height)
				{
					unsigned int Alpha = Color >> 24;
					unsigned int Red = (Color >> 16) & 0xFF;
					unsigned int Green = (Color >> 8) & 0xFF;
					unsigned int Blue = (Color) & 0xFF;
					unsigned int OldRed = (Buf[x + LDesc.Width * z] >> 16) & 0xFF;;
					unsigned int OldGreen = (Buf[x + LDesc.Width * z] >> 8) & 0xFF;
					unsigned int OldBlue = Buf[x + LDesc.Width * z] & 0xFF;
					// Alpha
					Buf[x + LDesc.Width * z] = MinimapAlpha << 24;
					// Red
					Buf[x + LDesc.Width * z] += (Red * Alpha + OldRed * (0xFF - Alpha)) / 0x100 << 16;
					// Green
					Buf[x + LDesc.Width * z] += (Green * Alpha + OldGreen * (0xFF - Alpha)) / 0x100 << 8;
					// Blue
					Buf[x + LDesc.Width * z] += (Blue * Alpha + OldBlue * (0xFF - Alpha)) / 0x100;
				}
				State = 11;
			}
		}*/
		State = 12;
		delete[] OwnerMap;
		// Objects
		for(int i = 0; i < Objects.Count(); ++i)
		{
			unsigned Color = 0xff000000;
			switch(Objects[i]->GetOwner())
			{
			case 0:
				Color = 0xffffffff;
				break;
			case 1:
				Color = 0xff00ff00;
				break;
			case 2:
				Color = 0xff0000ff;
				break;
			case 3:
				Color = 0xffff0000;
				break;
			}
			int Size = Objects[i]->GetMinimapSize();
			float px = (-Objects[i]->GetXZPos().x / Sp.Surface->GetSize() + 0.5f) * LDesc.Width;
			float pz = (Objects[i]->GetXZPos().z / Sp.Surface->GetSize() + 0.5f) * LDesc.Height;
			State = 13;
			for(int x = -Size; x <= Size; ++x)
				for(int z = -Size; z <= Size; ++z)
				{
					float a = px + x;
					float b = pz + z;
					if(a >= 0 && a < (int)LDesc.Width && b >= 0 && b < (int)LDesc.Height)
						Buf[(int)a + LDesc.Width * (int)b] = Color;
						//I3D::TextureManager::SetPixel(Texture, (int)a, (int)b, Color);
				}
			// highlight
			State = 14;
			if(Objects[i]->GetHighlight())
			{
				int Radius = 5 + ((int)(5 * Sp.DS->Sp->GameTime)) % 15;
				float px = (-Objects[i]->GetXZPos().x / Sp.Surface->GetSize() + 0.5f) * LDesc.Width;
				float pz = (Objects[i]->GetXZPos().z / Sp.Surface->GetSize() + 0.5f) * LDesc.Height;
				Circle(Buf, LDesc.Width, LDesc.Height, (int)px, (int)pz, Radius, 1, Color);
			}
		}
		State = 15;
		// viewing direction
		IMATH::Vector2DXZ a, b;
		a.x = (-(ViewingPos.x - 256.0f * ViewingVector.x) / Sp.Surface->GetSize() + 0.5f) * LDesc.Width;
		a.z = ((ViewingPos.z - 256.0f * ViewingVector.z) / Sp.Surface->GetSize() + 0.5f) * LDesc.Height;
		b.x = (-(ViewingPos.x + 256.0f * ViewingVector.x) / Sp.Surface->GetSize() + 0.5f) * LDesc.Width;
		b.z = ((ViewingPos.z + 256.0f * ViewingVector.z) / Sp.Surface->GetSize() + 0.5f) * LDesc.Height;
		Line(Buf, LDesc.Width, LDesc.Height, (int)a.x, (int)a.z, (int)b.x, (int)b.z, 1, 0xffffffff);
		IMATH::Vector2DXZ o(-(b - a).z, (b - a).x);
		Line(Buf, LDesc.Width, LDesc.Height, (int)b.x, (int)b.z, (int)(b.x - 0.2f * (b - a).x + 0.2f * o.x), (int)(b.z - 0.2f * (b - a).z + 0.2f * o.z), 1, 0xffffffff);
		Line(Buf, LDesc.Width, LDesc.Height, (int)b.x, (int)b.z, (int)(b.x - 0.2f * (b - a).x - 0.2f * o.x), (int)(b.z - 0.2f * (b - a).z - 0.2f * o.z), 1, 0xffffffff);
		State = 16;
		// unlock
		if(Texture->UnlockRect(0) != D3D_OK)
			throw("Minimap::ProcessTexture can't unlock texture");
		// mipmaps
		//I3D::TextureManager::CreateMipMaps(Texture, 0);
		State = 17;
	}
	catch(char *s)
	{
		throw(s);
	}
	catch(...)
	{
		char Str[1024];
		sprintf(Str, "Minimap::ProcessTexture#%d", State);
		throw(Str);
	}
}

void Minimap::Circle(unsigned *Buf, int Width, int Height, int x, int y, int Radius, int Size, unsigned Color)
{
	for(int i = -(int)(1/sqrt(2.0) * Radius); i <= (int)(1/sqrt(2.0) * Radius); ++i)
	{
		int j = (int)sqrt((double)(Radius * Radius - i * i));
		for(int k = -Size; k <= Size; ++k)
		{
			Buf[x + i     + Width * (y + j + k)] = Color;
			Buf[x + i     + Width * (y - j + k)] = Color;
			Buf[x + j + k + Width * (y + i    )] = Color;
			Buf[x - j + k + Width * (y + i    )] = Color;
			/*I3D::TextureManager::SetPixel(Texture, x + i    , y + j + k, Color);
			I3D::TextureManager::SetPixel(Texture, x + i    , y - j + k, Color);
			I3D::TextureManager::SetPixel(Texture, x + j + k, y + i    , Color);
			I3D::TextureManager::SetPixel(Texture, x - j + k, y + i    , Color);*/
		}
	}
}

void Minimap::Line(unsigned *Buf, int Width, int Height, int x1, int y1, int x2, int y2, int Size, unsigned Color)
{
	D3DSURFACE_DESC LDesc;
	Texture->GetLevelDesc(0, &LDesc);
	if(x1 == x2 && y1 == y2)
	{
		//I3D::TextureManager::SetPixel(Texture, x1, y1, Color);
		if(x1 >= 0 && x1 < (int)LDesc.Width && y1 >= 0 && y1 < (int)LDesc.Height)
			Buf[x1 + Width * y1] = Color;
		return;
	}
	if(abs(x2 - x1) > abs(y2 - y1))
	{
		if(x1 > x2)
		{
			x1 ^= x2;
			x2 ^= x1;
			x1 ^= x2;
			y1 ^= y2;
			y2 ^= y1;
			y1 ^= y2;
		}
		for(int i = x1; i <= x2; ++i)
			for(int x = -Size; x <= Size; ++x)
				for(int y = -Size; y <= Size; ++y)
				{
					int xx = i + x;
					int yy = y1 + (i - x1) * (y2 - y1) / (x2 - x1) + y;
					if(xx >= 0 && xx < (int)LDesc.Width && yy >= 0 && yy < (int)LDesc.Height)
						Buf[xx + Width * yy] = Color;
						//I3D::TextureManager::SetPixel(Texture, xx, yy, Color);
				}
	}
	else
	{
		if(y1 > y2)
		{
			x1 ^= x2;
			x2 ^= x1;
			x1 ^= x2;
			y1 ^= y2;
			y2 ^= y1;
			y1 ^= y2;
		}
		for(int i = y1; i <= y2; ++i)
			for(int x = -Size; x <= Size; ++x)
				for(int y = -Size; y <= Size; ++y)
				{
					int xx = x1 + (i - y1) * (x2 - x1) / (y2 - y1) + x;
					int yy = i + y;
					if(xx >= 0 && xx < (int)LDesc.Width && yy >= 0 && yy < (int)LDesc.Height)
						Buf[xx + Width * yy] = Color;
						//I3D::TextureManager::SetPixel(Texture, xx, yy, Color);
				}
	}
}

bool Minimap::ClickTest(const IMATH::Vector2D &Pos)
{
	IMATH::Vector RPos((Pos.x - (0.9f - (0.5f * 2) / 3)) / ((0.5f * 2) / 3), 0, (Pos.y - 0.4f) / 0.5f);
	RPos = I3D::Matrix::RotationYMatrix(Angle + PI, 0.5f, 0.5f) * RPos;
	return RPos.x >= 0 && RPos.x <= 1 && RPos.z >= 0 && RPos.z <=1;
}

IMATH::Vector Minimap::Click(const IMATH::Vector2D &Pos)
{
	IMATH::Vector RPos(0.5f - (Pos.x - (0.9f - (0.5f * 2) / 3)) / ((0.5f * 2) / 3), 0, 0.5f - (Pos.y - 0.4f) / 0.5f);
	RPos = I3D::Matrix::RotationYMatrix(Angle + PI, 0.0f, 0.0f) * RPos;
	//if(RPos.x >= 0 && RPos.x <= 1 && RPos.y >= 0 && RPos.y <=1)
	IMATH::Vector Ret = Sp.Surface->GetSize() * RPos;
	Ret.y = max(Sp.Surface->GetY(Ret.x, Ret.z), WATERLEVEL);
	return Ret;
}

void Minimap::Init3D(I3D::D3DStruct *DS, bool recurse)
{
	RS.CreateVBuf(Sp.DS->D3DDevice);
	RS.CreateIBuf(Sp.DS->D3DDevice);
	float Size = Sp.Surface->GetSize()/2;
	SurfaceTexture = Sp.Surface->GetTexture(Sp.DS, Size, Size, -Size, -Size);
	LPDIRECT3DSURFACE9 STSurface;
	SurfaceTexture->GetSurfaceLevel(0, &STSurface);
	D3DSURFACE_DESC Desc;
	STSurface->GetDesc(&Desc);
	// TODO: D3DPOOL_MANAGED, D3DPOOL_DEFAULT?
	Sp.DS->D3DDevice->CreateTexture(Desc.Width, Desc.Height, 1, 0, Desc.Format, D3DPOOL_MANAGED, &Texture, NULL);
	LPDIRECT3DSURFACE9 TSurface;
	Texture->GetSurfaceLevel(0, &TSurface);
	Sp.DS->D3DDevice->UpdateSurface(STSurface, 0, TSurface, 0);
	ProcessSurface();
}

void Minimap::Update()
{
	static float LastDraw = Sp.DS->Sp->GameTime - 0.75f;
	if(Sp.DS->Sp->GameTime - LastDraw >= 0.75f)
	{
//		float Size = Sp.Surface->GetSize()/2;
//		Texture = Sp.Surface->GetTexture(Sp.DS, Size, Size, -Size, -Size);
		LPDIRECT3DSURFACE9 STSurface;
		SurfaceTexture->GetSurfaceLevel(0, &STSurface);
		LPDIRECT3DSURFACE9 TSurface;
		Texture->GetSurfaceLevel(0, &TSurface);
		Sp.DS->D3DDevice->UpdateSurface(STSurface, 0, TSurface, 0);
		ProcessTexture();
		LastDraw = Sp.DS->Sp->GameTime;
	}
}

void Minimap::Render()
{
	Update();
	// render
	I3D::Matrix Mtx, WMtx;
	Mtx.SetIdentityMatrix();
	WMtx.SetRotationZMatrix(Angle + PI, 0.5, 0.5);
	WMtx *= WMtx.ScaleTranslationMatrix(IMATH::Vector((0.5f*2)/3, 0.5f, 0), IMATH::Vector(0.9f - (0.5f*2)/3, 0.4f, 0));
	Sp.DS->D3DDevice->SetTransform(D3DTS_VIEW, &Mtx);
	Sp.DS->D3DDevice->SetTransform(D3DTS_PROJECTION, &Mtx);
	Sp.DS->D3DDevice->SetTransform(D3DTS_WORLD, &WMtx);
	/*Sp.DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
	Sp.DS->D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	Sp.DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
	Sp.DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE);
	Sp.DS->D3DDevice->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATER);
	Sp.DS->D3DDevice->SetRenderState(D3DRS_ALPHAREF, 0x10);*/
	Sp.DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
	Sp.DS->D3DDevice->SetTexture(0, Texture);
	RS.RenderIndex(Sp.DS->D3DDevice);
	Sp.DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, TRUE);
	/*Sp.DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, FALSE);
	Sp.DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);*/
	Sp.DS->D3DDevice->SetTransform(D3DTS_VIEW, &Sp.DS->ViewMtx);
	Sp.DS->D3DDevice->SetTransform(D3DTS_PROJECTION, &Sp.DS->ProjMtx);
}

}
