/***********************************************************/
/*                                                         */
/* Minimap.h                                               */
/*                                                         */
/* created 16.01.2004                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef MINIMAP_H
#define MINIMAP_H

#include "D3DStruct.h"
#include "RenderStruct.h"
#include "D3DStructures.h"
#include "Terrain.h"
#include "DynamicArray.h"
#include "MathStructures.h"

#include <d3d9.h>

// www.infinitecode.com game library
namespace IGAME
{

class Minimap
{
public:
	class Specs
	{
	public:
		I3D::D3DStruct *DS;
		I3D::Terrain *Surface;
	};

	// object interface
	class ObjectInterface
	{
	public:
		virtual IMATH::Vector2DXZ GetXZPos() const = 0;
		virtual int GetOwner() const = 0;
		virtual int GetMinimapSize() const = 0;
		virtual bool GetHighlight() const = 0;
	};

private:
	LPDIRECT3DTEXTURE9 Texture, SurfaceTexture;
	I3D::RenderStruct<I3D::Vertex, I3D_FVFVERTEX, 4, 4> RS;
	Specs Sp;
	ISL::DynamicArray<ObjectInterface *> Objects;
	IMATH::Vector2DXZ ViewingVector, ViewingPos;
	static const int MinimapAlpha = 0xe0;
	float Angle;

public:
	Minimap(): Texture(0), Angle(0) {};

	~Minimap()
	{
		Destroy();
	}

	void Destroy()
	{
		RELEASE(Texture);
		RELEASE(SurfaceTexture);
		Objects.Clear();
	}

	void AddObject(ObjectInterface *Obj)
	{
		Objects.Add(Obj);
	}

	void DeleteObject(ObjectInterface *Obj)
	{
		Objects.Delete(Obj);
	}

	void SetViewingVector(IMATH::Vector2DXZ &V)
	{
		ViewingVector = V;
	}

	const IMATH::Vector2DXZ &GetViewingVector() const
	{
		return ViewingVector;
	}

	void SetViewingPos(IMATH::Vector2DXZ &V)
	{
		ViewingPos = V;
	}

	const IMATH::Vector2DXZ &GetSetViewingPos() const
	{
		return ViewingPos;
	}

	void SetAngle(float Value)
	{
		Angle = Value;
	}

	float GetAngle()
	{
		return Angle;
	}

	void Create(Specs &Sp);
	void Render();
	void Update();
	void Init3D(I3D::D3DStruct *DS, bool recurse);
	void ProcessSurface();
	void ProcessTexture();
	void Line(unsigned *Buf, int Width, int Height, int x1, int y1, int x2, int y2, int Size, unsigned Color);
	void Circle(unsigned *Buf, int Width, int Height, int x, int y, int Radius, int Size, unsigned Color);
	bool ClickTest(const IMATH::Vector2D &Pos);
	IMATH::Vector Click(const IMATH::Vector2D &Pos);
};

}

#endif
