/***********************************************************/
/*                                                         */
/* MissionEditor.cpp                                       */
/*                                                         */
/* created 15.03.2003                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "MissionEditor.h"

#include "MathStructures.h"
#include "LogFile.h"
#include "MessageWindow.h"
#include "Input.h"
#include "IMath.h"
#include "TextureManager.h"
#include "Text.h"
#include "IDType.h"
#include "IDTank.h"
#include "IDTerrain.h"
#include "IDParticleFX.h"
#include "Tank.h"
#include "IDMissionGoal.h"
#include "WaterLevel.h"
#include "DialogWindow.h"
#include "DebugPos.h"
#include "IDOwner.h"

#include <direct.h>
#include <ctime>
#include <sstream>

// debug
#include <fstream>

// www.infinitecode.com game library
namespace IGAME
{

void MissionEditor::MOPropertiesImpl::AIMove(const char *Name, IMATH::Vector2DXZ Pos)
{
	if(ME->MLib.Missions[ME->Mission]->GetMissionObject(Name))
	{
		ME->MLib.Missions[ME->Mission]->GetMissionObject(Name)->AI = Tank::TNK_AI_MOVE;
		ME->MLib.Missions[ME->Mission]->GetMissionObject(Name)->TargetX = Pos.x;
		ME->MLib.Missions[ME->Mission]->GetMissionObject(Name)->TargetZ = Pos.z;
	}
}

void MissionEditor::MOPropertiesImpl::AIAttack(const char *Name, const char *Target)
{
	if(ME->MLib.Missions[ME->Mission]->GetMissionObject(Name))
	{
		ME->MLib.Missions[ME->Mission]->GetMissionObject(Name)->AI = Tank::TNK_AI_ATTACK;
		strcpy(ME->MLib.Missions[ME->Mission]->GetMissionObject(Name)->Target, Target);
	}
}

void MissionEditor::MOPropertiesImpl::SetPos(const char *Name, IMATH::Vector Pos)
{
	if(ME->MLib.Missions[ME->Mission]->GetMissionObject(Name))
	{
		ME->MLib.Missions[ME->Mission]->GetMissionObject(Name)->x = Pos.x;
		//ME->MLib.Missions[ME->Mission]->GetMissionObject(Name)->y = Pos.y;
		ME->MLib.Missions[ME->Mission]->GetMissionObject(Name)->z = Pos.z;
	}
}

void MissionEditor::MOPropertiesImpl::SetWayPos(const char *Name, int ID, IMATH::Vector2DXZ Pos)
{
	if(ME->MLib.Missions[ME->Mission]->GetWay(Name))
	{
		if(ID == 1)
			ME->MLib.Missions[ME->Mission]->GetWay(Name)->Specs.Pos1 = Pos;
		else
			ME->MLib.Missions[ME->Mission]->GetWay(Name)->Specs.Pos2 = Pos;
	}
}

void MissionEditor::MOPropertiesImpl::SetTerrainPos(const char *Name, int ID, IMATH::Vector2DXZ Pos)
{
	if(ME->MLib.Missions[ME->Mission]->GetTerrainDsc(Name))
	{
		ME->MLib.Missions[ME->Mission]->GetTerrainDsc(Name)->SetPos(ID, Pos);
	}
}

const char *MissionEditor::CIData::GetData(const char *Str)
{
	if(!stricmp(Str, "orders"))
	{
		return ME->MLib.Missions[ME->Mission]->Orders;
	}
	return 0;
}

void MissionEditor::CreateWindowMenu(HWND Window)
{
	int State = 0;
	try
	{
		// menu stuff
		MENUITEMINFO MenuInfo;
		State = 1;
		memset(&MenuInfo, 0, sizeof(MenuInfo));
		MenuInfo.cbSize = sizeof(MenuInfo);
		int i;
		HMENU Menu = CreateMenu(); // menu

		State = 2;
		HMENU FileMenu = CreateMenu(); // file menu
		// item: Load...
		MenuInfo.fMask = MIIM_ID | MIIM_TYPE | MIIM_SUBMENU;
		MenuInfo.fType = MFT_STRING;
		MenuInfo.wID = 1024;
		MenuInfo.dwTypeData = "load...";
		InsertMenuItem(FileMenu, -1, true, &MenuInfo); // add to file menu
		// item: Save
		MenuInfo.wID = 1025;
		MenuInfo.dwTypeData = "save";
		InsertMenuItem(FileMenu, -1, true, &MenuInfo); // add to file menu
		// item: new
		MenuInfo.wID = 1026;
		MenuInfo.dwTypeData = "new";
		InsertMenuItem(FileMenu, -1, true, &MenuInfo); // add to file menu
		// menu: file
		MenuInfo.wID = 1030;
		MenuInfo.dwTypeData = "File";
		MenuInfo.hSubMenu = FileMenu;
		InsertMenuItem(Menu, -1, true, &MenuInfo); // add to menu

		State = 3;
		// mission menu
		HMENU MissionMenu = CreateMenu(); // mission menu
		// item: cam pos
		MenuInfo.hSubMenu = 0;
		MenuInfo.wID = 1401;
		MenuInfo.dwTypeData = "set initial camera position";
		InsertMenuItem(MissionMenu, -1, true, &MenuInfo); // add to file menu
		// add mission goal submenu
		HMENU AddGoalMenu = CreateMenu();
		// item: clear mission goals of selected tanks
		MenuInfo.hSubMenu = 0;
		MenuInfo.wID = 1431;
		MenuInfo.dwTypeData = "clear mission goals of selected tanks";
		InsertMenuItem(AddGoalMenu, -1, true, &MenuInfo); // add to menu
		// item: destroy selected tanks
		MenuInfo.hSubMenu = 0;
		MenuInfo.wID = 1432;
		MenuInfo.dwTypeData = "destroy selected tanks";
		InsertMenuItem(AddGoalMenu, -1, true, &MenuInfo); // add to menu
		// item: selected tanks must survive
		MenuInfo.hSubMenu = 0;
		MenuInfo.wID = 1433;
		MenuInfo.dwTypeData = "selected tanks must survive";
		InsertMenuItem(AddGoalMenu, -1, true, &MenuInfo); // add to menu
		// item: all player vehicles must be destroyed
		MenuInfo.hSubMenu = 0;
		MenuInfo.fMask = MIIM_ID | MIIM_TYPE | MIIM_SUBMENU | MIIM_STATE;
		MenuInfo.wID = 1434;
		MenuInfo.dwTypeData = "all player vehicles must be destroyed";
		InsertMenuItem(AddGoalMenu, -1, true, &MenuInfo); // add to menu
		// item: at least one player vehicle must survive
		MenuInfo.hSubMenu = 0;
		MenuInfo.wID = 1435;
		MenuInfo.dwTypeData = "at least one player vehicle must survive";
		InsertMenuItem(AddGoalMenu, -1, true, &MenuInfo); // add to menu
		// item: all enemy vehicles must be destroyed
		MenuInfo.hSubMenu = 0;
		MenuInfo.wID = 1436;
		MenuInfo.dwTypeData = "all enemy vehicles must be destroyed";
		InsertMenuItem(AddGoalMenu, -1, true, &MenuInfo); // add to menu
		// item: at least one enemy vehicle must survive
		MenuInfo.hSubMenu = 0;
		MenuInfo.wID = 1437;
		MenuInfo.dwTypeData = "at least one enemy vehicle must survive";
		InsertMenuItem(AddGoalMenu, -1, true, &MenuInfo); // add to menu
		// item: move selected tanks to selected area
		MenuInfo.hSubMenu = 0;
		MenuInfo.fMask = MIIM_ID | MIIM_TYPE | MIIM_SUBMENU;
		MenuInfo.wID = 1438;
		MenuInfo.dwTypeData = "move selected tanks to selected area";
		InsertMenuItem(AddGoalMenu, -1, true, &MenuInfo); // add to menu
		// item: selected tanks must not move to selected area
		MenuInfo.hSubMenu = 0;
		MenuInfo.wID = 1439;
		MenuInfo.dwTypeData = "selected tanks must not move to selected area";
		InsertMenuItem(AddGoalMenu, -1, true, &MenuInfo); // add to menu
		// item: any player tank must move to selected area
		MenuInfo.hSubMenu = 0;
		MenuInfo.wID = 1440;
		MenuInfo.dwTypeData = "any player tank must move to selected area";
		InsertMenuItem(AddGoalMenu, -1, true, &MenuInfo); // add to menu
		// item: no player (owner 2) tank is allowed to move to selected area
		MenuInfo.hSubMenu = 0;
		MenuInfo.wID = 1441;
		MenuInfo.dwTypeData = "no player (owner 2) tank is allowed to move to selected area";
		InsertMenuItem(AddGoalMenu, -1, true, &MenuInfo); // add to menu
		// item: any enemy tank must move to selected area
		MenuInfo.hSubMenu = 0;
		MenuInfo.wID = 1442;
		MenuInfo.dwTypeData = "any enemy tank must move to selected area";
		InsertMenuItem(AddGoalMenu, -1, true, &MenuInfo); // add to menu
		// item: enemy: no enemy (owner 3) tank is allowed to move to selected area
		MenuInfo.hSubMenu = 0;
		MenuInfo.wID = 1443;
		MenuInfo.dwTypeData = "enemy: no enemy (owner 3) tank is allowed to move to selected area";
		InsertMenuItem(AddGoalMenu, -1, true, &MenuInfo); // add to menu
		// submenu: add mission goal
		MenuInfo.wID = 1444;
		MenuInfo.dwTypeData = "add mission goal";
		MenuInfo.hSubMenu = AddGoalMenu;
		InsertMenuItem(MissionMenu, -1, true, &MenuInfo); // add to menu
		// item: change random seed
		MenuInfo.hSubMenu = 0;
		MenuInfo.wID = 1446;
		MenuInfo.dwTypeData = "change random seed";
		InsertMenuItem(MissionMenu, -1, true, &MenuInfo); // add to menu
		// item: edit mission orders
		MenuInfo.hSubMenu = 0;
		MenuInfo.wID = 1447;
		MenuInfo.dwTypeData = "edit mission orders";
		InsertMenuItem(MissionMenu, -1, true, &MenuInfo); // add to menu
		// side menu
		HMENU SideMenu = CreateMenu();
		// item: Israel
		MenuInfo.hSubMenu = 0;
		MenuInfo.wID = 1451;
		MenuInfo.dwTypeData = "Israel";
		InsertMenuItem(SideMenu, -1, true, &MenuInfo); // add to menu
		// item: Palestine
		MenuInfo.hSubMenu = 0;
		MenuInfo.wID = 1452;
		MenuInfo.dwTypeData = "Palestine";
		InsertMenuItem(SideMenu, -1, true, &MenuInfo); // add to menu
		// submenu: side
		MenuInfo.wID = 1450;
		MenuInfo.dwTypeData = "side";
		MenuInfo.hSubMenu = SideMenu;
		InsertMenuItem(MissionMenu, -1, true, &MenuInfo); // add to menu
		// menu: mission
		MenuInfo.wID = 1400;
		MenuInfo.dwTypeData = "Mission";
		MenuInfo.hSubMenu = MissionMenu;
		InsertMenuItem(Menu, -1, true, &MenuInfo); // add to menu

		State = 4;
		// tank menu
		HMENU TankMenu = CreateMenu();
		// items: tanks
		for(i = 0; i < TLib.Tanks.Count(); i++)
		{
			if(TLib.Tanks[i].Name && *TLib.Tanks[i].Name)
			{
				MenuInfo.wID = 38700 + TLib.Tanks[i].ID;
				MenuInfo.dwTypeData = TLib.Tanks[i].Name;
				MenuInfo.hSubMenu = 0;
				InsertMenuItem(TankMenu, -1, true, &MenuInfo); // add to menu
			}
		}
		// decal menu
		HMENU DecalMenu = CreateMenu();
		// items: decals
		for(i = 0; i < DLib.Decals.Count(); i++)
		{
			MenuInfo.wID = 37700 + DLib.Decals[i].ID;
			MenuInfo.dwTypeData = DLib.Decals[i].Name;
			MenuInfo.hSubMenu = 0;
			InsertMenuItem(DecalMenu, -1, true, &MenuInfo); // add to menu
		}

		State = 5;
		// edit menu
		HMENU EditMenu = CreateMenu();
		// item: edit terrain and ways
		MenuInfo.wID = 1302;
		MenuInfo.hSubMenu = 0;
		MenuInfo.dwTypeData = "edit terrain and ways";
		InsertMenuItem(EditMenu, -1, true, &MenuInfo); // add to menu
		// item: connect streets
		MenuInfo.wID = 1303;
		MenuInfo.hSubMenu = 0;
		MenuInfo.dwTypeData = "connect streets";
		InsertMenuItem(EditMenu, -1, true, &MenuInfo); // add to menu
		// item: mark area
		MenuInfo.hSubMenu = 0;
		MenuInfo.wID = 1500;
		MenuInfo.dwTypeData = "mark area";
		InsertMenuItem(EditMenu, -1, true, &MenuInfo); // add to menu
		// item: mark area
		MenuInfo.hSubMenu = 0;
		MenuInfo.wID = 1502;
		MenuInfo.dwTypeData = "remove area marker";
		InsertMenuItem(EditMenu, -1, true, &MenuInfo); // add to menu
		// item: highlight vehicle/building in minimap
		MenuInfo.hSubMenu = 0;
		MenuInfo.wID = 1445;
		MenuInfo.dwTypeData = "highlight vehicle/building in minimap";
		InsertMenuItem(EditMenu, -1, true, &MenuInfo); // add to menu
		// owner menu
		HMENU OwnerMenu = CreateMenu();
		// items: owner
		char Str[1024];
		for(i = 0; i < 10; i++)
		{
			MenuInfo.wID = 8192 + 2048 + i;
			switch(i)
			{
			case 0: sprintf(Str, "%d neutral, peaceful", i); break;
			case 1: sprintf(Str, "%d neutral, hostile", i); break;
			case 2: sprintf(Str, "%d player", i); break;
			case 3: sprintf(Str, "%d enemy", i); break;
			case 4: sprintf(Str, "%d enemy 2", i); break;
			case 5: sprintf(Str, "%d enemy 3", i); break;
			case 6: sprintf(Str, "%d enemy 4", i); break;
			case 7: sprintf(Str, "%d enemy 5", i); break;
			case 8: sprintf(Str, "%d enemy 6", i); break;
			case 9: sprintf(Str, "%d enemy 7", i); break;
			}
			MenuInfo.dwTypeData = Str;
			MenuInfo.hSubMenu = 0;
			InsertMenuItem(OwnerMenu, -1, true, &MenuInfo); // add to menu
		}
		// submenu: owner
		MenuInfo.wID = 1032;
		MenuInfo.dwTypeData = "owner of selected and new tanks";
		MenuInfo.hSubMenu = OwnerMenu;
		InsertMenuItem(EditMenu, -1, true, &MenuInfo); // add to menu
		// menu: Edit
		MenuInfo.wID = 1300;
		MenuInfo.dwTypeData = "Edit";
		MenuInfo.hSubMenu = EditMenu;
		InsertMenuItem(Menu, -1, true, &MenuInfo); // add to menu

		State = 6;
		// new menu
		HMENU NewMenu = CreateMenu();
		// item: airfield
		MenuInfo.hSubMenu = 0;
		MenuInfo.wID = 1501;
		MenuInfo.dwTypeData = "airfield";
		InsertMenuItem(NewMenu, -1, true, &MenuInfo); // add to menu
		// submenu: decal
		MenuInfo.wID = 1502;
		MenuInfo.dwTypeData = "decal";
		MenuInfo.hSubMenu = DecalMenu;
		InsertMenuItem(NewMenu, -1, true, &MenuInfo); // add to menu
		// submenu: tank
		MenuInfo.wID = 1031;
		MenuInfo.dwTypeData = "tank";
		MenuInfo.hSubMenu = TankMenu;
		InsertMenuItem(NewMenu, -1, true, &MenuInfo); // add to menu
		// ways / streets menu
		HMENU WayMenu = CreateMenu();
		// item: way
		MenuInfo.hSubMenu = 0;
		MenuInfo.wID = 1034;
		MenuInfo.dwTypeData = "way";
		InsertMenuItem(WayMenu, -1, true, &MenuInfo); // add to menu
		// item: street
		MenuInfo.wID = 1035;
		MenuInfo.dwTypeData = "street";
		InsertMenuItem(WayMenu, -1, true, &MenuInfo); // add to menu
		// item: highway
		MenuInfo.wID = 1036;
		MenuInfo.dwTypeData = "highway";
		InsertMenuItem(WayMenu, -1, true, &MenuInfo); // add to menu
		// submenu: ways / streets
		MenuInfo.wID = 1033;
		MenuInfo.dwTypeData = "ways / streets";
		MenuInfo.hSubMenu = WayMenu;
		InsertMenuItem(NewMenu, -1, true, &MenuInfo); // add to menu
		// trees menu
	/*	HMENU TreeMenu = CreateMenu();
		// item: tree 1
		MenuInfo.wID = 1101;
		MenuInfo.hSubMenu = 0;
		MenuInfo.dwTypeData = "tree 1";
		InsertMenuItem(TreeMenu, -1, true, &MenuInfo); // add to menu
		// item: tree 2
		MenuInfo.wID = 1102;
		MenuInfo.dwTypeData = "tree 2";
		InsertMenuItem(TreeMenu, -1, true, &MenuInfo); // add to menu
		// submenu: tree
		MenuInfo.wID = 1100;
		MenuInfo.dwTypeData = "trees";
		MenuInfo.hSubMenu = TreeMenu;
		InsertMenuItem(NewMenu, -1, true, &MenuInfo); // add to menu*/
		HMENU TerrainMenu = CreateMenu();
		// item: mountain
		MenuInfo.wID = 1201;
		MenuInfo.hSubMenu = 0;
		MenuInfo.dwTypeData = "mountain";
		InsertMenuItem(TerrainMenu, -1, true, &MenuInfo); // add to menu
		// item: high mountain
		MenuInfo.wID = 1202;
		MenuInfo.hSubMenu = 0;
		MenuInfo.dwTypeData = "high mountain";
		InsertMenuItem(TerrainMenu, -1, true, &MenuInfo); // add to menu
		// item: low area
		MenuInfo.wID = 1203;
		MenuInfo.hSubMenu = 0;
		MenuInfo.dwTypeData = "low area";
		InsertMenuItem(TerrainMenu, -1, true, &MenuInfo); // add to menu
		// item: water
		MenuInfo.wID = 1204;
		MenuInfo.hSubMenu = 0;
		MenuInfo.dwTypeData = "water";
		InsertMenuItem(TerrainMenu, -1, true, &MenuInfo); // add to menu
		// item: islands
		MenuInfo.wID = 1205;
		MenuInfo.hSubMenu = 0;
		MenuInfo.dwTypeData = "islands";
		InsertMenuItem(TerrainMenu, -1, true, &MenuInfo); // add to menu
		// item: deep water
		MenuInfo.wID = 1206;
		MenuInfo.hSubMenu = 0;
		MenuInfo.dwTypeData = "deep water";
		InsertMenuItem(TerrainMenu, -1, true, &MenuInfo); // add to menu
		// item: small islands
		MenuInfo.wID = 1207;
		MenuInfo.hSubMenu = 0;
		MenuInfo.dwTypeData = "small islands";
		InsertMenuItem(TerrainMenu, -1, true, &MenuInfo); // add to menu
		// submenu: terrain
		MenuInfo.wID = 1200;
		MenuInfo.dwTypeData = "terrain";
		MenuInfo.hSubMenu = TerrainMenu;
		InsertMenuItem(NewMenu, -1, true, &MenuInfo); // add to menu
		// generate submenu
		HMENU GenerateMenu = CreateMenu();
		// item: mission: capture airbases
		MenuInfo.wID = 1805;
		MenuInfo.hSubMenu = 0;
		MenuInfo.dwTypeData = "mission: capture airbases";
		InsertMenuItem(GenerateMenu, -1, true, &MenuInfo); // add to menu
		// item: mission: protect convoy
		MenuInfo.wID = 1806;
		MenuInfo.hSubMenu = 0;
		MenuInfo.dwTypeData = "mission: protect convoy";
		InsertMenuItem(GenerateMenu, -1, true, &MenuInfo); // add to menu
		// item: mission: destroy convoy
		MenuInfo.wID = 1807;
		MenuInfo.hSubMenu = 0;
		MenuInfo.dwTypeData = "mission: destroy convoy";
		InsertMenuItem(GenerateMenu, -1, true, &MenuInfo); // add to menu
		// item: mission: capture city
		MenuInfo.wID = 1808;
		MenuInfo.hSubMenu = 0;
		MenuInfo.dwTypeData = "mission: capture city";
		InsertMenuItem(GenerateMenu, -1, true, &MenuInfo); // add to menu
		// item: mission: capture base
		MenuInfo.wID = 1809;
		MenuInfo.hSubMenu = 0;
		MenuInfo.dwTypeData = "mission: capture base";
		InsertMenuItem(GenerateMenu, -1, true, &MenuInfo); // add to menu
		// item: mission: move prototype
		MenuInfo.wID = 1810;
		MenuInfo.hSubMenu = 0;
		MenuInfo.dwTypeData = "mission: move prototype";
		InsertMenuItem(GenerateMenu, -1, true, &MenuInfo); // add to menu
		// item: mission: move fleet
		MenuInfo.wID = 1811;
		MenuInfo.hSubMenu = 0;
		MenuInfo.dwTypeData = "mission: move fleet";
		InsertMenuItem(GenerateMenu, -1, true, &MenuInfo); // add to menu
		// item: mission: destroy moving fleet
		MenuInfo.wID = 1812;
		MenuInfo.hSubMenu = 0;
		MenuInfo.dwTypeData = "mission: destroy moving fleet";
		InsertMenuItem(GenerateMenu, -1, true, &MenuInfo); // add to menu
		// 1813 is used!
		// item: player base
		MenuInfo.wID = 1803;
		MenuInfo.hSubMenu = 0;
		MenuInfo.dwTypeData = "player base";
		InsertMenuItem(GenerateMenu, -1, true, &MenuInfo); // add to menu
		// item: enemy base
		MenuInfo.wID = 1813;
		MenuInfo.hSubMenu = 0;
		MenuInfo.dwTypeData = "enemy base";
		InsertMenuItem(GenerateMenu, -1, true, &MenuInfo); // add to menu
		// item: player airbase
		MenuInfo.wID = 1804;
		MenuInfo.hSubMenu = 0;
		MenuInfo.dwTypeData = "player airbase";
		InsertMenuItem(GenerateMenu, -1, true, &MenuInfo); // add to menu
		// item: enemy airbase
		MenuInfo.wID = 1814;
		MenuInfo.hSubMenu = 0;
		MenuInfo.dwTypeData = "enemy airbase";
		InsertMenuItem(GenerateMenu, -1, true, &MenuInfo); // add to menu
		// item: city
		MenuInfo.wID = 1801;
		MenuInfo.hSubMenu = 0;
		MenuInfo.dwTypeData = "city";
		InsertMenuItem(GenerateMenu, -1, true, &MenuInfo); // add to menu
		// item: mountains / sea
		MenuInfo.wID = 1802;
		MenuInfo.hSubMenu = 0;
		MenuInfo.dwTypeData = "landscape";
		InsertMenuItem(GenerateMenu, -1, true, &MenuInfo); // add to menu
		// submenu: generate
		MenuInfo.wID = 1800;
		MenuInfo.dwTypeData = "generate";
		MenuInfo.hSubMenu = GenerateMenu;
		InsertMenuItem(NewMenu, -1, true, &MenuInfo); // add to menu
		// menu: new
		MenuInfo.wID = 1032;
		MenuInfo.dwTypeData = "New";
		MenuInfo.hSubMenu = NewMenu;
		InsertMenuItem(Menu, -1, true, &MenuInfo); // add to menu

		State = 7;
		// Story Menu
		HMENU StoryMenu = CreateMenu();
		// item: new story item
		MenuInfo.wID = 1702;
		MenuInfo.hSubMenu = 0;
		MenuInfo.dwTypeData = "new...";
		InsertMenuItem(StoryMenu, -1, true, &MenuInfo); // add to menu
		// item: stop playing story
		MenuInfo.wID = 1701;
		MenuInfo.hSubMenu = 0;
		MenuInfo.dwTypeData = "stop playing story";
		InsertMenuItem(StoryMenu, -1, true, &MenuInfo); // add to menu
		//HMENU PlayStoryMenu = CreateMenu();
		for(int i = 0; i < MLib.Missions[Mission]->StoryItems.Count(); ++i)
		{
			HMENU StoryItemMenu = CreateMenu();
			// item: play story item
			MenuInfo.wID = 11700 + i;
			MenuInfo.hSubMenu = 0;
			MenuInfo.dwTypeData = "play this storyitem";
			InsertMenuItem(StoryItemMenu, -1, true, &MenuInfo); // add to menu
			{
				HMENU NextItemMenu = CreateMenu();
				for(int j = 0; j < MLib.Missions[Mission]->StoryItems.Count(); ++j)
				{
					// item: next storyitem list
					MenuInfo.wID = 54700 + 100 * i + j;
					MenuInfo.hSubMenu = 0;
					MenuInfo.dwTypeData = MLib.Missions[Mission]->StoryItems[j]->Name;
					InsertMenuItem(NextItemMenu, -1, true, &MenuInfo); // add to menu
				}
				// item: set as last story item
				MenuInfo.wID = 15700 + i;
				MenuInfo.hSubMenu = 0;
				MenuInfo.dwTypeData = "set as last story item";
				InsertMenuItem(NextItemMenu, -1, true, &MenuInfo); // add to menu
				char Info[1024];
				// submenu: set next story item
				MenuInfo.wID = 13700 + i;
				sprintf(Info, "set next story item (is: %s)", MLib.Missions[Mission]->StoryItems[i]->Next);
				MenuInfo.dwTypeData = Info;
				MenuInfo.hSubMenu = NextItemMenu;
				InsertMenuItem(StoryItemMenu, -1, true, &MenuInfo); // add to menu
			}
			{
				HMENU SetTimeMenu = CreateMenu();
				// item: time
				MenuInfo.wID = 17700 + i;
				MenuInfo.hSubMenu = 0;
				MenuInfo.dwTypeData = "0";
				InsertMenuItem(SetTimeMenu, -1, true, &MenuInfo); // add to menu
				// item: time
				MenuInfo.wID = 18700 + i;
				MenuInfo.dwTypeData = "2";
				InsertMenuItem(SetTimeMenu, -1, true, &MenuInfo); // add to menu
				// item: time
				MenuInfo.wID = 19700 + i;
				MenuInfo.dwTypeData = "4";
				InsertMenuItem(SetTimeMenu, -1, true, &MenuInfo); // add to menu
				// item: time
				MenuInfo.wID = 20700 + i;
				MenuInfo.dwTypeData = "6";
				InsertMenuItem(SetTimeMenu, -1, true, &MenuInfo); // add to menu
				// item: time
				MenuInfo.wID = 21700 + i;
				MenuInfo.dwTypeData = "8";
				InsertMenuItem(SetTimeMenu, -1, true, &MenuInfo); // add to menu
				// item: time
				MenuInfo.wID = 22700 + i;
				MenuInfo.dwTypeData = "12";
				InsertMenuItem(SetTimeMenu, -1, true, &MenuInfo); // add to menu
				// item: time
				MenuInfo.wID = 23700 + i;
				MenuInfo.dwTypeData = "15";
				InsertMenuItem(SetTimeMenu, -1, true, &MenuInfo); // add to menu
				// submenu: set time
				char Info[1024];
				MenuInfo.wID = 16700 + i;
				sprintf(Info, "set time (is: %f)", MLib.Missions[Mission]->StoryItems[i]->TimeDelta);
				MenuInfo.dwTypeData = Info;
				MenuInfo.hSubMenu = SetTimeMenu;
				InsertMenuItem(StoryItemMenu, -1, true, &MenuInfo); // add to menu
			}
			{
				// item type menu
				HMENU BumperIDMenu = CreateMenu();
				MenuInfo.hSubMenu = 0;
				// item: item type list
				MenuInfo.wID = 28700 + i;
				MenuInfo.dwTypeData = "GNN News";
				InsertMenuItem(BumperIDMenu, -1, true, &MenuInfo); // add to menu
				// item: item type list
				MenuInfo.wID = 29700 + i;
				MenuInfo.dwTypeData = "GNN Breaking News";
				InsertMenuItem(BumperIDMenu, -1, true, &MenuInfo); // add to menu
				// item: item type list
				MenuInfo.wID = 30700 + i;
				MenuInfo.dwTypeData = "GNN Headlines";
				InsertMenuItem(BumperIDMenu, -1, true, &MenuInfo); // add to menu
				// item: item type list
				MenuInfo.wID = 31700 + i;
				MenuInfo.dwTypeData = "Al Gazeera News";
				InsertMenuItem(BumperIDMenu, -1, true, &MenuInfo); // add to menu
				// item: item type list
				MenuInfo.wID = 32700 + i;
				MenuInfo.dwTypeData = "Al Gazeera Breaking News";
				InsertMenuItem(BumperIDMenu, -1, true, &MenuInfo); // add to menu
				// item: item type list
				MenuInfo.wID = 33700 + i;
				MenuInfo.dwTypeData = "Al Jazeera Headlines";
				InsertMenuItem(BumperIDMenu, -1, true, &MenuInfo); // add to menu
				// item: item type list
				MenuInfo.wID = 34700 + i;
				MenuInfo.dwTypeData = "Text Fade";
				InsertMenuItem(BumperIDMenu, -1, true, &MenuInfo); // add to menu
				// submenu: item type
				MenuInfo.wID = 27700 + i;
				switch(MLib.Missions[Mission]->StoryItems[i]->BumperID)
				{
				case 1: MenuInfo.dwTypeData = "item type (is: GNN News)"; break;
				case 2: MenuInfo.dwTypeData = "item type (is: GNN Breaking News)"; break;
				case 3: MenuInfo.dwTypeData = "item type (is: Al Gazeera News)"; break;
				case 4: MenuInfo.dwTypeData = "item type (is: Al Gazeera Breaking News)"; break;
				case 5: MenuInfo.dwTypeData = "item type (is: GNN Headlines)"; break;
				case 6: MenuInfo.dwTypeData = "item type (is: Al Gazeera Headlines)"; break;
				case 7: MenuInfo.dwTypeData = "item type (is: Text Fade)"; break;
				}
				MenuInfo.hSubMenu = BumperIDMenu;
				InsertMenuItem(StoryItemMenu, -1, true, &MenuInfo); // add to menu
			}
			{
				HMENU RenderWorldMenu = CreateMenu();
				MenuInfo.hSubMenu = 0;
				// item: render world
				MenuInfo.wID = 35700 + i;
				MenuInfo.dwTypeData = "yes";
				InsertMenuItem(RenderWorldMenu, -1, true, &MenuInfo); // add to menu
				// item: render world
				MenuInfo.wID = 36700 + i;
				MenuInfo.dwTypeData = "no";
				InsertMenuItem(RenderWorldMenu, -1, true, &MenuInfo); // add to menu
				// submenu: render world
				MenuInfo.wID = 28700 + i;
				MenuInfo.dwTypeData = "render world";
				MenuInfo.hSubMenu = RenderWorldMenu;
				InsertMenuItem(StoryItemMenu, -1, true, &MenuInfo); // add to menu
			}
			// item: topic
			char Info[1024];
			MenuInfo.wID = 24700 + i;
			MenuInfo.hSubMenu = 0;
			if(strlen(MLib.Missions[Mission]->StoryItems[i]->Topic) > 60)
				sprintf(Info, "topic: %.60s [...]", MLib.Missions[Mission]->StoryItems[i]->Topic);
			else if(*MLib.Missions[Mission]->StoryItems[i]->Topic)
				sprintf(Info, "topic: %.60s", MLib.Missions[Mission]->StoryItems[i]->Topic);
			else
				sprintf(Info, "topic: <none>");
			MenuInfo.dwTypeData = Info;
			InsertMenuItem(StoryItemMenu, -1, true, &MenuInfo); // add to menu
			// item: text
			MenuInfo.wID = 25700 + i;
			if(strlen(MLib.Missions[Mission]->StoryItems[i]->Text) > 60)
				sprintf(Info, "text: %.60s [...]", MLib.Missions[Mission]->StoryItems[i]->Text);
			else if(*MLib.Missions[Mission]->StoryItems[i]->Text)
				sprintf(Info, "text: %.60s", MLib.Missions[Mission]->StoryItems[i]->Text);
			else
				sprintf(Info, "text: <none>");
			MenuInfo.dwTypeData = Info;
			InsertMenuItem(StoryItemMenu, -1, true, &MenuInfo); // add to menu
			// item: newsticker
			MenuInfo.wID = 26700 + i;
			if(strlen(MLib.Missions[Mission]->StoryItems[i]->NewsTicker) > 60)
				sprintf(Info, "ticker: %.60s [...]", MLib.Missions[Mission]->StoryItems[i]->NewsTicker);
			else if(*MLib.Missions[Mission]->StoryItems[i]->NewsTicker)
				sprintf(Info, "ticker: %.60s", MLib.Missions[Mission]->StoryItems[i]->NewsTicker);
			else
				sprintf(Info, "ticker: <none>");
			MenuInfo.dwTypeData = Info;
			InsertMenuItem(StoryItemMenu, -1, true, &MenuInfo); // add to menu
			// submenu: story item list
			MenuInfo.wID = 12700 + i;
			MenuInfo.dwTypeData = MLib.Missions[Mission]->StoryItems[i]->Name;
			MenuInfo.hSubMenu = StoryItemMenu;
			InsertMenuItem(StoryMenu, -1, true, &MenuInfo); // add to menu
		}
		// menu: Story
		MenuInfo.wID = 1700;
		MenuInfo.dwTypeData = "Story";
		MenuInfo.hSubMenu = StoryMenu;
		InsertMenuItem(Menu, -1, true, &MenuInfo); // add to menu

		State = 8;
		// Render menu
		HMENU RenderMenu = CreateMenu();
		// item: Accessibility Map
		MenuInfo.wID = 1301;
		MenuInfo.hSubMenu = 0;
		MenuInfo.dwTypeData = "Accessibility Map";
		InsertMenuItem(RenderMenu, -1, true, &MenuInfo); // add to menu
		// menu: Render
		MenuInfo.wID = 1300;
		MenuInfo.dwTypeData = "Render";
		MenuInfo.hSubMenu = RenderMenu;
		InsertMenuItem(Menu, -1, true, &MenuInfo); // add to menu

		State = 9;
		// compile menu
		HMENU CompileMenu = CreateMenu();
		// item: mountain
		MenuInfo.wID = 1601;
		MenuInfo.hSubMenu = 0;
		MenuInfo.dwTypeData = "compile ways, terrain, ...";
		InsertMenuItem(CompileMenu, -1, true, &MenuInfo); // add to menu
		// menu: compile
		MenuInfo.wID = 1600;
		MenuInfo.dwTypeData = "compile";
		MenuInfo.hSubMenu = CompileMenu;
		InsertMenuItem(Menu, -1, true, &MenuInfo); // add to menu
		// add menu to window
		State = 10;
		SetMenu(Window, Menu);
		State = 11;

		// check mission goals
		if(MLib.Missions[Mission]->TestMissionGoal(MissionLib::Mission::MG_DESTROYALLPLAYER | MissionLib::Mission::MG_PLAYER))
			MenuCheck(1434, true);
		else
			MenuCheck(1434, false);
		if(MLib.Missions[Mission]->TestMissionGoal(MissionLib::Mission::MG_DESTROYALLPLAYER | MissionLib::Mission::MG_ENEMY))
			MenuCheck(1435, true);
		else
			MenuCheck(1435, false);
		if(MLib.Missions[Mission]->TestMissionGoal(MissionLib::Mission::MG_DESTROYALLENEMY | MissionLib::Mission::MG_PLAYER))
			MenuCheck(1436, true);
		else
			MenuCheck(1436, false);
		if(MLib.Missions[Mission]->TestMissionGoal(MissionLib::Mission::MG_DESTROYALLENEMY | MissionLib::Mission::MG_ENEMY))
			MenuCheck(1437, true);
		else
			MenuCheck(1437, false);
	}
	catch(...)
	{
		char Str[1024];
		sprintf(Str, "MissionEditor::CreateWindowMenu#%d", State);
		throw Str;
	}
}

void MissionEditor::Create(HINSTANCE ThisInstance, int WindowStyle)
{
	IData.Create(this);
	WindowStyle = SW_SHOWNORMAL;
	StoryItem = 0;
	//ShowCursor(false);
	//SetCursor(0);
	MouseDrag = false;
	Reinitialize = false;
	// clear the error output file
	ITL::LogFile::Clear("Error_MissionEditor.txt");
	MissionEditor::Instance = ThisInstance;
	// tank lib
	TLib.Create(); // before CreateWindowMenu!
	DLib.Create();
	MLib.Create("Missions"); // before CreateWindowMenu!
	Mission = MLib.GetMissionNum(MLib.FirstMissionPA);
	// Create the main window and initialize D3D
	try
	{
		Wnd.Create(WindowStyle, "Mission Editor", 800, 600);
		Wnd.AddListener(this);
		DS.Create(Wnd.D3DDevice, &Sp);
		CreateWindowMenu(Wnd.GetHWnd()); // resizes window
		Reinitialize = false;
		Create3D();
		//Init3D();
		LoadingScreen("Window");
	}
	catch(exception& Ex)
	{
		ITL::LogFile::Msg("ErrorMissionEditor.txt", (char *)Ex.what());
		ITL::MessageWindow::Message((char *)Ex.what(), "Error during D3D Window initialisation");
		exit(-1);
	}
	LoadingScreen("Mouse");
	In.Create(Wnd.GetHWnd());
	Mouse.Create(&DS, Wnd.GetHWnd());

	I3D::Button::Specs MSpecs;
	MSpecs.DS = &DS;
	MSpecs.Pos = IMATH::Vector2D(0.94f, 0.94f);
	MSpecs.Size = IMATH::Vector2D(0.05f, 0.05f);
	MaximizeButton.Create(MSpecs);

	LoadingScreen("Brush");
	//Mouse.SetCursor(false);
	Brush::BrushSpecs BSpecs;
	BSpecs.DS = &DS;
	BSpecs.Wrld = &Wrld;
	BSpecs.Mouse = &Mouse;
	BSpecs.TLib = &TLib;
	BSpecs.DLib = &DLib;
	BSpecs.MLib = &MLib;
	BSpecs.Mission = Mission;
	BSpecs.NextID = 0;
	//BSpecs.QT = &Wrld.QTree;
	Brush.Create(&BSpecs);

	LoadingScreen("InfoBox");
	InfoBox.Create(&DS);
	InfoBox << "foo";

	// story
	LoadingScreen("Story Renderer");
	StoryRnd.Create(&DS);

	//LoadingScreen("World");
	//Wrld.Create(&DS);
	LoadingScreen("Mission Library");

	LoadingScreen("Mission");
	Wrld.AddListener(this);
	LoadMission();
}

void MissionEditor::Create3D(void)
{
	DS.D3DDevice->SetRenderState(D3DRS_LIGHTING,TRUE);
	DS.D3DDevice->SetRenderState(D3DRS_ZENABLE,TRUE);
	DS.D3DDevice->SetRenderState(D3DRS_CULLMODE,D3DCULL_CW);
	DS.D3DDevice->SetSamplerState(0,D3DSAMP_MAGFILTER, D3DTEXF_LINEAR); 
	DS.D3DDevice->SetSamplerState(0,D3DSAMP_MINFILTER, D3DTEXF_LINEAR); 
	DS.D3DDevice->SetSamplerState(0,D3DSAMP_MIPFILTER, D3DTEXF_POINT); 
	DS.D3DDevice->SetSamplerState(1,D3DSAMP_MAGFILTER, D3DTEXF_LINEAR); 
	DS.D3DDevice->SetSamplerState(1,D3DSAMP_MINFILTER, D3DTEXF_LINEAR); 
	DS.D3DDevice->SetSamplerState(1,D3DSAMP_MIPFILTER, D3DTEXF_POINT); 
//	DS.D3DDevice->SetRenderState(D3DRS_AMBIENT, 0x00000000);
	DS.D3DDevice->SetRenderState(D3DRS_AMBIENT, 0x80808080);
	DS.D3DDevice->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATER);
	DS.D3DDevice->SetRenderState(D3DRS_ALPHAREF, 0x60);
	DS.D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	DS.D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
	if(D3DFlags & GAME_D3DFLAG_COLORKEY)
		DS.D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE);
	else
		DS.D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, FALSE);
	if(D3DFlags & GAME_D3DFLAG_ALPHABLEND)
		DS.D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
	else
		DS.D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
	if(D3DFlags & GAME_D3DFLAG_NOCULLING)
		DS.D3DDevice->SetRenderState(D3DRS_CULLMODE,D3DCULL_NONE);
	else
		DS.D3DDevice->SetRenderState(D3DRS_CULLMODE,D3DCULL_CW);

	// for debugging:
	//DS.D3DDevice->SetRenderState(D3DRS_AMBIENT, 0xFFFFFFFF);
	if(D3DFlags & GAME_D3DFLAG_WIREFRAME)
		DS.D3DDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME);

	D3DMATERIAL9 Material;
	memset(&Material, 0, sizeof(D3DMATERIAL9));
	Material.Diffuse.r = 1.0f;
	Material.Diffuse.g = 1.0f;
	Material.Diffuse.b = 1.0f;
	Material.Diffuse.a = 1.0f;
	Material.Ambient.r = 1.0f;
	Material.Ambient.g = 1.0f;
	Material.Ambient.b = 1.0f;
	Material.Ambient.a = 1.0f;
	Material.Specular.r = 1.0f;
	Material.Specular.g = 1.0f;
	Material.Specular.b = 1.0f;
	Material.Specular.a = 1.0f;
	Material.Power = 0.0f;
	//  Material.power = (float)10.0;
	DS.D3DDevice->SetMaterial(&Material);
	Wrld.InitLight(&DS);

	// load mouse cursors
	MouseTextures[0] = I3D::TextureManager::GetTexture(&DS, "Textures\\cursor_move.bmp", 0, I3D::TM_FLAG_ALPHACC0);
	MouseTextures[1] = I3D::TextureManager::GetTexture(&DS, "Textures\\cursor_impassable.bmp", 0, I3D::TM_FLAG_ALPHACC0);
	MouseTextures[2] = I3D::TextureManager::GetTexture(&DS, "Textures\\cursor_select.bmp", 0, I3D::TM_FLAG_ALPHACC0);
	MouseTextures[3] = I3D::TextureManager::GetTexture(&DS, "Textures\\cursor_attack.bmp", 0, I3D::TM_FLAG_ALPHACC0);
}

void MissionEditor::Init3D(bool recurse)
{
	int Stage = 0;
	try
	{
		Stage = 1;
		DS.D3DDevice = Wnd.D3DDevice;
		Stage = 2;
		if(DS.D3DDevice && recurse)
		{
			Stage = 3;
			I3D::TextureManager::Clear();
			I3D::Text::Init3D(&DS, true);
			LoadingScreen("reinitializing Direct3D");
			Stage = 4;
		}
		LoadingScreen("reinitializing Direct3D (Game)");
		Create3D();
		if(DS.D3DDevice && recurse)
		{
			LoadingScreen("reinitializing Direct3D (Mouse)");
			Mouse.Init3D(&DS, true);
			LoadingScreen("reinitializing Direct3D (Menu)");
			Menu.Init3D(&DS, true);
			Stage = 5;
			MaximizeButton.Init3D(&DS, true);
			Stage = 6;
			InfoBox.Init3D(&DS, true);
			Stage = 7;
			LoadingScreen("reinitializing Direct3D (World)");
			Wrld.Init3D(&DS, true);
			LoadingScreen("reinitializing Direct3D (done)");
			Stage = 8;
		}
	}
	catch(char *s)
	{
		throw(s);
	}
	catch(...)
	{
		// unknown exceptions
		char Str[1024];
		sprintf(Str, "Unknown Exception in MissionEditor::Init3D#%d", Stage);
		throw(Str);
	}
}

// called asynchronously!
void MissionEditor::OnEvent(I3D::D3DWEvent Event)
{
	switch(Event.GetID())
	{
	case I3D::D3DW_EVENT_RESET:
		Reinitialize = true;
		break;
	default:
		Events[Events.Count()] = Event;
	}
}

void MissionEditor::MenuCheck(int ID)
{
	MENUITEMINFO MII;
	MII.cbSize = sizeof(MII);
	MII.fMask = MIIM_STATE;
	GetMenuItemInfo(GetMenu(Wnd.GetHWnd()), ID, false, &MII);
	if(MII.fState == MFS_CHECKED)
		MII.fState = MFS_UNCHECKED;
	else
		MII.fState = MFS_CHECKED;
	SetMenuItemInfo(GetMenu(Wnd.GetHWnd()), ID, false, &MII);
}

void MissionEditor::MenuCheck(int ID, bool Value)
{
	MENUITEMINFO MII;
	MII.cbSize = sizeof(MII);
	MII.fMask = MIIM_STATE;
	GetMenuItemInfo(GetMenu(Wnd.GetHWnd()), ID, false, &MII);
	MII.fState = (Value ? MFS_CHECKED : MFS_UNCHECKED);
	SetMenuItemInfo(GetMenu(Wnd.GetHWnd()), ID, false, &MII);
}

void MissionEditor::Scrolling(void)
{
	bool Inside = true;
	if(Mouse.GetX() < -1 || Mouse.GetX() > 1
	|| Mouse.GetY() < -1 || Mouse.GetY() > 1)
		Inside = false;
	if(Inside)
	{
		float d = Sp.Delta;
		if(Mouse.GetX() < -0.9f)
			Wrld.MoveCamera(IMATH::Vector(-48 * d, 0, 0));
		if(Mouse.GetX() > 0.9f)
			Wrld.MoveCamera(IMATH::Vector( 48 * d, 0, 0));
		if(Mouse.GetY() < -0.9f)
			Wrld.MoveCamera(IMATH::Vector(0, 0, -48 * d));
		if(Mouse.GetY() > 0.9f)
			Wrld.MoveCamera(IMATH::Vector(0, 0,  48 * d));
		static float LastZ = Mouse.GetZ();
		float dZ = Mouse.GetZ() - LastZ;
		static float InterpolateddZ = 0;
		InterpolateddZ *= 0.9f;
		InterpolateddZ += 0.1f * dZ;
		if(absf(InterpolateddZ) > 0.001f)
			Wrld.MoveCameraDirect(IMATH::Vector(0, 0, 0.1f * InterpolateddZ));
		LastZ = Mouse.GetZ();
	}
}

void MissionEditor::MouseHandler(void)
{
	Scrolling();
	if(MouseEvents.Count())
	{
		MouseEvent Ev = MouseEvents.Last();
		MouseEvents.Delete(MouseEvents.Count() - 1);
		switch(Ev.Event)
		{
		case Ev.CLICK:
			if(MaximizeButton.Click(IMATH::Vector2D(Ev.Pos.x, Ev.Pos.y)))
				Wnd.ToggleFullscreen();
			else if(StoryItem && StoryRnd.Click(IMATH::Vector2D(Ev.Down.x, Ev.Down.y)))
			{
				CheckStory();
			}
			else
			{
				if(!Brush.Click())
					if(!Menu.Click(IMATH::Vector2D(Ev.Pos.x, Ev.Pos.y)))
						Wrld.Click(IMATH::Vector2D(Ev.Pos.x, Ev.Pos.y));
			}
			break;
		case Ev.RIGHTCLICK:
			Brush.RightClick();
			Wrld.RightClick(IMATH::Vector2D(Ev.Pos.x, Ev.Pos.y));
			break;
		case Ev.DRAG:
			Wrld.Drag(IMATH::Vector2D(Ev.Pos.x, Ev.Pos.y), IMATH::Vector2D(Ev.Down.x, Ev.Down.y));
			break;
		case Ev.DRAG_START:
			Wrld.DragStart(IMATH::Vector2D(Ev.Pos.x, Ev.Pos.y));
			break;
		case Ev.DRAG_END:
			Wrld.DragEnd(IMATH::Vector2D(Ev.Pos.x, Ev.Pos.y), IMATH::Vector2D(Ev.Down.x, Ev.Down.y));
			break;
		}
	}

	/*I3D::MouseCursor::MouseCursorEvent Event;
	if(Mouse.GetEvent(&Event))
	{
		if(GetFocus() == Wnd.GetHWnd())
			switch(Event.Code)
			{
			case I3D::MC_CLICK:
				Brush.Click();
				Wrld.Click(IMATH::Vector2D(Event.x, Event.y));
				break;
			case I3D::MC_RIGHTCLICK:
				Brush.RightClick();
				Wrld.RightClick(IMATH::Vector2D(Event.x, Event.y));
				break;
			case I3D::MC_DRAG:
				Wrld.Drag(IMATH::Vector2D(Event.x, Event.y), IMATH::Vector2D(Event.x2, Event.y2));
				break;
			case I3D::MC_DRAGSTART:
				Wrld.DragStart(IMATH::Vector2D(Event.x, Event.y));
				break;
			case I3D::MC_DRAGEND:
				Wrld.DragEnd(IMATH::Vector2D(Event.x, Event.y), IMATH::Vector2D(Event.x2, Event.y2));
				break;
			}
	}*/
}

void MissionEditor::Compile()
{
	LoadingScreen("compiling");
	Brush.Compile();
}

void MissionEditor::CheckStory(void)
{
	if(StoryItem)
	{
		if(Sp.GameTime - StoryTime >= /*StoryItem->TimeDeltaStory*/ StoryRnd.GetTimeDelta())
			LoadStory(StoryItem->Next);
	}
}

void MissionEditor::LoadStory(const char *Name)
{
	// Story
	int Num = MLib.Missions[Mission]->GetStoryNum(Name);
	if(Num == -1)
	{
		StoryItem = 0;
		StoryRnd.SetBumper(0);
		StoryRnd.SetTicker("");
		StoryRnd.SetTopic("");
	}
	else
	{
		StoryItem = MLib.Missions[Mission]->StoryItems[Num];
		StoryRnd.SetBumper(StoryItem->BumperID);
		StoryRnd.SetTicker(StoryItem->NewsTicker);
		StoryRnd.SetTickerStyle(StoryItem->TickerStyle);
		StoryRnd.SetTopic(StoryItem->Topic);
		StoryRnd.SetText(StoryItem->Text);
		StoryRnd.SetTimeDelta(StoryItem->TimeDelta);
	}
	StoryTime = Sp.GameTime;
}

void MissionEditor::OrdersDialog()
{
	// clean up old dialogs
	for(int i = 0; i < Dialogs.Count(); ++i)
	{
		if(!Dialogs[i]->GetRunning())
		{
			Dialogs.Delete(i--);
		}
	}
	// start dialog
	ITL::DialogWindow::Specs Specs;
	Specs.DlgTmp.style = WS_BORDER | WS_SYSMENU | WS_CAPTION | WS_VISIBLE;
	Specs.DlgTmp.x = 10;
	Specs.DlgTmp.y = 10;
	Specs.DlgTmp.cx = 400;
	Specs.DlgTmp.cy = 100;
	strcpy(Specs.Menu, "");
	strcpy(Specs.WClass, "");
	strcpy(Specs.Title, "Orders Editor");
	ITL::DialogWindow *DBT = new ITL::DialogWindow(Specs);
	Dialogs.Add(DBT);
	// items
	int y = 5;

	ITL::DialogWindow::Item It;
	It.Tmp.style = WS_VISIBLE | SS_SIMPLE;
	It.Tmp.x = 5;
	It.Tmp.y = y;
	It.Tmp.cx = 390;
	It.Tmp.cy = 12;
	It.Tmp.id = 100;
	It.BWClass = ITL::DialogWindow::STATIC;
	strcpy(It.Title, "Orders:");
	DBT->AddItem(It);

	y += It.Tmp.cy + 5;
	It.Tmp.style = WS_VISIBLE | WS_BORDER | WS_VSCROLL | ES_MULTILINE | ES_WANTRETURN;
	It.Tmp.y = y;
	It.Tmp.cy = 60;
	It.Tmp.id = 2;
	It.BWClass = ITL::DialogWindow::EDIT;
	strcpy(It.Title, MissionLib::StoryItem::IniToPrintable(MLib.Missions[Mission]->Orders));
	DBT->AddItem(It);

	y += It.Tmp.cy + 5;
	int x = 5;
	It.Tmp.style = WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON;
	It.Tmp.x = x;
	It.Tmp.y = y;
	It.Tmp.cx = 60;
	It.Tmp.cy = 12;
	It.Tmp.id = 200;
	It.BWClass = ITL::DialogWindow::BUTTON;
	strcpy(It.Title, "OK");
	DBT->AddItem(It);

	x += It.Tmp.cx + 5;
	It.Tmp.style = WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON;
	It.Tmp.x = x;
	It.Tmp.y = y;
	It.Tmp.cx = 60;
	It.Tmp.cy = 12;
	It.Tmp.id = 201;
	It.BWClass = ITL::DialogWindow::BUTTON;
	strcpy(It.Title, "Display");
	DBT->AddItem(It);

	y += It.Tmp.cy + 5;
	DBT->SetSize(400, y);
	ITL::DialogWindow::Handler *Hnd = new OrdersDialogHandler(DBT, this);
	Handlers.Add(Hnd);
	DBT->Run(MissionEditor::Instance, Wnd.GetHWnd(), Hnd);
}

void MissionEditor::OrdersDialogHandler::OnEvent(ITL::DlgWndEvent Ev)
{
	switch(Ev.id)
	{
	case 200:
		// ok
		char Buf[1025];
		GetDlgItemText(Ev.Dlg, 2, Buf, 1024);
		strcpy(ME->MLib.Missions[ME->Mission]->Orders, MissionLib::StoryItem::PrintableToIni(Buf));
		Dlg->Stop();
		break;
	case 201:
		// display
		GetDlgItemText(Ev.Dlg, 2, Buf, 1024);
		strcpy(ME->MLib.Missions[ME->Mission]->Orders, MissionLib::StoryItem::PrintableToIni(Buf));
		I3D::Menu::MenuSpecs Sp;
		Sp.DS = &ME->DS;
		Sp.Directory = "Menus";
		Sp.CommandInterface = 0;
		Sp.MenuFile = "Orders.ini";
		Sp.Visible = true;
		Sp.DataIf = &ME->IData;
		ME->Menu.Create(&Sp);
		break;
	case 1:
		break;
	}
}

void MissionEditor::StoryItemDialog(int ID)
{
	// clean up old dialogs
	for(int i = 0; i < Dialogs.Count(); ++i)
	{
		if(!Dialogs[i]->GetRunning())
		{
			Dialogs.Delete(i--);
		}
	}
	// start dialog
	ITL::DialogWindow::Specs Specs;
	Specs.DlgTmp.style = WS_BORDER | WS_SYSMENU | WS_CAPTION | WS_VISIBLE;
	Specs.DlgTmp.x = 10;
	Specs.DlgTmp.y = 10;
	Specs.DlgTmp.cx = 400;
	Specs.DlgTmp.cy = 185;
	strcpy(Specs.Menu, "");
	strcpy(Specs.WClass, "");
	strcpy(Specs.Title, "Story Item Editor");
	ITL::DialogWindow *DBT = new ITL::DialogWindow(Specs);
	Dialogs.Add(DBT);
	// Items
	int y = 5;
	ITL::DialogWindow::Item It;

	It.Tmp.style = WS_VISIBLE | SS_SIMPLE;
	It.Tmp.x = 5;
	It.Tmp.y = y;
	It.Tmp.cx = 390;
	It.Tmp.cy = 12;
	It.Tmp.id = 100;
	It.BWClass = ITL::DialogWindow::STATIC;
	char Str[1024];
	sprintf(Str, "Name: %s", MLib.Missions[Mission]->StoryItems[ID]->Name);
	strcpy(It.Title, Str);
	DBT->AddItem(It);

	y += It.Tmp.cy + 5;
	It.Tmp.style = WS_VISIBLE | SS_SIMPLE;
	It.Tmp.y = y;
	It.Tmp.cy = 12;
	It.Tmp.id = 255;
	It.BWClass = ITL::DialogWindow::STATIC;
	strcpy(It.Title, "Text:");
	DBT->AddItem(It);

	y += It.Tmp.cy + 5;
	It.Tmp.style = WS_VISIBLE | WS_BORDER | WS_VSCROLL | ES_MULTILINE | ES_WANTRETURN;
	It.Tmp.y = y;
	It.Tmp.cy = 40;
	It.Tmp.id = 1;
	It.BWClass = ITL::DialogWindow::EDIT;
	strcpy(It.Title, MissionLib::StoryItem::IniToPrintable(MLib.Missions[Mission]->StoryItems[ID]->Text));
	DBT->AddItem(It);

	y += It.Tmp.cy + 5;
	It.Tmp.style = WS_VISIBLE | SS_SIMPLE;
	It.Tmp.y = y;
	It.Tmp.cy = 12;
	It.Tmp.id = 255;
	It.BWClass = ITL::DialogWindow::STATIC;
	strcpy(It.Title, "News Topic:");
	DBT->AddItem(It);

	y += It.Tmp.cy + 5;
	It.Tmp.style = WS_VISIBLE | WS_BORDER | WS_VSCROLL | ES_MULTILINE | ES_WANTRETURN;
	It.Tmp.y = y;
	It.Tmp.cy = 40;
	It.Tmp.id = 2;
	It.BWClass = ITL::DialogWindow::EDIT;
	strcpy(It.Title, MissionLib::StoryItem::IniToPrintable(MLib.Missions[Mission]->StoryItems[ID]->Topic));
	DBT->AddItem(It);

	y += It.Tmp.cy + 5;
	It.Tmp.style = WS_VISIBLE | SS_SIMPLE;
	It.Tmp.y = y;
	It.Tmp.cy = 12;
	It.Tmp.id = 255;
	It.BWClass = ITL::DialogWindow::STATIC;
	strcpy(It.Title, "News Ticker:");
	DBT->AddItem(It);

	y += It.Tmp.cy + 5;
	It.Tmp.style = WS_VISIBLE | WS_BORDER | WS_VSCROLL | ES_MULTILINE | ES_WANTRETURN;
	It.Tmp.y = y;
	It.Tmp.cy = 40;
	It.Tmp.id = 3;
	It.BWClass = ITL::DialogWindow::EDIT;
	strcpy(It.Title, MissionLib::StoryItem::IniToPrintable(MLib.Missions[Mission]->StoryItems[ID]->NewsTicker));
	DBT->AddItem(It);

	y += It.Tmp.cy + 5;
	int x = 5;
	It.Tmp.style = WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON;
	It.Tmp.x = x;
	It.Tmp.y = y;
	It.Tmp.cx = 60;
	It.Tmp.cy = 12;
	It.Tmp.id = 200;
	It.BWClass = ITL::DialogWindow::BUTTON;
	strcpy(It.Title, "OK");
	DBT->AddItem(It);

	x += It.Tmp.cx + 5;
	It.Tmp.style = WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON;
	It.Tmp.x = x;
	It.Tmp.y = y;
	It.Tmp.cx = 60;
	It.Tmp.cy = 12;
	It.Tmp.id = 201;
	It.BWClass = ITL::DialogWindow::BUTTON;
	strcpy(It.Title, "Apply");
	DBT->AddItem(It);

	x += It.Tmp.cx + 5;
	It.Tmp.style = WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON;
	It.Tmp.x = x;
	It.Tmp.y = y;
	It.Tmp.cx = 60;
	It.Tmp.cy = 12;
	It.Tmp.id = 202;
	It.BWClass = ITL::DialogWindow::BUTTON;
	strcpy(It.Title, "Play item");
	DBT->AddItem(It);

	x += It.Tmp.cx + 5;
	It.Tmp.style = WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON;
	It.Tmp.x = x;
	It.Tmp.y = y;
	It.Tmp.cx = 60;
	It.Tmp.cy = 12;
	It.Tmp.id = 203;
	It.BWClass = ITL::DialogWindow::BUTTON;
	strcpy(It.Title, "Go to next item");
	DBT->AddItem(It);

	x += It.Tmp.cx + 5;
	It.Tmp.style = WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON;
	It.Tmp.x = x;
	It.Tmp.y = y;
	It.Tmp.cx = 60;
	It.Tmp.cy = 12;
	It.Tmp.id = 204;
	It.BWClass = ITL::DialogWindow::BUTTON;
	strcpy(It.Title, "Pause");
	DBT->AddItem(It);

	y += It.Tmp.cy + 5;
	DBT->SetSize(400, y);
	ITL::DialogWindow::Handler *Hnd = new StoryItemDialogHandler(ID, DBT, this);
	Handlers.Add(Hnd);
	DBT->Run(MissionEditor::Instance, Wnd.GetHWnd(), Hnd);
}

void MissionEditor::StoryItemDialogHandler::OnEvent(ITL::DlgWndEvent Ev)
{
	switch(Ev.id)
	{
	case 200:
		char Buf[1025];
		GetDlgItemText(Ev.Dlg, 1, Buf, 1024);
		strcpy(ME->MLib.Missions[ME->Mission]->StoryItems[ID]->Text, MissionLib::StoryItem::PrintableToIni(Buf));
		GetDlgItemText(Ev.Dlg, 2, Buf, 1024);
		strcpy(ME->MLib.Missions[ME->Mission]->StoryItems[ID]->Topic, MissionLib::StoryItem::PrintableToIni(Buf));
		GetDlgItemText(Ev.Dlg, 3, Buf, 1024);
		strcpy(ME->MLib.Missions[ME->Mission]->StoryItems[ID]->NewsTicker, MissionLib::StoryItem::PrintableToIni(Buf));
		ME->CreateWindowMenu(ME->Wnd.GetHWnd());
		ME->LoadStory(ME->MLib.Missions[ME->Mission]->StoryItems[ID]->Name);
		Dlg->Stop();
		break;
	case 201:
		GetDlgItemText(Ev.Dlg, 1, Buf, 1024);
		strcpy(ME->MLib.Missions[ME->Mission]->StoryItems[ID]->Text, MissionLib::StoryItem::PrintableToIni(Buf));
		GetDlgItemText(Ev.Dlg, 2, Buf, 1024);
		strcpy(ME->MLib.Missions[ME->Mission]->StoryItems[ID]->Topic, MissionLib::StoryItem::PrintableToIni(Buf));
		GetDlgItemText(Ev.Dlg, 3, Buf, 1024);
		strcpy(ME->MLib.Missions[ME->Mission]->StoryItems[ID]->NewsTicker, MissionLib::StoryItem::PrintableToIni(Buf));
		// update menu and restart story item
		ME->CreateWindowMenu(ME->Wnd.GetHWnd());
		ME->LoadStory(ME->MLib.Missions[ME->Mission]->StoryItems[ID]->Name);
		break;
	case 202:
		ME->LoadStory(ME->MLib.Missions[ME->Mission]->StoryItems[ID]->Name);
		break;
	case 203:
		if(ID < ME->MLib.Missions[ME->Mission]->StoryItems.Count() - 1)
		{
			ID++;
			SetDlgItemText(Ev.Dlg, 100, ME->MLib.Missions[ME->Mission]->StoryItems[ID]->Name);
			SetDlgItemText(Ev.Dlg, 1, MissionLib::StoryItem::PrintableToIni(ME->MLib.Missions[ME->Mission]->StoryItems[ID]->Text));
			SetDlgItemText(Ev.Dlg, 2, MissionLib::StoryItem::PrintableToIni(ME->MLib.Missions[ME->Mission]->StoryItems[ID]->Topic));
			SetDlgItemText(Ev.Dlg, 3, MissionLib::StoryItem::PrintableToIni(ME->MLib.Missions[ME->Mission]->StoryItems[ID]->NewsTicker));
		}
		break;
	case 204:
		ME->SetPause(!ME->GetPause());
		break;
	}
}

void MissionEditor::GenerateStreet(GenMap *Map, int x1, int z1, int x2, int z2)
{
	if(x1 > x2)
	{
		x1 ^= x2;
		x2 ^= x1;
		x1 ^= x2;
	}
	if(z1 > z2)
	{
		z1 ^= z2;
		z2 ^= z1;
		z1 ^= z2;
	}
	if(x1==x2)
		for(int i = z1; i <= z2; i += 32)
		{
			if(!(Map->Get(x1, i) & World::STREET_VERT))
			{
				Decal *D;
				if(Map->Get(x1, i) & World::STREET_HORZ && (D = Map->GetDecal(x1, i)) != 0)
				{
					D->SetID(2);
					Map->AddDecal(D);
				}
				else
				{
					Decal::DecalSpecs Specs;
					sprintf(Specs.Name, "MissionEditor Decal %d", Brush.GetNextID());
					Brush.SetNextID(Brush.GetNextID() + 1);
					Specs.Pos = IMATH::Vector2DXZ(Map->Pos[0]) + IMATH::Vector2DXZ((float)x1, (float)i);
					//Specs.DLib = Brush::Specs.DLib;
					Specs.ID = 1;
					Decal *D = new Decal(Specs);
					Map->GetME()->Wrld.AddDecalPre(D);
					Map->AddDecal(D);
					MLib.Missions[Mission]->AddDecal(D);
					MLib.Missions[Mission]->NextID = Brush.GetNextID();
				}
			}
		}
	else
		for(int i = x1; i <= x2; i += 32)
		{
			if(!(Map->Get(i, z1) & World::STREET_HORZ))
			{
				Decal *D;
				if((Map->Get(i, z1) & World::STREET_VERT) && (D = Map->GetDecal(i, z1)) != 0)
				{
					D->SetID(2);
					Map->AddDecal(D);
				}
				else
				{
					Decal::DecalSpecs Specs;
					sprintf(Specs.Name, "MissionEditor Decal %d", Brush.GetNextID());
					Brush.SetNextID(Brush.GetNextID() + 1);
					Specs.Pos = IMATH::Vector2DXZ(Map->Pos[0]) + IMATH::Vector2DXZ((float)i, (float)z1);
					//Specs.DLib = Brush::Specs.DLib;
					Specs.ID = 0;
					Decal *D = new Decal(Specs);
					Map->GetME()->Wrld.AddDecalPre(D);
					Map->AddDecal(D);
					MLib.Missions[Mission]->AddDecal(D);
					MLib.Missions[Mission]->NextID = Brush.GetNextID();
				}
			}
		}
}

struct Point
{
	int x;
	int z;

	Point() {}
	Point(int xn, int zn) { x = xn; z = zn; }
};

IMATH::Vector2DXZ MissionEditor::FindStreetLink(GenMap *Map)
{
	ISL::DynamicArray<Point> RandomArray;
	if(Map->GetStreetXOffset() > Map->Width || Map->GetStreetZOffset() > Map->Height)
		return IMATH::Vector2DXZ(-1, -1);
	for(int x = Map->GetStreetXOffset(); x <= Map->Width; x += 32)
		for(int z = Map->GetStreetZOffset(); z <= Map->Height; z += 32)
		{
			if(Map->Get(x, z) & World::STREET_MASK
				&& (Map->Get(x + 32, z) == World::NOTHING || Map->Get(x - 32, z) == World::NOTHING
					|| Map->Get(x, z + 32) == World::NOTHING || Map->Get(x, z - 32) == World::NOTHING))
				RandomArray.Add(Point(x, z));
		}
	if(RandomArray.Count())
	{
		Point P = RandomArray[rand() % RandomArray.Count()];
		return IMATH::Vector2DXZ((float)P.x, (float)P.z);
	}
	else
		return IMATH::Vector2DXZ(-1, -1);
}

void MissionEditor::GenerateLinkedStreet(GenMap *Map, const IMATH::Vector2DXZ &Link)
{
	int Side = rand() % 4;
	for(int i = 0; i < 4; ++i)
	{
		int SideI = (i + Side) % 4;
		int id, dx, dz;
		switch(SideI)
		{
		case 0:
			id = Map->Get((int)Link.x - 32, (int)Link.z);
			dx = -32;
			dz = 0;
			break;
		case 1:
			id = Map->Get((int)Link.x + 32, (int)Link.z);
			dx = 32;
			dz = 0;
			break;
		case 2:
			id = Map->Get((int)Link.x, (int)Link.z - 32);
			dx = 0;
			dz = -32;
			break;
		case 3:
			id = Map->Get((int)Link.x, (int)Link.z + 32);
			dx = 0;
			dz = 32;
			break;
		}
		if(id == World::NOTHING)
		{
			int k;
			int last;
			for(k = 1;;	++k)
			{
				int id = Map->Get((int)Link.x + k * dx, (int)Link.z + k * dz);
				if((int)Link.x + k * dx >= Map->Width  || (int)Link.x + k * dx < 0
					|| (int)Link.z + k * dz >= Map->Height || (int)Link.z + k * dz < 0)
					break;
				if(id != World::NOTHING && !(id & World::STREET_MASK))
					break;
				if(id & World::STREET_MASK && last & World::STREET_MASK)
					break;
				last = id;
			}
			k--;
			//std::ofstream dbg("gen.txt", ios::app);
			//dbg << (int)Link.x << ' ' << (int)Link.z << " -> " << (int)Link.x + k * dx << ' ' << (int)Link.z + k * dz << std::endl;
			GenerateStreet(Map, (int)Link.x, (int)Link.z, max(min((int)Link.x + k * dx, Map->Width - 1), 0), max(min((int)Link.z + k * dz, Map->Height - 1), 0));
			break;
		}
	}
}

bool MissionEditor::GenerateMainStreet(GenMap *Map)
{
	ISL::DynamicArray<Point> RandomArray;
	//return GenerateAbstractStreet(Map, false, true);
	//IMATH::Vector2DXZ Link;
	IMATH::Vector2DXZ Link = IMATH::Vector2DXZ((float)(((Map->Width / 2) >> 5) << 5) + Map->GetStreetXOffset(), (float)(((Map->Height / 2) >> 5) << 5) + Map->GetStreetZOffset());
	if(Map->Get((int)Link.x, (int)Link.z) != World::NOTHING)
	{
		for(int x = Map->GetStreetXOffset(); x < Map->Width; x += 32)
			for(int z = Map->GetStreetZOffset(); z < Map->Height; z += 32)
				if(Map->Get(x, z) == World::NOTHING)
					RandomArray.Add(Point(x, z));
		if(RandomArray.Count())
		{
			Point P = RandomArray[rand() % RandomArray.Count()];
			Link = IMATH::Vector2DXZ((float)P.x, (float)P.z);
		}
		else
			return false;
	}
	GenerateLinkedStreet(Map, Link);
	GenerateLinkedStreet(Map, Link);
	GenStreets.Add(GenStreet((int)(Link.x + Map->Pos[0].x), (int)(Link.z + Map->Pos[0] .z), Map->Get((int)Link.x, (int)Link.z)));
	return true;
}

bool MissionEditor::GenerateSideStreet(GenMap *Map)
{
	IMATH::Vector2DXZ Link = FindStreetLink(Map);
	if(Link.x != -1)
	{
		GenerateLinkedStreet(Map, Link);
		return true;
	}
	return false;
}

int MissionEditor::GetBuildingID(int Type, int Owner)
{
	ISL::DynamicArray<int> List;
	for(int i = 0; i < TLib.Tanks.Count(); ++i)
	{
		if(!TLib.Tanks[i].IsSpecialBuilding)
		{
			int WrongSide = SIDE_ISRAEL;
			if(MLib.Missions[Mission]->Side == SIDE_ISRAEL)
				WrongSide ^= SIDE_PALESTINE ^ SIDE_ISRAEL;
			if(Owner == OWNER_ENEMY)
				WrongSide ^= SIDE_PALESTINE ^ SIDE_ISRAEL;
			if((Owner != OWNER_PLAYER && Owner != OWNER_ENEMY) || TLib.Tanks[i].Side != WrongSide)
			{
				if(Type == 0 && TLib.Tanks[i].IsCivilBuilding)
					List.Add(i);
				if(Type == 1 && TLib.Tanks[i].IsBaseBuilding)
					List.Add(i);
				if(Type == 2 && TLib.Tanks[i].IsAirbaseBuilding)
					List.Add(i);
				if(Type == 3 && TLib.Tanks[i].IsBaseVehicle)
					List.Add(i);
			}
		}
	}
	if(List.Count())
		return List[rand() % List.Count()];
	else
		return 0;
}

void MissionEditor::GenerateTank(GenMap *Map, int Type, int Owner)
{
	for(int k = 0; k < 20; ++k)
	{
		ISL::DynamicArray<I3D::Way> *Ways = Wrld.GetWays();
		int Number = Ways->Count();
		ISL::DynamicArray<I3D::Way *> RandomList;
		for(int j = 0; j < Number; j++)
		{
			I3D::Way *W = &((*Ways)[j]);
			if(W->GetID() == I3D::IDT_WAY_HIGHWAY || W->GetID() == I3D::IDT_WAY_STREET || W->GetID() == I3D::IDT_WAY_WAY)
					RandomList.Add(W);
		}
		if(RandomList.Count())
		{
			I3D::Way *W = RandomList[rand() % RandomList.Count()];
			float Coeff = 0.01f * (rand() % 101);
			IMATH::Vector2DXZ Diff = W->GetPos(1) - W->GetPos(0);
			IMATH::Vector2DXZ Pos = W->GetPos(0) + Coeff * Diff;
			IMATH::Vector2DXZ Orth(Diff.z, -Diff.x);
			bool Building = TLib.Tanks[Type].IsBaseBuilding || TLib.Tanks[Type].IsAirbaseBuilding || TLib.Tanks[Type].IsCivilBuilding || TLib.Tanks[Type].IsSpecialBuilding;
			if(Building)
				Pos += (float)(2 * (rand() % 2) - 1) * 26 * Orth.Normalize();
			else
				Pos += (float)(2 * (rand() % 2) - 1) * 12 * Orth.Normalize();
			Pos -= Map->Pos[0];
			int i = (int)Pos.x;
			int j = (int)Pos.z;
			if(i >= 0 && i <= Map->Width && j >= 0 && j <= Map->Height)
			{
				/*if(Building)
				{
					i = (rand() * 32) % Map->Width;
					j = (rand() * 32) % Map->Height;
				}
				else
				{
					i = (rand() * 32 + 16) % Map->Width;
					j = (rand() * 32 + 16) % Map->Height;
				}*/
				int Landscape = Map->Get(i, j);
				if((Landscape == World::NOTHING || (!Building && (Landscape & World::STREET_MASK))) && Map->GetStreet(i, j))
				{
					// spawn tank or building
					SpawnInterface::SpawnInterfaceSpecs SS;
					memset(&SS, 0, sizeof(SS));
					SS.Type = IDTYPE_TANK;
					SS.ID = Type;
					SS.Owner = TLib.Tanks[Type].NeutralInBase ? 0 : Owner;
					sprintf(SS.Name, "MissionEditor Object O:%d T:%d I:%d ID:%d", SS.Owner, SS.Type, SS.ID, Brush.GetNextID());
					if(rand() % 12 == 0)
						SS.Angle = (rand() % 8) * PI / 4;
					else
						SS.Angle = (rand() % 4) * PI / 2;
					IMATH::Vector2DXZ Pos = Map->Pos[0] + IMATH::Vector2DXZ((float)i, (float)j);
					SS.Pos = IMATH::Vector(Pos.x, Wrld.GetSurfaceY(Pos.x, Pos.z), Pos.z);
					Wrld.GetSpawnInterface()->Spawn(&SS);
					MissionLib::Mission::MissionObjectSpecs MOS;
					MOS.Type = SS.Type;
					MOS.ID = SS.ID;
					MOS.Owner = SS.Owner;
					MOS.Pos = Pos;
					MOS.Angle = SS.Angle;
					strcpy(MOS.Name, SS.Name);
					MLib.Missions[Mission]->AddObject(&MOS);
					MLib.Missions[Mission]->NextID = Brush.GetNextID() ;
					Brush.SetNextID(Brush.GetNextID() + 1);
					Map->AddTank(Map->GetME()->Wrld.GetTank(MOS.Name));
					break;
				}
			}
		}
	}
}

void MissionEditor::GenerateBuilding(GenMap *Map, int Type, int Owner)
{
	GenerateTank(Map, GetBuildingID(Type, Owner), Owner);
}

MissionEditor::StreetDirectionInfo MissionEditor::TestStreetDirection(GenMap *Map, const IMATH::Vector2DXZ &Pos, const IMATH::Vector2DXZ &Direction)
{
	IMATH::Vector2DXZ Dir = Direction.Normalize();
	StreetDirectionInfo Info;
	Info.MaxLength = 0;
	Info.BlockingWorldID = 0;
	IMATH::Vector2DXZ TestPos = Pos;
	for(int i = 0; ;i++)
	{
		int Terrain = Map->Get((int)(TestPos.x - Map->Pos[0].x), (int)(TestPos.z - Map->Pos[0].z));
		if(i > 2 * I3D::IDT_MAX_WAYSIZE && Terrain != World::NOTHING)
		{
			Info.BlockingID = StreetDirectionInfo::OBSTACLE_IN_WORLD;
			Info.BlockingWorldID = Terrain;
			if(Terrain == World::WAY)
				Info.MaxLength += 6;
			break;
		}
		if(TestPos.x < Map->Pos[0].x || TestPos.x > Map->Pos[1].x || TestPos.z < Map->Pos[0].z || TestPos.z > Map->Pos[1].z)
		{
			Info.BlockingID = StreetDirectionInfo::END_OF_WORLD;
			break;
		}
		Info.MaxLength = (float)i;
		TestPos += Dir;
		//Map->Set((int)(TestPos.x - Map->Pos[0].x), (int)(TestPos.z - Map->Pos[0].z), 0);
	}
	return Info;
}

void MissionEditor::GrowStreets(GenMap *Map)
{
	using namespace IMATH;
	const int MinWayLength = 105;
	const int MinSideDist = 100;
	ISL::DynamicArray<I3D::Way> *Ways = Wrld.GetWays();
	int Number = Ways->Count();
	ISL::DynamicArray<I3D::Way *> RandomList;
	for(int i = 0; i < Number; i++)
	{
		I3D::Way *W = &((*Ways)[i]);
		if(W->GetID() == I3D::IDT_WAY_HIGHWAY || W->GetID() == I3D::IDT_WAY_STREET)
		{
			if((W->GetPos(1) - W->GetPos(0)).Length() > MinSideDist)
				RandomList.Add(W);
		}
	}
	if(RandomList.Count())
	{
		I3D::Way *W = RandomList[rand() % RandomList.Count()];
		Vector2DXZ Diff = W->GetPos(1) - W->GetPos(0);
		Vector2DXZ Orth = Vector2DXZ(Diff.z, -Diff.x);
		if(rand() % 2)
			Orth = -Orth;
		Vector2DXZ OrthNormed = Orth.Normalize();
		Vector2DXZ NewDirection = OrthNormed;
		if(rand() % 20 == 0)
		{
			NewDirection += 0.01f * (rand() % 401 - 200) * Diff.Normalize();
			NewDirection = NewDirection.Normalize();
		}
		else if(rand() % 20 == 0)
		{
			NewDirection += 0.01f * (rand() % 51 - 50) * Diff.Normalize();
			NewDirection = NewDirection.Normalize();
		}
		I3D::Way::WaySpecs WS;
		int Num = (int)floor(Diff.Length() / MinSideDist);
		Vector2DXZ StartPos = W->GetPos(0) + (float)(rand() % (Num + 1)) * MinSideDist * Diff.Normalize();
		//Vector2DXZ StartPos = W->GetPos(0) + 0.01f * (rand() % 101) * Diff;
		if(StartPos.x > Map->Pos[0].x && StartPos.x < Map->Pos[1].x
			&& StartPos.z > Map->Pos[0].z && StartPos.z < Map->Pos[1].z)
		{
			StreetDirectionInfo Info = TestStreetDirection(Map, StartPos, NewDirection);
			GenerationLog << "MaxLength: " << Info.MaxLength << std::endl;
			if(Info.MaxLength > MinWayLength)
			{
				WS.ID = I3D::IDT_WAY_STREET;
				sprintf(WS.Name, "Generated Street %d", Brush.GetNextID());
				Brush.SetNextID(Brush.GetNextID() + 1);
				WS.Pos1 = StartPos;
				WS.Pos1 += 0.5f * W->GetSize() * OrthNormed;
				//WS.Pos2 = WS.Pos1 + 0.01f * (20 + rand() % 61) * Info.MaxLength * NewDirection;
				WS.Pos2 = WS.Pos1 + Info.MaxLength * NewDirection;
				if(Info.BlockingID & Wrld.WAY)
					Wrld.SplitWay(StartPos);
				I3D::Way W(&WS);
				Wrld.AddWayNoHMap(W);
				Map->AddWay(&W);
				MLib.Missions[Mission]->AddWay(&WS);
			}
		}
	}
}

void MissionEditor::GrowMainStreet(GenMap *Map, bool Highway = true)
{
	using namespace IMATH;
	const int MinWayLength = 50;
	GenerationLog << "generating main street" << std::endl;
	for(int k = 0; k < 20; k++)
	{
		int x1 = rand() % Map->Width;
		int z1 = 0;
		if(rand() % 2)
		{
			z1 = x1;
			x1 = 0;
		}
		int x2 = Map->Width - x1;
		int z2 = Map->Height - x1;
		I3D::Way::WaySpecs WS;
		WS.Pos1 = Map->Pos[0] + Vector2DXZ((float)x1, (float)z1);
		WS.Pos2 = Map->Pos[0] + Vector2DXZ((float)x2, (float)z2);
		IMATH::Vector2DXZ Diff = WS.Pos2 - WS.Pos1;
		StreetDirectionInfo Info = TestStreetDirection(Map, WS.Pos1, Diff);
		if(Info.MaxLength > MinWayLength)
		{
			WS.Pos2 = WS.Pos1 + Diff.Normalize() * Info.MaxLength;
			WS.ID = Highway ? I3D::IDT_WAY_HIGHWAY : I3D::IDT_WAY_STREET;
			sprintf(WS.Name, "Generated Street %d", Brush.GetNextID());
			Brush.SetNextID(Brush.GetNextID() + 1);
			if(Info.BlockingID & Wrld.WAY)
				Wrld.SplitWay(WS.Pos2);
			GenerationLog << *Map;
			I3D::Way W(&WS);
			Wrld.AddWayNoHMap(W);
			Map->AddWay(&W);
			MLib.Missions[Mission]->AddWay(&WS);
			break;
		}
		else
			GenerationLog << "main street too short" << std::endl;
	}
}

void MissionEditor::GrowCity(GenMap *Map)
{
	LoadingScreen("generating streets");
	GrowMainStreet(Map);
	for(int i = 0; i < (Map->Width + Map->Height) / 15; ++i)
		GrowStreets(Map);
	LoadingScreen("updating street textures");
	Wrld.UpdateSurfaceTextures();
	LoadingScreen("updating proximity map");
	Map->NearMap();
	LoadingScreen("generating buildings");
	for(int i = 0; i < Map->Width * Map->Height / 1000; ++i)
	{
		if(i == 5)
			GenerateTank(Map, IDT_HOUSE_MOSQ, 0);
		else
			GenerateBuilding(Map, 0, 0);
	}
	GenerationLog << *Map;
}

void MissionEditor::GenerateCity(GenMap *Map)
{
	LoadingScreen("generating city");
	/*if(GenerateMainStreet(Map))
	{
		for(int i = 0; i < max((Map->Width + Map->Height) / 80, 2); ++i)
			GenerateSideStreet(Map);
		Map->NearMap();
		for(int i = 0; i < Map->Width * Map->Height / 1000; ++i)
		{
			if(i == 5)
				GenerateTank(Map, IDT_HOUSE_MOSQ, 0);
			else
				GenerateBuilding(Map, 0, 0);
		}
		Map->GetME()->Wrld.CompileDecalsAdditive();
	}*/
	GrowCity(Map);
}

#define LTREE_LEVEL 19
class LandscapeTree
{
	class Node
	{
		bool IsEmpty, EmptyKnown;

	public:
		Node *Children[2];
		IMATH::Vector2DXZ Points[3];
		LandscapeTree *Host;

		Node(int Level, LandscapeTree *Host, IMATH::Vector2DXZ a, IMATH::Vector2DXZ b, IMATH::Vector2DXZ c)
			: IsEmpty(false), EmptyKnown(false)
		{
			Node::Host = Host;
			Points[0] = a;
			Points[1] = b;
			Points[2] = c;
			if(Level)
			{
				Children[0] = new Node(Level - 1, Host, 0.5f * (b + c), a, c);
				Children[1] = new Node(Level - 1, Host, 0.5f * (b + c), a, b);
			}
			else
			{
				Children[0] = 0;
				Children[1] = 0;
			}
		}

		~Node()
		{
			delete Children[0];
			delete Children[1];
		}

		bool Empty(GenMap *Map)
		{
			if(EmptyKnown)
				return IsEmpty;
			if(Children[0])
			{
				IsEmpty = Children[0]->Empty(Map) && Children[1]->Empty(Map);
				EmptyKnown = true;
				return IsEmpty;
			}
			else
			{
				//IMATH::Vector2DXZ Pos = (1 / 3.0f) * (Points[0] + Points[1] + Points[2]) - Map->Pos[0];
				EmptyKnown = true;
				IsEmpty = true;
				IsEmpty = IsEmpty && (Map->Get((int)(Points[0].x - Map->Pos[0].x), (int)(Points[0].z - Map->Pos[0].z)) == World::NOTHING);
				IsEmpty = IsEmpty && (Map->Get((int)(Points[1].x - Map->Pos[0].x), (int)(Points[1].z - Map->Pos[0].z)) == World::NOTHING);
				IsEmpty = IsEmpty && (Map->Get((int)(Points[2].x - Map->Pos[0].x), (int)(Points[2].z - Map->Pos[0].z)) == World::NOTHING);
				IsEmpty = IsEmpty && (Map->Get((int)(21 * Points[0].x - 20 * Points[1].x - Map->Pos[0].x), (int)(21 * Points[0].z - 20 * Points[1].z - Map->Pos[0].z)) == World::NOTHING);
				IsEmpty = IsEmpty && (Map->Get((int)(21 * Points[2].x - 20 * Points[0].x - Map->Pos[0].x), (int)(21 * Points[2].z - 20 * Points[0].z - Map->Pos[0].z)) == World::NOTHING);
				IsEmpty = IsEmpty && (Map->Get((int)(21 * Points[1].x - 20 * Points[2].x - Map->Pos[0].x), (int)(21 * Points[1].z - 20 * Points[2].z - Map->Pos[0].z)) == World::NOTHING);
				IsEmpty = IsEmpty && (Map->Get((int)(3 * Points[0].x - 2 * Points[1].x - Map->Pos[0].x), (int)(3 * Points[0].z - 2 * Points[1].z - Map->Pos[0].z)) == World::NOTHING);
				IsEmpty = IsEmpty && (Map->Get((int)(3 * Points[2].x - 2 * Points[0].x - Map->Pos[0].x), (int)(3 * Points[2].z - 2 * Points[0].z - Map->Pos[0].z)) == World::NOTHING);
				IsEmpty = IsEmpty && (Map->Get((int)(3 * Points[1].x - 2 * Points[2].x - Map->Pos[0].x), (int)(3 * Points[1].z - 2 * Points[2].z - Map->Pos[0].z)) == World::NOTHING);
				return IsEmpty;
			}
		}

		void Generate(GenMap *Map, MissionEditor *ME, int Level = LTREE_LEVEL)
		{
			if(Children[0])
			{
				bool Empty1 = Children[0]->Empty(Map);
				bool Empty2 = Children[1]->Empty(Map);
				if(Empty1 && Empty2 && (Points[0] - Points[1]).Length() <= 64 + 128 * (rand() % 2) && (Points[0] - Points[1]).Length() > 16)
				{
					// get outline
					IMATH::Vector2DXZ Pos = (1 / 3.0f) * (Points[0] + Points[1] + Points[2]);
					float x = (Pos.x - Map->Pos[0].x) / (Map->Pos[1].x - Map->Pos[0].x);
					float z = (Pos.z - Map->Pos[0].z) / (Map->Pos[1].z - Map->Pos[0].z);
					float Out = ((1 - x) * (1 - z) * Host->TerrainOutline[0][0]
						+ (1 - x) * z * Host->TerrainOutline[0][1]
						+ x * (1 - z) * Host->TerrainOutline[1][0]
						+ x * z * Host->TerrainOutline[1][1]) * (4.0f / 3);
					Out += (rand() % 2) - 1.0f;
					int Outline = (int)Out;
					if(Outline < 0)
						Outline = 0;
					if(Outline > 5)
						Outline = 5;
					// add terraindsc
					I3D::TerrainDsc::TerrainDscSpecs Specs;
					switch(Outline)
					{
					case 0: Specs.ID = 3 + 2 * (rand() % 2); break;
					case 1: Specs.ID = 3 + (rand() % 2); break;
					case 2: Specs.ID = 2 + 4 * (rand() % 2); break;
					case 3: Specs.ID = 2 - 3 * (rand() % 2); break;
					case 4: Specs.ID = 0 - 1 * (rand() % 2); break;
					case 5: Specs.ID = rand() % 2; break;
					}
					if(Specs.ID > 0)
					{
						Specs.Pos[0] = Points[0];
						Specs.Pos[1] = Points[1];
						Specs.Pos[2] = Points[2];
						sprintf(Specs.Name, "MissionEditor Terrain I:%d ID:%d", Specs.ID, ME->GetNextBrushID());
						ME->SetNextBrushID(ME->GetNextBrushID() + 1);
						ME->AddTerrainNoHMap(Specs);
					}
				}
				else
				{
					if((Points[0] - Points[1]).Length() > 32)
					{
						Children[0]->Generate(Map, ME, Level - 1);
						Children[1]->Generate(Map, ME, Level - 1);
					}
				}
			}
		}
	};

	Node *Root[2];
	GenMap *Map;
	int TerrainOutline[2][2];

public:
	LandscapeTree(GenMap *Map)
	{
		LandscapeTree::Map = Map;
		Root[0] = new Node(LTREE_LEVEL, this, Map->Pos[0], IMATH::Vector2DXZ(Map->Pos[0].x, Map->Pos[1].z), IMATH::Vector2DXZ(Map->Pos[1].x, Map->Pos[0].z));
		Root[1] = new Node(LTREE_LEVEL, this, Map->Pos[1], IMATH::Vector2DXZ(Map->Pos[0].x, Map->Pos[1].z), IMATH::Vector2DXZ(Map->Pos[1].x, Map->Pos[0].z));
	}

	~LandscapeTree()
	{
		delete Root[0];
		delete Root[1];
	}

	void Generate(MissionEditor *ME)
	{
		srand(ME->MLib.Missions[ME->Mission]->Seed);
		TerrainOutline[0][0] = rand() % 6;
		TerrainOutline[0][1] = rand() % 6;
		TerrainOutline[1][0] = rand() % 6;
		TerrainOutline[1][1] = rand() % 6;
		ME->LoadingScreen("generating terrain left");
		Root[0]->Generate(Map, ME);
		ME->LoadingScreen("generating terrain right");
		Root[1]->Generate(Map, ME);
		ME->LoadingScreen("generating terrain heightmap");
		ME->Wrld.CreateHMap(ME->MLib.Missions[ME->Mission]->Seed);
	}
};

void MissionEditor::GenerateLandscape(GenMap *Map)
{
	LoadingScreen("generating landscape tree");
	LandscapeTree LT(Map);
	LT.Generate(this);
}

void MissionEditor::GenerateBaseStreets(GenMap *Map)
{
	LoadingScreen("generating streets");
	GenerationLog << *Map;
	GrowMainStreet(Map, false);
	for(int i = 0; i < (Map->Width + Map->Height) / 15; ++i)
		GrowStreets(Map);
	LoadingScreen("updating street textures");
	Wrld.UpdateSurfaceTextures();
	GenerationLog << *Map;
	/*GenerateMainStreet(Map);
	GenerateMainStreet(Map);
	GenerateMainStreet(Map);
	for(int i = 0; i < 5; ++i)
		GenerateSideStreet(Map);
	GenerateConnectedStreets(Map);*/
}

void MissionEditor::GenerateBaseBuildings(GenMap *Map, int Owner)
{
	Map->NearMap();
	for(int i = 0; i < Map->Width * Map->Height / 2000; ++i)
		GenerateBuilding(Map, 1, Owner);
	for(int i = 0; i < Map->Width * Map->Height / 2000; ++i)
		GenerateBuilding(Map, 3, Owner);
}

void MissionEditor::GenerateBase(GenMap *Map, int Owner)
{
	LoadingScreen("generating base streets");
	GenerateBaseStreets(Map);
	LoadingScreen("generating base buildings");
	GenerateBaseBuildings(Map, Owner);
	LoadingScreen("generating decals");
	Map->GetME()->Wrld.CompileDecalsAdditive();
}

bool MissionEditor::GenerateAirbaseAirfield(GenMap *Map, int Owner)
{
	Airfield::AirfieldSpecs Specs;
	sprintf(Specs.Name, "MissionEditor Airfield %d", Brush.GetNextID());
	FreeArea A = FindFreeAreaXZ(Map, 240, 80, 240, 80);
	if(A.Successful)
	{
		Specs.Pos = 0.5f * (A.Area[0] + A.Area[1]);
		GenerationLog << "Adding Airfield at " << Specs.Pos.x << ", " << Specs.Pos.z << std::endl;
		Airfield *AF = new Airfield(Specs);
		MLib.Missions[Mission]->AddAirfield(AF);
		MLib.Missions[Mission]->NextID = Brush.GetNextID();
		LoadingScreen("Adding Airfield...");
		Wrld.AddAirfieldPre(AF);
		Map->FillMap();
		Brush.SetNextID(Brush.GetNextID() + 1);
		return true;
	}
	return false;
}

void MissionEditor::GenerateAirbaseBuildings(GenMap *Map, int Owner)
{
	int State = 0;
	try
	{
		State = 1;
		Map->FillMap();
		Map->NearMap();
		for(int i = 0; i < Map->Width * Map->Height / 6000; ++i)
			GenerateBuilding(Map, 2, Owner);
		State = 9;
		for(int i = 0; i < Map->Width * Map->Height / 6000; ++i)
			GenerateBuilding(Map, 3, Owner);
		State = 10;
	}
	catch(char *s)
	{
		throw s;
	}
	catch(...)
	{
		char Str[1024];
		sprintf(Str, "MissionEditor::GenerateAirbaseBuildings#%d", State);
		throw Str;
	}
}

void MissionEditor::GenerateAirbase(GenMap *Map, int Owner)
{
	LoadingScreen("generating airbase airfield");
	if(GenerateAirbaseAirfield(Map, Owner))
	{
		LoadingScreen("generating airbase streets");
		GenerateBaseStreets(Map);
		LoadingScreen("generating airbase buildings");
		GenerateAirbaseBuildings(Map, Owner);
		LoadingScreen("generating airbase decals");
		Map->GetME()->Wrld.CompileDecalsAdditive();
	}
}

MissionEditor::FreeArea MissionEditor::FindFreeAreaXZ(GenMap *Map, int MinSizeX, int MinSizeZ, int MaxSizeX, int MaxSizeZ)
{
	GenerationLog << "FindFreeAreaXZ: size " << MinSizeX << "x" << MinSizeZ << " to " << MaxSizeX << "x" << MaxSizeZ << std::endl;
	FreeArea Ret;
	Ret.Successful = false;
	int x1, z1, x2, z2;
	for(int k = 0; k < 40; ++k)
	{
		int x = rand() % Map->Width;
		int z = rand() % Map->Height;
		if(Map->Get(x, z) != World::NOTHING)
			continue;
		x1 = x; z1 = z;
		x2 = x; z2 = z;
		bool ok[4];
		ok[0] = true; ok[1] = true; ok[2] = true; ok[3] = true;
		// try to grow area
		bool success = true;
		while(success)
		{
			success = false;
			if(ok[0] && x1 > 0 && x2 - x1 < MaxSizeX)
			{
				// left
				for(int i = z1; i <= z2; ++i)
					if(Map->Get(x1 - 1, i) != World::NOTHING)
					{
						ok[0] = false;
						break;
					}
				if(ok[0])
				{
					x1--;
					success = true;
				}
			}
			if(ok[1] && x2 < Map->Width - 1 && x2 - x1 < MaxSizeX)
			{
				// right
				for(int i = z1; i <= z2; ++i)
					if(Map->Get(x2 + 1, i) != World::NOTHING)
					{
						ok[1] = false;
						break;
					}
				if(ok[1])
				{
					x2++;
					success = true;
				}
			}
			if((!(ok[0] || ok[1])) && x2 - x1 < MinSizeX)
				return Ret;
			if(ok[2] && z1 > 0 && z2 - z1 < MaxSizeZ)
			{
				// top
				for(int i = x1; i <= x2; ++i)
					if(Map->Get(i, z1 - 1) != World::NOTHING)
					{
						ok[2] = false;
						break;
					}
				if(ok[2])
				{
					z1--;
					success = true;
				}
			}
			if(ok[3] && z2 < Map->Height - 1 && z2 - z1 < MaxSizeZ)
			{
				// bottom
				for(int i = x1; i <= x2; ++i)
					if(Map->Get(i, z1 + 1) != World::NOTHING)
					{
						ok[3] = false;
						break;
					}
				if(ok[3])
				{
					z2++;
					success = true;
				}
			}
			if((!(ok[2] || ok[3])) && z2 - z1 < MinSizeZ)
				return Ret;
		}
		if(x2 - x1 >= MinSizeX && z2 - z1 >= MinSizeZ)
		{
			Ret.Area[0].x = Map->Pos[0].x + x1;
			Ret.Area[0].z = Map->Pos[0].z + z1;
			Ret.Area[1].x = Map->Pos[0].x + x2;
			Ret.Area[1].z = Map->Pos[0].z + z2;
			GenerationLog << "FindFreeAreaXZ: found area " << Ret.Area[0].x << ", " << Ret.Area[0].z << "->" << Ret.Area[1].x << ", " << Ret.Area[1].z << std::endl;
			Ret.Successful = true;
			return Ret;
		}
		else
		{
			GenerationLog << "FindFreeAreaXZ: too small " << x1 << ", " << z1 << "->" << x2 << ", " << z2 << std::endl;
		}
	}
	return Ret;
}

MissionEditor::FreeArea MissionEditor::FindFreeArea(GenMap *Map, int MinSize, int MaxSize)
{
	return FindFreeAreaXZ(Map, MinSize, MinSize, MaxSize, MaxSize);
}

class GenStreetC: public MissionEditor::GenStreet
{
public:
	int Flags;
	GenStreetC() {}
	GenStreetC(GenStreet a, int f): GenStreet(a), Flags(f) {}
	bool operator ==(const GenStreetC &a) { return (GenStreet &)*this == (GenStreet &)a && Flags == a.Flags;}
};

bool MissionEditor::IsConnected(GenMap *Map, GenStreet &a, GenStreet &b)
{
	//static std::ofstream log("conn2.txt");
	//log << "(" << a.x << ", " << a.z << ") -> (" << b.x << ", " << b.z << "): ";
	/*if(a.is_connected_with(b))
	{
		//log << "directly connected" << std::endl;
		return true;
	}*/
	//log << std::endl;
	ISL::DynamicArray<GenStreetC> List;
	List.Add(GenStreetC(a, 0));
	List.Add(GenStreetC(b, 1));
	int FirstUnchecked = 0;
	for(;;)
	{
		if(FirstUnchecked >= List.Count())
			return false;
		GenStreetC &s = List[FirstUnchecked];
		//log << "checking " << FirstUnchecked << "of" << List.Count() << ", (" << s.x << ", " << s.z << ", " << s.Flags << ")";
		// test if connected
		for(int i = 0; i < FirstUnchecked; ++i)
			if(s.is_connected_with(List[i]) && s.Flags != List[i].Flags)
			{
				//log << " is connected to #" << i << std::endl;
				return true;
			}
		//log << std::endl;
		// add neighbors
		int ThisType = Map->Get(s.x - (int)Map->Pos[0].x, s.z - (int)Map->Pos[0].z);
		if(ThisType & World::STREET_HORZ)
		{
			int Left = Map->Get(s.x - (int)Map->Pos[0].x - 32, s.z - (int)Map->Pos[0].z);
			if((Left & World::STREET_HORZ) && !List.IsIn(GenStreetC(GenStreet(s.x - 32, s.z, Left), s.Flags)))
				List.Add(GenStreetC(GenStreet(s.x - 32, s.z, Left), s.Flags));
			int Right = Map->Get(s.x - (int)Map->Pos[0].x + 32, s.z - (int)Map->Pos[0].z);
			if((Right & World::STREET_HORZ) && !List.IsIn(GenStreetC(GenStreet(s.x + 32, s.z, Right), s.Flags)))
				List.Add(GenStreetC(GenStreet(s.x + 32, s.z, Right), s.Flags));
		}
		if(ThisType & World::STREET_VERT)
		{
			int Far = Map->Get(s.x - (int)Map->Pos[0].x, s.z - (int)Map->Pos[0].z - 32);
			if((Far & World::STREET_VERT) && !List.IsIn(GenStreetC(GenStreet(s.x, s.z - 32, Far), s.Flags)))
				List.Add(GenStreetC(GenStreet(s.x, s.z - 32, Far), s.Flags));
			int Near = Map->Get(s.x - (int)Map->Pos[0].x, s.z - (int)Map->Pos[0].z + 32);
			if((Near & World::STREET_VERT) && !List.IsIn(GenStreetC(GenStreet(s.x, s.z + 32, Near), s.Flags)))
				List.Add(GenStreetC(GenStreet(s.x, s.z + 32, Near), s.Flags));
		}
		FirstUnchecked++;
	}
}

void MissionEditor::GenerateConnection(GenMap *Map, GenStreet &a_world, GenStreet &b_world)
{
	// local coordinates
	GenStreet a(a_world.x - (int)Map->Pos[0].x, a_world.z - (int)Map->Pos[0].z, a_world.GetDir());
	GenStreet b(b_world.x - (int)Map->Pos[0].x, b_world.z - (int)Map->Pos[0].z, b_world.GetDir());
	bool ok = true;
	for(int i = min(a.z, b.z); i <= max(a.z, b.z); ++i)
		if(Map->Get(a.x, i) & ~World::STREET_MASK)
		{
			ok = false;
			break;
		}
	if(ok)
	{
		for(int i = min(a.x, a.x); i <= max(a.x, b.x); ++i)
			if(Map->Get(i, b.z) & ~World::STREET_MASK)
			{
				ok = false;
				break;
			}
		if(ok)
		{
			GenerateStreet(Map, a.x, a.z, a.x, b.z);
			GenerateStreet(Map, a.x, b.z, b.x, b.z);
			return;
		}
	}
	ok = true;
	for(int i = min(a.z, b.z); i <= max(a.z, b.z); ++i)
		if(Map->Get(b.x, i) & ~World::STREET_MASK)
		{
			ok = false;
			break;
		}
	if(ok)
	{
		for(int i = min(a.x, a.x); i <= max(a.x, b.x); ++i)
			if(Map->Get(i, a.z) & ~World::STREET_MASK)
			{
				ok = false;
				break;
			}
		if(ok)
		{
			GenerateStreet(Map, b.x, b.z, b.x, a.z);
			GenerateStreet(Map, b.x, a.z, a.x, a.z);
			return;
		}
	}
}

struct Conn
{
	MissionEditor::GenStreet *Nodes[2];
	float Distance;
	bool done;
	Conn() {}
	Conn(MissionEditor::GenStreet *a, MissionEditor::GenStreet *b)
	{
		Nodes[0] = a;
		Nodes[1] = b;
		Distance = absf((float)a->x - b->x) + absf((float)a->z - b->z);
		done = false;
	}
};

void MissionEditor::GenerateConnectedStreets(GenMap *Map)
{
	/*std::ofstream log("conn.txt");
	// map
	const step = 32;
	char *lmap = new char[Map->Width * Map->Height / (step * step)];
	memset(lmap, ' ', Map->Width * Map->Height / (step * step));
	for(int j = 0; j < Map->Height / step; j++)
		for(int i = 0; i < Map->Width / step; i++)
			switch(Map->Get(i * step, j * step))
			{
			case World::STREET_HORZ: lmap[j * (Map->Width / step) + i] = '-'; break;
			case World::STREET_VERT: lmap[j * (Map->Width / step) + i] = '|'; break;
			case World::STREET_CROSS: lmap[j * (Map->Width / step) + i] = '+'; break;
			case World::AIRFIELD: lmap[j * (Map->Width / step) + i] = '#'; break;
			case World::TERRAIN: lmap[j * (Map->Width / step) + i] = '.'; break;
			case World::VEHICLE: lmap[j * (Map->Width / step) + i] = 'v'; break;
			}
	// genstreets
	for(int i = 0; i < GenStreets.Count(); ++i)
		lmap[((int)(GenStreets[i].z - Map->Pos[0].z) / step) * (Map->Width / step) + (int)(GenStreets[i].x - Map->Pos[0].x) / step] = 'o';
	// log
	for(int j = 0; j < Map->Height / step; ++j)
	{
		for(int i = 0; i < Map->Width / step; ++i)
			log << (char)lmap[j * (Map->Width / step) + i];
		log << std::endl;
	}
	//
	log << "map: (" << Map->Pos[0].x << ", " << Map->Pos[0].z << ") -> (" << Map->Pos[1].x << ", " << Map->Pos[1].z << ")" << std::endl;*/
	ISL::DynamicArray<Conn> Conns;
	for(int i = 0; i < GenStreets.Count(); ++i)
		if(GenStreets[i].x >= Map->Pos[0].x && GenStreets[i].z >= Map->Pos[0].z
			&& GenStreets[i].x <= Map->Pos[1].x && GenStreets[i].z <= Map->Pos[1].z)
			for(int j = 0; j < i; ++j)
				if(GenStreets[j].x >= Map->Pos[0].x && GenStreets[j].z >= Map->Pos[0].z
					&& GenStreets[j].x <= Map->Pos[1].x && GenStreets[j].z <= Map->Pos[1].z)
					if(!IsConnected(Map, GenStreets[i], GenStreets[j]))
					{
						Conns.Add(Conn(&GenStreets[i], &GenStreets[j]));
						//log << "conn: (" << GenStreets[i].x << ", " << GenStreets[i].z << ") -> (" << GenStreets[j].x << ", " << GenStreets[j].z << ")" << std::endl;
					}
	float Near;
	Conn *Next;
	//log << Conns.Count() << " connections" << std::endl;
	do
	{
		//log << "next" << std::endl;
		Near = -1.0f;
		Next = 0;
		for(int j = 0; j < Conns.Count(); ++j)
			if(!Conns[j].done && (Conns[j].Distance < Near || Near == -1.0f) && !IsConnected(Map, *Conns[j].Nodes[0], *Conns[j].Nodes[1]))
			{
				Next = &Conns[j];
				Near = Next->Distance;
				//log << "ok, distance: " << Near << std::endl;
			}
			else
			{
				/*if(Conns[j].done)
					log << "bad, done" << std::endl;
				else if(Conns[j].Distance >= Near && Near != -1.0f)
					log << "bad, too far: " << Conns[j].Distance << std::endl;
				else if(IsConnected(Map, *Conns[j].Nodes[0], *Conns[j].Nodes[1]))
				{
					Conns[j].done = true;
					log << "bad, already connected" << std::endl;
				}*/
			}
		if(Next)
		{
			/*log << "generating: " << Near << std::endl;
			log << "generating: (" << Next->Nodes[0]->x << ", " << Next->Nodes[0]->z << ") -> (" << Next->Nodes[1]->x << ", " << Next->Nodes[1]->z << ")" << std::endl;*/
			GenerateConnection(Map, *Next->Nodes[0], *Next->Nodes[1]);
			Next->done = true;
		}
	} while(Next);
	/*Map->FillMap();
	// map
	char *lmap2 = new char[Map->Width * Map->Height / (step * step)];
	memset(lmap2, ' ', Map->Width * Map->Height / (step * step));
	for(int j = 0; j < Map->Height / step; j++)
		for(int i = 0; i < Map->Width / step; i++)
			switch(Map->Get(i * step, j * step))
			{
			case World::STREET_HORZ: lmap2[j * (Map->Width / step) + i] = '-'; break;
			case World::STREET_VERT: lmap2[j * (Map->Width / step) + i] = '|'; break;
			case World::STREET_CROSS: lmap2[j * (Map->Width / step) + i] = '+'; break;
			case World::AIRFIELD: lmap2[j * (Map->Width / step) + i] = '#'; break;
			case World::TERRAIN: lmap2[j * (Map->Width / step) + i] = '.'; break;
			case World::VEHICLE: lmap2[j * (Map->Width / step) + i] = 'v'; break;
			}
	// genstreets
	for(int i = 0; i < GenStreets.Count(); ++i)
		lmap2[((int)(GenStreets[i].z - Map->Pos[0].z) / step) * (Map->Width / step) + (int)(GenStreets[i].x - Map->Pos[0].x) / step] = 'o';
	// log
	for(int j = 0; j < Map->Height / step; ++j)
	{
		for(int i = 0; i < Map->Width / step; ++i)
		{
			if(lmap[j * (Map->Width / step) + i] == lmap2[j * (Map->Width / step) + i])
				log << (char)lmap2[j * (Map->Width / step) + i];
			else
				switch(lmap2[j * (Map->Width / step) + i])
				{
					case '-': log << (char)'~'; break;
					case '|': log << (char)'I'; break;
					case '+': log << (char)'*'; break;
					case 'o': log << (char)'O'; break;
				}
		}
		log << std::endl;
	}
	delete[] lmap;
	delete[] lmap2;*/
}

struct NewAirbase
{
	GenMap *Map;
	int Owner;

	NewAirbase(): Map(0) {}
	NewAirbase(GenMap *NewMap, int NewOwner): Map(NewMap), Owner(NewOwner) {}
};

void MissionEditor::GenerateMissionCaptureAirbases(GenMap *Map)
{
	int State = 0;
	ISL::DynamicArray<NewAirbase> NewAirbases;
	try
	{
		State = 1;
		{
			IMATH::Vector2DXZ Area[2] = {IMATH::Vector2DXZ(Map->Pos[0].x, Map->Pos[0].z), IMATH::Vector2DXZ(Map->Pos[0].x * 0.25f + Map->Pos[1].x * 0.75f, Map->Pos[1].z)};
			LoadingScreen("airbase: generating leftmap");
			GenMap LeftMap(Area, Map->GetME());
			LoadingScreen("airbase: searching free area");
			FreeArea FArea = FindFreeAreaXZ(&LeftMap, 310, 150, 512, 512);
			if(FArea.Successful)
			{
				LoadingScreen("airbase: generating map");
				GenMap *NewMap = new GenMap(FArea.Area, LeftMap.GetME());
				LoadingScreen("airbase: generating streets");
				GenerateAirbaseAirfield(NewMap, 2);
				GenerateBaseStreets(NewMap);
				NewAirbases.Add(NewAirbase(NewMap, 2));
			}
		}
		State = 2;
		{
			IMATH::Vector2DXZ Area[2] = {IMATH::Vector2DXZ(Map->Pos[0].x * 0.75f + Map->Pos[1].x * 0.25f, Map->Pos[0].z), IMATH::Vector2DXZ(Map->Pos[1].x, Map->Pos[1].z)};
			LoadingScreen("airbase: generating rightmap");
			GenMap RightMap(Area, Map->GetME());
			LoadingScreen("airbase: searching free area");
			FreeArea FArea = FindFreeAreaXZ(&RightMap, 310, 150, 512, 512);
			if(FArea.Successful)
			{
				LoadingScreen("airbase: generating map");
				GenMap *NewMap = new GenMap(FArea.Area, RightMap.GetME());
				LoadingScreen("airbase: generating streets");
				GenerateAirbaseAirfield(NewMap, 3);
				GenerateBaseStreets(NewMap);
				NewAirbases.Add(NewAirbase(NewMap, 3));
			}
		}
		State = 3;
		for(int k = 0; k < 4; ++k)
		{
			LoadingScreen("airbase: searching free area");
			State = 4;
			FreeArea Area = FindFreeAreaXZ(Map, 310, 150, 512, 512);
			State = 5;
			if(Area.Successful)
			{
				LoadingScreen("airbase: generating map");
				State = 6;
				GenMap *NewMap = new GenMap(Area.Area, Map->GetME());
				State = 7;
				LoadingScreen("airbase: generating airfield");
				State = 8;
				GenerateAirbaseAirfield(NewMap, 3);
				State = 9;
				LoadingScreen("airbase: generating streets");
				GenerateBaseStreets(NewMap);
				State = 10;
				NewAirbases.Add(NewAirbase(NewMap, 3));
			}
		}
		State = 11;
		LoadingScreen("fill map");
		Map->FillMap();
		LoadingScreen("macroconnect streets");
		State = 12;
		GenerateConnectedStreets(Map);
		State = 13;
		LoadingScreen("compile decals");
		Map->GetME()->Wrld.CompileDecalsAdditive();
		LoadingScreen("generating buildings");
		State = 14;
		for(int i = 0; i < NewAirbases.Count(); ++i)
		{
			GenerateAirbaseBuildings(NewAirbases[i].Map, NewAirbases[i].Owner);
			delete NewAirbases[i].Map;
		}
		State = 15;
		MLib.Missions[Mission]->AddMissionGoal(MissionLib::Mission::MG_DESTROYALLENEMY | MissionLib::Mission::MG_PLAYER);
		MenuCheck(1436, true);
		MLib.Missions[Mission]->AddMissionGoal(MissionLib::Mission::MG_DESTROYALLPLAYER | MissionLib::Mission::MG_ENEMY);
		MenuCheck(1435, true);
		State = 16;
	}
	catch(char *s)
	{
		throw s;
	}
	catch(...)
	{
		char Str[1024];
		sprintf(Str, "MissionEditor::GenerateMissionCaptureAirbases#%d", State);
		throw Str;
	}
}

void MissionEditor::GenerateMissionProtectConvoy(GenMap *Map)
{
	MLib.Missions[Mission]->AddMissionGoal(MissionLib::Mission::MG_DESTROYALLENEMY | MissionLib::Mission::MG_PLAYER);
	MenuCheck(1436, true);
	MLib.Missions[Mission]->AddMissionGoal(MissionLib::Mission::MG_DESTROYALLPLAYER | MissionLib::Mission::MG_ENEMY);
	MenuCheck(1435, true);
}

void MissionEditor::GenerateMissionDestroyConvoy(GenMap *Map)
{
	MLib.Missions[Mission]->AddMissionGoal(MissionLib::Mission::MG_DESTROYALLENEMY | MissionLib::Mission::MG_PLAYER);
	MenuCheck(1436, true);
	MLib.Missions[Mission]->AddMissionGoal(MissionLib::Mission::MG_DESTROYALLPLAYER | MissionLib::Mission::MG_ENEMY);
	MenuCheck(1435, true);
}

void MissionEditor::GenerateMissionCaptureCity(GenMap *Map)
{
	FreeArea Area = FindFreeArea(Map, 128, 1024);
	if(Area.Successful)
	{
		GenMap NewMap(Area.Area, Map->GetME());
		GenerateCity(&NewMap);
	}
	MLib.Missions[Mission]->AddMissionGoal(MissionLib::Mission::MG_DESTROYALLENEMY | MissionLib::Mission::MG_PLAYER);
	MenuCheck(1436, true);
	MLib.Missions[Mission]->AddMissionGoal(MissionLib::Mission::MG_DESTROYALLPLAYER | MissionLib::Mission::MG_ENEMY);
	MenuCheck(1435, true);
}

void MissionEditor::GenerateMissionCaptureBase(GenMap *Map)
{
	FreeArea Area = FindFreeArea(Map, 128, 1024);
	if(Area.Successful)
	{
		GenMap NewMap(Area.Area, Map->GetME());
		GenerateBase(&NewMap, 3);
	}
	MLib.Missions[Mission]->AddMissionGoal(MissionLib::Mission::MG_DESTROYALLENEMY | MissionLib::Mission::MG_PLAYER);
	MenuCheck(1436, true);
	MLib.Missions[Mission]->AddMissionGoal(MissionLib::Mission::MG_DESTROYALLPLAYER | MissionLib::Mission::MG_ENEMY);
	MenuCheck(1435, true);
}

void MissionEditor::GenerateMissionMovePrototype(GenMap *Map)
{
	MLib.Missions[Mission]->AddMissionGoal(MissionLib::Mission::MG_DESTROYALLENEMY | MissionLib::Mission::MG_PLAYER);
}

void MissionEditor::GenerateMissionMoveFleet(GenMap *Map)
{
	MLib.Missions[Mission]->AddMissionGoal(MissionLib::Mission::MG_DESTROYALLENEMY | MissionLib::Mission::MG_PLAYER);
	MenuCheck(1436, true);
	MLib.Missions[Mission]->AddMissionGoal(MissionLib::Mission::MG_DESTROYALLPLAYER | MissionLib::Mission::MG_ENEMY);
	MenuCheck(1435, true);
}

void MissionEditor::GenerateMissionDestroyFleet(GenMap *Map)
{
	MLib.Missions[Mission]->AddMissionGoal(MissionLib::Mission::MG_DESTROYALLENEMY | MissionLib::Mission::MG_PLAYER);
	MenuCheck(1436, true);
	MLib.Missions[Mission]->AddMissionGoal(MissionLib::Mission::MG_DESTROYALLPLAYER | MissionLib::Mission::MG_ENEMY);
	MenuCheck(1435, true);
}

// called synchronously
void MissionEditor::EventHandler(void)
{
	int State = 0;
	try
	{
		State = 1;
		if(Events.Count())
		{
			for(int i = 0; i < Events.Count(); i++)
			{
				int ID = Events[i].GetWParam() & 0xFFFF;
				State = 2;
				switch(Events[i].GetID())
				{
				case I3D::D3DW_EVENT_WINDOWSEVENT:
					State = 100;
					switch(Events[i].GetInfo())
					{
					case WM_KEYDOWN:
						State = 101;
						KeyboardInput(Events[i].GetWParam());
						break;
					case WM_COMMAND:
						State = 102;
						switch(ID)
						{
						case 1024:
							State = 201;
							char Dir[1024];
							getcwd(Dir, 1024);
							// load
							OPENFILENAME OFN;
							memset(&OFN, 0, sizeof(OFN));
							OFN.lStructSize = sizeof(OFN);
							OFN.hwndOwner = Wnd.GetHWnd();
							OFN.lpstrFilter = "Mission Files (*.ini)\0*.INI\0\0";
							OFN.lpstrDefExt = "ini";
							OFN.lpstrInitialDir = "Missions";
							OFN.lpstrTitle = "Load Mission";
							OFN.lpstrFile = new char[1024];
							OFN.nMaxFile = 1024;
							*OFN.lpstrFile = 0;
							OFN.lpstrFileTitle = new char[1024];
							OFN.nMaxFileTitle = 1024;
							*OFN.lpstrFileTitle = 0;
							//OFN.lpstrFile = NULL;
							/*strcpy(OFN.lpstrFile, "Models\\Tree.t4m");
							OFN.nFileOffset = 8;
							OFN.nFileExtension = 13;*/
							OFN.Flags = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY /*| OFN_SHAREAWARE*/; // OFN_OVERWRITEPROMPT
							//ShowCursor(true);
							if(GetOpenFileName(&OFN))
							{
								chdir(Dir);
								// reload mission library
								MLib.Create("Missions");
								// load mission
								Mission = MLib.GetMissionNum(OFN.lpstrFileTitle);
								State = 202;
								LoadMission();
								State = 203;
							}
							else
								chdir(Dir);
							//ShowCursor(false);
							delete[] OFN.lpstrFile;
							break;
						case 1025:
							// save
							MLib.Missions[Mission]->Save("Missions");
							break;
						case 1026:
							// new
							NewMission();
							break;
						case 1034:
							// way
							Brush.SetWay(0);
							break;
						case 1035:
							// street
							Brush.SetWay(1);
							break;
						case 1036:
							// highway
							Brush.SetWay(2);
							break;
						// terrain
						case 1201:
							Brush.SetTerrain(I3D::IDT_TERRAIN_MOUNTAIN);
							break;
						case 1202:
							Brush.SetTerrain(I3D::IDT_TERRAIN_HIGHMOUNTAIN);
							break;
						case 1203:
							Brush.SetTerrain(I3D::IDT_TERRAIN_LOW);
							break;
						case 1204:
							Brush.SetTerrain(I3D::IDT_TERRAIN_WATER);
							break;
						case 1205:
							Brush.SetTerrain(I3D::IDT_TERRAIN_ISLANDS);
							break;
						case 1206:
							Brush.SetTerrain(I3D::IDT_TERRAIN_DEEPWATER);
							break;
						case 1207:
							Brush.SetTerrain(I3D::IDT_TERRAIN_SMALLISLANDS);
							break;
						case 1301:
							Wrld.SetDrawAccMap(!Wrld.GetDrawAccMap());
							MenuCheck(1301);
							break;
						case 1302:
							Wrld.SetDrawMELines(!Wrld.GetDrawMELines());
							MenuCheck(1302);
							break;
						case 1303:
							{
								IMATH::Vector2DXZ Area[2];
								Area[0] = IMATH::Vector2DXZ(-0.5f * Wrld.GetSize(), -0.5f * Wrld.GetSize());
								Area[1] = IMATH::Vector2DXZ(0.5f * Wrld.GetSize(), 0.5f * Wrld.GetSize());
								GenMap Map(Area, this);
								GenerateConnectedStreets(&Map);
							}
							break;
						case 1401:
							// set cam position
							MLib.Missions[Mission]->SetInitialCameraPos(Wrld.GetCameraPos());
							MLib.Missions[Mission]->SetInitialCameraAngle(Wrld.GetCameraAngle());
							break;
						case 1431:
							for(int i = 0; i < Wrld.GetSelection().Count(); ++i)
								if(MLib.Missions[Mission]->TestMissionGoal(0, Wrld.GetSelection()[i]->GetName()))
									MLib.Missions[Mission]->RemoveMissionGoal(0, Wrld.GetSelection()[i]->GetName());
							Wrld.CreateLines();
							break;
						case 1432:
							for(int i = 0; i < Wrld.GetSelection().Count(); ++i)
								MLib.Missions[Mission]->AddMissionGoal(MissionLib::Mission::MG_DESTROY | MissionLib::Mission::MG_PLAYER, Wrld.GetSelection()[i]->GetName());
							break;
						case 1433:
							for(int i = 0; i < Wrld.GetSelection().Count(); ++i)
								MLib.Missions[Mission]->AddMissionGoal(MissionLib::Mission::MG_DESTROY | MissionLib::Mission::MG_ENEMY, Wrld.GetSelection()[i]->GetName());
							break;
						case 1434:
							if(MLib.Missions[Mission]->TestMissionGoal(MissionLib::Mission::MG_DESTROYALLPLAYER | MissionLib::Mission::MG_PLAYER))
								MLib.Missions[Mission]->RemoveMissionGoal(MissionLib::Mission::MG_DESTROYALLPLAYER | MissionLib::Mission::MG_PLAYER);
							else
								MLib.Missions[Mission]->AddMissionGoal(MissionLib::Mission::MG_DESTROYALLPLAYER | MissionLib::Mission::MG_PLAYER);
							MenuCheck(1434, MLib.Missions[Mission]->TestMissionGoal(MissionLib::Mission::MG_DESTROYALLPLAYER | MissionLib::Mission::MG_PLAYER));
							break;
						case 1435:
							if(MLib.Missions[Mission]->TestMissionGoal(MissionLib::Mission::MG_DESTROYALLPLAYER | MissionLib::Mission::MG_ENEMY))
								MLib.Missions[Mission]->RemoveMissionGoal(MissionLib::Mission::MG_DESTROYALLPLAYER | MissionLib::Mission::MG_ENEMY);
							else
								MLib.Missions[Mission]->AddMissionGoal(MissionLib::Mission::MG_DESTROYALLPLAYER | MissionLib::Mission::MG_ENEMY);
							MenuCheck(1435, MLib.Missions[Mission]->TestMissionGoal(MissionLib::Mission::MG_DESTROYALLPLAYER | MissionLib::Mission::MG_ENEMY));
							break;
						case 1436:
							if(MLib.Missions[Mission]->TestMissionGoal(MissionLib::Mission::MG_DESTROYALLENEMY | MissionLib::Mission::MG_PLAYER))
								MLib.Missions[Mission]->RemoveMissionGoal(MissionLib::Mission::MG_DESTROYALLENEMY | MissionLib::Mission::MG_PLAYER);
							else
								MLib.Missions[Mission]->AddMissionGoal(MissionLib::Mission::MG_DESTROYALLENEMY | MissionLib::Mission::MG_PLAYER);
							MenuCheck(1436, MLib.Missions[Mission]->TestMissionGoal(MissionLib::Mission::MG_DESTROYALLENEMY | MissionLib::Mission::MG_PLAYER));
							break;
						case 1437:
							if(MLib.Missions[Mission]->TestMissionGoal(MissionLib::Mission::MG_DESTROYALLENEMY | MissionLib::Mission::MG_ENEMY))
								MLib.Missions[Mission]->RemoveMissionGoal(MissionLib::Mission::MG_DESTROYALLENEMY | MissionLib::Mission::MG_ENEMY);
							else
								MLib.Missions[Mission]->AddMissionGoal(MissionLib::Mission::MG_DESTROYALLENEMY | MissionLib::Mission::MG_ENEMY);
							MenuCheck(1437, MLib.Missions[Mission]->TestMissionGoal(MissionLib::Mission::MG_DESTROYALLENEMY | MissionLib::Mission::MG_ENEMY));
							break;
						case 1438:
							for(int i = 0; i < Wrld.GetSelection().Count(); ++i)
								MLib.Missions[Mission]->AddMissionGoal(MissionLib::Mission::MG_MOVETO | MissionLib::Mission::MG_PLAYER, Wrld.GetSelection()[i]->GetName(), Brush.GetArea());
							break;
						case 1439:
							for(int i = 0; i < Wrld.GetSelection().Count(); ++i)
								MLib.Missions[Mission]->AddMissionGoal(MissionLib::Mission::MG_MOVETO | MissionLib::Mission::MG_ENEMY, Wrld.GetSelection()[i]->GetName(), Brush.GetArea());
							break;
						case 1440:
							MLib.Missions[Mission]->AddMissionGoal(MissionLib::Mission::MG_ANYPLAYERMOVETO | MissionLib::Mission::MG_PLAYER, 0, Brush.GetArea());
							break;
						case 1441:
							MLib.Missions[Mission]->AddMissionGoal(MissionLib::Mission::MG_ANYPLAYERMOVETO | MissionLib::Mission::MG_ENEMY, 0, Brush.GetArea());
							break;
						case 1442:
							MLib.Missions[Mission]->AddMissionGoal(MissionLib::Mission::MG_ANYENEMYMOVETO | MissionLib::Mission::MG_PLAYER, 0, Brush.GetArea());
							break;
						case 1443:
							MLib.Missions[Mission]->AddMissionGoal(MissionLib::Mission::MG_ANYENEMYMOVETO | MissionLib::Mission::MG_ENEMY, 0, Brush.GetArea());
							break;
						case 1445:
							for(int i = 0; i < Wrld.GetSelection().Count(); ++i)
							{
								Wrld.GetSelection()[i]->SetHighlight(!Wrld.GetSelection()[i]->GetHighlight());
								MLib.Missions[Mission]->Objects[i].Highlight = !MLib.Missions[Mission]->GetMissionObject(Wrld.GetSelection()[i]->GetName())->Highlight;
							}
							break;
						case 1446:
							LoadingScreen("creating new terrain");
							MLib.Missions[Mission]->Seed = time(0) + rand();
							Wrld.CreateHMap(MLib.Missions[Mission]->Seed);
							break;
						case 1447:
							OrdersDialog();
							break;
						case 1451:
							MLib.Missions[Mission]->Side = SIDE_ISRAEL;
							break;
						case 1452:
							MLib.Missions[Mission]->Side = SIDE_PALESTINE;
							break;
						case 1500:
							Brush.SetArea();
							break;
						case 1502:
							Brush.ClearArea();
							break;
						case 1501:
							Brush.SetAirfield();
							break;
						case 1601:
							Compile();
							break;
						case 1701:
							LoadStory("");
							break;
						case 1702:
							{
								MissionLib::StoryItem *It = new MissionLib::StoryItem;
								It->BumperID = 1;
								sprintf(It->Name, "MissionEditor StoryItem ID:%d", MLib.Missions[Mission]->NextID++);
								It->TimeDelta = 6;
								It->Render = false;
								MLib.Missions[Mission]->StoryItems.Add(It);
								CreateWindowMenu(Wnd.GetHWnd());
								int id = MLib.Missions[Mission]->StoryItems.Count() - 1;
								LoadStory(MLib.Missions[Mission]->StoryItems[id]->Name);
								SetPause(true);
								StoryItemDialog(id);
							}
							break;
						case 1801:
						case 1802:
						case 1803:
						case 1804:
						case 1805:
						case 1806:
						case 1807:
						case 1808:
						case 1809:
						case 1810:
						case 1811:
						case 1812:
						case 1813:
						case 1814:
							{
								// city
								const IMATH::Vector2DXZ *BArea = Brush.GetArea();
								IMATH::Vector2DXZ Area[2];
								Area[0] = BArea[0];
								Area[1] = BArea[1];
								if(!(Area[0].x != 0 || Area[0].z != 0 || Area[1].x != 0 || Area[1].z != 0))
								{
									Area[0] = IMATH::Vector2DXZ(-0.5f * Wrld.GetSize(), -0.5f * Wrld.GetSize());
									Area[1] = IMATH::Vector2DXZ(0.5f * Wrld.GetSize(), 0.5f * Wrld.GetSize());
								}
								GenerationLog << "ID: " << ID << std::endl;
								GenMap Map(Area, this);
								switch(ID)
								{
								case 1801: GenerateCity(&Map); break;
								case 1802: GenerateLandscape(&Map); break;
								case 1803: GenerateBase(&Map, 2); break;
								case 1813: GenerateBase(&Map, 3); break;
								case 1804: GenerateAirbase(&Map, 2); break;
								case 1814: GenerateAirbase(&Map, 3); break;
								case 1805: GenerateMissionCaptureAirbases(&Map); break;
								case 1806: GenerateMissionProtectConvoy(&Map); break;
								case 1807: GenerateMissionDestroyConvoy(&Map); break;
								case 1808: GenerateMissionCaptureCity(&Map); break;
								case 1809: GenerateMissionCaptureBase(&Map); break;
								case 1810: GenerateMissionMovePrototype(&Map); break;
								case 1811: GenerateMissionMoveFleet(&Map); break;
								case 1812: GenerateMissionDestroyFleet(&Map); break;
								// 1813 is used!
								// 1814 is used!
								}
								Wrld.MinimapUpdate();
								CreateWindowMenu(Wnd.GetHWnd());
							}
							break;
						}
						State = 106;
						// tank menu
						if(ID >= 38700 && ID < 54700)
						{
							int TankID = ID - 38700;
							State = 1061;
							Brush.SetTank(TankID);
							State = 1062;
						}
						State = 1063;
						// story menu
						if(ID >= 11700 && ID < 16700)
						{
							int StID = ID - 11700;
							StoryItem = MLib.Missions[Mission]->StoryItems[StID];
							LoadStory(StoryItem->Name);
						}
						State = 1064;
						// next story item story menu
						if(ID >= 54700 && ID <= 65535)
						{
							int ToID = (ID - 54700) % 100;
							int FromID = ((ID - 54700) - ToID) / 100;
							strcpy(MLib.Missions[Mission]->StoryItems[FromID]->Name, MLib.Missions[Mission]->StoryItems[ToID]->Name);
							CreateWindowMenu(Wnd.GetHWnd());
						}
						State = 1065;
						// story time menu
						if(ID >= 17700 && ID < 24700)
						{
							int Time, StID;
							if(ID >= 23700)
							{
								Time = 15;
								StID = ID - 23700;
							}
							else if(ID >= 22700)
							{
								Time = 12;
								StID = ID - 22700;
							}
							else if(ID >= 21700)
							{
								Time = 8;
								StID = ID - 21700;
							}
							else if(ID >= 20700)
							{
								Time = 6;
								StID = ID - 20700;
							}
							else if(ID >= 19700)
							{
								Time = 4;
								StID = ID - 19700;
							}
							else if(ID >= 18700)
							{
								Time = 2;
								StID = ID - 18700;
							}
							else
							{
								Time = 0;
								StID = ID - 17700;
							}
							MLib.Missions[Mission]->StoryItems[StID]->TimeDelta = (float)Time;
							CreateWindowMenu(Wnd.GetHWnd());
							LoadStory(MLib.Missions[Mission]->StoryItems[StID]->Name);
						}
						State = 1066;
						// story edit item
						if(ID >= 24700 && ID < 27700)
						{
							int StID = (ID - 700) % 1000;
							StoryItemDialog(StID);
							LoadStory(MLib.Missions[Mission]->StoryItems[StID]->Name);
						}
						State = 1067;
						// item type menu
						if(ID >= 28700 && ID < 35700)
						{
							int BumperID, StID;
							if(ID >= 34700)
							{
								BumperID = 7;
								StID = ID - 34700;
							}
							else if(ID >= 33700)
							{
								BumperID = 6;
								StID = ID - 33700;
							}
							else if(ID >= 32700)
							{
								BumperID = 4;
								StID = ID - 32700;
							}
							else if(ID >= 31700)
							{
								BumperID = 3;
								StID = ID - 31700;
							}
							else if(ID >= 30700)
							{
								BumperID = 5;
								StID = ID - 30700;
							}
							else if(ID >= 29700)
							{
								BumperID = 2;
								StID = ID - 29700;
							}
							else
							{
								BumperID = 1;
								StID = ID - 28700;
							}
							MLib.Missions[Mission]->StoryItems[StID]->BumperID = BumperID;
							CreateWindowMenu(Wnd.GetHWnd());
							LoadStory(MLib.Missions[Mission]->StoryItems[StID]->Name);
						}
						State = 1068;
						if(ID >= 35700 && ID < 36700)
						{
							int StID = ID - 35700;
							MLib.Missions[Mission]->StoryItems[StID]->Render = true;
							LoadStory(MLib.Missions[Mission]->StoryItems[StID]->Name);
						}
						State = 1069;
						if(ID >= 36700 && ID < 37700)
						{
							int StID = ID - 36700;
							MLib.Missions[Mission]->StoryItems[StID]->Render = false;
							LoadStory(MLib.Missions[Mission]->StoryItems[StID]->Name);
						}
						// next story item: none
						if(ID >= 15700 && ID < 16700)
						{
							int StID = ID - 15700;
							*MLib.Missions[Mission]->StoryItems[StID]->Next = 0;
							CreateWindowMenu(Wnd.GetHWnd());
						}
						State = 105;
						// decal menu
						if(ID >= 37700 && ID < 38700)
						{
							int DecalID = ID - 37700;
							Brush.SetDecal(DecalID);
						}
						State = 104;
						// owner menu
						if(ID >= 8192 + 2048 && ID < 8192 + 2048 + 1024)
						{
							int OwnerID = ID - (8192 + 2048);
							Brush.SetOwner(OwnerID);
							State = 1040;
							for(int i = 0; i < Wrld.GetSelection().Count(); ++i)
							{
								/*Wrld.GetSelection()[i]->SetOwner(OwnerID);
								MissionLib::Mission::MissionObjectSpecs MOS;
								MOS.Type = IDTYPE_TANK;
								MOS.ID = Wrld.GetSelection()[i]->GetID();
								MOS.Owner = Wrld.GetSelection()[i]->GetOwner();
								MOS.Pos = IMATH::Vector2DXZ(Wrld.GetSelection()[i]->GetPos().x, Wrld.GetSelection()[i]->GetPos().z);
								MOS.Angle = Wrld.GetSelection()[i]->GetAngle();
								strcpy(MOS.Name, Wrld.GetSelection()[i]->GetName());
								MLib.Missions[Mission]->UpdateObject(Wrld.GetSelection()[i]->GetName(), &MOS);*/
								State = 1041;
								Wrld.GetSelection()[i]->SetOwner(OwnerID);
								State = 1042;
								const char *name = Wrld.GetSelection()[i]->GetName();
								State = 1043;
								MissionObject *Obj = MLib.Missions[Mission]->GetMissionObject(name);
								State = 1044;
								Obj->Owner = OwnerID;
								State = 1045;
							}
						}
						State = 103;
						break;
					case WM_LBUTTONDOWN:
						{
							State = 19;
							IMATH::Vector2D Pos;
							Pos.x = (float)(Events[i].GetLParam() & 0xffff);
							Pos.y = (float)(Events[i].GetLParam() >> 16);
							RECT Rect;
							GetClientRect(Wnd.GetHWnd(), &Rect);
							if(Rect.left != Rect.right && Rect.top != Rect.bottom)
							{
								if(Pos.x > Rect.left && Pos.x < Rect.right && Pos.y > Rect.top && Pos.y < Rect.bottom)
								{
									MouseDown = Pos;
									MouseDrag = true;
									MouseEvent Ev;
									Ev.Pos.x = 2 * (float)Pos.x / (Rect.right - Rect.left) - 1;
									Ev.Pos.y = 2 * (1 - (float)Pos.y / (Rect.bottom - Rect.top)) - 1;
									Ev.Event = Ev.DRAG_START;
									MouseEvents.Add(Ev);
								}
							}
						}
						break;
					case WM_LBUTTONUP:
						{
							State = 10;
							IMATH::Vector2D Pos;
							Pos.x = (float)(Events[i].GetLParam() & 0xffff);
							Pos.y = (float)(Events[i].GetLParam() >> 16);
							RECT Rect;
							GetClientRect(Wnd.GetHWnd(), &Rect);
							if(Rect.left != Rect.right && Rect.top != Rect.bottom)
							{
								MouseDrag = false;
								if(absf(Pos.x - MouseDown.x) < 10
									&& absf(Pos.y - MouseDown.y) < 10)
								{
									MouseEvent Ev;
									Ev.Pos.x = 2 * (float)Pos.x / (Rect.right - Rect.left) - 1;
									Ev.Pos.y = 2 * (1 - (float)Pos.y / (Rect.bottom - Rect.top)) - 1;
									Ev.Event = Ev.CLICK;
									MouseEvents.Add(Ev);
								}
								else
								{
									MouseEvent Ev;
									Ev.Pos.x = 2 * (float)Pos.x / (Rect.right - Rect.left) - 1;
									Ev.Pos.y = 2 * (1 - (float)Pos.y / (Rect.bottom - Rect.top)) - 1;
									Ev.Event = Ev.DRAG_END;
									Ev.Down.x = 2 * (float)MouseDown.x / (Rect.right - Rect.left) - 1;
									Ev.Down.y = 2 * (1 - (float)MouseDown.y / (Rect.bottom - Rect.top)) - 1;
									MouseEvents.Add(Ev);
								}
							}
						}
						break;
					case WM_RBUTTONUP:
						{
							State = 11;
							IMATH::Vector2D Pos;
							Pos.x = (float)(Events[i].GetLParam() & 0xffff);
							Pos.y = (float)(Events[i].GetLParam() >> 16);
							RECT Rect;
							GetClientRect(Wnd.GetHWnd(), &Rect);
							if(Rect.left != Rect.right && Rect.top != Rect.bottom)
							{
								if(Pos.x > Rect.left && Pos.x < Rect.right && Pos.y > Rect.top && Pos.y < Rect.bottom)
								{
									MouseEvent Ev;
									Ev.Pos.x = 2 * (float)Pos.x / (Rect.right - Rect.left) - 1;
									Ev.Pos.y = 2 * (1 - (float)Pos.y / (Rect.bottom - Rect.top)) - 1;
									Ev.Event = Ev.RIGHTCLICK;
									MouseEvents.Add(Ev);
								}
							}
						}
						break;
					case WM_MOUSEMOVE:
						{
							State = 12;
							IMATH::Vector2D Pos;
							Pos.x = (float)(Events[i].GetLParam() & 0xffff);
							Pos.y = (float)(Events[i].GetLParam() >> 16);
							RECT Rect;
							GetClientRect(Wnd.GetHWnd(), &Rect);
							if(Rect.left != Rect.right && Rect.top != Rect.bottom)
							{
								if(Events[i].GetWParam() & MK_LBUTTON)
								{
									if(!MouseDrag)
									{
										if(Pos.x > Rect.left && Pos.x < Rect.right && Pos.y > Rect.top && Pos.y < Rect.bottom)
										{
											MouseDrag = true;
											MouseDown = Pos;
											// drag start
											MouseEvent Ev;
											Ev.Pos.x = 2 * (float)Pos.x / (Rect.right - Rect.left) - 1;
											Ev.Pos.y = 2 * (1 - (float)Pos.y / (Rect.bottom - Rect.top)) - 1;
											Ev.Event = Ev.DRAG_START;
											MouseEvents.Add(Ev);
										}
									}
								}
								else
								{
									if(MouseDrag)
									{
										MouseDrag = false;
										// drag end
										MouseEvent Ev;
										Ev.Pos.x = 2 * (float)Pos.x / (Rect.right - Rect.left) - 1;
										Ev.Pos.y = 2 * (1 - (float)Pos.y / (Rect.bottom - Rect.top)) - 1;
										Ev.Event = Ev.DRAG_END;
										Ev.Down.x = 2 * (float)MouseDown.x / (Rect.right - Rect.left) - 1;
										Ev.Down.y = 2 * (1 - (float)MouseDown.y / (Rect.bottom - Rect.top)) - 1;
										MouseEvents.Add(Ev);
									}
								}
								if(MouseDrag)
								{
									// drag
									MouseEvent Ev;
									Ev.Pos.x = 2 * (float)Pos.x / (Rect.right - Rect.left) - 1;
									Ev.Pos.y = 2 * (1 - (float)Pos.y / (Rect.bottom - Rect.top)) - 1;
									Ev.Down.x = 2 * (float)MouseDown.x / (Rect.right - Rect.left) - 1;
									Ev.Down.y = 2 * (1 - (float)MouseDown.y / (Rect.bottom - Rect.top)) - 1;
									Ev.Event = Ev.DRAG;
									MouseEvents.Add(Ev);
								}
							}
						}
						break;
					/*case WM_INITMENUPOPUP:
						ShowCursor(true);
						break;*/
					}
					break;
				}
				State = 3;
			}
			Events.Clear();
		}
	}
	catch(char *s)
	{
		throw(s);
	}
	catch(...)
	{
		char Str[1024];
		sprintf(Str, "MissionEditor::EventHandler#%d", State);
		throw(Str);
	}
}

void MissionEditor::KeyboardInput(char c)
{
	switch(c)
	{
	case 27:
		// Escape
		PostQuitMessage(0);
		break;
	case 'w':
	case 'W':
		D3DFlags ^= GAME_D3DFLAG_WIREFRAME;
		if(D3DFlags & GAME_D3DFLAG_WIREFRAME)
			DS.D3DDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME);
		else
			DS.D3DDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);
		break;
	// pause
	case 'p':
	case 'P':
		Sp.Paused = !Sp.Paused;
		break;
	case 's':
	case 'S':
		for(int i = 0; i < Wrld.GetSelection().Count(); ++i)
		{
			Wrld.GetSelection()[i]->Idle();
			MLib.Missions[Mission]->GetMissionObject(Wrld.GetSelection()[i]->GetName())->AI = Tank::TNK_AI_IDLE;
		}
		Wrld.CreateLines();
		break;
	case 'q':
	case 'Q':
		Wrld.SetQTreeProfShow(!Wrld.GetQTreeProfShow());
		break;
	case 'r':
	case 'R':
		Wrld.SetProfShow(!Wrld.GetProfShow());
		break;
	// angle
	case ' ':
		Brush.SetAngle(Brush.GetAngle() + PI / 4);
		for(int i = 0; i < Wrld.GetSelection().Count(); ++i)
		{
			Wrld.GetSelection()[i]->SetAngle(Wrld.GetSelection()[i]->GetAngle() + PI / 4);
			Wrld.GetSelection()[i]->UpdateMatrices();
			MissionLib::Mission::MissionObjectSpecs MOS;
			MOS.Type = IDTYPE_TANK;
			MOS.ID = Wrld.GetSelection()[i]->GetID();
			MOS.Owner = Wrld.GetSelection()[i]->GetOwner();
			MOS.Pos = IMATH::Vector2DXZ(Wrld.GetSelection()[i]->GetPos().x, Wrld.GetSelection()[i]->GetPos().z);
			MOS.Angle = Wrld.GetSelection()[i]->GetAngle();
			strcpy(MOS.Name, Wrld.GetSelection()[i]->GetName());
			MLib.Missions[Mission]->UpdateObject(Wrld.GetSelection()[i]->GetName(), &MOS);
		}
		break;
	// scrolling
	case VK_NUMPAD4:
	case VK_LEFT:
		Wrld.MoveCamera(I3D::Vector(-4, 0, 0));
		break;
	case VK_NUMPAD6:
	case VK_RIGHT:
		Wrld.MoveCamera(I3D::Vector(4, 0, 0));
		break;
	case VK_NUMPAD8:
	case VK_UP:
		Wrld.MoveCamera(I3D::Vector(0, 0, 4));
		break;
	case VK_NUMPAD2:
	case VK_DOWN:
		Wrld.MoveCamera(I3D::Vector(0, 0, -4));
		break;
	case VK_NUMPAD7:
	case VK_HOME:
		Wrld.MoveCamera(I3D::Vector(0, 4, 0));
		break;
	case VK_NUMPAD1:
	case VK_END:
		Wrld.MoveCamera(I3D::Vector(0, -4, 0));
		break;
	case VK_ADD:
		Wrld.MoveCameraDirect(I3D::Vector(0, 0, 4));
		break;
	case VK_SUBTRACT:
		Wrld.MoveCameraDirect(I3D::Vector(0, 0, -4));
		break;
	case VK_NUMPAD9:
	case VK_PRIOR:
		Wrld.RotateCamera(I3D::Vector(-0.02f, 0, 0));
		break;
	case VK_NUMPAD3:
	case VK_NEXT:
		Wrld.RotateCamera(I3D::Vector(0.02f, 0, 0));
		break;
	case VK_NUMPAD0:
	case VK_INSERT:
		Wrld.RotateCamera(I3D::Vector(0, -0.02f, 0));
		break;
	case VK_DECIMAL:
		Wrld.RotateCamera(I3D::Vector(0, 0.02f, 0));
		break;
	case VK_DIVIDE:
		Wrld.RotateCamera(I3D::Vector(0, 0, 0.02f));
		break;
	case VK_MULTIPLY:
		Wrld.RotateCamera(I3D::Vector(0, 0, -0.02f));
		break;
	// mission editor
	case VK_DELETE:
		World::SelectionType Selection;
		Selection = Wrld.GetSelection();
		for(int i = 0; i < Selection.Count(); i++)
		{
			Wrld.GetSpawnInterface()->DeleteTank(Selection[i]);
			MLib.Missions[Mission]->DeleteObject(Selection[i]->GetName());
		}
		break;
	}
}

void MissionEditor::MouseClick(void)
{
}

void MissionEditor::LoadingScreen(const char *Str)
{
	DS.D3DDevice->Clear(0, NULL, D3DCLEAR_ZBUFFER | D3DCLEAR_TARGET, 0x00000000, 1.0f, 0);
	D3DMATRIX Matrix = {1,0,0,0,  0,1,0,0,  0,0,1,0,  0,0,0,1};
	// world and view matrix
	DS.D3DDevice->SetTransform(D3DTS_WORLD, &Matrix);
	DS.D3DDevice->SetTransform(D3DTS_VIEW, &Matrix);
	DS.D3DDevice->SetTransform(D3DTS_PROJECTION, &Matrix);
	// loading text
	char Text[1024] = "loading...";
	HRESULT BeginSceneResult = DS.D3DDevice->BeginScene();
	I3D::Text::DrawString(&DS, Text, -0.05f * 0.5f * strlen(Text), 0, 0.05f, 0.05f);
	I3D::Text::DrawString(&DS, Str, -0.05f * 0.5f * strlen(Str), -0.1f, 0.05f, 0.05f);
	if(BeginSceneResult == D3D_OK)
		DS.D3DDevice->EndScene(); // end scene, if we needed to begin one
	// display
	DS.D3DDevice->Present(NULL, NULL, NULL, NULL);
}

void MissionEditor::DisplayFPS(void)
{
	if(Sp.DeltaLongTerm)
	{
		char Buf[1024];
		itoa((int)(1.0f / Sp.DeltaLongTerm), Buf, 10);
		strcat(Buf, " fps");
		I3D::Text::DrawString(&DS, Buf, 0.5f, 0.97f, 0.05f, 0.05f);
	}
}

void MissionEditor::NewMission(void)
{
	char Dir[1024];
	getcwd(Dir, 1024);
	// load
	OPENFILENAME OFN;
	memset(&OFN, 0, sizeof(OFN));
	OFN.lStructSize = sizeof(OFN);
	OFN.hwndOwner = Wnd.GetHWnd();
	OFN.lpstrFilter = "Mission Files (*.ini)\0*.ini\0\0";
	OFN.lpstrDefExt = "ini";
	OFN.lpstrInitialDir = "Missions";
	OFN.lpstrTitle = "new mission";
	OFN.lpstrFile = new char[1024];
	OFN.nMaxFile = 1024;
	*OFN.lpstrFile = 0;
	OFN.lpstrFileTitle = new char[1024];
	OFN.nMaxFileTitle = 1024;
	*OFN.lpstrFileTitle = 0;
	//OFN.lpstrFile = NULL;
	/*strcpy(OFN.lpstrFile, "Models\\Tree.t4m");
	OFN.nFileOffset = 8;
	OFN.nFileExtension = 13;*/
	OFN.Flags = OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT /*| OFN_SHAREAWARE*/; // OFN_OVERWRITEPROMPT
	ShowCursor(true);
	if(GetSaveFileName(&OFN))
	{
		chdir(Dir);

		{
			// copy empty.ini
			char line[1024];
			ifstream in("Missions\\empty.ini");
			ostringstream oss;
			oss << "Missions\\" << OFN.lpstrFileTitle;
			ofstream out(oss.str().c_str());
			while(!in.eof())
			{
				in.getline(line, 1024);
				out << line << std::endl;
			}
		}

		// reload mission library
		MLib.Create("Missions");

		Mission = MLib.GetMissionNum(OFN.lpstrFileTitle);
		MLib.Missions[Mission]->Seed = time(0);
		LoadMission();
		MLib.Missions[Mission]->Save("Missions");
		Wrld.SetCameraPos(IMATH::Vector(0, 50, -0.5f * Wrld.GetSize()));
		Wrld.SetCameraAngle(IMATH::Vector(PI / 8.0f, 0, 0));
		MLib.Missions[Mission]->SetInitialCameraPos(Wrld.GetCameraPos());
		MLib.Missions[Mission]->SetInitialCameraAngle(Wrld.GetCameraAngle());
	}
	else
		chdir(Dir);
	ShowCursor(false);
	delete[] OFN.lpstrFile;
}

void MissionEditor::LoadMission(void)
{
	int State = 0;
	try
	{
		State = 1;
		LoadingScreen("World");
		int i;
		World::WorldSpecs WS;
		WS.DS = &DS;
		WS.MOPropIf = &MOPropImpl;
		WS.Seed = MLib.Missions[Mission]->Seed;
		WS.EditMode = true;
		for(i = 0; i < MLib.Missions[Mission]->Ways.Count(); i++)
			WS.Ways[i] = &MLib.Missions[Mission]->Ways[i];
		for(i = 0; i < MLib.Missions[Mission]->TerrainDscs.Count(); i++)
			WS.TerrainDscs[i] = &MLib.Missions[Mission]->TerrainDscs[i];
		for(i = 0; i < MLib.Missions[Mission]->Airfields.Count(); i++)
			WS.Airfields[i] = MLib.Missions[Mission]->Airfields[i];
		for(i = 0; i < MLib.Missions[Mission]->Decals.Count(); i++)
			WS.Decals[i] = MLib.Missions[Mission]->Decals[i];
		// plains under buildings
		for(i = 0; i < MLib.Missions[Mission]->Objects.Count(); i++)
			if(!TLib.Tanks[MLib.Missions[Mission]->Objects[i].Type].CanMove)
				WS.Plains.Add(World::Plain(IMATH::Vector2DXZ(MLib.Missions[Mission]->Objects[i].x, MLib.Missions[Mission]->Objects[i].z), 2));
		State = 2;
		Wrld.Create(&WS);
		State = 3;
		Wrld.SetDrawMELines(false);
		Wrld.SetCameraPos(MLib.Missions[Mission]->InitialCameraPos);
		Wrld.SetCameraAngle(MLib.Missions[Mission]->InitialCameraAngle);
		State = 4;
		// load tanks
		for(i = 0; i < MLib.Missions[Mission]->Objects.Count(); i++)
		{
			State = 5;
			char Str[1024];
			sprintf(Str, "Mission Object %d of %d", i + 1, MLib.Missions[Mission]->Objects.Count());
			LoadingScreen(Str);
			SpawnInterface::SpawnInterfaceSpecs SS;
			IGAME::MissionObject *MObj = &MLib.Missions[Mission]->Objects[i];
			SS.Type = MObj->ObjType;
			SS.ID = MObj->Type;
			SS.Owner = MObj->Owner;
			SS.Pos = IMATH::Vector(MObj->x, TLib.Tanks[MObj->Type].IsShip ? WATERLEVEL : Wrld.GetSurfaceY(MObj->x, MObj->z), MObj->z);
			SS.Target = 0;
			SS.Angle = MObj->Angle;
			SS.AI = MObj->AI;
			SS.TargetX = MObj->TargetX;
			SS.TargetZ = MObj->TargetZ;
			SS.Commandable = MObj->Commandable;
			SS.Highlight = MObj->Highlight;
			strcpy(SS.Name, MObj->Name);
			State = 6;
			SpawnInterface *SI = Wrld.GetSpawnInterface();
			State = 60;
			SI->Spawn(&SS);
			State = 7;
		}
		State = 8;
		// mark mission goals
		static ISL::DynamicArray<I3D::TerrainLine *> Lines;
		Lines.ClearPtr();
		for(i = 0; i < MLib.Missions[Mission]->Goals.Count(); ++i)
		{
			State = 9;
			if(MLib.Missions[Mission]->Goals[i]->Type == MLIB_GOAL_POS)
			{
				State = 10;
				Lines.Add(new I3D::TerrainLine());
				I3D::TerrainLine::TerrainLineSpecs Specs;
				Specs.DS = &DS;
				Specs.Color = 0xFF0000FF;
				Specs.TI = Wrld.GetTerrainInterface();
				Specs.TLR = Wrld.GetTerrainLineRenderer();
				Specs.Pos[0] = IMATH::Vector2DXZ(MLib.Missions[Mission]->Goals[i]->x1, MLib.Missions[Mission]->Goals[i]->z1);
				Specs.Pos[1] = IMATH::Vector2DXZ(MLib.Missions[Mission]->Goals[i]->x2, MLib.Missions[Mission]->Goals[i]->z1);
				State = 11;
				Lines.Last()->Create(&Specs);
				State = 12;
				Specs.Pos[0] = IMATH::Vector2DXZ(MLib.Missions[Mission]->Goals[i]->x2, MLib.Missions[Mission]->Goals[i]->z1);
				Specs.Pos[1] = IMATH::Vector2DXZ(MLib.Missions[Mission]->Goals[i]->x2, MLib.Missions[Mission]->Goals[i]->z2);
				Lines.Add(new I3D::TerrainLine());
				Lines.Last()->Create(&Specs);
				Specs.Pos[0] = IMATH::Vector2DXZ(MLib.Missions[Mission]->Goals[i]->x2, MLib.Missions[Mission]->Goals[i]->z2);
				Specs.Pos[1] = IMATH::Vector2DXZ(MLib.Missions[Mission]->Goals[i]->x1, MLib.Missions[Mission]->Goals[i]->z2);
				Lines.Add(new I3D::TerrainLine());
				Lines.Last()->Create(&Specs);
				Specs.Pos[0] = IMATH::Vector2DXZ(MLib.Missions[Mission]->Goals[i]->x1, MLib.Missions[Mission]->Goals[i]->z2);
				Specs.Pos[1] = IMATH::Vector2DXZ(MLib.Missions[Mission]->Goals[i]->x1, MLib.Missions[Mission]->Goals[i]->z1);
				Lines.Add(new I3D::TerrainLine());
				Lines.Last()->Create(&Specs);
				State = 13;
			}
		}
		State = 14;
		Brush.SetNextID(MLib.Missions[Mission]->NextID);
		Brush.SetMission(Mission);
		State = 141;
		CreateWindowMenu(Wnd.GetHWnd());
		State = 142;
		//MissionState = GAME_MIS_STATE_ACTIVE;
	}
	catch(char *s)
	{
		throw(s);
	}
	catch(...)
	{
		char Str[1024];
		sprintf(Str, "MissionEditor::LoadMission#%d", State);
		throw(Str);
	}
}

void MissionEditor::RenderInfoBox(I3D::D3DStruct *DS)
{
	// vehicles
	Tank *Vehicle = Wrld.GetTankAtPos(IMATH::Vector2DXZ(Mouse.GetPos().x, Mouse.GetPos().y));
	if(Vehicle)
	{
		InfoBox.Clear();
		InfoBox << Vehicle->GetName() << '\n';
		InfoBox << TLib.Tanks[Vehicle->GetID()].Name << '\n';
		// pos
		InfoBox << "x: " << Vehicle->GetPos().x << '\n';
		InfoBox << "y: " << Vehicle->GetPos().y << '\n';
		InfoBox << "z: " << Vehicle->GetPos().z << '\n';
		// owner
		switch(Vehicle->GetOwner())
		{
		case 0: InfoBox << "owner: " << Vehicle->GetOwner() << " (neutral)" << '\n'; break;
		case 1: InfoBox << "owner: " << Vehicle->GetOwner() << " (beast)" << '\n'; break;
		case 2: InfoBox << "owner: " << Vehicle->GetOwner() << " (player)" << '\n'; break;
		case 3: InfoBox << "owner: " << Vehicle->GetOwner() << " (enemy)" << '\n'; break;
		}
		// goals
		for(int i = 0; i < MLib.Missions[Mission]->Goals.Count(); ++i)
		{
			MissionLib::Goal *Goal = MLib.Missions[Mission]->Goals[i];
			switch(Goal->Type)
			{
			case IGAME::MLIB_GOAL_DESTROY:
				if(Wrld.GetTank(Goal->Object) == Vehicle)
					InfoBox << "to be destroyed" << '\n';
				break;
			case IGAME::MLIB_GOAL_DESTROYALL:
				if(Goal->Owner == Vehicle->GetOwner())
					InfoBox << "to be destroyed\n  (because of owner)" << '\n';
				break;
			case IGAME::MLIB_GOAL_POS:
				if(Wrld.GetTank(Goal->Object) == Vehicle)
					InfoBox << "has a position goal" << '\n';
				break;
			case IGAME::MLIB_GOAL_OWNERPOS:
				if(Goal->Owner == Vehicle->GetOwner())
					InfoBox << "has a position goal\n  (because of owner)" << '\n';
				break;
			}
		}
		InfoBox.Render(DS, 0.58f, -0.42f, 0.4f, 0.8f);
	}
}

void MissionEditor::MainLoop(void)
{
	int State = 0;
	try
	{
		if(Reinitialize)
		{
			Reinitialize = false;
			Init3D(true);
		}
		State = 1;
		Sp.Update();
		State = 10;
		In.Update();
		State = 11;
		EventHandler();
		State = 12;
		MouseHandler();
		State = 13;
		Wrld.PhysicsPFX(&DS);
		State = 14;
		Wrld.ManageDeletionLists();
		State = 2;
		// mouse cursor
		switch(Wrld.GetMouseTarget(Mouse.GetPos()))
		{
		case IGAME::World::MOUSE_TARGET_NOTHING:
			Mouse.SetCursorTexture(MouseTextures[0]);
			break;
		case IGAME::World::MOUSE_TARGET_IMPASSABLE:
			Mouse.SetCursorTexture(MouseTextures[1]);
			break;
		case IGAME::World::MOUSE_TARGET_FRIEND:
			Mouse.SetCursorTexture(MouseTextures[2]);
			break;
		case IGAME::World::MOUSE_TARGET_ENEMY:
			Mouse.SetCursorTexture(MouseTextures[3]);
			break;
		}
		// render
		if(DS.D3DDevice->BeginScene())
		{
			if(D3DFlags & GAME_D3DFLAG_WIREFRAME || (StoryItem && !StoryItem->Render))
				DS.D3DDevice->Clear(0, NULL, D3DCLEAR_ZBUFFER | D3DCLEAR_TARGET, 0x00000000, 1.0f, 0);
			else
				DS.D3DDevice->Clear(0, NULL, D3DCLEAR_ZBUFFER | D3DCLEAR_TARGET, 0x0040B0FF, 1.0f, 0);
			D3DMATRIX Matrix = {1,0,0,0,  0,1,0,0,  0,0,1,0,  0,0,0,1};
			// world and view matrix
			DS.D3DDevice->SetTransform(D3DTS_WORLD, &Matrix);
			DS.D3DDevice->SetTransform(D3DTS_VIEW, &Matrix);
			DS.D3DDevice->SetTransform(D3DTS_PROJECTION, &Matrix);
			// projection matrix (perspective)
			// distance eye->monitor in meters
			const float EyeDistance = 0.5f;
			// (vertical) monitor size in meters
			const float MonitorSize = 0.33f;
			// minimum distance of objects in game world space (for z-buffer optimization)
			const float MinDistance = 30.0f;
			// perspective projection: [x, y, z] /= (1 + z)
			D3DMATRIX PMatrix = {1,0,0,0,  0,1,0,0,  0,0,1,1,  0,0,0,MinDistance};
			DS.D3DDevice->MultiplyTransform(D3DTS_PROJECTION, &PMatrix);
			// adjust to monitor size and position
			// screen 3:4  = 0.75f
			D3DMATRIX TMatrix3 = {0.75f * 2.0f / MonitorSize,0,0,0,  0,2.0f / MonitorSize,0,0,  0,0,1.0f / EyeDistance,0,  0,0,0,1};
			DS.D3DDevice->MultiplyTransform(D3DTS_PROJECTION, &TMatrix3);
			DS.D3DDevice->GetTransform(D3DTS_PROJECTION, &DS.ProjMtx);

			State = 3;
			const bool RenderWorld = !StoryItem || StoryItem->Render;
			//QTree.Render(&DS);
			State = 4;
			if(RenderWorld)
			{
				Wrld.Render(&DS);
				State = 5;
				Brush.Render(&DS);
				State = 51;
			}
			State = 52;
			CheckStory();
			State = 53;
			if(StoryItem)
			{
				State = 54;
				// destroy light
				Wrld.DestroyLight(&DS);
				State = 55;
				StoryRnd.Render(&DS);
				// reset
				Wrld.InitLight(&DS);
				if(D3DFlags & GAME_D3DFLAG_WIREFRAME)
					DS.D3DDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME);
				State = 56;
			}
			State = 57;

			static float LastSpawn = DS.Sp->GameTime;
			// spawn pfx at mouse pos
			/*if(DS.Sp->GameTime > LastSpawn + 0.1f)
			{
				SpawnInterface::SpawnInterfaceSpecs SS;
				SS.Type = IDTYPE_PARTICLEFX;
				SS.ID = IDPFX_SMALLRISINGSMOKE;
				IMATH::Vector2D Pos = Mouse.GetPos();
				IMATH::Vector2DXZ XZPos = Wrld.GetSurfaceXZ(Pos);
				SS.Pos = IMATH::Vector(XZPos.x, Wrld.GetSurfaceY(XZPos.x, XZPos.z), XZPos.z);
				Wrld.GetSpawnInterface()->Spawn(&SS);
				LastSpawn = DS.Sp->GameTime;
			}*/

			// spawn debug pfx
			if(DS.Sp->GameTime > LastSpawn + 0.1f)
			{
				DebugPos::ArrayType &Array = DebugPos::GetPosArray();
				SpawnInterface::SpawnInterfaceSpecs SS;
				SS.Type = IDTYPE_PARTICLEFX;
				SS.ID = IDPFX_SMALLRISINGSMOKE;
				for(int i = 0; i < Array.Count(); ++i)
				{
					SS.Pos = IMATH::Vector(Array[i].x, Wrld.GetSurfaceY(Array[i].x, Array[i].z), Array[i].z);
					Wrld.GetSpawnInterface()->Spawn(&SS);
				}
				LastSpawn = DS.Sp->GameTime;
				//DebugPos::Clear();
			}

			// overlay
			Menu.Render(&DS);
			DisplayFPS();
			State = 58;
			MaximizeButton.Render(&DS);
			State = 59;
			RenderInfoBox(&DS);
			State = 60;
			Mouse.Render(&DS);
			State = 6;

			DS.D3DDevice->EndScene();

			DS.D3DDevice->Present(0, 0, 0, 0);
			State = 7;
		}
	}
	catch(char *s)
	{
		throw(s);
	}
	catch(...)
	{
		char Str[1024];
		sprintf(Str, "MissionEditor::MainLoop#%d", State);
		throw(Str);
	}
}

std::ostream &operator << (std::ostream &os, const GenMap &Map)
{
	os << "Map:" << std::endl;
	for(int i = 0; i < Map.Height / MAP_RES; i++)
	{
		for(int j = 0; j < Map.Width / MAP_RES; j++)
		{
			int Value = Map.Get((int)Map.Pos[0].x + i * MAP_RES, (int)Map.Pos[1].z + j * MAP_RES);
			if(Value >= '0' && Value <= '9')
				os << Value;
			switch(Value)
			{
			case World::NOTHING:
				os << ' ';
				break;
			case World::WAY:
				os << '+';
				break;
			case World::STREET_MASK:
				os << '#';
				break;
			case World::TERRAIN:
				os << 'X';
				break;
			case World::VEHICLE:
				os << 'o';
				break;
			case World::DECAL:
				os << 'D';
				break;
			case World::AIRFIELD:
				os << 'A';
				break;
			}
		}
		os << std::endl;
	}
	os << std::endl;
	return os;
}

}
