/***********************************************************/
/*                                                         */
/* MissionLib.h                                            */
/*                                                         */
/* created 01.03.2003                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "MissionLib.h"
#include "IDType.h"
#include "IDMissionGoal.h"
#include "IDSide.h"

#include <sstream>
#include <stdio.h>
#include <windows.h>
#include <time.h>

#include <fstream>
using namespace std;

// www.infinitecode.com game library
namespace IGAME
{

const char *MissionLib::StoryItem::IniToPrintable(const char *Str)
{
	static char *Buf = 0;
	delete[] Buf;
	Buf = new char[3 * strlen(Str) + 20];
    const unsigned char *i = (const unsigned char *)Str;
	unsigned char *j = (unsigned char *)Buf;
	while(*i)
	{
		if(*i >= 0x80 && *i < 0x90)
		{
			*j++ = '\\';
			*j++ = *i / 0x10 + ((*i / 0x10 < 0xa) ? 48 : 87);
			*j++ = *i % 0x10 + ((*i % 0x10 < 0xa) ? 48 : 87);
		}
		else if(*i == '\n')
		{
			*j++ = '\r';
			*j++ = '\n';
		}
		else if(*i == '\r') {}
		else
			*j++ = *i;
		++i;
	}
	*j = 0;
	return Buf;
}

const char *MissionLib::StoryItem::PrintableToIni(const char *Str)
{
	static char *Buf = 0;
	delete[] Buf;
	Buf = new char[strlen(Str) + 1];
    const unsigned char *i = (const unsigned char *)Str;
	unsigned char *j = (unsigned char *)Buf;
	while(*i)
	{
		if(*i == '\r') {}
		else if(*i == '\\' && *(i + 1) == '\\')
		{
			*j++ = '\\';
			++i;
		}
		else if(*i == '\\' && *(i + 1) && *(i + 2))
		{
			if(*(i + 1) < 58)
				*j = *(i + 1) - 48;
			else if(*(i + 1) < 97)
				*j = *(i + 1) - 55;
			else
				*j = *(i + 1) - 87;
			*j <<= 4;
			if(*(i + 2) < 58)
				*j += *(i + 2) - 48;
			else if(*(i + 2) < 97)
				*j += *(i + 2) - 55;
			else
				*j += *(i + 2) - 87;
			++j;
			i += 2;
		}
		else
			*j++ = *i;
		++i;
	}
	*j = 0;
	return Buf;
}

int MissionLib::Mission::GetStoryNum(const char *Name)
{
	for(int i = 0; i < StoryItems.Count(); i++)
	{
		if(!strcmp(StoryItems[i]->Name, Name))
			return i;
	}
	return -1;
}

MissionLib::Goal *MissionLib::Mission::GetGoal(const char *Name)
{
	for(int i = 0; i < Goals.Count(); i++)
		if(!strcmp(Goals[i]->Name, Name))
			return Goals[i];
	return 0;
}

MissionLib::Mission::~Mission(void)
{
	int j;
	for(j = 0; j < Goals.Count(); j++)
		delete Goals[j];
	for(j = 0; j < StoryItems.Count(); j++)
		delete StoryItems[j];
	for(j = 0; j < Airfields.Count(); j++)
		delete Airfields[j];
	Goals.Clear();
	StoryItems.Clear();
}

void MissionLib::Mission::Save(const char *Directory)
{
	// get time
	time_t Time;
	time(&Time);
	struct tm *LocalTime = localtime(&Time);
	// save
	char AbsFilename[1024];
	sprintf(AbsFilename, "%s\\%s", Directory, FileName);
		/*ofstream d("debug_save", ios::app);
		d << "ml::save" << FileName << ' ' << TerrainDscs.Count() << endl;*/
	ofstream f(AbsFilename);
	f << "; Mission File" << endl << ";" << endl;
	f << "; created by mission editor" << endl;
	f << "; save date: " << asctime(LocalTime);
	f << "; filename:  " << FileName << endl;
	// properties
	f << endl;
	f << "[Properties]" << endl;
	f << "Name = \"" << Name << "\"" << endl;
	f << "Next = \"" << Next << "\"" << endl;
	f << "NextID = " << NextID << endl;
	f << "Seed = " << Seed << endl;
	f << "Side = " << ((Side == SIDE_ISRAEL) ? "Israel" : "Palestine") << endl;
	f << "Orders = \"" << Orders << '\"' << endl;
	f << "InitialCameraPosX = " << InitialCameraPos.x << endl;
	f << "InitialCameraPosY = " << InitialCameraPos.y << endl;
	f << "InitialCameraPosZ = " << InitialCameraPos.z << endl;
	f << "InitialCameraAngleX = " << InitialCameraAngle.x << endl;
	f << "InitialCameraAngleY = " << InitialCameraAngle.y << endl;
	f << "InitialCameraAngleZ = " << InitialCameraAngle.z << endl;
	int i;
	// objects
	for(i = 0 ; i < Objects.Count(); i++)
	{
		f << endl;
		f << "[" << Objects[i].Name << "]" << endl;
		f << "GroupType = \"Object\"" << endl;
		f << "ObjectType = \"Tank\"" << endl;
		f << "Type = " << Objects[i].Type << endl;
		f << "x = " << Objects[i].x << endl;
		f << "z = " << Objects[i].z << endl;
		f << "Angle = " << Objects[i].Angle << endl;
		f << "Owner = " << Objects[i].Owner << endl;
		f << "AI = " << Objects[i].AI << endl;
		f << "TargetX = " << Objects[i].TargetX << endl;
		f << "TargetZ = " << Objects[i].TargetZ << endl;
		f << "Target = \"" << Objects[i].Target << "\"" << endl;
		f << "Commandable = \"" << (Objects[i].Commandable ? "true" : "false") << "\"" << endl;
		f << "Highlight = \"" << (Objects[i].Highlight ? "true" : "false") << "\"" << endl;
		f << "MaxSpeed = " << Objects[i].MaxSpeed << endl;
	}
	// ways
	for(i = 0; i < Ways.Count(); i++)
	{
		f << endl;
		f << "[" << Ways[i].Specs.Name << "]" << endl;
		f << "GroupType = \"Way\"" << endl;
		f << "ID = " << Ways[i].Specs.ID << endl;
		f << "Pos1X = " << Ways[i].Specs.Pos1.x << endl;
		f << "Pos1Z = " << Ways[i].Specs.Pos1.z << endl;
		f << "Pos2X = " << Ways[i].Specs.Pos2.x << endl;
		f << "Pos2Z = " << Ways[i].Specs.Pos2.z << endl;
	}
	// airfields
	for(i = 0; i < Airfields.Count(); i++)
	{
		f << endl;
		f << "[" << Airfields[i]->GetName() << "]" << endl;
		f << "GroupType = \"Airfield\"" << endl;
		f << "PosX = " << Airfields[i]->GetPos().x << endl;
		f << "PosZ = " << Airfields[i]->GetPos().z << endl;
	}
	// decals
	for(i = 0; i < Decals.Count(); i++)
	{
		f << endl;
		f << "[" << Decals[i]->GetName() << "]" << endl;
		f << "GroupType = \"Decal\"" << endl;
		f << "PosX = " << Decals[i]->GetPos().x << endl;
		f << "PosZ = " << Decals[i]->GetPos().z << endl;
		f << "ID = " << Decals[i]->GetID() << endl;
	}
	// TerrainDscs
	for(i = 0; i < TerrainDscs.Count(); i++)
	{
		f << endl;
		f << "[" << TerrainDscs[i].Specs.Name << "]" << endl;
		f << "GroupType = \"TerrainDsc\"" << endl;
		f << "ID = " << TerrainDscs[i].Specs.ID << endl;
		f << "Pos1X = " << TerrainDscs[i].Specs.Pos[0].x << endl;
		f << "Pos1Z = " << TerrainDscs[i].Specs.Pos[0].z << endl;
		f << "Pos2X = " << TerrainDscs[i].Specs.Pos[1].x << endl;
		f << "Pos2Z = " << TerrainDscs[i].Specs.Pos[1].z << endl;
		f << "Pos3X = " << TerrainDscs[i].Specs.Pos[2].x << endl;
		f << "Pos3Z = " << TerrainDscs[i].Specs.Pos[2].z << endl;
	}
	// events
	for(i = 0 ; i < Events.Count(); i++)
	{
		f << endl;
		f << "[" << Events[i]->Name << "]" << endl;
		f << "GroupType = \"Event\"" << endl;
		f << "Trigger = \"" << Events[i]->Trigger << "\"" << endl;
		f << "Action = \"" << Events[i]->Action << "\"" << endl;
	}
	// goals
	for(i = 0 ; i < Goals.Count(); i++)
	{
		f << endl;
		f << "[" << Goals[i]->Name << "]" << endl;
		f << "GroupType = \"Goal\"" << endl;
		if(*Goals[i]->Object)
			f << "Object = \"" << Goals[i]->Object << "\"" << endl;
		f << "GoalType = ";
		switch(Goals[i]->Type)
		{
		case MLIB_GOAL_DESTROY:
			f << "\"destroy\"";
			break;
		case MLIB_GOAL_AND:
			f << "\"and\"";
			break;
		case MLIB_GOAL_OR:
			f << "\"or\"";
			break;
		case MLIB_GOAL_DESTROYALL:
			f << "\"destroyall\"";
			break;
		case MLIB_GOAL_POS:
			f << "\"pos\"";
			break;
		case MLIB_GOAL_STORY:
			f << "\"story\"";
			break;
		case MLIB_GOAL_TRUE:
			f << "\"true\"";
			break;
		case MLIB_GOAL_FALSE:
			f << "\"false\"";
			break;
		case MLIB_GOAL_TIME:
			f << "\"time\"";
			break;
		}
		f << endl;
		f << "Owner = " << Goals[i]->Owner << endl;
		f << "x1 = " << Goals[i]->x1 << endl;
		f << "x2 = " << Goals[i]->x2 << endl;
		f << "z1 = " << Goals[i]->z1 << endl;
		f << "z2 = " << Goals[i]->z2 << endl;
		f << "Value = " << Goals[i]->Value << endl;
		for(int j = 0; j < Goals[i]->Params.Count(); j++)
		{
			f << "Param" << j + 1 << " = \"" << Goals[i]->Params[j]->Name << "\"" << endl;
		}
	}
	// story items
	for(i = 0 ; i < StoryItems.Count(); i++)
	{
		f << endl;
		f << "[" << StoryItems[i]->Name << "]" << endl;
		f << "GroupType = \"StoryItem\"" << endl;
		f << "Render = " << (StoryItems[i]->Render?"true":"false") << endl;
		f << "BumperID = " << StoryItems[i]->BumperID << endl;
		f << "NewsTicker = \"" << StoryItems[i]->NewsTicker << "\"" << endl;
		f << "TickerStyle = " << StoryItems[i]->TickerStyle << endl;
		f << "Topic = \"" << StoryItems[i]->Topic << "\"" << endl;
		f << "Text = \"" << StoryItems[i]->Text << "\"" << endl;
		f << "Next = \"" << StoryItems[i]->Next << "\"" << endl;
		f << "TimeDelta = " << StoryItems[i]->TimeDelta << endl;
		f << "CameraX = " << StoryItems[i]->Camera.x << endl;
		f << "CameraY = " << StoryItems[i]->Camera.y << endl;
		f << "CameraZ = " << StoryItems[i]->Camera.z << endl;
		f << "CameraAngleX = " << StoryItems[i]->CameraAngle.x << endl;
		f << "CameraAngleY = " << StoryItems[i]->CameraAngle.y << endl;
		f << "CameraAngleZ = " << StoryItems[i]->CameraAngle.z << endl;
	}
}

MissionObject *MissionLib::Mission::GetMissionObject(const char *Name)
{
	for(int i = 0; i < Objects.Count(); ++i)
		if(!strcmp(Name, Objects[i].Name))
			return &Objects[i];
	return 0;
}

I3D::Way *MissionLib::Mission::GetWay(const char *Name)
{
	for(int i = 0; i < Ways.Count(); ++i)
		if(!strcmp(Name, Ways[i].Specs.Name))
			return &Ways[i];
	return 0;
}

I3D::TerrainDsc *MissionLib::Mission::GetTerrainDsc(const char *Name)
{
	for(int i = 0; i < TerrainDscs.Count(); ++i)
		if(!strcmp(Name, TerrainDscs[i].Specs.Name))
			return &TerrainDscs[i];
	return 0;
}

void MissionLib::Mission::AddObject(MissionLib::Mission::MissionObjectSpecs *MOS)
{
	MissionObject MO;
	MO.ObjType = MOS->Type;
	MO.Type = MOS->ID;
	MO.Owner = MOS->Owner;
	MO.x = MOS->Pos.x;
	MO.z = MOS->Pos.z;
	MO.Angle = MOS->Angle;
	strcpy(MO.Name, MOS->Name);
	Objects.Add(MO);
}

void MissionLib::Mission::UpdateObject(const char *Name, MissionLib::Mission::MissionObjectSpecs *MOS)
{
	for(int i = 0; i < Objects.Count(); ++i)
		if(!strcmp(Objects[i].Name, Name))
		{
			Objects[i].Angle = MOS->Angle;
			Objects[i].ObjType = MOS->Type;
			Objects[i].Owner = MOS->Owner;
			strcpy(Objects[i].Name, MOS->Name);
			Objects[i].x = MOS->Pos.x;
			Objects[i].z = MOS->Pos.z;
			Objects[i].Type = MOS->ID;
		}
}

void MissionLib::Mission::AddWay(I3D::Way::WaySpecs *WS)
{
	Ways.Add(I3D::Way(WS));
}

void MissionLib::Mission::AddAirfield(Airfield *AF)
{
	Airfields.Add(AF);
}

void MissionLib::Mission::AddDecal(Decal *D)
{
	Decals.Add(D);
}

void MissionLib::Mission::AddTerrainDsc(I3D::TerrainDsc::TerrainDscSpecs *TS)
{
	/*ofstream d("debug_save", ios::app);
	d << "ml::addterraindsc" << FileName << endl;*/
	TerrainDscs.Add(I3D::TerrainDsc(TS));
}

MissionLib::Event *MissionLib::Mission::GetEventComplete()
{
	Event *EvComplete = 0;
	for(int i = 0; i < Events.Count(); ++i)
	{
		if(!stricmp(Events[i]->Action, "MissionComplete") || !stricmp(Events[i]->Action, "NextMission"))
		{
			EvComplete = Events[i];
		}
	}
	return EvComplete;
}

MissionLib::Event *MissionLib::Mission::GetEventFailed()
{
	Event *EvFailed;
	for(int i = 0; i < Events.Count(); ++i)
	{
		if(!stricmp(Events[i]->Action, "MissionFailed"))
		{
			EvFailed = Events[i];
		}
	}
	return EvFailed;
}

MissionLib::Goal *MissionLib::Mission::GetGoalComplete(MissionLib::Event *EvComplete)
{
	char Complete[1024];
	strcpy(Complete, EvComplete->Trigger);
	Goal *GoalComplete = 0;
	for(int i = 0; i < Goals.Count(); ++i)
	{
		if(!stricmp(Goals[i]->Name, Complete))
			GoalComplete = Goals[i];
	}
	return GoalComplete;
}

MissionLib::Goal *MissionLib::Mission::GetGoalFailed(MissionLib::Event *EvFailed)
{
	char Failed[1024];
	strcpy(Failed, EvFailed->Trigger);
	Goal *GoalFailed = 0;
	for(int i = 0; i < Goals.Count(); ++i)
	{
		if(!stricmp(Goals[i]->Name, Failed))
			GoalFailed = Goals[i];
	}
	return GoalFailed;
}

MissionLib::Goal *MissionLib::Mission::FindGoal(Goal *Tree, bool Conjunctions, int Type, const char *Tank)
{
	// translate types
	int Owner = -1, GoalType = -1;
	if(Type == MG_DESTROYALLPLAYER)
	{
		GoalType  = MLIB_GOAL_DESTROYALL;
		Owner = 2;
	}
	else if(Type == MG_DESTROYALLENEMY)
	{
		GoalType  = MLIB_GOAL_DESTROYALL;
		Owner = 3;
	}
	else if(Type != 0)
		return 0;
	ofstream log("goals.txt", ios::app);
	log << "searching: " << GoalType << " " << Owner << ", seeing: " << Tree->Type << " " << Tree->Owner << std::endl;
	if(Tree->Type == GoalType && Tree->Owner == Owner)
	{
		log << "returning" << std::endl;
		return Tree;
	}
	if(Type == 0 && !stricmp(Tree->Object, Tank))
		return Tree;
	if(Tree->Type == (Conjunctions ? MLIB_GOAL_AND : MLIB_GOAL_OR))
	{
		for(int i = 0; i < Tree->Params.Count(); ++i)
		{
			Goal *G = FindGoal(Tree->Params[i], Conjunctions, Type, Tank);
			if(G)
				return G;
		}
	}
	return 0;
}

void MissionLib::Mission::RemoveMissionGoal(int Type, const char *Tank)
{
	Event *EvComplete = GetEventComplete();
	Event *EvFailed = GetEventFailed();
	Goal *GoalComplete = GetGoalComplete(EvComplete);
	Goal *GoalFailed = GetGoalFailed(EvFailed);
	if(!GoalComplete || !GoalFailed)
		return;
	Goal  *G;
	while(Type & MG_PLAYER && (G = FindGoal(GoalComplete, true, Type & MG_TYPES_MASK, Tank)) != 0)
	{
		G->Type = MLIB_GOAL_TRUE;
	}
	while(Type & MG_ENEMY && (G = FindGoal(GoalFailed, false, Type & MG_TYPES_MASK, Tank)) != 0)
	{
		G->Type = MLIB_GOAL_FALSE;
	}
	while(Type == 0 && (G = FindGoal(GoalComplete, true, 0, Tank)) != 0)
	{
		G->Type = MLIB_GOAL_TRUE;
		*G->Object = 0;
	}
	while(Type == 0 && (G = FindGoal(GoalFailed, false, 0, Tank)) != 0)
	{
		G->Type = MLIB_GOAL_FALSE;
		*G->Object = 0;
	}
}

bool MissionLib::Mission::TestMissionGoal(int Type, const char *Tank)
{
	Event *EvComplete = GetEventComplete();
	Event *EvFailed = GetEventFailed();
	Goal *GoalComplete = GetGoalComplete(EvComplete);
	Goal *GoalFailed = GetGoalFailed(EvFailed);
	if(!GoalComplete || !GoalFailed)
		return false;
	if(Type & MG_PLAYER && FindGoal(GoalComplete, true, Type & MG_TYPES_MASK, Tank))
	{
		ofstream log("goals.txt", ios::app);
		log << "returning true" << std::endl;
		return true;
	}
	if(Type & MG_ENEMY && FindGoal(GoalFailed, false, Type & MG_TYPES_MASK, Tank))
	{
		ofstream log("goals.txt", ios::app);
		log << "returning true" << std::endl;
		return true;
	}
	ofstream log("goals.txt", ios::app);
	log << "returning false" << std::endl;
	return false;
}

void MissionLib::Mission::AddMissionGoal(int Type, const char *Tank, const IMATH::Vector2DXZ *Area)
{
	Event *EvComplete = GetEventComplete();
	Event *EvFailed = GetEventFailed();
	Goal *GoalComplete = GetGoalComplete(EvComplete);
	Goal *GoalFailed = GetGoalFailed(EvFailed);
	if(!GoalComplete || !GoalFailed)
		return;
	Goal *NewGoal = new Goal();
	std::ostringstream name;
	name << "Mission Editor Goal " << NextID++;
	// goal
	if(Type & MG_DESTROY)
	{
		NewGoal->Type = MLIB_GOAL_DESTROY;
		strcpy(NewGoal->Object, Tank);
	}
	if(Type & MG_DESTROYALLPLAYER)
	{
		NewGoal->Type = MLIB_GOAL_DESTROYALL;
		NewGoal->Owner = 2;
	}
	if(Type & MG_DESTROYALLENEMY)
	{
		NewGoal->Type = MLIB_GOAL_DESTROYALL;
		NewGoal->Owner = 3;
	}
	if(Type & MG_MOVETO)
	{
		NewGoal->Type = MLIB_GOAL_POS;
		NewGoal->x1 = Area[0].x;
		NewGoal->z1 = Area[0].z;
		NewGoal->x2 = Area[1].x;
		NewGoal->z2 = Area[1].z;
		strcpy(NewGoal->Object, Tank);
	}
	if(Type & MG_ANYPLAYERMOVETO)
	{
		NewGoal->Type = MLIB_GOAL_OWNERPOS;
		NewGoal->Owner = 2;
		NewGoal->x1 = Area[0].x;
		NewGoal->z1 = Area[0].z;
		NewGoal->x2 = Area[1].x;
		NewGoal->z2 = Area[1].z;
	}
	if(Type & MG_ANYENEMYMOVETO)
	{
		NewGoal->Type = MLIB_GOAL_OWNERPOS;
		NewGoal->Owner = 3;
		NewGoal->x1 = Area[0].x;
		NewGoal->z1 = Area[0].z;
		NewGoal->x2 = Area[1].x;
		NewGoal->z2 = Area[1].z;
	}
	strcpy(NewGoal->Name, name.str().c_str());
    Goals.Add(NewGoal);
	Goal *Junction = new Goal();
	Junction->Type = MLIB_GOAL_AND;
	if(Type & MG_ENEMY)
		Junction->Type = MLIB_GOAL_OR;
	std::ostringstream name2;
	name2 << "Mission Editor Goal " << NextID++;
	strcpy(Junction->Name, name2.str().c_str());
	Junction->Params[0] = NewGoal;
	Junction->Params[1] = ((Type & MG_ENEMY) ? GoalFailed : GoalComplete);
    Goals.Add(Junction);
	MainGoal = Junction;
	if(Type & MG_ENEMY)
		strcpy(EvFailed->Trigger, Junction->Name);
	else
		strcpy(EvComplete->Trigger, Junction->Name);
}

void MissionLib::Mission::DeleteObject(const char *Name)
{
	for(int i = 0; i < Objects.Count(); i++)
	{
		if(!stricmp(Objects[i].Name, Name))
			Objects.Delete(i);
	}
}

void MissionLib::Create(const char *Directory)
{
	int State = 0;
	try
	{
		State = 1;
		// load Missions.ini
		char FileName[1024];
		sprintf(FileName, "%s\\Missions.ini", Directory);
		ITL::IniLoader Mini(FileName);
		strcpy(FirstMissionPA, Mini.GetValue("Game", "FirstMissionPA"));
		strcpy(FirstMissionIS, Mini.GetValue("Game", "FirstMissionIS"));
		sprintf(FileName, "Missions\\*.ini", Directory);
		// get mission files
		WIN32_FIND_DATA FindData;
		HANDLE Handle;
		Handle = FindFirstFile(FileName, &FindData);
		BOOL Succeeded = (Handle != INVALID_HANDLE_VALUE);
		while(Succeeded)
		{
			State = 2;
			if(stricmp(FindData.cFileName, "Missions.ini"))
			{
				Mission *Mis = new Mission();
				strcpy(Mis->FileName, FindData.cFileName);
				sprintf(FileName, "%s\\%s", Directory, FindData.cFileName);
				ITL::IniLoader MData(FileName);
				int i;
				State = 3;
				for(i = 0; i < MData.CountGroups(); i++)
				{
					const char *Group = MData.GetGroupName(i);
					// Properties
					if(!stricmp(Group, "Properties"))
					{
						State = 4;
						for(int j = 0; j < MData.CountKeys(i); j++)
						{
							const char *Key = MData.GetKey(i, j);
							// Name
							if(!stricmp(Key, "Name"))
								strcpy(Mis->Name, MData.GetValue(i, j));
							if(!stricmp(Key, "Next"))
								strcpy(Mis->Next, MData.GetValue(i, j));
							if(!stricmp(Key, "NextID"))
								Mis->NextID = atoi(MData.GetValue(i, j));
							if(!stricmp(Key, "Seed"))
								Mis->Seed = atoi(MData.GetValue(i, j));
							if(!stricmp(Key, "Orders"))
								strcpy(Mis->Orders, MData.GetValue(i, j));
							if(!stricmp(Key, "Side"))
							{
								if(!stricmp(MData.GetValue(i, j), "Israel"))
									Mis->Side = SIDE_ISRAEL;
								else
									Mis->Side = SIDE_PALESTINE;
							}
							// initial camera pos
							if(!stricmp(Key, "InitialCameraPosX"))
								Mis->InitialCameraPos.x = (float)atof(MData.GetValue(i, j));
							if(!stricmp(Key, "InitialCameraPosY"))
								Mis->InitialCameraPos.y = (float)atof(MData.GetValue(i, j));
							if(!stricmp(Key, "InitialCameraPosZ"))
								Mis->InitialCameraPos.z = (float)atof(MData.GetValue(i, j));
							if(!stricmp(Key, "InitialCameraAngleX"))
								Mis->InitialCameraAngle.x = (float)atof(MData.GetValue(i, j));
							if(!stricmp(Key, "InitialCameraAngleY"))
								Mis->InitialCameraAngle.y = (float)atof(MData.GetValue(i, j));
							if(!stricmp(Key, "InitialCameraAngleZ"))
								Mis->InitialCameraAngle.z = (float)atof(MData.GetValue(i, j));
						}
					}
					// Object
					if(MData.GetValue(i, "GroupType") && !stricmp(MData.GetValue(i, "GroupType"), "Object"))
					{
						State = 5;
						MissionObject Obj;
						strcpy(Obj.Name, Group);
						for(int j = 0; j < MData.CountKeys(i); j++)
						{
							const char *Key = MData.GetKey(i, j);
							const char *Value = MData.GetValue(i, j);
							if(!stricmp(Key, "ObjectType"))
							{
								if(!stricmp(Value, "Tank"))
									Obj.ObjType = IDTYPE_TANK;
							}
							if(!stricmp(Key, "Type"))
								Obj.Type = atoi(Value);
							if(!stricmp(Key, "x"))
								Obj.x = (float)atof(Value);
							if(!stricmp(Key, "z"))
								Obj.z = (float)atof(Value);
							if(!stricmp(Key, "Angle"))
								Obj.Angle = (float)atof(Value);
							if(!stricmp(Key, "Owner"))
								Obj.Owner = atoi(Value);
							if(!stricmp(Key, "AI"))
								Obj.AI = atoi(Value);
							if(!stricmp(Key, "TargetX"))
								Obj.TargetX = (float)atof(Value);
							if(!stricmp(Key, "TargetZ"))
								Obj.TargetZ = (float)atof(Value);
							if(!stricmp(Key, "Target"))
								strcpy(Obj.Target, Value);
							if(!stricmp(Key, "Commandable"))
								Obj.Commandable = !stricmp(Value, "true");
							if(!stricmp(Key, "Highlight"))
								Obj.Highlight = !stricmp(Value, "true");
							if(!stricmp(Key, "MaxSpeed"))
								Obj.MaxSpeed = (float)atof(Value);
						}
						Mis->Objects.Add(Obj);
					}
					// Way
					if(MData.GetValue(i, "GroupType") && !stricmp(MData.GetValue(i, "GroupType"), "Way"))
					{
						State = 6;
						I3D::Way Way;
						strcpy(Way.Specs.Name, Group);
						for(int j = 0; j < MData.CountKeys(i);j++)
						{
							const char *Key = MData.GetKey(i, j);
							const char *Value = MData.GetValue(i, j);
							if(!stricmp(Key, "ID"))
								Way.Specs.ID = atoi(Value);
							if(!stricmp(Key, "Pos1X"))
								Way.Specs.Pos1.x = (float)atof(Value);
							if(!stricmp(Key, "Pos1Z"))
								Way.Specs.Pos1.z = (float)atof(Value);
							if(!stricmp(Key, "Pos2X"))
								Way.Specs.Pos2.x = (float)atof(Value);
							if(!stricmp(Key, "Pos2Z"))
								Way.Specs.Pos2.z = (float)atof(Value);
						}
						Mis->Ways.Add(Way);
					}
					// Airfields
					if(MData.GetValue(i, "GroupType") && !stricmp(MData.GetValue(i, "GroupType"), "Airfield"))
					{
						State = 7;
						IGAME::Airfield *AF = new Airfield;
						AF->SetName(Group);
						for(int j = 0; j < MData.CountKeys(i);j++)
						{
							const char *Key = MData.GetKey(i, j);
							const char *Value = MData.GetValue(i, j);
							if(!stricmp(Key, "PosX"))
								AF->SetPosX((float)atof(Value));
							if(!stricmp(Key, "PosZ"))
								AF->SetPosZ((float)atof(Value));
						}
						Mis->Airfields.Add(AF);
					}
					// Decals
					if(MData.GetValue(i, "GroupType") && !stricmp(MData.GetValue(i, "GroupType"), "Decal"))
					{
						State = 8;
						float x, z;
						int id = 0;
						for(int j = 0; j < MData.CountKeys(i); j++)
						{
							const char *Key = MData.GetKey(i, j);
							const char *Value = MData.GetValue(i, j);
							if(!stricmp(Key, "PosX"))
								x = (float)atof(Value);
							if(!stricmp(Key, "PosZ"))
								z = (float)atof(Value);
							if(!stricmp(Key, "id"))
								id = atoi(Value);
						}
						IGAME::Decal::DecalSpecs Sp;
						strcpy(Sp.Name, Group);
						Sp.Pos.x = x;
						Sp.Pos.z = z;
						Sp.ID = id;
						IGAME::Decal *D = new Decal(Sp);
						Mis->Decals.Add(D);
					}
					// TerrainDsc
					if(MData.GetValue(i, "GroupType") && !stricmp(MData.GetValue(i, "GroupType"), "TerrainDsc"))
					{
						State = 9;
						I3D::TerrainDsc TerrainDsc;
						strcpy(TerrainDsc.Specs.Name, Group);
						for(int j = 0; j < MData.CountKeys(i);j++)
						{
							const char *Key = MData.GetKey(i, j);
							const char *Value = MData.GetValue(i, j);
							if(!stricmp(Key, "ID"))
								TerrainDsc.Specs.ID = atoi(Value);
							if(!stricmp(Key, "Pos1X"))
								TerrainDsc.Specs.Pos[0].x = (float)atof(Value);
							if(!stricmp(Key, "Pos1Z"))
								TerrainDsc.Specs.Pos[0].z = (float)atof(Value);
							if(!stricmp(Key, "Pos2X"))
								TerrainDsc.Specs.Pos[1].x = (float)atof(Value);
							if(!stricmp(Key, "Pos2Z"))
								TerrainDsc.Specs.Pos[1].z = (float)atof(Value);
							if(!stricmp(Key, "Pos3X"))
								TerrainDsc.Specs.Pos[2].x = (float)atof(Value);
							if(!stricmp(Key, "Pos3Z"))
								TerrainDsc.Specs.Pos[2].z = (float)atof(Value);
						}
						Mis->TerrainDscs.Add(TerrainDsc);
					}
					// Event
					if(MData.GetValue(i, "GroupType") && !stricmp(MData.GetValue(i, "GroupType"), "Event"))
					{
						State = 10;
						// create event
						Event *NewEvent = new Event();
						strcpy(NewEvent->Name, Group);
						// read data
						for(int j = 0; j < MData.CountKeys(i); j++)
						{
							const char *Key = MData.GetKey(i, j);
							const char *Value = MData.GetValue(i, j);
							if(!stricmp(Key, "Trigger"))
								strcpy(NewEvent->Trigger, Value);
							if(!stricmp(Key, "Action"))
								strcpy(NewEvent->Action, Value);
						}
						// add
						Mis->Events.Add(NewEvent);
					}
					// Goal
					if(MData.GetValue(i, "GroupType") && !stricmp(MData.GetValue(i, "GroupType"), "Goal"))
					{
						State = 11;
						// create goal
						Goal *NewGoal = new Goal();
						strcpy(NewGoal->Name, Group);
						// read data
						for(int j = 0; j < MData.CountKeys(i); j++)
						{
							const char *Key = MData.GetKey(i, j);
							const char *Value = MData.GetValue(i, j);
							if(!stricmp(Key, "GoalType"))
							{
								// read goal type
								if(!stricmp(Value, "destroy"))
									NewGoal->Type = MLIB_GOAL_DESTROY;
								if(!stricmp(Value, "and"))
									NewGoal->Type = MLIB_GOAL_AND;
								if(!stricmp(Value, "or"))
									NewGoal->Type = MLIB_GOAL_OR;
								if(!stricmp(Value, "destroyall"))
									NewGoal->Type = MLIB_GOAL_DESTROYALL;
								if(!stricmp(Value, "true"))
									NewGoal->Type = MLIB_GOAL_TRUE;
								if(!stricmp(Value, "false"))
									NewGoal->Type = MLIB_GOAL_FALSE;
								if(!stricmp(Value, "time"))
									NewGoal->Type = MLIB_GOAL_TIME;
								if(!stricmp(Value, "story"))
									NewGoal->Type = MLIB_GOAL_STORY;
								if(!stricmp(Value, "pos"))
									NewGoal->Type = MLIB_GOAL_POS;
							}
							if(!stricmp(Key, "Object"))
								strcpy(NewGoal->Object, Value);
							if(!stricmp(Key, "Owner"))
								NewGoal->Owner = atoi(Value);
							if(!stricmp(Key, "Value"))
								NewGoal->Value = (float)atof(Value);
							if(!stricmp(Key, "x1"))
								NewGoal->x1 = (float)atof(Value);
							if(!stricmp(Key, "x2"))
								NewGoal->x2 = (float)atof(Value);
							if(!stricmp(Key, "z1"))
								NewGoal->z1 = (float)atof(Value);
							if(!stricmp(Key, "z2"))
								NewGoal->z2 = (float)atof(Value);
						}
						// read parameters
						int Param = 1;
						char Str[1024];
						sprintf(Str, "Param%d", Param);
						while(MData.GetValue(i, Str))
						{
							NewGoal->ParamsTxt.Add((char *)MData.GetValue(i, Str)); // pointers are only valid in MissionLib::Create!
							Param++;
							sprintf(Str, "Param%d", Param);
						}
						// check
						if(NewGoal->Type == -1)
							throw("MissionLib::Create: no goal type found");
						// add
						Mis->Goals.Add(NewGoal);
						// main goal
						if(!stricmp(Group, "MainGoal"))
							Mis->MainGoal = NewGoal;
						// enemy goal
						if(!stricmp(Group, "MainGoalEnemy"))
							Mis->MainGoalEnemy = NewGoal;
					}
					// StoryItem
					if(MData.GetValue(i, "GroupType") && !stricmp(MData.GetValue(i, "GroupType"), "StoryItem"))
					{
						State = 12;
						// create StoryItem
						StoryItem *NewStoryItem = new StoryItem();
						strcpy(NewStoryItem->Name, Group);
						// read data
						for(int j = 0; j < MData.CountKeys(i); j++)
						{
							const char *Key = MData.GetKey(i, j);
							const char *Value = MData.GetValue(i, j);
							// int
							if(!stricmp(Key, "BumperID"))
								NewStoryItem->BumperID = atoi(Value);
							if(!stricmp(Key, "CameraX"))
								NewStoryItem->Camera.x = (float)atof(Value);
							if(!stricmp(Key, "CameraY"))
								NewStoryItem->Camera.z = (float)atof(Value);
							if(!stricmp(Key, "CameraZ"))
								NewStoryItem->Camera.z = (float)atof(Value);
							if(!stricmp(Key, "CameraAngleX"))
								NewStoryItem->CameraAngle.x = (float)atof(Value);
							if(!stricmp(Key, "CameraAngleY"))
								NewStoryItem->CameraAngle.z = (float)atof(Value);
							if(!stricmp(Key, "CameraAngleZ"))
								NewStoryItem->CameraAngle.z = (float)atof(Value);
							if(!stricmp(Key, "Next"))
								strcpy(NewStoryItem->Next, Value);
							// string
							if(!stricmp(Key, "NewsTicker"))
								strcpy(NewStoryItem->NewsTicker, Value);
							if(!stricmp(Key, "Text"))
							{
								char Str[MISL_STRMEM];
								strcpy(Str, Value);
								char *Ptr;
								while(Ptr = strchr(Str, '\\'))
								{
									switch(*(Ptr + 1))
									{
									case 'n':
										*Ptr = '\n';
										Ptr++;
										while(*Ptr = *(Ptr + 1)) Ptr++;
										break;
									case 0:
										*Ptr = 0;
										break;
									default:
										while(*Ptr = *(Ptr + 1)) Ptr++;
										break;
									}
								}
								strcpy(NewStoryItem->Text, Str);
							}
							// int
							if(!stricmp(Key, "TickerStyle"))
								NewStoryItem->TickerStyle = atoi(Value);
							if(!stricmp(Key, "Topic"))
							{
								char Str[MISL_STRMEM];
								strcpy(Str, Value);
								char *Ptr;
								while(Ptr = strchr(Str, '\\'))
								{
									switch(*(Ptr + 1))
									{
									case 'n':
										*Ptr = '\n';
										Ptr++;
										while(*Ptr = *(Ptr + 1)) Ptr++;
										break;
									case 0:
										*Ptr = 0;
										break;
									default:
										while(*Ptr = *(Ptr + 1)) Ptr++;
										break;
									}
								}
								strcpy(NewStoryItem->Topic, Str);
							}
							// float
							if(!stricmp(Key, "TimeDelta"))
								NewStoryItem->TimeDelta = (float)atof(Value);
							if(!stricmp(Key, "Render"))
								NewStoryItem->Render = !stricmp(Value, "true");
						}
						// add
						Mis->StoryItems.Add(NewStoryItem);
						// main story
						/*if(!stricmp(Group, "MainStory"))
							Mis.MainStory = NewStoryItem;*/
					}
				}
				// init Goal::Params
				State = 13;
				for(i = 0; i < Mis->Goals.Count(); i++)
					for(int j = 0; j < Mis->Goals[i]->ParamsTxt.Count(); j++)
						for(int k = 0; k < Mis->Goals.Count(); k++)
							if(!stricmp(Mis->Goals[i]->ParamsTxt[j], Mis->Goals[k]->Name))
							{
								Mis->Goals[i]->Params.Add(Mis->Goals[k]);
								break;
							}
				Missions[Missions.Count()] = Mis;
				State = 14;
				// check for errors
				if(!Mis->MainGoal)
				{
					char Str[1024];
					sprintf(Str, "MissionLib::Create: Mission has no main goal: %s", FileName);
					throw(Str);
				}
				State = 15;
			}
			State = 16;
			Succeeded = FindNextFile(Handle, &FindData);
		}
		State = 17;
		if(Handle != INVALID_HANDLE_VALUE)
			FindClose(Handle);
		State = 18;
	}
	catch(char *s)
	{
		throw(s);
	}
	catch(...)
	{
		char Str[1024];
		sprintf(Str, "MissionLib::Create#%d", State);
		throw(Str);
	}
}

MissionLib::~MissionLib(void)
{
	for(int i = 0; i < Missions.Count(); i++)
		delete Missions[i];
	Missions.Clear();
}

int MissionLib::GetMissionNum(const char *FileName)
{
	for(int i = 0; i < Missions.Count(); i++)
	{
		if(!strcmp(Missions[i]->FileName, FileName))
			return i;
	}
	char str[1024];
	sprintf(str, "MissionLib::GetMissionNum: Mission not found: %s", FileName);
	throw str;
	return -1;
}

}
