/***********************************************************/
/*                                                         */
/* Model.cpp                                               */
/*                                                         */
/* created 20.08.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "Model.h"
#include "IniLoader.h"

#include <stdio.h>

// www.infinitecode.com 3d library
namespace I3D
{

ISL::DynamicArray<Model::LoadedModel, 0, 1> Model::LoadedModels;

Model::Model(void)
{
	*StandardTexture = 0;
}

bool Model::IsSeperator(char c)
{
		return (c == ' ' || c == '\t' || c == '\n' || c == '\r' || c ==';' || c ==',');
}

float Model::GetSizeX()
{
	if(RS.GetVertexNumber())
	{
		float mi = RS.GetVertex(0).x, ma = RS.GetVertex(0).x;
		for(int i = 0; i < RS.GetVertexNumber(); i++)
		{
			if(RS.GetVertex(i).x < mi)
				mi = RS.GetVertex(i).x;
			if(RS.GetVertex(i).x > ma)
				ma = RS.GetVertex(i).x;
		}
		return ma - mi;
	}
	else
		return 0;
}

float Model::GetSizeY()
{
	if(RS.GetVertexNumber())
	{
		float mi = RS.GetVertex(0).y, ma = RS.GetVertex(0).y;
		for(int i = 0; i < RS.GetVertexNumber(); i++)
		{
			if(RS.GetVertex(i).y < mi)
				mi = RS.GetVertex(i).y;
			if(RS.GetVertex(i).y > ma)
				ma = RS.GetVertex(i).y;
		}
		return ma - mi;
	}
	else
		return 0;
}

float Model::GetSizeZ()
{
	if(RS.GetVertexNumber())
	{
		float mi = RS.GetVertex(0).z, ma = RS.GetVertex(0).z;
		for(int i = 0; i < RS.GetVertexNumber(); i++)
		{
			if(RS.GetVertex(i).z < mi)
				mi = RS.GetVertex(i).z;
			if(RS.GetVertex(i).z > ma)
				ma = RS.GetVertex(i).z;
		}
		return ma - mi;
	}
	else
		return 0;
}

/*int Model::ReadInt(char *&Str)
{
	int Number = 0;
	while(*Str >= 48 && *Str < 58)
	{
		Number = Number * 10 + (*Str - 48);
		Str++;
	}
	return Number;
}

float Model::ReadFloat(char *&Str)
{
	float Number = 0;
	bool Neg = false;
	if(*Str == '-')
	{
		Neg = true;
		Str++;
	}
	while(*Str >= 48 && *Str < 58)
	{
		Number = Number * 10 + (*Str - 48);
		Str++;
	}
	if(*Str == '.')
	{
		float Scalar = 0.1f;
		Str++;
		while(*Str >= 48 && *Str < 58)
		{
			Number = Number + Scalar * (*Str - 48);
			Str++;
			Scalar *= 0.1f;
		}
	}
	return Neg?-Number:Number;
}*/
/*
void Model::FindStr(char *Srch, char *In, char *Pos, int Brackets)
{
	int Match = 0;
	char *OrigIn = In;
	Brackets = 0;
	while(*In)
	{
		if(*Srch == '{')
			Brackets++;
		if(*Srch == '}')
		{
			Brackets--;
		}
		if(Srch[Match] == *In && (Match || OrigIn == In || IsSeperator(In[-1])))
		{
			Match++;
			if(Match == (int)strlen(Srch) && IsSeperator(In[1]))
				return In - (strlen(Srch) - 1);
		}
		else
			Match = 0;
		In++;
	}
	return 0;
}*/

void Model::BuildModel(XTree *Tree, Model *Mdl, Matrix *Mtx)
{
	int i;
	// recurse FrameTransformMatrices and SkinWeights
	for(i = 0; i < Tree->Child.Count(); i++)
	{
		if(!strcmp(Tree->Child[i]->Type, "FrameTransformMatrix") || !strcmp(Tree->Child[i]->Type, "SkinWeights"))
			BuildModel(Tree->Child[i], Mdl, Mtx);
	}
	if(!strcmp(Tree->Type, "FrameTransformMatrix"))
	{
		Matrix M;
		// read matrix
		for(int i = 0; i < 4; i++)
			for(int j = 0; j < 4; j++)
			{
				M.m[i][j] = Tree->Data[i * 4 + j];
			}
		// apply matrix
		*Mtx = *Mtx * M;
	}
	if(!strcmp(Tree->Type, "SkinWeights"))
	{
		int Number = (int)Tree->Data[0];
		Matrix M;
		// read matrix
		for(int i = 0; i < 4; i++)
			for(int j = 0; j < 4; j++)
			{
				M.m[i][j] = Tree->Data[1 + Number * 2 + i * 4 + j];
			}
		// apply matrix
		//*Mtx = *Mtx * M;
	}
	if(!strcmp(Tree->Type, "TextureFilename"))
	{
		strcpy(StandardTexture, Tree->Text);
	}
	// original indices
	static ISL::DynamicArray<int> OrigIndices;
	// read mesh and add it to the model
	static int FirstVertexOrig = -1;
	if(!strcmp(Tree->Type, "Mesh"))
	{
		XTree *DataTree = Tree;
		// Vertices
		int VNumber = (int)DataTree->Data[0];
		int i;
		FirstVertexOrig = RS.GetVertexNumber();
		int FirstVertex = FirstVertexOrig;
		/*for(i = 0; i < VNumber; i++)
		{
			Vertex V;
			V.x = DataTree->Data[1 + 3 * i];
			V.y = DataTree->Data[1 + 3 * i + 1];
			V.z = DataTree->Data[1 + 3 * i + 2];
			V.v = *Mtx * V.v;
			RS.AddVertex(V);
		}*/
		// Indices
		int INumber = (int)DataTree->Data[3 * VNumber + 1];
		for(i = 0; i < INumber; i++)
		{
/*			// add indices to renderstruct
			int Index1 = (int)DataTree->Data[3 * VNumber + 4 * i + 2 + 1] + FirstVertex;
			int Index3 = (int)DataTree->Data[3 * VNumber + 4 * i + 2 + 2] + FirstVertex;
			//    ^ swapping Index2 and Index3!
			int Index2 = (int)DataTree->Data[3 * VNumber + 4 * i + 2 + 3] + FirstVertex;*/

			// add indices to renderstruct
			int Index1 = (int)DataTree->Data[3 * VNumber + 4 * i + 2 + 1];
			int Index3 = (int)DataTree->Data[3 * VNumber + 4 * i + 2 + 2];
			//    ^ swapping Index2 and Index3!
			int Index2 = (int)DataTree->Data[3 * VNumber + 4 * i + 2 + 3];
			for(int j = 0; j < 3; ++j)
			{
				int k;
				switch(j)
				{
					case 0: k = Index1; break;
					case 1: k = Index2; break;
					case 2: k = Index3; break;
				}
				Vertex V;
				V.x = DataTree->Data[1 + 3 * k];
				V.y = DataTree->Data[1 + 3 * k + 1];
				V.z = DataTree->Data[1 + 3 * k + 2];
				V.v = *Mtx * V.v;
				RS.AddVertex(V);
				OrigIndices[FirstVertex + j] = k;
			}
			RS.AddIndex(FirstVertex);
			RS.AddIndex(FirstVertex + 1);
			RS.AddIndex(FirstVertex + 2);
			// calculate normal
			Vector Vector1 = RS.RenderList[FirstVertex].v;
			Vector Vector2 = RS.RenderList[FirstVertex + 1].v;
			Vector Vector3 = RS.RenderList[FirstVertex + 2].v;
			Vector Cross1 = Vector1 - Vector2;
			Vector Cross2 = Vector3 - Vector2;
			Vector Norm = Vector::Normalize(Vector::CrossProduct(Cross1, Cross2));
			// assign normal to vertices
			RS.RenderList[FirstVertex].n = Norm;
			RS.RenderList[FirstVertex + 1].n = Norm;
			RS.RenderList[FirstVertex + 2].n = Norm;
			FirstVertex += 3;

/*			RS.AddIndex(Index1);
			RS.AddIndex(Index2);
			RS.AddIndex(Index3);
			// calculate normal
			Vector Vector1 = RS.RenderList[Index1].v;
			Vector Vector2 = RS.RenderList[Index2].v;
			Vector Vector3 = RS.RenderList[Index3].v;
			Vector Cross1 = Vector1 - Vector2;
			Vector Cross2 = Vector3 - Vector2;
			Vector Norm = Vector::Normalize(Vector::CrossProduct(Cross1, Cross2));
			// assign normal to vertices
			RS.RenderList[Index1].n = Norm;
			RS.RenderList[Index2].n = Norm;
			RS.RenderList[Index3].n = Norm;*/
		}
	}
	// Add Texture Coordinates to the Mesh
	if(!strcmp(Tree->Type, "MeshTextureCoords"))
	{
		if(Tree->Parent && !strcmp(Tree->Parent->Type, "Mesh"))
		{
			XTree *DataTree = Tree->Parent;
/*			for(int i = 0; i < DataTree->Child.Count(); i++)
			{
				if(!strcmp(DataTree->Child[i]->Name, "Mesh"))
				{
					DataTree = DataTree->Child[i];*/
					/*int Number = (int)Tree->Data[0];
					for(int i = 0; i < min(Number, DataTree->Data[0]); i++)
					{
						RS.RenderList[(RS.GetVertexNumber() - Number) + i].t = Vector2D(Tree->Data[1 + 2 * i], Tree->Data[1 + 2 * i + 1]);
					}*/

					if(FirstVertexOrig != -1)
						for(int i = FirstVertexOrig; i < RS.RenderList.Count(); ++i)
						{
							int j = OrigIndices[i];
							RS.RenderList[i].t = Vector2D(Tree->Data[1 + 2 * j], Tree->Data[1 + 2 * j + 1]);
						}
					//break;
/*				}
			}*/
		}
	}
	// recurse other structures
	for(i = 0; i < Tree->Child.Count(); i++)
	{
		if(!(!strcmp(Tree->Child[i]->Type, "FrameTransformMatrix") || !strcmp(Tree->Child[i]->Type, "SkinWeights")))
		{
			Matrix SubMtx = *Mtx;
			BuildModel(Tree->Child[i], Mdl, &SubMtx);
		}
	}
}

Model::XTree *Model::BuildXTree(char *&Srch)
{
	const char *Seperators = " \n\r\t;,";
	int State = 0;
	try
	{
		State = 1;
		XTree *Tree = new XTree();
		*Tree->Text = 0;
		*Tree->Type = 0;
		*Tree->Name = 0;
		Tree->Parent = 0;
		char Name[1024], Type[1024];
		State = 2;
		while(*Srch && *Srch != '}')
		{
			State = 3;
			while(IsSeperator(*Srch)) Srch++;
			//Srch += strcspn(Srch, Seperators);
			// read blocks ( Frame Wheel {...} )
			if(isalpha(*Srch) || *Srch == '_' || *Srch == '{')
			{
				State = 4;
				sscanf(Srch, "%s", Type);
				// jump over unneeded blocks
				if(!strnicmp(Type, "template", 8)
					|| !strnicmp(Type, "MeshNormals", 8)
					|| !strnicmp(Type, "VertexDuplicationIndices", 8))
				{
					State = 5;
					// jump over template block
					int Brackets = 0;
					while(*Srch)
					{
						if(*Srch == '{')
							Brackets++;
						if(*Srch == '}')
						{
							Brackets--;
							if(Brackets <= 0)
							{
								Srch++;
								break;
							}
						}
						Srch++;
					}
				}
				else
				{
					State = 6;
					// get name
					Srch += strlen(Type);
					if(Srch)
					{
						while(IsSeperator(*Srch)) Srch++;
						if(isalnum(*Srch))
							sscanf(Srch, "%s", Name);
						else
							*Name = 0;
						Srch += strlen(Name);
						// recurse subtree
						while(IsSeperator(*Srch)) Srch++;
						if(*Srch == '{')
						{
							Srch++;
							Tree->Child[Tree->Child.Count()] = BuildXTree(Srch);
							strcpy(Tree->Child.Last()->Type, Type);
							strcpy(Tree->Child.Last()->Name, Name);
							Tree->Child.Last()->Parent = Tree;
							if(*Srch == '}') Srch++;
						}
					}
				}
			}
			State = 7;
			// read data (Float numbers)
			if(isdigit(*Srch) || *Srch == '-')
			{
				// data
				State = 71;
				float Value = (float)atof(Srch);
				State = 72;
				Tree->Data.Add(Value);
				while(isdigit(*Srch) || *Srch == '-' || *Srch == '.') Srch++;
				while(IsSeperator(*Srch)) Srch++;
			}
			State = 8;
			// read string
			if(*Srch == '\"')
			{
				char *Start = ++Srch;
				while(*Srch && *Srch != '\"')
					Srch++;
				strncpy(Tree->Text, Start, min(Srch - Start, 1023));
				Tree->Text[min(Srch - Start, 1023)] = 0;
				if(Srch)
					Srch++;
			}
		}
		State = 9;
		return Tree;
	}
	catch(char *s)
	{
		throw(s);
	}
	catch(...)
	{
		char Str[1024];
		sprintf(Str, "Model::BuildXTree#%d", State);
		throw(Str);
	}
	return 0;
}

Model::XTree::~XTree(void)
{
	for (int i = 0; i < Child.Count(); i++)
		delete Child[i];
}

void Model::LoadModel(D3DStruct *DS, char *FileName, int Part)
{
	int State = 0;
	try
	{
		/*if(!Part)
			TurretOffset = RS.GetIndexNumber();*/
		if(strstr(FileName, ".t4m") == FileName + strlen(FileName) - 4)
		{
			// .t4m file
			ITL::IniLoader Loader(FileName);
			int Vertices = Loader.CountKeys("Vertices");
			int i;
			for(i = 0; i < Vertices; i++)
			{
				char Str[1024];
				char Value[1024];
				Vertex V;
				strcpy(Value, Loader.GetValue("Vertices", itoa(i, Str, 10)));
				// parse
				char *Srch = Value;
				while((*Srch == ' ' || *Srch == '\t') && Srch != 0) Srch++;
				V.x = (float)atof(Srch);
				while(*Srch != ',' && *Srch) Srch++;
				Srch++;
				while((*Srch == ' ' || *Srch == '\t') && Srch != 0) Srch++;
				V.y = (float)atof(Srch);
				while(*Srch != ',' && *Srch) Srch++;
				Srch++;
				while((*Srch == ' ' || *Srch == '\t') && Srch != 0) Srch++;
				V.z = (float)atof(Srch);
				while(*Srch != ',' && *Srch) Srch++;
				Srch++;
				while((*Srch == ' ' || *Srch == '\t') && Srch != 0) Srch++;
				V.tu = (float)atof(Srch);
				while(*Srch != ',' && *Srch) Srch++;
				Srch++;
				while((*Srch == ' ' || *Srch == '\t') && Srch != 0) Srch++;
				V.tv = (float)atof(Srch);
				// add vertex to renderstruct
				RS.AddVertex(V);
			}
			int Triangles = Loader.CountKeys("Triangles");
			for(i = 0; i < Triangles; i++)
			{
				char Str[1024];
				char Value[1024];
				strcpy(Value, Loader.GetValue("Triangles", itoa(i, Str, 10)));
				// parse
				char *Srch = Value;
				while((*Srch == ' ' || *Srch == '\t') && Srch != 0) Srch++;
				int Index1 = atoi(Srch);
				while(*Srch != ',' && *Srch) Srch++;
				Srch++;
				while((*Srch == ' ' || *Srch == '\t') && Srch != 0) Srch++;
				int Index2 = atoi(Srch);
				while(*Srch != ',' && *Srch) Srch++;
				Srch++;
				while((*Srch == ' ' || *Srch == '\t') && Srch != 0) Srch++;
				int Index3 = atoi(Srch);
				// add indices to renderstruct
				RS.AddIndex(Index1);
				RS.AddIndex(Index2);
				RS.AddIndex(Index3);
				// calculate normal
				Vector Vector1 = RS.RenderList[Index1].v;
				Vector Vector2 = RS.RenderList[Index2].v;
				Vector Vector3 = RS.RenderList[Index3].v;
				Vector Cross1 = Vector1 - Vector2;
				Vector Cross2 = Vector3 - Vector2;
				Vector Norm = Vector::Normalize(Vector::CrossProduct(Cross1, Cross2));
				// assign normal to vertices
				RS.RenderList[Index1].n = Norm;
				RS.RenderList[Index2].n = Norm;
				RS.RenderList[Index3].n = Norm;
			}
			RS.CreateVBuf(DS->D3DDevice); // after normal calculations!
			RS.CreateIBuf(DS->D3DDevice);
			const char *StdTexture = Loader.GetValue("Texture", "Name");
			if(StdTexture)
				strcpy(StandardTexture, StdTexture);
			/*else
				strcpy(Part?StandardTexture2:StandardTexture, "");*/
		}
		else
		{
			// .x file
			State = 1;
			*StandardTexture = 0;
			FILE *f = fopen(FileName, "rb");
			if(!f)
			{
				char Str[1024];
				sprintf(Str, "Model::LoadModel#loading x file#fopen: couldn't open file: %s", FileName?FileName:"NULL");
				throw(Str);
			}
			State = 2;
			fseek(f, 0, SEEK_END);
			int FileSize = ftell(f);
			fseek(f, 0, SEEK_SET);
			State = 3;
			char *Buf = new char[FileSize + 1];
			fread(Buf, FileSize, 1, f);
			Buf[FileSize] = 0;
			State = 4;
			if(Buf[8] == 't' && Buf[9] == 'x' && Buf[10] == 't' && Buf[11] == ' ')
			{
				State = 5;
				char *Srch;
				if(strlen(Buf) < 16)
				{
					char Str[1024];
					sprintf(Str, "Model::LoadModel#unexpected end of x file %s", FileName);
					throw(Str);
				}
				Srch = Buf + 16;
				State = 6;
				XTree *Tree = BuildXTree(Srch);
				State = 7;
				I3D::Matrix Mtx;
				Mtx.SetIdentityMatrix();
				BuildModel(Tree, this, &Mtx);
				State = 8;
				delete Tree;
				State = 9;
			}
			State = 10;
			delete Buf;
			fclose(f);
			State = 11;
			RS.CreateVBuf(DS->D3DDevice); // after normal calculations!
			RS.CreateIBuf(DS->D3DDevice);
			State = 12;
		}
	}
	catch(char *s)
	{
		throw(s);
	}
	catch(...)
	{
		char Str[1024];
		sprintf(Str, "Model::LoadModel#%d", State);
		throw(Str);
	}
}

void Model::Create(D3DStruct *DS, char *FileName, char *FileName2)
{
	char Std[1024];
	int State = 0;
	//TurretOffset = -1;
	try
	{
		State = 1;
		LoadModel(DS, FileName);
		State = 2;
		strcpy(Std, StandardTexture);
		State = 3;
		if(FileName2 && *FileName2)
		{
			State = 4;
			LoadModel(DS, FileName2);
			State = 5;
		}
		State = 6;
	}
	catch(char *s)
	{
		throw(s);
	}
	catch(...)
	{
		char Str[1024];
		sprintf(Str, "Model::Create#LoadModel#%d", State);
		throw(Str);
	}
	strcpy(StandardTexture, Std);
}

void Model::RenderSetup(D3DStruct *DS)
{
	RS.SetupIndex(DS->D3DDevice);
}

}
