/***********************************************************/
/*                                                         */
/* Model.h                                                 */
/*                                                         */
/* created 20.08.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef MODEL_H
#define MODEL_H

#include "D3DStruct.h"
#include "Renderable.h"
#include "DynamicArray.h"

// www.infinitecode.com 3d library
namespace I3D
{

class Model: public RenderableIndex
{
private:
	// Parser
	class XTree
	{
	public:
		char Type[1024], Name[1024], Text[1024];
		ISL::DynamicArray<XTree *> Child;
		ISL::DynamicArray<float, 0, 32> Data;
		XTree *Parent;

		~XTree(void);
	};

	bool IsSeperator(char c);
	XTree *BuildXTree(char *&Srch);
	void BuildModel(XTree *Tree, Model *Mdl, Matrix *Mtx);
/*	int ReadInt(char *&Str);
	float ReadFloat(char *&Str);
	char *FindStr(char *Srch, char *In);*/
	void LoadModel(D3DStruct *DS, char *FileName, int Part = 0);

protected:
	class LoadedModel
	{
	public:
		LoadedModel(void)
			:Name(0)
		{
		}

		LoadedModel(const char *Name, Model *Mdl)
		{
			LoadedModel::Name = new char[1024];
			strcpy(LoadedModel::Name, Name);
			LoadedModel::Mdl = Mdl;
		}

		~LoadedModel(void)
		{
			if(Name) delete[] Name;
		}

		LoadedModel& operator = (LoadedModel &LM)
		{
			if(!Name)
				Name = new char[1024];
			strcpy(Name, LM.Name);
			Mdl = LM.Mdl;
			return *this;
		}

		LoadedModel(LoadedModel& LM)
		{
			*this = LM;
		}

		char *Name;
		Model *Mdl;

	};

	static ISL::DynamicArray<LoadedModel, 0, 1> LoadedModels;

public:
	char StandardTexture[1024];

	Model(void);

	static Model *GetModel(D3DStruct *DS, char *Name, char *Name2 = 0)
	{
		for(int i = 0; i < LoadedModels.Count(); i++)
			if(!strcmpi(Name, LoadedModels[i].Name))
				return LoadedModels[i].Mdl;
		Model *NewModel = new Model();
		/*char FileName[1024];
		strcpy(FileName, "Models\\");
		strcat(FileName, Name);
		strcat(FileName, ".t4m");*/
		NewModel->Create(DS, Name, Name2);
		LoadedModels[LoadedModels.Count()] = LoadedModel(Name, NewModel);
		return NewModel;
	}

	virtual void PreRender(D3DStruct *DS) 
	{
		DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
		DS->D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
		DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATER);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHAREF, 0x60);
	}

	virtual void PostRender(D3DStruct *DS)
	{
		DS->D3DDevice->SetRenderState(D3DRS_CULLMODE,D3DCULL_CW);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, FALSE);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
	}

	void Create(D3DStruct *DS, char *FileName, char *FileName2 = 0);
	virtual void CreateView(D3DStruct *DS) {}
	//virtual void Render(D3DStruct *DS); // uses RenderableIndex::Render
	void RenderSetup(D3DStruct *DS);
	float GetSizeX();
	float GetSizeY();
	float GetSizeZ();

	const char *GetStandardTexture()
	{
		return StandardTexture;
	}
};

}

#endif
