/***********************************************************/
/*                                                         */
/* ModelEditor.cpp                                         */
/*                                                         */
/* created 13.07.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "ModelEditor.h"

#include "MathStructures.h"
#include "D3DStructures.h"
#include "LogFile.h"
#include "MessageWindow.h"
#include "Input.h"
#include "IMath.h"
#include "TextureManager.h"

#include <direct.h>

// www.infinitecode.com game library
namespace IGAME
{

void ModelEditor::Create(HINSTANCE ThisInstance, int WindowStyle)
{
	// TODO: implement fullscreen display
	WindowStyle = SW_SHOWNORMAL;

	Flags = 0;
	Turn = 0;
	Pitch = 0;
	Roll = 0;
	TurretTurn = 0;
	TurretPitch = 0;
	Mdl = 0;
	Mdl2 = 0;
	Prj = 0;
	// clear the error output file
	ITL::LogFile::Clear("ErrorModelEditor.txt");
	ModelEditor::Instance = ThisInstance;
	//strcpy(ModelFile, "Models\\tank.t4m");
	strcpy(ModelFile, "Models\\Art1_vehicle.x");
	strcpy(ModelFile2, "Models\\art1_weapon.x");
	//strcpy(ModelFile, "Models\\first.x");
	strcpy(TextureFile, "Textures\\art1_vehicleTEX.bmp");
	strcpy(TextureFile2, "Textures\\art1_weaponTEX.bmp");
	// library
	TLib.Create();
	Lib = &TLib.Tanks[0];
	// Create the main window and initialize D3D
	try
	{
		Wnd.Create(WindowStyle, "Model Editor", 400, 400);
		// menu stuff
		MENUITEMINFO MenuInfo;
		memset(&MenuInfo, 0, sizeof(MenuInfo));

		HMENU Menu = CreateMenu(); // menu
		HMENU FileMenu = CreateMenu(); // file menu
		HMENU RenderMenu = CreateMenu(); // render menu
		HMENU LibMenu = CreateMenu(); // library menu
		// Load...
		MenuInfo.cbSize = sizeof(MenuInfo);
		MenuInfo.fMask = MIIM_ID | MIIM_TYPE | MIIM_SUBMENU | MIIM_STATE;
		MenuInfo.fType = MFT_STRING;
		MenuInfo.wID = 1024;
		MenuInfo.dwTypeData = "Load...";
		InsertMenuItem(FileMenu, -1, true, &MenuInfo); // add to file menu
		// export
/*		MenuInfo.wID = 1028;
		MenuInfo.dwTypeData = "Export to 3DS...";
		InsertMenuItem(FileMenu, -1, true, &MenuInfo); // add to file menu*/
		// load turret
		MenuInfo.wID = 1029;
		MenuInfo.dwTypeData = "Load Turret...";
		InsertMenuItem(FileMenu, -1, true, &MenuInfo); // add to file menu
		MenuInfo.wID = 1033;
		MenuInfo.dwTypeData = "Size Info";
		InsertMenuItem(FileMenu, -1, true, &MenuInfo); // add to file menu
		// import
/*		MenuInfo.wID = 1029;
		MenuInfo.dwTypeData = "Import from 3DS...";
		InsertMenuItem(FileMenu, -1, true, &MenuInfo); // add to file menu*/
		// Alpha Key
		MenuInfo.wID = 1025;
		MenuInfo.fState = MFS_UNCHECKED;
		MenuInfo.dwTypeData = "Color Key";
		InsertMenuItem(RenderMenu, -1, true, &MenuInfo); // add to render menu
		// Alpha blending
		MenuInfo.wID = 1026;
		MenuInfo.dwTypeData = "Alpha Blending";
		InsertMenuItem(RenderMenu, -1, true, &MenuInfo); // add to render menu
		// no culling
		MenuInfo.wID = 1027;
		MenuInfo.dwTypeData = "No Backface Culling";
		InsertMenuItem(RenderMenu, -1, true, &MenuInfo); // add to render menu
		// white background
		MenuInfo.wID = 1028;
		MenuInfo.dwTypeData = "White Background";
		InsertMenuItem(RenderMenu, -1, true, &MenuInfo); // add to render menu
		// rotation
		MenuInfo.wID = 1100;
		MenuInfo.dwTypeData = "Rotation";
		InsertMenuItem(RenderMenu, -1, true, &MenuInfo); // add to render menu
		// 1029 not free!
		// library
		for(int i = 0; i < TLib.Tanks.Count(); i++)
		{
			if(*TLib.Tanks[i].Name)
			{
				MenuInfo.wID = 2048 + i;
				char *Str = new char[1024];
				sprintf(Str, "%d %s", i, TLib.Tanks[i].Name);
				MenuInfo.dwTypeData = Str;
				InsertMenuItem(LibMenu, -1, true, &MenuInfo); // add to library menu
			}
		}
		// file menu
		MenuInfo.wID = 1030;
		MenuInfo.dwTypeData = "File";
		MenuInfo.hSubMenu = FileMenu;
		InsertMenuItem(Menu, -1, true, &MenuInfo); // add to menu
		// render menu
		MenuInfo.wID = 1031;
		MenuInfo.dwTypeData = "Render";
		MenuInfo.hSubMenu = RenderMenu;
		InsertMenuItem(Menu, -1, true, &MenuInfo); // add to menu
		// lib menu
		MenuInfo.wID = 1032;
		MenuInfo.dwTypeData = "Library";
		MenuInfo.hSubMenu = LibMenu;
		InsertMenuItem(Menu, -1, true, &MenuInfo); // add to menu
		// add menu to window
		SetMenu(Wnd.GetHWnd(), Menu);
		// 1033 not free
		// 1100 not free
	}
	catch(std::exception& Ex)
	{
		ITL::LogFile::Msg("ErrorModelEditor.txt", (char *)Ex.what());
		ITL::MessageWindow::Message((char *)Ex.what(), "Error during D3D Window initialisation");
		exit(-1);
	}
	Wnd.AddListener(this);
	//In.Create(Wnd.GetHWnd());
	DS.Create(Wnd.D3DDevice, &Sp);
	Mouse.Create(&DS, Wnd.GetHWnd());
	//Mouse.SetDragDrop(false);
	Angle = 0.5f * 2 * PI;
	Angle2 = 1.0f * 2 * PI;
	Zoom = 30.0f;
	BackgroundColor = 0x00000000;

	Init3D();
}

void ModelEditor::Init3D(void)
{
	DS.D3DDevice->SetRenderState(D3DRS_LIGHTING,TRUE);
	DS.D3DDevice->SetRenderState(D3DRS_ZENABLE,TRUE);
	DS.D3DDevice->SetRenderState(D3DRS_CULLMODE,D3DCULL_CW);
	DS.D3DDevice->SetSamplerState(0,D3DSAMP_MAGFILTER, D3DTEXF_LINEAR); 
	DS.D3DDevice->SetSamplerState(0,D3DSAMP_MINFILTER, D3DTEXF_LINEAR); 
	DS.D3DDevice->SetSamplerState(0,D3DSAMP_MIPFILTER, D3DTEXF_POINT); 
	DS.D3DDevice->SetSamplerState(1,D3DSAMP_MAGFILTER, D3DTEXF_LINEAR); 
	DS.D3DDevice->SetSamplerState(1,D3DSAMP_MINFILTER, D3DTEXF_LINEAR); 
	DS.D3DDevice->SetSamplerState(1,D3DSAMP_MIPFILTER, D3DTEXF_POINT); 
//	DS.D3DDevice->SetRenderState(D3DRS_AMBIENT, 0x00000000);
	DS.D3DDevice->SetRenderState(D3DRS_AMBIENT, 0x80808080);
	DS.D3DDevice->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATER);
	DS.D3DDevice->SetRenderState(D3DRS_ALPHAREF, 0x60);
	DS.D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	DS.D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
	if(D3DFlags & GAME_D3DFLAG_COLORKEY)
		DS.D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE);
	else
		DS.D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, FALSE);
	if(D3DFlags & GAME_D3DFLAG_ALPHABLEND)
		DS.D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
	else
		DS.D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
	if(D3DFlags & GAME_D3DFLAG_NOCULLING)
		DS.D3DDevice->SetRenderState(D3DRS_CULLMODE,D3DCULL_NONE);
	else
		DS.D3DDevice->SetRenderState(D3DRS_CULLMODE,D3DCULL_CW);

	// for debugging:
	//DS.D3DDevice->SetRenderState(D3DRS_AMBIENT, 0xFFFFFFFF);
	if(D3DFlags & GAME_D3DFLAG_WIREFRAME)
		DS.D3DDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME);

	D3DMATERIAL9 Material;
	memset(&Material, 0, sizeof(D3DMATERIAL9));
	Material.Diffuse.r = 1.0f;
	Material.Diffuse.g = 1.0f;
	Material.Diffuse.b = 1.0f;
	Material.Diffuse.a = 1.0f;
	Material.Ambient.r = 1.0f;
	Material.Ambient.g = 1.0f;
	Material.Ambient.b = 1.0f;
	Material.Ambient.a = 1.0f;
	Material.Specular.r = 1.0f; // Licht spiegeln
	Material.Specular.g = 1.0f;
	Material.Specular.b = 1.0f;
	Material.Specular.a = 1.0f;
	Material.Power = 0.0f;
	//  Material.power = (float)10.0;
	DS.D3DDevice->SetMaterial(&Material);
	// lighting
	D3DLIGHT9 Light;
	memset(&Light, 0, sizeof(D3DLIGHT9));
	Light.Type = D3DLIGHT_DIRECTIONAL;
	Light.Diffuse.r = 1.0f;
	Light.Diffuse.g = 1.0f;
	Light.Diffuse.b = 0.8f;
	Light.Specular.r=Light.Diffuse.r;
	Light.Specular.g=Light.Diffuse.g;
	Light.Specular.b=Light.Diffuse.b;
	Light.Attenuation0 = 0.0;
	Light.Attenuation1 = 0.0;
	Light.Attenuation2 = 0.0;
	Light.Range = 0;
	Light.Position.x = 0;
	Light.Position.y = 0;
	Light.Position.z = 0;
	Light.Direction.x = 1.0f;
	Light.Direction.y = 1.0f;
	Light.Direction.z = 1.0f;
	DS.D3DDevice->SetLight(1, &Light);
	DS.D3DDevice->LightEnable(1, true);

	memset(&Light, 0, sizeof(D3DLIGHT9));
	Light.Type = D3DLIGHT_DIRECTIONAL;
	Light.Diffuse.r = 0.0f;
	Light.Diffuse.g = 0.0f;
	Light.Diffuse.b = 0.2f;
	Light.Specular.r=Light.Diffuse.r;
	Light.Specular.g=Light.Diffuse.g;
	Light.Specular.b=Light.Diffuse.b;
	Light.Attenuation0 = 0.0;
	Light.Attenuation1 = 0.0;
	Light.Attenuation2 = 0.0;
	Light.Range = 0;
	Light.Position.x = 0;
	Light.Position.y = 0;
	Light.Position.z = 0;
	Light.Direction.x = -1.0f;
	Light.Direction.y = -1.0f;
	Light.Direction.z = -1.0f;
	DS.D3DDevice->SetLight(2, &Light);
	DS.D3DDevice->LightEnable(2, true);
	//Wrld.Init3D(DS);
}

// called asynchronously!
void ModelEditor::OnEvent(I3D::D3DWEvent Event)
{
	switch(Event.GetID())
	{
	case I3D::D3DW_EVENT_RESET:
		Init3D();
		break;
	default:
		Events[Events.Count()] = Event;
	}
}

void ModelEditor::MenuCheck(int ID)
{
	MENUITEMINFO MII;
	MII.cbSize = sizeof(MII);
	MII.fMask = MIIM_STATE;
	GetMenuItemInfo(GetMenu(Wnd.GetHWnd()), ID, false, &MII);
	if(MII.fState == MFS_CHECKED)
		MII.fState = MFS_UNCHECKED;
	else
		MII.fState = MFS_CHECKED;
	SetMenuItemInfo(GetMenu(Wnd.GetHWnd()), ID, false, &MII);
}

void ModelEditor::MouseHandler(void)
{
	bool Inside = true;
	if(Mouse.GetX() < -1 || Mouse.GetX() > 1
	|| Mouse.GetY() < -1 || Mouse.GetY() > 1)
		Inside = false;
	if(Inside)
	{
		static float LastZ = Mouse.GetZ();
		float dZ = Mouse.GetZ() - LastZ;
		static float InterpolateddZ = 0;
		InterpolateddZ *= 0.9f;
		InterpolateddZ += 0.1f * dZ;
		if(absf(InterpolateddZ) > 0.001f)
		{
			// zoom
			Zoom *= 1 - 0.002f * InterpolateddZ;
		}
		LastZ = Mouse.GetZ();
	}
}

// called synchronously
void ModelEditor::EventHandler(void)
{
	if(Events.Count())
	{
		for(int i = 0; i < Events.Count(); i++)
		{
			switch(Events[i].GetID())
			{
			case I3D::D3DW_EVENT_WINDOWSEVENT:
				switch(Events[i].GetInfo())
				{
				case WM_KEYDOWN:
					KeyboardInput(Events[i].GetWParam());
					break;
				case WM_COMMAND:
				{
					const int WParam = Events[i].GetWParam() & 0xFFFF;
					switch(WParam)
					{
					case 1025:
						// color key
						D3DFlags ^= GAME_D3DFLAG_COLORKEY;
						if(D3DFlags & GAME_D3DFLAG_COLORKEY)
							DS.D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE);
						else
							DS.D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, FALSE);
						MenuCheck(1025);
						break;
					case 1026:
						// alpha blending
						D3DFlags ^= GAME_D3DFLAG_ALPHABLEND;
						if(D3DFlags & GAME_D3DFLAG_ALPHABLEND)
							DS.D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
						else
							DS.D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
						MenuCheck(1026);
						break;
					case 1027:
						// no culling
						D3DFlags ^= GAME_D3DFLAG_NOCULLING;
						if(D3DFlags & GAME_D3DFLAG_NOCULLING)
							DS.D3DDevice->SetRenderState(D3DRS_CULLMODE,D3DCULL_NONE);
						else
							DS.D3DDevice->SetRenderState(D3DRS_CULLMODE,D3DCULL_CW);
						MenuCheck(1027);
						break;
					case 1028:
						// white background
						D3DFlags ^= GAME_D3DFLAG_WHITEBACK;
						if(D3DFlags & GAME_D3DFLAG_WHITEBACK)
							BackgroundColor = 0xFFFFFFFF;
						else
							BackgroundColor = 0x00000000;
						MenuCheck(1028);
						break;
					case 1100:
						// rotation
						Flags ^= FLAG_ROTATION;
						MenuCheck(1100);
						break;
					case 1033:
						// size info
						char Info[1024];
						sprintf(Info, "X Size: %f\r\nY Size: %f\r\nZ Size: %f\r\n", Mdl->GetSizeX(), Mdl->GetSizeY(), Mdl->GetSizeZ());
						MessageBox(0, Info, "Model Sizes", MB_OK);
						break;
					case 1024:
						char Dir[1024];
						getcwd(Dir, 1024);
						// load
						OPENFILENAME OFN;
						memset(&OFN, 0, sizeof(OFN));
						OFN.lStructSize = sizeof(OFN);
						OFN.hwndOwner = Wnd.GetHWnd();
						OFN.lpstrFilter = "Model Files (*.t4m; *.x)\0*.T4M;*.X\0\0";
						OFN.lpstrDefExt = "t4m";
						OFN.lpstrInitialDir = "Models";
						OFN.lpstrTitle = "Load Model";
						OFN.lpstrFile = new char[1024];
						OFN.nMaxFile = 1024;
						*OFN.lpstrFile = 0;
						//OFN.lpstrFile = NULL;
						/*strcpy(OFN.lpstrFile, "Models\\Tree.t4m");
						OFN.nFileOffset = 8;
						OFN.nFileExtension = 13;*/
						OFN.Flags = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY /*| OFN_SHAREAWARE*/; // OFN_OVERWRITEPROMPT
						if(GetOpenFileName(&OFN))
						{
							strcpy(ModelFile, OFN.lpstrFile);
							*ModelFile2 = 0;
							delete OFN.lpstrFile;
							if(Mdl)
								delete Mdl;
							Mdl = 0;
							if(Mdl2)
								delete Mdl2;
							Mdl2 = 0;
							RELEASE(Texture);
							Texture = 0;
							Angle = 0.5f * 2 * PI;
							Angle2 = 1.0f * 2 * PI;
						}
						chdir(Dir);
						break;
					case 1029:
						getcwd(Dir, 1024);
						// load
						memset(&OFN, 0, sizeof(OFN));
						OFN.lStructSize = sizeof(OFN);
						OFN.hwndOwner = Wnd.GetHWnd();
						OFN.lpstrFilter = "Model Files (*.t4m; *.x)\0*.T4M;*.X\0\0";
						OFN.lpstrDefExt = "t4m";
						OFN.lpstrInitialDir = "Models";
						OFN.lpstrTitle = "Load Model";
						OFN.lpstrFile = new char[1024];
						OFN.nMaxFile = 1024;
						*OFN.lpstrFile = 0;
						//OFN.lpstrFile = NULL;
						/*strcpy(OFN.lpstrFile, "Models\\Tree.t4m");
						OFN.nFileOffset = 8;
						OFN.nFileExtension = 13;*/
						OFN.Flags = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY /*| OFN_SHAREAWARE*/; // OFN_OVERWRITEPROMPT
						if(GetOpenFileName(&OFN))
						{
							strcpy(ModelFile2, OFN.lpstrFile);
							delete OFN.lpstrFile;
							/*if(Mdl)
								delete Mdl;
							Mdl = 0;*/
							if(Mdl2)
								delete Mdl2;
							Mdl2 = 0;
							Mdl = 0;
							RELEASE(Texture);
							Texture = 0;
							Angle = 0.5f * 2 * PI;
							Angle2 = 1.0f * 2 * PI;
						}
						else
							*ModelFile2 = 0;
						chdir(Dir);
						break;
					}
					if(WParam >= 2048 && WParam < 2048 + 10024)
					{
						int ID = Events[i].GetWParam() & 0xFFFF - 2048;
						TLib.Create();
						Lib = &TLib.Tanks[ID];
						delete Mdl;
						Mdl = 0;
						delete Mdl2;
						Mdl2 = 0;
						strcpy(ModelFile, Lib->Model);
						strcpy(ModelFile2, Lib->TurretModel);
					}
					break;
				}
				case WM_MOUSEMOVE:
					if(Events[i].GetWParam() & MK_LBUTTON)
					{
						int x = Events[i].GetLParam() & 0xFFFF;
						int y = Events[i].GetLParam() >> 16;
						Angle = 2 * PI * x / Wnd.GetWidth();
						Angle2 = 2 * PI * (0.5f + (float)y / Wnd.GetHeight());
					}
					break;
				}
				break;
			}
		}
		Events.Clear();
	}
}

void ModelEditor::KeyboardInput(char c)
{
	switch(c)
	{
	case 27:
		// Escape
		PostQuitMessage(0);
		break;
	case 'w':
	case 'W':
		D3DFlags ^= GAME_D3DFLAG_WIREFRAME;
		if(D3DFlags & GAME_D3DFLAG_WIREFRAME)
			DS.D3DDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME);
		else
			DS.D3DDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);
		break;
	}
}

void ModelEditor::MouseClick(void)
{
}

void ModelEditor::MainLoop(void)
{
	//In.Update();
	EventHandler();
	MouseHandler();
	Sp.Update();
	if(DS.D3DDevice->BeginScene())
	{
		DS.D3DDevice->Clear(0, NULL, D3DCLEAR_ZBUFFER | D3DCLEAR_TARGET, BackgroundColor, 1.0f, 0);
		D3DMATRIX Matrix = {1,0,0,0,  0,1,0,0,  0,0,1,0,  0,0,0,1};
		// world and view matrix
		DS.D3DDevice->SetTransform(D3DTS_WORLD, &Matrix);
		DS.D3DDevice->SetTransform(D3DTS_VIEW, &Matrix);
		DS.D3DDevice->SetTransform(D3DTS_PROJECTION, &Matrix);
		// projection matrix
		// distance eye->monitor in meters
		const float EyeDistance = 0.6f;
		// (vertical) monitor size in meters
		const float MonitorSize = 0.33f;
		// perspective projection: [x, y, z] /= (1 + z)
		D3DMATRIX PMatrix = {1,0,0,0,  0,1,0,0,  0,0,1,1,  0,0,0,1};
		DS.D3DDevice->MultiplyTransform(D3DTS_PROJECTION, &PMatrix);
		// adjust to monitor size and position
		// screen 3:4  = 0.75f
		D3DMATRIX TMatrix3 = {0.75f * 2.0f / MonitorSize,0,0,0,  0,2.0f / MonitorSize,0,0,  0,0,1.0f / EyeDistance,0,  0,0,0,1};
		DS.D3DDevice->MultiplyTransform(D3DTS_PROJECTION, &TMatrix3);
		DS.D3DDevice->GetTransform(D3DTS_PROJECTION, &DS.ProjMtx);

		// view matrix
		//Angle += 0.07f;
		/*D3DMATRIX VMatrix = {1,0,0,0,  0,1,0,0,  0,0,1,0,  0,-3.0f,0,1};
		DS.D3DDevice->MultiplyTransform(D3DTS_VIEW, &VMatrix);*/
		// camera distance
		D3DMATRIX VMatrix2 = {1,0,0,0,  0,1,0,0,  0,0,1,0,  0,0,Zoom,1};
		DS.D3DDevice->MultiplyTransform(D3DTS_VIEW, &VMatrix2);
		// Look from above
		D3DMATRIX VMatrix3 = {1,0,0,0,  0,(float)cos(Angle2),-(float)sin(Angle2),0,  0,(float)sin(Angle2),(float)cos(Angle2),0,  0,0,0,1};
		DS.D3DDevice->MultiplyTransform(D3DTS_VIEW, &VMatrix3);
		// rotate camera
		D3DMATRIX VMatrix4 = {(float)cos(Angle),0,(float)sin(Angle),0,  0,1,0,0,  -(float)sin(Angle),0,(float)cos(Angle),0,  0,0,0,1};
		DS.D3DDevice->MultiplyTransform(D3DTS_VIEW, &VMatrix4);
		DS.D3DDevice->GetTransform(D3DTS_VIEW, &DS.ViewMtx);

		DS.CreateProjViewMtx();

		if(Flags & FLAG_ROTATION)
		{
			Turn += 0.4f * Sp.Delta;
			//Pitch = PI / 16.0f * (float)sin(Sp.GameTime);
			//Roll = PI / 16.0f * (float)sin(2 * Sp.GameTime);
			TurretTurn += 0.5f * Sp.Delta;
			TurretPitch = PI / 4.0f * ((float)cos(Sp.GameTime) + 1) / 2.0f;
		}

		// render
		if(!Mdl)
		{
			// load model
			Mdl = new I3D::Model();
			Mdl->Create(&DS, ModelFile);
		}
		if(!Mdl2 && *ModelFile2)
		{
			Mdl2 = new I3D::Model();
			Mdl2->Create(&DS, ModelFile2);
		}
		if(!Prj)
		{
			Prj = new I3D::Model();
			Prj->Create(&DS, "Models\\projectile_marker.x");
		}
		// set texture
		if(!Texture)
		{
			if(Mdl->GetStandardTexture() && *Mdl->GetStandardTexture())
			{
				char TexName[1024];
				strcpy(TexName, "Textures\\");
				strcat(TexName, Mdl->GetStandardTexture());
				int Flags = I3D::TM_FLAG_ALPHACC0;
				if(D3DFlags & GAME_D3DFLAG_COLORKEY)
					Flags |= I3D::TM_FLAG_ACCBILOPT;
				if(D3DFlags & GAME_D3DFLAG_ALPHABLEND)
					Flags |= I3D::TM_FLAG_MIPALPHA60;
				Texture = I3D::TextureManager::Load(DS.D3DDevice, TexName, NULL, Flags | GAME_D3DFLAG_COLORKEY);
			}
			else
				Texture = NULL;
		}
		DS.D3DDevice->SetTexture(0, Texture);
		// World Matrix
		I3D::Matrix WMatrix;
		IMATH::Vector Normal = IMATH::Vector(0, 1, 0);
		((I3D::Matrix *)&WMatrix)->SetAnglePitchMatrix(IMATH::Vector(0, 0, 0), Turn, Pitch, Normal);
		/*I3D::Matrix WMatrix(1,0,0,0,  0,1,0,0,  0,0,1,0,  0,0,0,1);
		I3D::Matrix WMatrix2;
		WMatrix2.SetRotationXMatrix(Pitch, Lib->PitchRoll.y, Lib->PitchRoll.z);
		WMatrix = WMatrix2 * WMatrix;
		WMatrix2.SetRotationZMatrix(Roll, Lib->PitchRoll.x, Lib->PitchRoll.y);
		WMatrix = WMatrix2 * WMatrix;
		WMatrix2.SetRotationYMatrix(Turn, Lib->Turn.x, Lib->Turn.z);
		WMatrix = WMatrix2 * WMatrix;*/
		DS.D3DDevice->SetTransform(D3DTS_WORLD, &WMatrix);
		Mdl->Render(&DS);
		if(Mdl2)
		{
			if(!Texture2)
			{
				// set texture
				if(Mdl2->GetStandardTexture() && *Mdl2->GetStandardTexture())
				{
					char TexName[1024];
					strcpy(TexName, "Textures\\");
					strcat(TexName, Mdl2->GetStandardTexture());
					int Flags = I3D::TM_FLAG_ALPHACC0;
					if(D3DFlags & GAME_D3DFLAG_COLORKEY)
						Flags |= I3D::TM_FLAG_ACCBILOPT;
					if(D3DFlags & GAME_D3DFLAG_ALPHABLEND)
						Flags |= I3D::TM_FLAG_MIPALPHA60;
					Texture2 = I3D::TextureManager::Load(DS.D3DDevice, TexName, NULL, Flags);
					//Texture = TextureManager::Load(DS.D3DDevice, Mdl->GetStandardTexture(), NULL, Flags);
				}
				else
					Texture2 = NULL;
			}
			DS.D3DDevice->SetTexture(0, Texture2);
			// World Matrix
			//Normal = WMatrix * IMATH::Vector(0, 1, 0);
			WMatrix.SetAnglePitchMatrix(IMATH::Vector(0, 0, 0), Turn, TurretTurn
			, Pitch, TurretPitch, Normal
			, Lib->PitchRoll, Lib->Turn, Lib->TurretPitch, Lib->TurretTurn);
			/*WMatrix.SetIdentityMatrix();
			// turret
			WMatrix2.SetRotationXMatrix(TurretPitch, Lib->TurretPitch.y, Lib->TurretPitch.z);
			WMatrix = WMatrix2 * WMatrix;
			WMatrix2.SetRotationYMatrix(TurretTurn, Lib->TurretTurn.x, Lib->TurretTurn.z);
			WMatrix = WMatrix2 * WMatrix;
			// tank
			WMatrix2.SetRotationXMatrix(Pitch, Lib->PitchRoll.y, Lib->PitchRoll.z);
			WMatrix = WMatrix2 * WMatrix;
			WMatrix2.SetRotationZMatrix(Roll, Lib->PitchRoll.x, Lib->PitchRoll.y);
			WMatrix = WMatrix2 * WMatrix;
			WMatrix2.SetRotationYMatrix(Turn, Lib->Turn.x, Lib->Turn.z);
			WMatrix = WMatrix2 * WMatrix;*/
			DS.D3DDevice->SetTransform(D3DTS_WORLD, &WMatrix);

			Mdl2->Render(&DS);

			//WMatrix.SetAnglePitchMatrix(Lib->ProjectileSpawnPos, Turn, Pitch, Normal);
			//WMatrix.SetTranslationMatrix(Lib->ProjectileSpawnPos);
			int Number = 1;
			if(Lib->Burst)
				Number = Lib->BurstNumber;
			I3D::Matrix LaunchMtx;
			DS.D3DDevice->SetTexture(0, 0);
			for(int i = 0; i < Number; i++)
			{
				WMatrix.SetTranslationMatrix(Lib->ProjectileSpawnPos);
				if(Lib->Burst)
				{
					WMatrix = I3D::Matrix::TranslationMatrix(((float)i - (float)(Number - 1) / 2.0f) * Lib->BurstDisplacement) * WMatrix;
				}
				LaunchMtx.SetAnglePitchMatrix(IMATH::Vector(0, 0, 0), Turn, TurretTurn
				, Pitch, TurretPitch, Normal
				, Lib->PitchRoll, Lib->Turn, Lib->TurretPitch, Lib->TurretTurn);
				WMatrix = LaunchMtx * WMatrix;
				DS.D3DDevice->SetTransform(D3DTS_WORLD, &WMatrix);
				Prj->Render(&DS);
			}
		}

		Mouse.Render(&DS);
/*			RELEASE(Texture);
		Texture = 0;
		RELEASE(Mdl);
		Mdl = 0;*/
		DS.D3DDevice->Present( NULL, NULL, NULL, NULL );
		DS.D3DDevice->EndScene();
	}
}

}
