/***********************************************************/
/*                                                         */
/* ModelEditor.h                                           */
/*                                                         */
/* created 13.07.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef MODELEDITOR_H
#define MODELEDITOR_H

#include "D3DWindow.h"
#include "D3DStruct.h"
#include "EventListener.h"
#include "DynamicArray.h"
#include "Model.h"
#include "RenderableModel.h"
#include "Input.h"
#include "MouseCursor.h"
#include "TankLib.h"

// www.infinitecode.com game library
namespace IGAME
{

enum
{
	GAME_D3DFLAG_WIREFRAME = 1,
	GAME_D3DFLAG_NOCULLING = 2,
	GAME_D3DFLAG_COLORKEY = 4,
	GAME_D3DFLAG_ALPHABLEND = 8,
	GAME_D3DFLAG_WHITEBACK = 16
};

class ModelEditor: public IEV::EventListener<I3D::D3DWEvent>
{
	enum Flags
	{
		FLAG_ROTATION = 1
	};

	I3D::D3DWindow Wnd;
	I3D::D3DStruct DS;
	int D3DFlags;
	I3D::Model *Mdl, *Mdl2, *Prj;
	LPDIRECT3DTEXTURE9 Texture, Texture2;
	HINSTANCE Instance;
	char ModelFile[1024], ModelFile2[1024], TextureFile[1024], TextureFile2[1024];
	float Angle, Angle2, Zoom, Pitch, Roll, Turn, TurretTurn, TurretPitch;
	int BackgroundColor, Flags;
	//IIN::Input In;
	I3D::MouseCursor Mouse;
	ITL::Speed Sp;
	IGAME::TankLib TLib;
	IGAME::TankLib::Tank *Lib;

	ISL::DynamicArray<I3D::D3DWEvent, 0, 4> Events;

	virtual void OnEvent(I3D::D3DWEvent Data);
	void EventHandler(void);
	void KeyboardInput(char c);
	void MouseClick(void);
	void Init3D(void);
	void MenuCheck(int ID);
	void MouseHandler(void);

public:
	ModelEditor(void)
		:D3DFlags(0), Mdl(0), Texture(0), Flags(0)
	{
	}

	~ModelEditor(void)
	{
		Wnd.DeleteListener(this);
		RELEASE(Texture);
		Texture = 0;
		if(Mdl)
			delete Mdl;
		Mdl = 0;
		if(Mdl2)
			delete Mdl;
		if(Prj)
			delete Prj;
		Mdl2 = 0;
	}

	void Create(HINSTANCE ThisInstance, int WindowStyle);
	// this is called continuously while the application is running
	void MainLoop(void);
};

}

#endif
