/***********************************************************/
/*                                                         */
/* MouseCursor.h                                           */
/*                                                         */
/* created 26.08.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "MouseCursor.h"
#include "Input.h"
#include "TextureManager.h"
#include "IMath.h"

// www.infinitecode.com 3d library
namespace I3D
{

void MouseCursor::CreateView(D3DStruct *DS)
{
	RECT Rect;
	GetClientRect(Wnd, &Rect);
	ClientRect = Rect;
	float x = 48.0f / (Rect.right - Rect.left);
	float y = 48.0f / (Rect.bottom - Rect.top);
	/*RELEASE(StdTexture);
	StdTexture = TextureManager::GetTexture(DS, "Textures\\Mouse.bmp", 0, TM_FLAG_ALPHACC0);
	Texture = StdTexture;*/
	RS.ClearVertex();
	// mouse
	RS.AddVertex(Vertex(Vector(0,  0, 0), Vector(0, 0, -1), Vector2D(0, 0)));
	RS.AddVertex(Vertex(Vector(0, -y, 0), Vector(0, 0, -1), Vector2D(0, 1)));
	RS.AddVertex(Vertex(Vector(x, -y, 0), Vector(0, 0, -1), Vector2D(1, 1)));
	RS.AddVertex(Vertex(Vector(x,  0, 0), Vector(0, 0, -1), Vector2D(1, 0)));
	// frame
	RS.AddVertex(Vertex(Vector(0, 0, 0), Vector(0, 0, -1), Vector2D(0, 0)));
	RS.AddVertex(Vertex(Vector(1, 0, 0), Vector(0, 0, -1), Vector2D(1, 0)));
	RS.AddVertex(Vertex(Vector(1, 1, 0), Vector(0, 0, -1), Vector2D(1, 1)));
	RS.AddVertex(Vertex(Vector(0, 1, 0), Vector(0, 0, -1), Vector2D(0, 1)));
	RS.CreateVBuf(DS->D3DDevice);
	RS.ClearIndex();
	// mouse or frame
	RS.AddIndex(0);
	RS.AddIndex(1);
	RS.AddIndex(3);
	RS.AddIndex(1);
	RS.AddIndex(2);
	RS.AddIndex(3);
	RS.CreateIBuf(DS->D3DDevice);
}

void MouseCursor::Init3D(D3DStruct *DS, bool recurse)
{
	//RELEASE(StdTexture);
	StdTexture = TextureManager::GetTexture(DS, "Textures\\Mouse.bmp", 0, TM_FLAG_ALPHACC0);
	Texture = StdTexture;

	CreateView(DS);

	/*RS.ClearVertex();
	// mouse
	RS.AddVertex(Vertex(Vector(0,  0, 0), Vector(0, 0, -1), Vector2D(0, 0)));
	RS.AddVertex(Vertex(Vector(0, -y, 0), Vector(0, 0, -1), Vector2D(0, 1)));
	RS.AddVertex(Vertex(Vector(x, -y, 0), Vector(0, 0, -1), Vector2D(1, 1)));
	RS.AddVertex(Vertex(Vector(x,  0, 0), Vector(0, 0, -1), Vector2D(1, 0)));
	// frame
	RS.AddVertex(Vertex(Vector(0, 0, 0), Vector(0, 0, -1), Vector2D(0, 0)));
	RS.AddVertex(Vertex(Vector(1, 0, 0), Vector(0, 0, -1), Vector2D(1, 0)));
	RS.AddVertex(Vertex(Vector(1, 1, 0), Vector(0, 0, -1), Vector2D(1, 1)));
	RS.AddVertex(Vertex(Vector(0, 1, 0), Vector(0, 0, -1), Vector2D(0, 1)));
	RS.CreateVBuf(DS->D3DDevice);
	RS.ClearIndex();
	// mouse or frame
	RS.AddIndex(0);
	RS.AddIndex(1);
	RS.AddIndex(3);
	RS.AddIndex(1);
	RS.AddIndex(2);
	RS.AddIndex(3);
	RS.CreateIBuf(DS->D3DDevice);*/
}

void MouseCursor::Create(D3DStruct *DS, HWND Wnd)
{
	Buttons = 0;
	//DragDrop = true;
	Cursor = true;
	In.Create(Wnd);
	MouseCursor::Wnd = Wnd;
	GetClientRect(Wnd, &ClientRect);
	/*StdTexture = TextureManager::GetTexture(DS, "Textures\\Mouse.bmp", 0, TM_FLAG_ALPHACC0);
	Texture = StdTexture;*/
	//DragTexture = TextureManager::GetTexture(DS, "Textures\\Drag.bmp", 0, TM_FLAG_ALPHACC0);
	Init3D(DS, false);
}

void MouseCursor::Render(D3DStruct *DS)
{
	// position
	In.Update();
	POINT Pos = In.GetPos();
	ScreenToClient(Wnd, &Pos);
	RECT Rect;
	GetClientRect(Wnd, &Rect);
	if(Rect.left != ClientRect.left
	|| Rect.right != ClientRect.right
	|| Rect.top != ClientRect.top
	|| Rect.bottom != ClientRect.bottom)
	{
		CreateView(DS);
	}
	x = 2 * (float)Pos.x / (Rect.right - Rect.left) - 1;
	y = 2 * (1 - (float)Pos.y / (Rect.bottom - Rect.top)) - 1;
	static float xv = x;
	static float yv = y;
	// drag
	if((x != xv || y != yv) && Drag)
	{
		// drag
		MouseCursorEvent Ev;
		Ev.Code = MC_DRAG;
		Ev.x = x;
		Ev.y = y;
		Ev.x2 = Downx;
		Ev.y2 = Downy;
		Events[Events.Count()] = Ev;
	}
	xv = x;
	yv = y;
	// buttons
	IIN::Input::MouseEvent Event;
	while(In.GetEvent(&Event))
	{
		if(Event.Button == 0)
		{
			if(Event.Code == IIN::INPUT_UP)
			{
				if(absf(Pos.x - IDownx) < 10
				&& absf(Pos.y - IDowny) < 10)
				{
					// click
					MouseCursorEvent Ev;
					Ev.Code = MC_CLICK;
					Ev.x = x;
					Ev.y = y;
					Events[Events.Count()] = Ev;
				}
				else
				{
					// drag & drop
					MouseCursorEvent Ev;
					Ev.Code = MC_DRAGEND;
					Ev.x = x;
					Ev.y = y;
					Ev.x2 = Downx;
					Ev.y2 = Downy;
					Events[Events.Count()] = Ev;
				}
				Drag = false;
			}
			else if(Event.Code == IIN::INPUT_DOWN)
			{
				// drag start
				Downx = x;
				Downy = y;
				IDownx = Pos.x;
				IDowny = Pos.y;
				MouseCursorEvent Ev;
				Ev.Code = MC_DRAGSTART;
				Ev.x = x;
				Ev.y = y;
				Events[Events.Count()] = Ev;
				Drag = true;
			}
		}
		if(Event.Button == 1)
		{
			if(Event.Code == IIN::INPUT_UP)
			{
				// rightclick
				MouseCursorEvent Ev;
				Ev.Code = MC_RIGHTCLICK;
				Ev.x = x;
				Ev.y = y;
				Events[Events.Count()] = Ev;
			}
		}
	}
	// render
	DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
	DS->D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATER);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHAREF, 0x10);
	DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
	DS->D3DDevice->SetSamplerState(0,D3DSAMP_ADDRESSU,D3DTADDRESS_CLAMP); 
	DS->D3DDevice->SetSamplerState(0,D3DSAMP_ADDRESSV,D3DTADDRESS_CLAMP); 
	//DS->D3DDevice->SetTextureStageState(0, D3DTSS_ADDRESSU, D3DTADDRESS_CLAMP);
	//DS->D3DDevice->SetTextureStageState(0, D3DTSS_ADDRESSV, D3DTADDRESS_CLAMP);
	DS->D3DDevice->SetTexture(0, Texture);
	D3DMATRIX Matrix = {1,0,0,0,  0,1,0,0,  0,0,1,0,  0,0,0,1};
	DS->D3DDevice->SetTransform(D3DTS_VIEW, &Matrix);
	DS->D3DDevice->SetTransform(D3DTS_PROJECTION, &Matrix);
	D3DMATRIX WMatrix = {1,0,0,0,  0,1,0,0,  0,0,1,0,  x,y,0,1};
	DS->D3DDevice->SetTransform(D3DTS_WORLD, &WMatrix);
	if(Cursor)
		RS.RenderIndex(DS->D3DDevice, 0, 4, 0, 6);
	/*if(Drag && DragDrop)
	{
		DS->D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);
		DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
		DS->D3DDevice->SetTexture(0, DragTexture);
		D3DMATRIX WMatrix = {Downx - x,0,0,0,  0,Downy - y,0,0,  0,0,1,0,  x,y,0,1};
		DS->D3DDevice->SetTransform(D3DTS_WORLD, &WMatrix);
		DS->D3DDevice->SetRenderState(D3DRS_CULLMODE,D3DCULL_NONE);
		RS.RenderIndex(DS->D3DDevice, 4, 4, 0, 6);
		DS->D3DDevice->SetRenderState(D3DRS_CULLMODE,D3DCULL_CW);
	}*/
	DS->D3DDevice->SetTransform(D3DTS_WORLD, &Matrix);
	DS->D3DDevice->SetTransform(D3DTS_VIEW, &DS->ViewMtx);
	DS->D3DDevice->SetTransform(D3DTS_PROJECTION, &DS->ProjMtx);
	DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, TRUE);
	DS->D3DDevice->SetSamplerState(0,D3DSAMP_ADDRESSU,D3DTADDRESS_WRAP); 
	DS->D3DDevice->SetSamplerState(0,D3DSAMP_ADDRESSV,D3DTADDRESS_WRAP); 
	//DS->D3DDevice->SetTextureStageState(0, D3DTSS_ADDRESSU, D3DTADDRESS_WRAP);
	//DS->D3DDevice->SetTextureStageState(0, D3DTSS_ADDRESSV, D3DTADDRESS_WRAP);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, FALSE);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
}

bool MouseCursor::GetEvent(MouseCursor::MouseCursorEvent *Event)
{
	if(Events.Count())
	{
		MouseCursorEvent Ret = Events[0];
		for(int i = 0; i < Events.Count() - 1; i++)
			Events[i] = Events[i + 1];
		Events.SetCount(Events.Count() - 1);
		*Event = Ret;		
		return true;
	}
	else
		return false;
}

}
