/***********************************************************/
/*                                                         */
/* MouseCursor.h                                           */
/*                                                         */
/* created 26.08.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef MOUSECURSOR_H
#define MOUSECURSOR_H

#include "D3DStruct.h"
#include "Input.h"
#include "DynamicArray.h"
#include "RenderStruct.h"
#include "MathStructures.h"

// www.infinitecode.com 3d library
namespace I3D
{

enum
{
	MC_CLICK =		1, // cancels drag
	MC_DRAGSTART =	2,
	MC_DRAGEND =	3,
	MC_RIGHTCLICK =	4,
	MC_DRAG =		5
};

class MouseCursor
{
public:
	class MouseCursorEvent
	{
	public:
		int Code;
		float x, y, x2, y2;
	};

private:
	IIN::Input In;
	HWND Wnd;
	LPDIRECT3DTEXTURE9 Texture, StdTexture;//, DragTexture;
	RECT ClientRect;
	float x, y;
	int Buttons;
	float Downx, Downy;
	int IDownx, IDowny;
	bool Drag, /*DragDrop, */Cursor;
	ISL::DynamicArray<MouseCursorEvent, 0, 1> Events;
	RenderStruct<Vertex, I3D_FVFVERTEX, 16, 16> RS;

	void CreateView(D3DStruct *DS);

public:

	void Create(D3DStruct *DS, HWND Wnd);
	void Init3D(D3DStruct *DS, bool recurse);
	bool GetEvent(MouseCursorEvent *Event);

	void SetCursorTexture(LPDIRECT3DTEXTURE9 Texture)
	{
		MouseCursor::Texture = Texture;
	}

	void SetStdCursor(void)
	{
		Texture = StdTexture;
	}

	void Render(D3DStruct *DS);

	float GetX(void)
	{
		return x;
	}

	float GetY(void)
	{
		return y;
	}

	IMATH::Vector2D GetPos(void)
	{
		return IMATH::Vector2D(x, y);
	}

	float GetZ(void)
	{
		return (float)In.GetZ();
	}

	/*void SetDragDrop(bool Value)
	{
		DragDrop = Value;
	}*/

	void SetCursor(bool Value)
	{
		Cursor = Value;
	}
};

}

#endif
