/***********************************************************/
/*                                                         */
/* MultiHashTable.h                                        */
/*                                                         */
/* created 24.07.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef MULTIHASHTABLE_H
#define MULTIHASHTABLE_H

#include "DynamicArray.h"

// www.infinitecode.com Standard Data Structure Library
namespace ISL
{

template <class KeyType, class ValueType, int StartSize = 0, int Increment = 4, int SetStartSize = 0, int SetIncrement = 4>
class MultiHashTable
{
public:	
	template <class KeyType, class ValueType, int SetStartSize = 8, int SetIncrement = 8>
	class MultiHashTableEntry
	{
	public:	
		KeyType Key;
		class HTEValues: public DynamicArray<ValueType, SetStartSize, SetIncrement> {};
		HTEValues Values;

		MultiHashTableEntry(void)
		{
		}

		// copy functions
		MultiHashTableEntry(MultiHashTableEntry& HT)
		{
			*this = HT;
		}

		MultiHashTableEntry& operator = (MultiHashTableEntry& HT)
		{
			Key = HT.Key;
			Values = HT.Values;
			return *this;
		}

		bool operator == (MultiHashTableEntry& HT)
		{
			return (Key == HT.Key) && (Values == HT.Values);
		}
	};

	class Entry: public MultiHashTableEntry<KeyType, ValueType, SetStartSize, SetIncrement> {};

private:
	DynamicArray<Entry, StartSize, Increment> Table;

public:

	MultiHashTable(void)
	{

	}

	// copy functions
	MultiHashTable(MultiHashTable& HT)
	{
		*this = HT;
	}

	MultiHashTable& operator = (MultiHashTable& HT)
	{
		Table = HT.Table;
		return *this;
	}

	bool operator == (MultiHashTable& HT)
	{
		return Table == HT.Table;
	}

	void Clear(void)
	{
		Table.Clear();
	}

	void Add(KeyType Key, ValueType Value)
	{
		for(int i = 0; i < Table.Count(); i++)
		{
			if(Table[i].Key == Key)
			{
				/*for(int j = 0; j < Table[i].Values.Count(); j++)
				{
					if(Table[i].Values[j] == Value)
						return;
				}*/
				// else
				Table[i].Values[Table[i].Values.Count()] = Value;
				return;
			}
		}
		// else
		int LastElement = Table.Count();
		Table[LastElement].Key = Key;
		Table[LastElement].Values[0] = Value;
	}

	// only add, if it's not yet an element of the set
	void AddToSet(KeyType Key, ValueType Value)
	{
		for(int i = 0; i < Table.Count(); i++)
		{
			if(Table[i].Key == Key)
			{
				// if it's already an element, return
				for(int j = 0; j < Table[i].Values.Count(); j++)
					if(Table[i].Values[j] == Value)
						return;
				// otherwise, add it
				Table[i].Values[Table[i].Values.Count()] = Value;
				return;
			}
		}
		// else
		int LastElement = Table.Count();
		Table[LastElement].Key = Key;
		Table[LastElement].Values[0] = Value;
	}

	void DeleteKey(KeyType Key)
	{
		for(int i = 0; i < Table.Count(); i++)
		{
			if(Table[i].Key == Key)
			{
				if(i != Table.GetSize() - 1)
				{
					Table[i] = Table[Table.Count() - 1];
				}
				Table.SetSize(Table.Count() - 1);
				return;
			}
		}
	}

	void Delete(KeyType Key, ValueType Value)
	{
		for(int i = 0; i < Table.Count(); i++)
		{
			if(Table[i].Key == Key)
			{
				Table[i].Values.Delete(Value);
				return;
			}
		}
	}

/*
	DynamicArray<ValueType, SetStartSize, SetIncrement> *GetValuesPtr(KeyType Key)
	{
		for(int i = 0; i < Table.Count(); i++)
		{
			if(Table[i].Key == Key)
			{
				return &Table[i].Values;
			}
		}
		return NULL;
	}*/

	KeyType GetKey(ValueType Value)
	{
		for(int i = 0; i < Table.Count(); i++)
		{
			for(int j = 0; j < Table[i].Values.Count(); j++)
			{
				if(Table[i].Values[j] == Value)
				{
					return Table[i].Key;
				}
			}
		}
		return NULL;
	}

	// return a pointer to the key
	KeyType *GetKeyPtr(ValueType Value)
	{
		for(int i = 0; i < Table.Count(); i++)
		{
			for(int j = 0; j < Table[i].Values.Count(); j++)
			{
				if(Table[i].Values[j] == Value)
				{
					return &Table[i].Key;
				}
			}
		}
		return NULL;
	}

	KeyType GetKey(int Num)
	{
		return Table[Num].Key;
	}

	// return a pointer to the key
	KeyType *GetKeyPtr(int Num)
	{
		return &Table[Num].Key;
	}

	ValueType GetValue(KeyType Key)
	{
		for(int i = 0; i < Table.Count(); i++)
		{
			if(Table[i].Key == Key)
			{
				return Table[i].Values[0];
			}
		}
		return NULL;
	}

	// return a pointer to the value
	ValueType *GetValuePtr(KeyType Key)
	{
		for(int i = 0; i < Table.Count(); i++)
		{
			if(Table[i].Key == Key)
			{
				return &Table[i].Values[0];
			}
		}
		return NULL;
	}

	ValueType *GetValuePtr(int Num)
	{
		return &Table[Num].Values[0];
	}

	// return a pointer to the values (dynamic array of values)
	typename Entry::HTEValues *GetValuesPtr(KeyType Key)
	{
		for(int i = 0; i < Table.Count(); i++)
		{
			if(Table[i].Key == Key)
			{
				return &Table[i].Values;
			}
		}
		return NULL;
	}

	// return a pointer to the values (dynamic array of values)
	typename Entry::HTEValues *GetValuesPtr(int Num)
	{
		return &Table[Num].Values;
	}

	// return the number of keys
	int CountKeys(void)
	{
		return Table.Count();
	}
};

}

#endif
