/***********************************************************/
/*                                                         */
/* ParticleEffect.cpp                                      */
/*                                                         */
/* created 17.02.2003                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "ParticleEffect.h"
#include "ParticleFXRenderer.h"
#include "WorldPFXInterface.h"
#include "IDParticleFX.h"
#include "IMath.h"

// www.infinitecode.com 3d library
namespace I3D
{

int ParticleEffect::TypeID;

void ParticleEffect::Create(ParticleEffect::ParticleEffectSpecs *Specs)
{
	Loaded = true;
	ParticleEffect::Specs = *Specs;
	ParticleEffect::Specs.Time += ParticleEffect::Specs.Delay;
	Pos = Specs->Pos;
	OverrideTexture = 0;
	RotationDir = ParticleEffect::Specs.PFXLib->Effects[ParticleEffect::Specs.ID].RotationDirRandom ? (float)((rand() % 2) * 2 - 1) : 1;
	if(ParticleEffect::Specs.PFXLib->Effects[ParticleEffect::Specs.ID].RotationStartRandom)
		Rotation = 2 * PI * ((rand() % 1000) / 1000.0f);
	else
		Rotation = 0;

	LifeTime = ParticleEffect::Specs.PFXLib->Effects[ParticleEffect::Specs.ID].LifeTime;
	RelTime = 0;
	int XFrames = ParticleEffect::Specs.PFXLib->Effects[ParticleEffect::Specs.ID].XFrames;
	int YFrames = ParticleEffect::Specs.PFXLib->Effects[ParticleEffect::Specs.ID].YFrames;
	Offset = ParticleEffect::Specs.PFXLib->Effects[ParticleEffect::Specs.ID].RandomizeOffset ? rand() % (XFrames * YFrames) : 0;

	ParticleEffect::Specs.SM->AddManagedObject(this);
	// create vertices
	RS.ClearVertex();
	RS.AddVertex(LVertex(Vector(-1, -1, 0), 0xffffffff, Vector2D(0, 0)));
	RS.AddVertex(LVertex(Vector( 1, -1, 0), 0xffffffff, Vector2D(1, 0)));
	RS.AddVertex(LVertex(Vector( 1,  1, 0), 0xffffffff, Vector2D(1, 1)));
	RS.AddVertex(LVertex(Vector(-1,  1, 0), 0xffffffff, Vector2D(0, 1)));
	RS.CreateVBuf(ParticleEffect::Specs.DS->D3DDevice);
	RS.ClearIndex();
	RS.AddIndex(0);
	RS.AddIndex(1);
	RS.AddIndex(3);
	RS.AddIndex(1);
	RS.AddIndex(2);
	RS.AddIndex(3);
	RS.CreateIBuf(ParticleEffect::Specs.DS->D3DDevice);
}

void ParticleEffect::Physics(D3DStruct *DS)
{
	float DTime = DS->Sp->GameTime - Specs.Time;
	RelTime = DTime / LifeTime;
	if(RelTime >= 0)
	{
		Pos += DS->Sp->Delta * Specs.PFXLib->Effects[Specs.ID].Movement;
		Rotation += DS->Sp->Delta * RotationDir * Specs.PFXLib->Effects[Specs.ID].Rotation;
		if(RelTime >= 1.0f)
			Specs.IWorldPFX->Delete(this);
	}
}

void ParticleEffect::SetVisible(bool Visible)
{
	if(Visible != ManagedObject::Visible)
	{
		if(Visible)
			Specs.PFXRenderer->AddEffect(this);
		else
			Specs.PFXRenderer->DeleteEffect(this);
	}
	ManagedObject::Visible = Visible;
}

ParticleEffect::~ParticleEffect(void)
{
	Specs.PFXRenderer->DeleteEffect(this);
	ParticleEffect::Specs.SM->DeleteManagedObject(this);
}

}
