/***********************************************************/
/*                                                         */
/* ParticleFXRenderer.cpp                                  */
/*                                                         */
/* created 19.02.2003                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "ParticleFXRenderer.h"
#include "TextureManager.h"
#include "IDParticleFX.h"

// www.infinitecode.com 3d library
namespace I3D
{

void ParticleFXRenderer::Create(ParticleFXRenderer::ParticleFXRendererSpecs *Specs)
{
	ParticleFXRenderer::Specs = *Specs;
	for(int i = 0; i < ParticleFXRenderer::Specs.PFXLib->Effects.Count(); i++)
	{
		if(*ParticleFXRenderer::Specs.PFXLib->Effects[i].Texture)
		{
			const char *TexName = ParticleFXRenderer::Specs.PFXLib->Effects[i].Texture;
			Texture[Texture.Count()] = TextureManager::GetTexture(ParticleFXRenderer::Specs.DS, TexName);
			ParticleFXRenderer::Specs.PFXLib->Effects[i].TextureID = Texture.Count() - 1;
		}
	}
	/*PFXData = ITL::IniLoader("ParticleEffects.ini");
	for(int i = 0; i < PFXData.CountGroups(); i++)
		for(int j = 0; j < PFXData.CountKeys(i);j++)
		{
			const char *Key = PFXData.GetKey(i, j);
			if(!strcmp(Key, "Texture"))
			{
				char Str[1024], FName[1024], *FNPtr;
				strcpy(FName, PFXData.GetValue(i, j));
				FNPtr = FName;
				if(*FName == '\"')
					FNPtr++;
				if(FName[strlen(FName) - 1] == '\"')
					FName[strlen(FName) - 1] = 0;
				sprintf(Str, "Textures\\%s", FNPtr);
				Texture[Texture.Count()] = TextureManager::GetTexture(DS, Str);
			}
		}*/
/*	Texture[0] = TextureManager::GetTexture(DS, "Textures\\Flare.bmp");
	Texture[1] = TextureManager::GetTexture(DS, "Textures\\explode_1.bmp");
	Texture[2] = TextureManager::GetTexture(DS, "Textures\\explode_2.bmp");
	Texture[3] = TextureManager::GetTexture(DS, "Textures\\smoke.bmp");*/

}

void ParticleFXRenderer::AddEffect(ParticleEffect *PFX)
{
	ParticleFXRenderer::PFX[ParticleFXRenderer::PFX.Count()] = PFX;
}

void ParticleFXRenderer::DeleteEffect(ParticleEffect *PFX)
{
	ParticleFXRenderer::PFX.Delete(PFX);
}

void ParticleFXRenderer::SortPFX(const I3D::D3DStruct *DS)
{
	// TODO: replace by faster sorting algorithm
	/*bool change;
	int Num = PFX.Count()  / 4;
	for(int k = 0; k < Num; k++)
	{
		change = false;
		for(int i = 0; i < PFX.Count() - 1; i++)
		{
			if((DS->ViewMtx * PFX[i]->GetPos()).z < (DS->ViewMtx * PFX[i + 1]->GetPos()).z)
			{
				ParticleEffect *Swap;
				Swap = PFX[i];
				PFX[i] = PFX[i + 1];
				PFX[i + 1] = Swap;
				change = true;
			}
		}
		if(!change)
			break;
	}*/

	// bucket by texture
	ISL::DynamicArray<SortablePFX *> *PFXTex = new ISL::DynamicArray<SortablePFX *>[Texture.Count()];
	for(int i = 0; i < PFX.Count(); i++)
	{
		ParticleFXLib::Effect *Lib = &Specs.PFXLib->Effects[PFX[i]->GetID()];
		int TexID = Lib->TextureID;
		PFXTex[TexID].Add(new SortablePFX(PFX[i], (DS->ViewMtx * PFX[i]->GetPos()).z));
	}
	// sort PFXTex
	for(int j = 0; j < Texture.Count(); j++)
	{
		int Num = PFXTex[j].Count();// / 4;
		bool change;
		for(int k = 0; k < Num; k++)
		{
			change = false;
			for(int i = 0; i < PFXTex[j].Count() - 1; i++)
			{
				if(PFXTex[j][i]->z < PFXTex[j][i + 1]->z)
				{
					SortablePFX *Swap;
					Swap = PFXTex[j][i];
					PFXTex[j][i] = PFXTex[j][i + 1];
					PFXTex[j][i + 1] = Swap;
					change = true;
				}
			}
			if(!change)
				break;
		}
	}
	PFX.ClearElements();
	// merge
	// get most far particle
	float z = -1E9;
	int CurrentTex = 0;
	for(int j = 0; j < Texture.Count(); j++)
	{
		if(PFXTex[j].Count() && PFXTex[j][0]->z > z)
		{
			z = PFXTex[j][0]->z;
			ParticleFXLib::Effect *Lib = &Specs.PFXLib->Effects[PFXTex[j][0]->PFX->GetID()];
			CurrentTex = Lib->TextureID;
		}
	}
	float Threshold = 5.0f;
	int *Ptr = new int[Texture.Count()];
	for(int j = 0; j < Texture.Count(); j++)
		Ptr[j] = 0;
	do
	{
		// add near particles
		while(Ptr[CurrentTex] != PFXTex[CurrentTex].Count() && z - PFXTex[CurrentTex][Ptr[CurrentTex]]->z < Threshold)
		{
			PFX.Add(PFXTex[CurrentTex][Ptr[CurrentTex]]->PFX);
			Ptr[CurrentTex]++;
		}
		// get next particle
		z = -1E9;
		for(int j = 0; j < Texture.Count(); j++)
		{
			if(Ptr[j] != PFXTex[j].Count() && PFXTex[j][Ptr[j]]->z > z)
			{
				z = PFXTex[j][Ptr[j]]->z;
				ParticleFXLib::Effect *Lib = &Specs.PFXLib->Effects[PFXTex[j][Ptr[j]]->PFX->GetID()];
				CurrentTex = Lib->TextureID;
			}
		}
	} while(z != -1E9);
	delete[] Ptr;
	for(int j = 0; j < Texture.Count(); j++)
		PFXTex[j].ClearPtr();
	delete[] PFXTex;
}

void ParticleFXRenderer::Init3D(I3D::D3DStruct *DS, bool recurse)
{
	for(int i = 0; i < ParticleFXRenderer::Specs.PFXLib->Effects.Count(); i++)
	{
		if(*ParticleFXRenderer::Specs.PFXLib->Effects[i].Texture)
		{
			const char *TexName = ParticleFXRenderer::Specs.PFXLib->Effects[i].Texture;
			Texture[Texture.Count()] = TextureManager::GetTexture(ParticleFXRenderer::Specs.DS, TexName);
			ParticleFXRenderer::Specs.PFXLib->Effects[i].TextureID = Texture.Count() - 1;
		}
	}
}

void ParticleFXRenderer::Render(const I3D::D3DStruct *DS)
{
	// sort particles
	SortPFX(DS);
	//qsort(PFX.GetPtr(), PFX.Count(), sizeof(ParticleEffect *), ParticleEffect::HeightCompare);
	int i;
	try
	{
		// set renderstates for alpha blending (transparency)
		DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
		DS->D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);
		DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATER);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHAREF, 0x10);
		DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
		DS->D3DDevice->SetSamplerState(0,D3DSAMP_ADDRESSU,D3DTADDRESS_CLAMP); 
		DS->D3DDevice->SetSamplerState(0,D3DSAMP_ADDRESSV,D3DTADDRESS_CLAMP); 
		//DS->D3DDevice->SetTextureStageState(0, D3DTSS_ADDRESSU, D3DTADDRESS_CLAMP);
		//DS->D3DDevice->SetTextureStageState(0, D3DTSS_ADDRESSV, D3DTADDRESS_CLAMP);
		// set transformation
		Matrix M;
		M.SetIdentityMatrix();
		//M.SetTranslationMatrix(Vector(0, 4, 0));
		DS->D3DDevice->SetTransform(D3DTS_WORLD, &M);
		DS->D3DDevice->SetTransform(D3DTS_VIEW, &M);
		LPDIRECT3DTEXTURE9 CurrentTexture = 0;
/*		RS.ClearVertexElements();
		RS.ClearIndexElements();*/
		int TexID;
		IMATH::Vector Shake(0, 0, 0);
		for(i = 0; i < PFX.Count(); i++)
		{
			if(PFX[i]->RelTime >= 0)
			{
				ParticleFXLib::Effect *Lib = &Specs.PFXLib->Effects[PFX[i]->GetID()];
				TexID = Lib->TextureID;
				//RS.ClearVertexElements();
				/*RS.RenderList[0].x = PFXPos.x - x - y;
				RS.RenderList[0].y = PFXPos.y + x - y;
				RS.RenderList[0].z = PFXPos.z;
				RS.RenderList[0].c = RGBA;
				RS.RenderList[0].tu = FrameX;
				RS.RenderList[0].tv = FrameY;
				RS.RenderList[1].x = PFXPos.x - x + y;
				RS.RenderList[1].y = PFXPos.y - x - y;
				RS.RenderList[1].z = PFXPos.z;
				RS.RenderList[1].c = RGBA;
				RS.RenderList[1].tu = FrameX;
				RS.RenderList[1].tv = FrameY + SizeY;
				RS.RenderList[2].x = PFXPos.x + x + y;
				RS.RenderList[2].y = PFXPos.y - x + y;
				RS.RenderList[2].z = PFXPos.z;
				RS.RenderList[2].c = RGBA;
				RS.RenderList[2].tu = FrameX + SizeX;
				RS.RenderList[2].tv = FrameY + SizeY;
				RS.RenderList[3].x = PFXPos.x + x + y;
				RS.RenderList[3].y = PFXPos.y - x + y;
				RS.RenderList[3].z = PFXPos.z;
				RS.RenderList[3].c = RGBA;
				RS.RenderList[3].tu = FrameX + SizeX;
				RS.RenderList[3].tv = FrameY + SizeY;
				RS.RenderList[4].x = PFXPos.x + x - y;
				RS.RenderList[4].y = PFXPos.y + x + y;
				RS.RenderList[4].z = PFXPos.z;
				RS.RenderList[4].c = RGBA;
				RS.RenderList[4].tu = FrameX + SizeX;
				RS.RenderList[4].tv = FrameY;
				RS.RenderList[5].x = PFXPos.x - x - y;
				RS.RenderList[5].y = PFXPos.y + x - y;
				RS.RenderList[5].z = PFXPos.z;
				RS.RenderList[5].c = RGBA;
				RS.RenderList[5].tu = FrameX;
				RS.RenderList[5].tv = FrameY;
				RS.CreateVBuf(DS->D3DDevice);*/
				// set texture
				LPDIRECT3DTEXTURE9 NewTexture = PFX[i]->OverrideTexture?PFX[i]->OverrideTexture:Texture[TexID];
				if(NewTexture != CurrentTexture)
				{
					DS->D3DDevice->SetTexture(0, NewTexture);
					CurrentTexture = NewTexture;
				}
				// render
				//const I3D::RenderStruct<I3D::LVertex, I3D_FVFLVERTEX, 4, 4> *R = &PFX[i]->GetRS();
				//PFX[i]->GetRS().RenderVertex(DS->D3DDevice);
				//R->RenderVertex(DS->D3DDevice);
				//PFX[i]->RenderToRS(&RS);
				PFX[i]->Render();
				// shake
				Shake = Specs.PFXLib->Effects[PFX[i]->GetID()].Shake * IMATH::Vector(0.01f * (rand() % 201 - 100), 0.01f * (rand() % 201 - 100), 0.01f * (rand() % 201 - 100));
				Specs.ForceFeedback->AddShakeVector(PFX[i]->GetPos(), Shake);
			}
		}
		/*RS.CreateVBuf(DS->D3DDevice);
		RS.CreateIBuf(DS->D3DDevice);
		RS.RenderIndex(DS->D3DDevice);*/
		// reset renderstates
		DS->D3DDevice->SetTransform(D3DTS_VIEW, &DS->ViewMtx);
		DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, TRUE);
		DS->D3DDevice->SetSamplerState(0,D3DSAMP_ADDRESSU,D3DTADDRESS_WRAP); 
		DS->D3DDevice->SetSamplerState(0,D3DSAMP_ADDRESSV,D3DTADDRESS_WRAP); 
		//DS->D3DDevice->SetTextureStageState(0, D3DTSS_ADDRESSU, D3DTADDRESS_WRAP);
		//DS->D3DDevice->SetTextureStageState(0, D3DTSS_ADDRESSV, D3DTADDRESS_WRAP);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, FALSE);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
	}
	catch(char *s)
	{
		throw(s);
	}
	catch(...)
	{
		throw("ParticleFXRenderer::Render");
	}
}

}
