/***********************************************************/
/*                                                         */
/* ParticleFXRenderer.h                                    */
/*                                                         */
/* created 19.02.2003                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef PARTICLEFXRENDERER_H
#define PARTICLEFXRENDERER_H

#include "ParticleEffect.h"
#include "D3DStruct.h"
#include "DynamicArray.h"
#include "RenderStruct.h"
#include "D3DStructures.h"
#include "INILoader.h"
#include "ParticleFXLib.h"
#include "ForceFeedbackInterface.h"

// www.infinitecode.com 3d library
namespace I3D
{

class ParticleFXRenderer
{
public:
	class ParticleFXRendererSpecs
	{
	public:
		D3DStruct *DS;
		ParticleFXLib *PFXLib;
		ForceFeedbackInterface *ForceFeedback;
	};

	struct SortablePFX
	{
		ParticleEffect *PFX;
		float z;
		SortablePFX(ParticleEffect *P, float Z): PFX(P), z(Z) {}
	};

private:
	ISL::DynamicArray<ParticleEffect *> PFX;
	I3D::RenderStruct<I3D::LVertex, I3D_FVFLVERTEX, 64, 64> RS;
	ISL::DynamicArray<LPDIRECT3DTEXTURE9, 0, 1> Texture;
	ParticleFXRendererSpecs Specs;

	void SortPFX(const I3D::D3DStruct *DS);

public:
	void Create(ParticleFXRendererSpecs *Specs);
	void Init3D(I3D::D3DStruct *DS, bool recurse);

	void AddEffect(ParticleEffect *PFX);
	void DeleteEffect(ParticleEffect *PFX);
	void Render(const I3D::D3DStruct *DS);
};

}

#endif
