/***********************************************************/
/*                                                         */
/* ParticleFXViewer.cpp                                    */
/*                                                         */
/* created 02.03.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "MathStructures.h"
#include "LogFile.h"
#include "MessageWindow.h"
#include "Input.h"
#include "IMath.h"
#include "TextureManager.h"
#include "ParticleFXViewer.h"

#include <direct.h>

// www.infinitecode.com 3d library
namespace I3D
{

void ParticleFXViewer::Create(HINSTANCE ThisInstance, int WindowStyle)
{
	IWorldPFX.SetPFXViewer(this);
	// TODO: implement fullscreen display
	WindowStyle = SW_SHOWNORMAL;
	// clear the error output file
	ITL::LogFile::Clear("ErrorPFXViewer.txt");
	ParticleFXViewer::Instance = ThisInstance;
	//strcpy(ModelFile, "Models\\tank.t4m");
	strcpy(PFXFile, "Textures\\smoke.bmp");
	//strcpy(ModelFile2, "Models\\art1_weapon.x");
	//strcpy(ModelFile, "Models\\first.x");
	// Create the main window and initialize D3D
	try
	{
		Wnd.Create(WindowStyle, "Model Editor", 400, 400);
		// menu stuff
		MENUITEMINFO MenuInfo;
		memset(&MenuInfo, 0, sizeof(MenuInfo));

		HMENU Menu = CreateMenu(); // menu
		HMENU FileMenu = CreateMenu(); // file menu
		HMENU RenderMenu = CreateMenu(); // render menu
		// Load...
		MenuInfo.cbSize = sizeof(MenuInfo);
		MenuInfo.fMask = MIIM_ID | MIIM_TYPE | MIIM_SUBMENU | MIIM_STATE;
		MenuInfo.fType = MFT_STRING;
		MenuInfo.wID = 1024;
		MenuInfo.dwTypeData = "Load...";
		InsertMenuItem(FileMenu, -1, true, &MenuInfo); // add to file menu
		// file menu
		MenuInfo.wID = 1030;
		MenuInfo.dwTypeData = "File";
		MenuInfo.hSubMenu = FileMenu;
		InsertMenuItem(Menu, -1, true, &MenuInfo); // add to menu
		// add menu to window
		SetMenu(Wnd.GetHWnd(), Menu);
	}
	catch(exception& Ex)
	{
		ITL::LogFile::Msg("ErrorPFXViewer.txt", (char *)Ex.what());
		ITL::MessageWindow::Message((char *)Ex.what(), "Error during D3D Window initialisation");
		exit(-1);
	}
	Wnd.AddListener(this);
	In.Create(Wnd.GetHWnd());
	DS.Create(Wnd.D3DDevice, &Sp);
	PFXLib.Create();
	ParticleFXRenderer::ParticleFXRendererSpecs Specs;
	Specs.DS = &DS;
	Specs.PFXLib = &PFXLib;
	PFXRenderer.Create(&Specs);
	ParticleEffect::ParticleEffectSpecs PFXSpecs;
	PFXSpecs.ID = 2;
	PFXSpecs.PFXLib = &PFXLib;
	PFXSpecs.PFXRenderer = &PFXRenderer;
	PFXSpecs.Pos = IMATH::Vector(0, 0, 0);
	PFXSpecs.Time = 0;
	PFXSpecs.SM = &SM;
	PFXSpecs.IWorldPFX = &IWorldPFX;
	PFX.Create(&PFXSpecs);
	PFXRenderer.AddEffect(&PFX);

	Mouse.Create(&DS, Wnd.GetHWnd());
	Mouse.SetDragDrop(false);
	Angle = 0.5f * 2 * PI;
	Angle2 = 1.0f * 2 * PI;
	Zoom = 60.0f;
	BackgroundColor = 0x00000000;
	Init3D();
}

void ParticleFXViewer::Init3D(void)
{
	DS.D3DDevice->SetRenderState(D3DRS_LIGHTING,TRUE);
	DS.D3DDevice->SetRenderState(D3DRS_ZENABLE,TRUE);
	DS.D3DDevice->SetRenderState(D3DRS_CULLMODE,D3DCULL_CW);
    DS.D3DDevice->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
    DS.D3DDevice->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
    DS.D3DDevice->SetTextureStageState(0, D3DTSS_MIPFILTER, D3DTEXF_POINT);
//	DS.D3DDevice->SetRenderState(D3DRS_AMBIENT, 0x00000000);
	DS.D3DDevice->SetRenderState(D3DRS_AMBIENT, 0x80808080);
	DS.D3DDevice->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATER);
	DS.D3DDevice->SetRenderState(D3DRS_ALPHAREF, 0x60);
	DS.D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	DS.D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
	if(D3DFlags & GAME_D3DFLAG_COLORKEY)
		DS.D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE);
	else
		DS.D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, FALSE);
	if(D3DFlags & GAME_D3DFLAG_ALPHABLEND)
		DS.D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
	else
		DS.D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
	if(D3DFlags & GAME_D3DFLAG_NOCULLING)
		DS.D3DDevice->SetRenderState(D3DRS_CULLMODE,D3DCULL_NONE);
	else
		DS.D3DDevice->SetRenderState(D3DRS_CULLMODE,D3DCULL_CW);

	// for debugging:
	//DS.D3DDevice->SetRenderState(D3DRS_AMBIENT, 0xFFFFFFFF);
	if(D3DFlags & GAME_D3DFLAG_WIREFRAME)
		DS.D3DDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME);

	D3DMATERIAL8 Material;
	memset(&Material, 0, sizeof(D3DMATERIAL8));
	Material.Diffuse.r = 1.0f;
	Material.Diffuse.g = 1.0f;
	Material.Diffuse.b = 1.0f;
	Material.Diffuse.a = 1.0f;
	Material.Ambient.r = 1.0f;
	Material.Ambient.g = 1.0f;
	Material.Ambient.b = 1.0f;
	Material.Ambient.a = 1.0f;
	Material.Specular.r = 1.0f; // Licht spiegeln
	Material.Specular.g = 1.0f;
	Material.Specular.b = 1.0f;
	Material.Specular.a = 1.0f;
	Material.Power = 0.0f;
	//  Material.power = (float)10.0;
	DS.D3DDevice->SetMaterial(&Material);
	// lighting
	D3DLIGHT8 Light;
	memset(&Light, 0, sizeof(D3DLIGHT8));
	Light.Type = D3DLIGHT_DIRECTIONAL;
	Light.Diffuse.r = 1.0f;
	Light.Diffuse.g = 1.0f;
	Light.Diffuse.b = 1.0f;
	Light.Specular.r=Light.Diffuse.r;
	Light.Specular.g=Light.Diffuse.g;
	Light.Specular.b=Light.Diffuse.b;
	Light.Attenuation0 = 0.0;
	Light.Attenuation1 = 0.0;
	Light.Attenuation2 = 0.0;
	Light.Range = 0;
	Light.Position.x = 0;
	Light.Position.y = 0;
	Light.Position.z = 0;
	Light.Direction.x = -1.0f;
	Light.Direction.y = -1.5f;
	Light.Direction.z = -1.0f;
	DS.D3DDevice->SetLight(1, &Light);
	DS.D3DDevice->LightEnable(1, true);
	//Wrld.Init3D(DS);
}

// called asynchronously!
void ParticleFXViewer::OnEvent(I3D::D3DWEvent Event)
{
	switch(Event.GetID())
	{
	case I3D::D3DW_EVENT_RESET:
		Init3D();
		break;
	default:
		Events[Events.Count()] = Event;
	}
}

void ParticleFXViewer::MenuCheck(int ID)
{
	MENUITEMINFO MII;
	MII.cbSize = sizeof(MII);
	MII.fMask = MIIM_STATE;
	GetMenuItemInfo(GetMenu(Wnd.GetHWnd()), ID, false, &MII);
	if(MII.fState == MFS_CHECKED)
		MII.fState = MFS_UNCHECKED;
	else
		MII.fState = MFS_CHECKED;
	SetMenuItemInfo(GetMenu(Wnd.GetHWnd()), ID, false, &MII);
}

void ParticleFXViewer::MouseHandler(void)
{
	bool Inside = true;
	if(Mouse.GetX() < -1 || Mouse.GetX() > 1
	|| Mouse.GetY() < -1 || Mouse.GetY() > 1)
		Inside = false;
	if(Inside)
	{
		static float LastZ = Mouse.GetZ();
		float dZ = Mouse.GetZ() - LastZ;
		static float InterpolateddZ = 0;
		InterpolateddZ *= 0.9f;
		InterpolateddZ += 0.1f * dZ;
		if(absf(InterpolateddZ) > 0.001f)
		{
			// zoom
			Zoom *= 1 - 0.002f * InterpolateddZ;
		}
		LastZ = Mouse.GetZ();
	}
}

// called synchronously
void ParticleFXViewer::EventHandler(void)
{
	if(Events.Count())
	{
		for(int i = 0; i < Events.Count(); i++)
		{
			switch(Events[i].GetID())
			{
			case I3D::D3DW_EVENT_WINDOWSEVENT:
				switch(Events[i].GetInfo())
				{
				case WM_KEYDOWN:
					KeyboardInput(Events[i].GetWParam());
					break;
				case WM_COMMAND:
					switch(Events[i].GetWParam() & 0xFFFF)
					{

					case 1024:
						char Dir[1024];
						getcwd(Dir, 1024);
						// load
						OPENFILENAME OFN;
						memset(&OFN, 0, sizeof(OFN));
						OFN.lStructSize = sizeof(OFN);
						OFN.hwndOwner = Wnd.GetHWnd();
						OFN.lpstrFilter = "Model Files (*.bmp)\0*.BMP\0\0";
						OFN.lpstrDefExt = "t4m";
						OFN.lpstrInitialDir = "Textures";
						OFN.lpstrTitle = "Load Model";
						OFN.lpstrFile = new char[1024];
						OFN.nMaxFile = 1024;
						*OFN.lpstrFile = 0;
						//OFN.lpstrFile = NULL;
						/*strcpy(OFN.lpstrFile, "Models\\Tree.t4m");
						OFN.nFileOffset = 8;
						OFN.nFileExtension = 13;*/
						OFN.Flags = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY /*| OFN_SHAREAWARE*/; // OFN_OVERWRITEPROMPT
						if(GetOpenFileName(&OFN))
						{
							strcpy(PFXFile, OFN.lpstrFile);
							ParticleEffect::ParticleEffectSpecs Specs;
							Specs.ID = 2;
							Specs.PFXLib = &PFXLib;
							Specs.PFXRenderer = &PFXRenderer;
							Specs.Pos = IMATH::Vector(0, 0, 0);
							Specs.Time = 0;
							Specs.SM = &SM;
							Specs.IWorldPFX = &IWorldPFX;
							PFX.Create(&Specs);
							PFXRenderer.AddEffect(&PFX);
							delete OFN.lpstrFile;
							RELEASE(Texture);
							Texture = 0;
							Angle = 0.5f * 2 * PI;
							Angle2 = 1.0f * 2 * PI;
							Texture = TextureManager::Load(DS.D3DDevice, PFXFile, 0, 0);
							PFX.SetTexture(Texture);
						}
						chdir(Dir);
						break;
					}
					break;
				case WM_MOUSEMOVE:
					if(Events[i].GetWParam() & MK_LBUTTON)
					{
						int x = Events[i].GetLParam() & 0xFFFF;
						int y = Events[i].GetLParam() >> 16;
						Angle = 2 * PI * x / Wnd.GetWidth();
						Angle2 = 2 * PI * (0.5f + (float)y / Wnd.GetHeight());
					}
					break;
				}
				break;
			}
		}
		Events.Clear();
	}
}

void ParticleFXViewer::KeyboardInput(char c)
{
	switch(c)
	{
	case 27:
		// Escape
		PostQuitMessage(0);
		break;
	case 'w':
	case 'W':
		D3DFlags ^= GAME_D3DFLAG_WIREFRAME;
		if(D3DFlags & GAME_D3DFLAG_WIREFRAME)
			DS.D3DDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME);
		else
			DS.D3DDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);
		break;
	}
}

void ParticleFXViewer::MouseClick(void)
{
}

void ParticleFXViewer::MainLoop(void)
{
	int State = 0;
	try
	{
		State = 1;
		In.Update();
		EventHandler();
		MouseHandler();
		Sp.Update();
		State = 2;
		if(DS.D3DDevice->BeginScene())
		{
			State = 3;
			DS.D3DDevice->Clear(0, NULL, D3DCLEAR_ZBUFFER | D3DCLEAR_TARGET, BackgroundColor, 1.0f, 0);
			D3DMATRIX Matrix = {1,0,0,0,  0,1,0,0,  0,0,1,0,  0,0,0,1};
			// world and view matrix
			DS.D3DDevice->SetTransform(D3DTS_WORLD, &Matrix);
			DS.D3DDevice->SetTransform(D3DTS_VIEW, &Matrix);
			DS.D3DDevice->SetTransform(D3DTS_PROJECTION, &Matrix);
			// projection matrix
			// distance eye->monitor in meters
			const float EyeDistance = 0.6f;
			// (vertical) monitor size in meters
			const float MonitorSize = 0.33f;
			// perspective projection: [x, y, z] /= (1 + z)
			D3DMATRIX PMatrix = {1,0,0,0,  0,1,0,0,  0,0,1,1,  0,0,0,1};
			DS.D3DDevice->MultiplyTransform(D3DTS_PROJECTION, &PMatrix);
			// adjust to monitor size and position
			// screen 3:4  = 0.75f
			D3DMATRIX TMatrix3 = {0.75f * 2.0f / MonitorSize,0,0,0,  0,2.0f / MonitorSize,0,0,  0,0,1.0f / EyeDistance,0,  0,0,0,1};
			DS.D3DDevice->MultiplyTransform(D3DTS_PROJECTION, &TMatrix3);
			DS.D3DDevice->GetTransform(D3DTS_PROJECTION, &DS.ProjMtx);

			// render
			//Angle += 0.07f;
			D3DMATRIX VMatrix = {1,0,0,0,  0,1,0,0,  0,0,1,0,  0,0,0,1};
			//DS.D3DDevice->MultiplyTransform(D3DTS_VIEW, &VMatrix);
			D3DMATRIX VMatrix2 = {1,0,0,0,  0,1,0,0,  0,0,1,0,  0,0,Zoom,1};
			DS.D3DDevice->MultiplyTransform(D3DTS_VIEW, &VMatrix2);

			D3DMATRIX WMatrix2 = {1,0,0,0,  0,(float)cos(Angle2),-(float)sin(Angle2),0,  0,(float)sin(Angle2),(float)cos(Angle2),0,  0,0,0,1};
			DS.D3DDevice->SetTransform(D3DTS_WORLD, &WMatrix2);

			D3DMATRIX WMatrix = {(float)cos(Angle),0,(float)sin(Angle),0,  0,1,0,0,  -(float)sin(Angle),0,(float)cos(Angle),0,  0,0,0,1};
			DS.D3DDevice->MultiplyTransform(D3DTS_WORLD, &WMatrix);

			I3D::Matrix M, P;
			DS.D3DDevice->GetTransform(D3DTS_VIEW, &M);
			DS.ViewMtx = M;
			DS.D3DDevice->GetTransform(D3DTS_PROJECTION, &P);
			DS.ProjMtx = P;
			DS.ProjViewMtx = P * M;

			State = 4;
			PFX.Physics(&DS);

			State = 5;
			// render
			PFXRenderer.Render(&DS);

			State = 6;
			Mouse.Render(&DS);

			State = 7;
			DS.D3DDevice->Present( NULL, NULL, NULL, NULL );
			DS.D3DDevice->EndScene();
			State = 8;
		}
		State = 9;
	}
	catch(char *s)
	{
		throw(s);
	}
	catch(...)
	{
		char Str[1024];
		sprintf(Str, "ParticleFXViewer::MainLoop#%d", State);
		throw(Str);
	}
}

}
