/***********************************************************/
/*                                                         */
/* ParticleFXViewer.h                                      */
/*                                                         */
/* created 02.03.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef PARTICLEFXVIEWER_H
#define PARTICLEFXVIEWER_H

#include "D3DWindow.h"
#include "D3DStruct.h"
#include "EventListener.h"
#include "DynamicArray.h"
#include "Model.h"
#include "RenderableModel.h"
#include "Input.h"
#include "MouseCursor.h"
#include "ParticleEffect.h"
#include "ParticleFXRenderer.h"
#include "ViewerSpaceManager.h"
#include "Speed.h"
#include "WorldPFXInterface.h"

// www.infinitecode.com 3d library
namespace I3D
{

enum
{
	GAME_D3DFLAG_WIREFRAME = 1,
	GAME_D3DFLAG_NOCULLING = 2,
	GAME_D3DFLAG_COLORKEY = 4,
	GAME_D3DFLAG_ALPHABLEND = 8,
	GAME_D3DFLAG_WHITEBACK = 16
};

class ParticleFXViewer: public IEV::EventListener<I3D::D3DWEvent>
{

	class CIWorldPFX: public I3D::WorldPFXInterface
	{
		ParticleFXViewer *PFXViewer;
	public:
		void SetPFXViewer(ParticleFXViewer *PFXViewer)
		{
			CIWorldPFX::PFXViewer = PFXViewer;
		}

		virtual void Delete(I3D::ParticleEffect *PFX)
		{
			PFX->SetTime(PFXViewer->Sp.GameTime);
		}
	};

	CIWorldPFX IWorldPFX;

	friend CIWorldPFX;

	D3DWindow Wnd;
	D3DStruct DS;
	int D3DFlags;
	LPDIRECT3DTEXTURE8 Texture;
	HINSTANCE Instance;
	char PFXFile[1024], TextureFile[1024];
	float Angle, Angle2, Zoom;
	int BackgroundColor;
	IIN::Input In;
	MouseCursor Mouse;
	ParticleEffect PFX;
	ParticleFXRenderer PFXRenderer;
	ParticleFXLib PFXLib;
	CIViewerSpaceManager SM;
	ITL::Speed Sp;

	ISL::DynamicArray<I3D::D3DWEvent, 0, 4> Events;

	virtual void OnEvent(I3D::D3DWEvent Data);
	void EventHandler(void);
	void KeyboardInput(char c);
	void MouseClick(void);
	void Init3D(void);
	void MenuCheck(int ID);
	void MouseHandler(void);

public:
	ParticleFXViewer(void)
		:D3DFlags(0)
	{
	}

	~ParticleFXViewer(void)
	{
		Wnd.DeleteListener(this);
		RELEASE(Texture);
		Texture = 0;
	}

	void Create(HINSTANCE ThisInstance, int WindowStyle);
	// this is called continuously while the application is running
	void MainLoop(void);
};

}

#endif
