/***********************************************************/
/*                                                         */
/* Profiler.cpp                                            */
/*                                                         */
/* created 25.03.2003                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "Profiler.h"
#include "TextureManager.h"
#include "Text.h"

#include <sstream>
#include <iomanip>

// www.infinitecode.com tool library
namespace ITL
{

void Profiler::Create(I3D::D3DStruct *DS)
{
	using namespace I3D;
	using namespace IMATH;
	Show = false;
	Timers.ClearPtr();
	// Bar texture
	Bar = I3D::TextureManager::GetTexture(DS, "Textures\\MenuButton.bmp");
	RS.AddVertex(Vertex(Vector(0, 0, 0), Vector(0, 0, -1), Vector2D(0, 0)));
	RS.AddVertex(Vertex(Vector(1, 0, 0), Vector(0, 0, -1), Vector2D(1, 0)));
	RS.AddVertex(Vertex(Vector(1, 1, 0), Vector(0, 0, -1), Vector2D(1, 1)));
	RS.AddVertex(Vertex(Vector(0, 1, 0), Vector(0, 0, -1), Vector2D(0, 1)));
	// Button
	RS.AddVertex(Vertex(Vector(0, 0, 0), Vector(0, 0, -1), Vector2D(0, 0.25f)));
	RS.AddVertex(Vertex(Vector(1, 0, 0), Vector(0, 0, -1), Vector2D(1, 0.25f)));
	RS.AddVertex(Vertex(Vector(1, 1, 0), Vector(0, 0, -1), Vector2D(1, 0)));
	RS.AddVertex(Vertex(Vector(0, 1, 0), Vector(0, 0, -1), Vector2D(0, 0)));
	RS.CreateVBuf(DS->D3DDevice);
	RS.AddIndex(0);
	RS.AddIndex(1);
	RS.AddIndex(2);
	RS.AddIndex(0);
	RS.AddIndex(2);
	RS.AddIndex(3);
	RS.CreateIBuf(DS->D3DDevice);
	FrameTime = ((float)clock()) / CLOCKS_PER_SEC;
}

void Profiler::SetToZero(int Index)
{
	AllocIndex(Index);
	Timers[Index]->Diff = 0;
}

void Profiler::SetName(int Index, const char *Name)
{
	AllocIndex(Index);
	strcpy(Timers[Index]->Name, Name);
}

void Profiler::Init3D(I3D::D3DStruct *DS, bool recurse)
{
	Bar = I3D::TextureManager::GetTexture(DS, "Textures\\MenuButton.bmp");
	RS.CreateVBuf(DS->D3DDevice);
	RS.CreateIBuf(DS->D3DDevice);
}

void Profiler::Render(I3D::D3DStruct *DS)
{
	if(Timers.Count())
	{
		// calculate
		for(int i = 0; i < Timers.Count(); i++)
		{
			Timers[i]->Delta = Timers[i]->Delta * 0.995f + 0.005f * Timers[i]->Diff;
			Timers[i]->Diff = 0;
		}
		// render
		if(Show)
		{
			const IMATH::Vector2D Pos(-0.9f, -0.9f);
			const IMATH::Vector2D Size(1.8f, 1.8f);
			I3D::Matrix Mtx, WMtx;
			Mtx.SetIdentityMatrix();
			// total
			float Total = 0;
			for(int i = 0; i < Timers.Count(); i++)
				Total += Timers[i]->Delta;
			// Frame time
			float FrameDelta = ((float)clock()) / CLOCKS_PER_SEC - FrameTime;
			FrameTime = ((float)clock()) / CLOCKS_PER_SEC;
			if(FrameDelta == 0)
				FrameDelta = 1e6;
			if(Total)
			{
				// draw bars
				for(int i = 0; i < Timers.Count(); i++)
				{
					float Length = Timers[i]->Delta / Total;
					IMATH::Vector BPos(Pos.x, Pos.y + Size.y - (i + 1) * Size.y / (Timers.Count() + 1), 0); 
					IMATH::Vector BSize(Size.x * Length, Size.y / (2 * Timers.Count()), 0);
					// bar
					DS->D3DDevice->SetTexture(0, Bar);
					WMtx.SetScaleTranslationMatrix(BSize, BPos);
					DS->D3DDevice->SetTransform(D3DTS_VIEW, &Mtx);
					DS->D3DDevice->SetTransform(D3DTS_PROJECTION, &Mtx);
					DS->D3DDevice->SetTransform(D3DTS_WORLD, &WMtx);
					DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
					RS.RenderIndex(DS->D3DDevice, 4, 4, 0, 6);
					// text
					std::ostringstream oss;
					Timers[i]->FDelta = 0.95f * Timers[i]->FDelta + 0.05f * Timers[i]->Delta / FrameDelta;
					oss << Timers[i]->Name << ' ' << std::setprecision(2) << Timers[i]->FDelta;
					I3D::Text::DrawString(DS, oss.str().c_str(), BPos.x, BPos.y + BSize.y * 2 / 3, BSize.y / 3, BSize.y / 3);
				}
			}
			// reset to standard values
			DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, TRUE);
			DS->D3DDevice->SetTransform(D3DTS_VIEW, &DS->ProjMtx);
			DS->D3DDevice->SetTransform(D3DTS_PROJECTION, &DS->ViewMtx);
		}
		// reset
		/*for(int i = 0; i < Timers.Count(); i++)
			Timers[i].Diff = 0;*/
			//Timers[i].Stop = Timers[i].Start;
	}
}

std::ostream &operator << (std::ostream &os, const Profiler &Prf)
{
	float sum = 0;
	for(int i = 0; i < Prf.GetNumber(); i++)
	{
		Profiler &P = const_cast<Profiler &>(Prf);
		sum += P.GetDelta(i);
	}
	if(sum)
		for(int i = 0; i < Prf.GetNumber(); i++)
		{
			Profiler &P = const_cast<Profiler &>(Prf);
			os << P.GetName(i) << ": " << P.GetDelta(i) / sum << std::endl;
			os << std::endl;
		}
	return os;
}

}
