/***********************************************************/
/*                                                         */
/* Profiler.h                                              */
/*                                                         */
/* created 25.03.2003                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef PROFILER_H
#define PROFILER_H

#include "DynamicArray.h"
#include "D3DStruct.h"
#include "RenderStruct.h"
#include "D3DStructures.h"

#include <ostream>
#include <time.h>

// www.infinitecode.com tool library
namespace ITL
{

class Profiler
{
	class Timer
	{
		friend ITL::Profiler;

		float Diff, Delta, FDelta;
		char Name[1024];

	public:
		Timer(void): Diff(0), Delta(0), FDelta(0)
		{
			*Name = 0;
		}
	};

	ISL::DynamicArray<Timer *> Timers;
	LPDIRECT3DTEXTURE9 Bar;
	I3D::RenderStruct<I3D::Vertex, I3D_FVFVERTEX, 1, 1> RS;
	bool Show;
	float FrameTime;

	void AllocIndex(int n)
	{
		if(n >= Timers.Count())
			for(int i = Timers.Count(); i <= n; i++)
				Timers[i] = new Timer;
	}

public:
	~Profiler()
	{
		Timers.ClearPtrElements();
	}

	bool GetShow(void)
	{
		return Show;
	}

	void SetShow(bool Value)
	{
		Show = Value;
	}

	void Start(int Index)
	{
		AllocIndex(Index);
		Timers[Index]->Diff -= ((float)clock()) / CLOCKS_PER_SEC;
	}

	void Stop(int Index)
	{
		AllocIndex(Index);
		if(Timers[Index]->Diff)
			Timers[Index]->Diff += ((float)clock()) / CLOCKS_PER_SEC;
	}

	int GetNumber() const
	{
		return Timers.Count();
	}

	float GetDiff(int n)
	{
		AllocIndex(n);
		return Timers[n]->Diff;
	}

	float GetDelta(int n)
	{
		AllocIndex(n);
		return Timers[n]->Delta;
	}

	const char *GetName(int n)
	{
		AllocIndex(n);
		return Timers[n]->Name;
	}

	void Create(I3D::D3DStruct *DS);
	void SetToZero(int Index);
	void SetName(int Index, const char *Name);
	void Render(I3D::D3DStruct *DS);
	void Init3D(I3D::D3DStruct *DS, bool recurse);
};

std::ostream &operator << (std::ostream &os, const Profiler &Prf);

}

#endif
