/***********************************************************/
/*                                                         */
/* Projectile.cpp                                          */
/*                                                         */
/* created 13.02.2003                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "Projectile.h"
#include "IMath.h"
#include "TextureManager.h"
#include "IDType.h"
#include "IDTank.h"
#include "IDParticleFX.h"
#include "Physics.h"
#include "Waterlevel.h"

// www.infinitecode.com game library
namespace IGAME
{

int Projectile::TypeID;

void Projectile::Create(Projectile::ProjectileSpecs *Specs)
{
	int State = 0;
	try
	{
		Projectile::Specs = *Specs;
		AIOrder = 0;
		Speed = 32;
		LastParticleSpawnTime = 0;
		RM.SetBase(this);
		LaunchTime = Specs->DS->Sp->GameTime;
		dPos = Specs->dPos;
		Lib = &Projectile::Specs.PLib->Projectiles[Projectile::Specs.ID];
		Owner = 1 + rand() % 2;
		State = 2;
		if(*Projectile::Specs.PLib->Projectiles[Projectile::Specs.ID].Model)
			RM.Mdl = I3D::Model::GetModel(Specs->DS, Projectile::Specs.PLib->Projectiles[Projectile::Specs.ID].Model);
		else
			RM.Mdl = 0;
		char TexName[1024];
		State = 3;
		if(RM.Mdl && RM.Mdl->GetStandardTexture() && *RM.Mdl->GetStandardTexture())
		{
			strcpy(TexName, "Textures\\");
			strcat(TexName, RM.Mdl->GetStandardTexture());
			RM.Texture = I3D::TextureManager::GetTexture(Specs->DS, TexName);
		}
		State = 4;
		LastShot = Specs->DS->Sp->GameTime;
		//Actor::Create(&Specs->AS);
		Owner = Specs->Owner;
		RM.SetPos(Specs->Pos);
		ScreenPos = Specs->DS->ProjViewMtx * RM.GetPos();
		State = 5;
		Specs->PosMgr->AddManagedObject(this);
		if(RM.Mdl)
			Specs->SM->AddManagedObject(&RM);
		((I3D::Matrix *)RM.Mtx)->SetTangentMatrix(RM.GetPos(), Projectile::Specs.Angle, Specs->QT->Specs.Surface->GetNormal(RM.GetPos().x, RM.GetPos().z));
		State = 6;
		// play sound
		if(Projectile::Specs.SndMgr && *Lib->LaunchSound)
		{
			IAU::DSSound *Snd = Projectile::Specs.SndMgr->Get(Lib->LaunchSound, 5);
			Snd->Play(ScreenPos.x);
		}
		// spawn particle effect
		if(Lib->LaunchPFX)
		{
			SpawnInterface::SpawnInterfaceSpecs SS;
			SS.Type = IDTYPE_PARTICLEFX;
			SS.ID = Lib->LaunchPFX;
			SS.Delay = 0;
			SS.Pos = Specs->Pos;
			Specs->ISpawn->Spawn(&SS);
		}
		State = 7;
	}
	catch(char *s)
	{
		throw(s);
	}
	catch(...)
	{
		char str[1024];
		sprintf(str, "exception in Projectile::Create#%s", State);
	}
}

void Projectile::Init3D(I3D::D3DStruct *DS, bool recurse)
{
	char TexName[1024];
	if(RM.Mdl && RM.Mdl->GetStandardTexture() && *RM.Mdl->GetStandardTexture())
	{
		strcpy(TexName, "Textures\\");
		strcat(TexName, RM.Mdl->GetStandardTexture());
		RM.Texture = I3D::TextureManager::GetTexture(DS, TexName);
	}
}

Projectile::~Projectile(void)
{
	UnLink();
}

void Projectile::UnLink(void)
{
	Specs.PosMgr->DeleteManagedObject(this);
	if(RM.Mdl)
		Specs.SM->DeleteManagedObject(&RM);
	// for debugging:
	UnLinked = true;
	RM.UnLinked = true;
}

bool Projectile::CheckCollision(Tank *Obj)
{
	// check if the object is hit
	return Obj != Specs.Shooter && (Obj->GetPos() - GetPos()).Length() < Obj->GetSize();
}

void Projectile::CollidedWith(Tank *Obj)
{
	// delete this projectile
	Specs.ISpawn->DeleteProjectile(this);
	// hit target
	Obj->Hit(Specs.Shooter, Lib->HitAmount);
	try
	{
		if(Specs.Shooter && Lib->HitFeedback)
			Specs.Shooter->ProjectileHit(true);
	}
	catch(char *s)
	{
		throw(s);
	}
	catch(...)
	{
		throw("exception in Projectile::Physics#Specs.Shooter->ProjectileHit(true);");
	}
	if(Specs.PLib->Projectiles[Specs.ID].HitPFX)
	{
		// spawn particle effect
		SpawnInterface::SpawnInterfaceSpecs SS;
		SS.Type = IDTYPE_PARTICLEFX;
		SS.ID = Specs.PLib->Projectiles[Specs.ID].HitPFX;
		for(int i = 0; i < Lib->HitPFXNum; i++)
		{
			SS.Delay = (rand() % 100) / 200.0f;
			SS.Pos = RM.GetPos()
				+ IMATH::Vector((rand() % 100 - 50) / 50.0f * Lib->HitPFXSpread
					, (rand() % 100 - 50) / 50.0f * Lib->HitPFXSpread
					, (rand() % 100 - 50) / 50.0f * Lib->HitPFXSpread);
			Specs.ISpawn->Spawn(&SS);
		}
	}
}

void Projectile::Physics(I3D::D3DStruct *DS)
{
	try
	{
		//Actor::Physics(DS);
		if(!AIOrder)
		{
			//Angle += 0.2f * DS->Sp->Delta;
			Specs.ISpawn->DeleteProjectile(this);
		}
		else
		{
			if(AIOrder & PRJ_PHYS_GUIDED)
			{
				MoveTo(IMATH::Vector2DXZ(Target->GetPos().x, Target->GetPos().z));
				// check for split
				if(Lib->Split && (Target->GetPos() - GetPos()).Length() < Lib->SplitDist)
				{
					// spawn parts
					SpawnInterface::SpawnInterfaceSpecs SS;
					SS.Type = IDTYPE_PROJECTILE;
					SS.ID = Lib->SplitID;
					SS.Pos = RM.GetPos();
					SS.Owner = Owner;
					SS.Spawner = Specs.Shooter;
					SS.Target = Target;
					for(int i = 0; i < Lib->SplitNum; i++)
					{
						SS.dPos = dPos
							+ IMATH::Vector((rand() % 101 - 50) / 50.0f * Lib->SplitSpeed
								, 0
								, (rand() % 101 - 50) / 50.0f * Lib->SplitSpeed);
						Specs.ISpawn->Spawn(&SS);
					}
					// delete this projectile
					Specs.ISpawn->DeleteProjectile(this);
					// split PFX
					if(Lib->SplitPFX)
					{
						SpawnInterface::SpawnInterfaceSpecs SS;
						SS.Type = IDTYPE_PARTICLEFX;
						SS.ID = Lib->SplitPFX;
						SS.Delay = 0;
						SS.Pos = RM.GetPos();
						Specs.ISpawn->Spawn(&SS);
					}
				}
			}
			//else if(AIOrder & PRJ_PHYS_TARGETED)
			//	MoveTo(TargetPos);
		}
		// gravity
		dPos.y += DS->Sp->Delta * PHYS_GRAVITY;
		RM.SetPos(RM.GetPos() + DS->Sp->Delta * dPos);
		// surface
		float y = Specs.QT->Specs.Surface->GetY(RM.GetPos().x, RM.GetPos().z);
		// ground hit
		if(RM.GetPos().y < y || RM.GetPos().y < WATERLEVEL || Specs.DS->Sp->GameTime - LaunchTime > PRJ_LIFETIME)
		{
			if(RM.GetPos().y < y)
			{
				LPDIRECT3DTEXTURE9 Texture = Specs.QT->GetTerrainTexture(GetPos());
				if(Texture)
				{
					int x = Specs.QT->GetTexturePosX(GetPos().x);
					int z = Specs.QT->GetTexturePosZ(GetPos().z);
					I3D::TextureManager::Spot(Texture, x, z, 0, -0x40);
				}
			}
			if(Specs.Shooter && Lib->HitFeedback)
				Specs.Shooter->ProjectileHit(false);
			Specs.ISpawn->DeleteProjectile(this);
			// spawn particle effect
			if(RM.GetPos().y < y)
			{
				if(Specs.PLib->Projectiles[Specs.ID].GroundHitPFX)
				{
					SpawnInterface::SpawnInterfaceSpecs SS;
					SS.Type = IDTYPE_PARTICLEFX;
					SS.ID = Specs.PLib->Projectiles[Specs.ID].GroundHitPFX;
					SS.Pos = RM.GetPos();
					SS.Delay = 0;
					Specs.ISpawn->Spawn(&SS);
				}
			}
			else if(RM.GetPos().y < WATERLEVEL)
			{
				if(Specs.PLib->Projectiles[Specs.ID].WaterHitPFX)
				{
					SpawnInterface::SpawnInterfaceSpecs SS;
					SS.Type = IDTYPE_PARTICLEFX;
					SS.ID = Specs.PLib->Projectiles[Specs.ID].WaterHitPFX;
					SS.Pos = RM.GetPos();
					SS.Delay = 0;
					Specs.ISpawn->Spawn(&SS);
				}
			}
		}
		IMATH::Vector x = IMATH::Vector(dPos.z, 0, -dPos.x);
		((I3D::Matrix *)RM.Mtx)->SetNormalMatrix(RM.GetPos(), IMATH::Vector::Normalize(x), IMATH::Vector::Normalize(-IMATH::Vector::CrossProduct(x, dPos)), IMATH::Vector::Normalize(dPos));
		Specs.PosMgr->MoveManagedObject(this);
		if(RM.Mdl)
			Specs.SM->MoveManagedObject(&RM);
		// particle
		if(Specs.PLib->Projectiles[Specs.ID].Trail && Specs.DS->Sp->GameTime - LastParticleSpawnTime > PRJ_PARTICLESPAWNINTERVALL)
		{
			LastParticleSpawnTime = Specs.DS->Sp->GameTime;
			if(Specs.PLib->Projectiles[Specs.ID].TrailPFX)
			{
				// spawn particle effect
				SpawnInterface::SpawnInterfaceSpecs SS;
				SS.Type = IDTYPE_PARTICLEFX;
				SS.ID = Specs.PLib->Projectiles[Specs.ID].TrailPFX;
				SS.Pos = RM.GetPos();
				Specs.ISpawn->Spawn(&SS);
			}
		}
	}
	catch(char *s)
	{
		throw(s);
	}
	catch(...)
	{
		throw("exception in Projectile::Physics");
	}
}

void Projectile::SetTargetPos(const IMATH::Vector2DXZ &TargetPos)
{
	Projectile::TargetPos = TargetPos;
	AIOrder = PRJ_PHYS_TARGETED;
}

void Projectile::Attack(Tank *Target)
{
	Projectile::Target = Target;
	AIOrder = PRJ_PHYS_GUIDED;
}

void Projectile::ClearTank(Tank *Target)
{
	if(Projectile::Target == Target)
		AIOrder = 0;
	if(Specs.Shooter == Target)
		Specs.Shooter = 0;
}

void Projectile::Idle(void)
{
}

void Projectile::MoveTo(const IMATH::Vector2DXZ &Target)
{
	IMATH::Vector2DXZ Dir = Target - IMATH::Vector2DXZ(RM.GetPos().x, RM.GetPos().z);
	if(Dir.z)
	{
		float TargetAngle = (float)atan(Dir.x / Dir.z);
		if(Dir.z < 0) TargetAngle += PI;
		Specs.Angle = TargetAngle;
	}
}

}
