/***********************************************************/
/*                                                         */
/* Projectile.h                                            */
/*                                                         */
/* created 13.02.2003                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef PROJECTILE_H
#define PROJECTILE_H

#include "MathStructures.h"
#include "ProjectileLib.h"
#include "Tank.h"
#include "D3DStruct.h"
#include "SoundManager.h"

// www.infinitecode.com game library
namespace IGAME
{

	class Projectile: public I3D::ManagedObject
{
public:

	class ProjectileSpecs
	{
	public:
		ProjectileSpecs(): SndMgr(0) {}

		I3D::D3DStruct *DS;
		Tank *Shooter;
		IMATH::Vector Pos;
		IMATH::Vector dPos;
		int ID, Owner;
		float Angle;
		ProjectileLib *PLib;
		SpawnInterface *ISpawn;
		I3D::QuadTree *QT;
		I3D::SpaceManagerInterface *SM, *PosMgr;
		IAU::SoundManager *SndMgr;
	};

private:
	enum
	{
		PRJ_PHYS_BALLISTIC = 0,
		PRJ_PHYS_TARGETED = 1,
		PRJ_PHYS_GUIDED = 2,

		FLAG_TURRET = 1,
		FLAG_VEHICLE = 2
	};

	#define PRJ_LIFETIME 30.0f
	#define PRJ_PARTICLESPAWNINTERVALL 0.08f

	IMATH::Vector ScreenPos;
	IMATH::Vector2DXZ TargetPos;
	IMATH::Vector dPos;
	Tank *Target;
	int AIOrder;
	float LastShot;
	float Speed;
	float LaunchTime;
	float LastParticleSpawnTime;
	int Owner;
	I3D::RenderableModel RM;
	int Flags;
	bool UnLinked;
	ProjectileLib::Projectile *Lib;
	static int TypeID;

	ProjectileSpecs Specs;

	// remove from QuadTree
	void UnLink(void);
	void MoveTo(const IMATH::Vector2DXZ &Target);

public:
	Projectile(void):Target(0), UnLinked(false), Flags(0), Owner(0) {}

	~Projectile(void);

	virtual void Create(ProjectileSpecs *Specs);
	void Init3D(I3D::D3DStruct *DS, bool recurse);
	virtual void Physics(I3D::D3DStruct *DS);
	virtual void SetTargetPos(const IMATH::Vector2DXZ &TargetPos);
	virtual void Attack(Tank *Target);
	virtual void Idle(void);
	virtual void ClearTank(Tank *Target);
	virtual void Hit(float Amount) {};
	virtual bool CheckCollision(Tank *Obj);
	virtual void CollidedWith(Tank *Obj);

	const IMATH::Vector &GetPos(void) const
	{
		return RM.GetPos();
	}

	Tank *GetShooter() const
	{
		return Specs.Shooter;
	}

	virtual void SetPos(const IMATH::Vector &Pos)
	{
		RM.SetPos(Pos);
	}

	virtual void *GetType() const
	{
		return &TypeID;
	}

	virtual void *GetBase() const
	{
		return (void *)this;
	}

	static const void *GetClassType()
	{
		return &TypeID;
	}
};

}

#endif
