/***********************************************************/
/*                                                         */
/* RenderableModel.h                                       */
/*                                                         */
/* created 20.08.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef RENDERABLEMODEL_H
#define RENDERABLEMODEL_H

#include "ManagedObject.h"
#include "Model.h"
#include "MathStructures.h"

#include <d3d9.h>

// www.infinitecode.com 3d library
namespace I3D
{

class RenderableModel: public ManagedObject
{
	static int TypeID;

	// data for managers etc, for example quadtree management
	void *Data;
	void *Base;
	IMATH::Vector Pos;

protected:

	virtual void CreateView(D3DStruct *DS) {}

public:
	LPDIRECT3DTEXTURE9 Texture;
	Model *Mdl;
	Matrix *Mtx;
	bool UnLinked;

	RenderableModel(void);

	void Create(void *Base);

	~RenderableModel(void)
	{
		// TODO: why does the Mtx deletion cause an exception?
		/*if(Mtx)
		{
			delete Mtx;
			Mtx = 0;
		}*/
	}

	virtual void Render(D3DStruct *DS);

	virtual void SetPos(const IMATH::Vector &Value)
	{
		Pos = Value;
	}

	virtual const IMATH::Vector &GetPos(void) const
	{
		return Pos;
	}

	virtual void *GetBase(void) const
	{
		return Base;
	}

	virtual void *GetType(void) const
	{
		return &TypeID;
	}

	void SetBase(void *Value)
	{
		Base = Value;
	}

	// data for managers etc, for example quadtree management
	void SetData(void *Data)
	{
		RenderableModel::Data = Data;
	}

	// data for managers etc, for example quadtree management
	void *GetData(void) const
	{
		return Data;
	}
};

}

#endif
