/***********************************************************/
/*                                                         */
/* SpawnInterface.h                                        */
/*                                                         */
/* created 13.02.2003                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef SPAWNINTERFACE_H
#define SPAWNINTERFACE_H

#include "MathStructures.h"
#include "DynamicArray.h"
#include "ManagedObject.h"

namespace IGAME
{

class Tank;
class Projectile;

class SpawnInterface
{
public:

	class SpawnInterfaceSpecs
	{
	public:
		SpawnInterfaceSpecs(void): Delay(0), Target(0), Spawner(0), Commandable(true), Highlight(false), MaxSpeed(10000) {}

		int Type, ID, Owner, AI;
		bool Commandable, Highlight;
		IMATH::Vector Pos, dPos;
		Tank *Target;
		Tank *Spawner;
		float Angle, Delay, TargetX, TargetZ, MaxSpeed;
		char Name[1024];
	};

	virtual void Spawn(SpawnInterfaceSpecs *SS) = 0;
	virtual void GetNearObjects(const IMATH::Vector &Pos, const float Area, I3D::ManagedObject::MgdObjArray *Objects) const = 0;
	virtual void GetNearObjects(const IMATH::Vector &Pos, const float Area, I3D::ManagedObject::MgdObjArray *Objects, const void *Type) const = 0;
	//virtual void GetNearTanks(const IMATH::Vector &Pos, const float Area, ISL::DynamicArray<Tank *> *Tanks) = 0;
	virtual void DeleteTank(Tank *Tnk) = 0;
	virtual void DeleteProjectile(Projectile *Prj) = 0;
	//virtual void Display(SpawnInterfaceSpecs *SS) = 0;
};

}

#endif
