/***********************************************************/
/*                                                         */
/* Speed.cpp                                               */
/*                                                         */
/* created 27.08.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "Speed.h"

#include <time.h>

// www.infinitecode.com tool library
namespace ITL
{

Speed::Speed(void)
{
	Delta = 1 / 30.0f;
	DeltaNoPause = Delta;
	DeltaLongTerm = 1 / 30.0f;
	Paused = false;
	GameTime = 0;
	GameTimeNoPause = 0;
}

void Speed::Update(void)
{
	static float Time1 = ((float)clock()) / CLOCKS_PER_SEC;
	float Time2;
	// get current time in seconds
	Time2 = ((float)clock()) / CLOCKS_PER_SEC;
	// calculate speed constant
	Delta = 0.9f * Delta + 0.1f * (Time2 - Time1);
	DeltaLongTerm = 0.99f * DeltaLongTerm + 0.01f * (Time2 - Time1);
	DeltaNoPause = Delta;
	if(Paused) Delta = 0;
	// don't let the speed below 10 fps, so the game is still playable on very slow computers (but with reduced speed)
	if(Delta > 1.0f / 10.0f) Delta = 1.0f / 10.0f;
	// calculate game time
	if(!Paused) GameTime += DeltaNoPause;
	GameTime += Delta;
	// remember the time for the next calculation
	Time1 = Time2;
}

}
