/***********************************************************/
/*                                                         */
/* StoryRenderer.cpp                                       */
/*                                                         */
/* created 06.04.2003                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "StoryRenderer.h"
#include "TextureManager.h"
#include "Text.h"
#include "IMath.h"
#include "Fader.h"

// www.infinitecode.com game library
namespace IGAME
{

void StoryRenderer::Create(I3D::D3DStruct *DS)
{
	StoryRenderer::DS = DS;
	BumperID = 0;
	*Ticker = 0;
	*Topic = 0;
	Earth = I3D::Model::GetModel(DS, "Models\\Earth.x");
	EarthMedium = I3D::Model::GetModel(DS, "Models\\Earth_medium.x");
	AlJazeera = I3D::Model::GetModel(DS, "Models\\logo_otherside.x");
	CreateBars(DS);
	I3D::Button::Specs BS;
	BS.DS = DS;
	BS.Pos = IMATH::Vector2D(0.6f, -1.0f);
	BS.Size = IMATH::Vector2D(0.4f, 0.1f);
	BS.Index = 2;
	BS.TilesX = 2;
	BS.TilesY = 8;
	strcpy(BS.Text, "Next");
	NextButton.Create(BS);
	Init3D(DS, false);
}

bool StoryRenderer::Click(IMATH::Vector2D &Pos)
{
	TimeDelta = 0;
	return true;
}

void StoryRenderer::CreateBars(I3D::D3DStruct *DS)
{
	// Ticker
	// CNN
	Bars.AddVertex(I3D::LVertex(IMATH::Vector(-1.0f, -0.85f * 0.75f, 0), 0xFF800000, IMATH::Vector2D(0, 0)));
	Bars.AddVertex(I3D::LVertex(IMATH::Vector(-1.0f, -1.0f * 0.75f, 0), 0xFF800000, IMATH::Vector2D(0, 1)));
	Bars.AddVertex(I3D::LVertex(IMATH::Vector( 1.0f, -0.85f * 0.75f, 0), 0xFF800000, IMATH::Vector2D(1, 0)));
	Bars.AddVertex(I3D::LVertex(IMATH::Vector( 1.0f, -0.85f * 0.75f, 0), 0xFF800000, IMATH::Vector2D(1, 0)));
	Bars.AddVertex(I3D::LVertex(IMATH::Vector(-1.0f, -1.0f * 0.75f, 0), 0xFF800000, IMATH::Vector2D(0, 1)));
	Bars.AddVertex(I3D::LVertex(IMATH::Vector( 1.0f, -1.0f * 0.75f, 0), 0xFF800000, IMATH::Vector2D(1, 1)));
	// AlJazeera
	Bars.AddVertex(I3D::LVertex(IMATH::Vector(-1.0f, -0.85f * 0.75f, 0), 0xFF0000A0, IMATH::Vector2D(0, 0)));
	Bars.AddVertex(I3D::LVertex(IMATH::Vector(-1.0f, -1.0f * 0.75f, 0), 0xFF0000A0, IMATH::Vector2D(0, 1)));
	Bars.AddVertex(I3D::LVertex(IMATH::Vector( 1.0f, -0.85f * 0.75f, 0), 0xFF8080A0, IMATH::Vector2D(1, 0)));
	Bars.AddVertex(I3D::LVertex(IMATH::Vector( 1.0f, -0.85f * 0.75f, 0), 0xFF8080A0, IMATH::Vector2D(1, 0)));
	Bars.AddVertex(I3D::LVertex(IMATH::Vector(-1.0f, -1.0f * 0.75f, 0), 0xFF0000A0, IMATH::Vector2D(0, 1)));
	Bars.AddVertex(I3D::LVertex(IMATH::Vector( 1.0f, -1.0f * 0.75f, 0), 0xFF8080A0, IMATH::Vector2D(1, 1)));
	// Topic
	// CNN
	Bars.AddVertex(I3D::LVertex(IMATH::Vector(-1.0f, -0.70f * 0.75f, 0), 0xFFA08000, IMATH::Vector2D(0, 0)));
	Bars.AddVertex(I3D::LVertex(IMATH::Vector(-1.0f, -0.85f * 0.75f, 0), 0xFFA08000, IMATH::Vector2D(0, 1)));
	Bars.AddVertex(I3D::LVertex(IMATH::Vector( 1.0f, -0.70f * 0.75f, 0), 0x00000000, IMATH::Vector2D(1, 0)));
	Bars.AddVertex(I3D::LVertex(IMATH::Vector( 1.0f, -0.70f * 0.75f, 0), 0x00000000, IMATH::Vector2D(1, 0)));
	Bars.AddVertex(I3D::LVertex(IMATH::Vector(-1.0f, -0.85f * 0.75f, 0), 0xFFA08000, IMATH::Vector2D(0, 1)));
	Bars.AddVertex(I3D::LVertex(IMATH::Vector( 1.0f, -0.85f * 0.75f, 0), 0x00000000, IMATH::Vector2D(1, 1)));
	// ALJazeera
	Bars.AddVertex(I3D::LVertex(IMATH::Vector(-1.0f, -0.70f * 0.75f, 0), 0xFF0000A0, IMATH::Vector2D(0, 0)));
	Bars.AddVertex(I3D::LVertex(IMATH::Vector(-1.0f, -0.85f * 0.75f, 0), 0xFF0000A0, IMATH::Vector2D(0, 1)));
	Bars.AddVertex(I3D::LVertex(IMATH::Vector( 1.0f, -0.70f * 0.75f, 0), 0x8000A0A0, IMATH::Vector2D(1, 0)));
	Bars.AddVertex(I3D::LVertex(IMATH::Vector( 1.0f, -0.70f * 0.75f, 0), 0x8000A0A0, IMATH::Vector2D(1, 0)));
	Bars.AddVertex(I3D::LVertex(IMATH::Vector(-1.0f, -0.85f * 0.75f, 0), 0xFF0000A0, IMATH::Vector2D(0, 1)));
	Bars.AddVertex(I3D::LVertex(IMATH::Vector( 1.0f, -0.85f * 0.75f, 0), 0x800080A0, IMATH::Vector2D(1, 1)));
	// Todays headlines
	// CNN
	Bars.AddVertex(I3D::LVertex(IMATH::Vector(-1.0f,  0.75f, 2.0f), 0xFFA00000, IMATH::Vector2D(0, 0)));
	Bars.AddVertex(I3D::LVertex(IMATH::Vector(-1.0f, -0.75f, 2.0f), 0xFFA00000, IMATH::Vector2D(0, 1)));
	Bars.AddVertex(I3D::LVertex(IMATH::Vector( 1.0f,  0.75f, 2.0f), 0xFFA0A000, IMATH::Vector2D(1, 0)));
	Bars.AddVertex(I3D::LVertex(IMATH::Vector( 1.0f,  0.75f, 2.0f), 0xFFA0A000, IMATH::Vector2D(1, 0)));
	Bars.AddVertex(I3D::LVertex(IMATH::Vector(-1.0f, -0.75f, 2.0f), 0xFFA00000, IMATH::Vector2D(0, 1)));
	Bars.AddVertex(I3D::LVertex(IMATH::Vector( 1.0f, -0.75f, 2.0f), 0xFFA0A000, IMATH::Vector2D(1, 1)));
	// ALJazeera
	Bars.AddVertex(I3D::LVertex(IMATH::Vector(-1.0f,  0.75f, 2.0f), 0xFF8080A0, IMATH::Vector2D(0, 0)));
	Bars.AddVertex(I3D::LVertex(IMATH::Vector(-1.0f, -0.75f, 2.0f), 0xFF8080A0, IMATH::Vector2D(0, 1)));
	Bars.AddVertex(I3D::LVertex(IMATH::Vector( 1.0f,  0.75f, 2.0f), 0x80A0D0D0, IMATH::Vector2D(1, 0)));
	Bars.AddVertex(I3D::LVertex(IMATH::Vector( 1.0f,  0.75f, 2.0f), 0x80A0D0D0, IMATH::Vector2D(1, 0)));
	Bars.AddVertex(I3D::LVertex(IMATH::Vector(-1.0f, -0.75f, 2.0f), 0xFF8080A0, IMATH::Vector2D(0, 1)));
	Bars.AddVertex(I3D::LVertex(IMATH::Vector( 1.0f, -0.75f, 2.0f), 0x80A0D0D0, IMATH::Vector2D(1, 1)));

	Bars.CreateVBuf(DS->D3DDevice);
	// CNN
	CNN.AddVertex(I3D::LVertex(IMATH::Vector(-1.0f, 1.0f, 0), 0xFFFFFFFF, IMATH::Vector2D(0, 0)));
	CNN.AddVertex(I3D::LVertex(IMATH::Vector(-1.0f, -1.0f, 0), 0xFFFFFFFF, IMATH::Vector2D(0, 0.5f)));
	CNN.AddVertex(I3D::LVertex(IMATH::Vector( 1.0f, 1.0f, 0), 0xFFFFFFFF, IMATH::Vector2D(1, 0)));
	CNN.AddVertex(I3D::LVertex(IMATH::Vector( 1.0f, 1.0f, 0), 0xFFFFFFFF, IMATH::Vector2D(1, 0)));
	CNN.AddVertex(I3D::LVertex(IMATH::Vector(-1.0f, -1.0f, 0), 0xFFFFFFFF, IMATH::Vector2D(0, 0.5f)));
	CNN.AddVertex(I3D::LVertex(IMATH::Vector( 1.0f, -1.0f, 0), 0xFFFFFFFF, IMATH::Vector2D(1, 0.5f)));
	CNN.CreateVBuf(DS->D3DDevice);
}

void StoryRenderer::SetBumper(int ID)
{
	BumperID = ID;
}

void StoryRenderer::SetTicker(const char *Text)
{
	strcpy(Ticker, Text);
}

void StoryRenderer::SetTickerStyle(int Style)
{
	TickerStyle = Style;
}

void StoryRenderer::SetTopic(const char *Text)
{
	strcpy(Topic, Text);
}

void StoryRenderer::SetText(const char *Text)
{
	strcpy(StoryRenderer::Text, Text);
}

void StoryRenderer::SetTimeDelta(float Time)
{
	TimeDelta = Time;
	TimeStart = DS->Sp->GameTime;
}

void StoryRenderer::RenderEarth(I3D::D3DStruct *DS, const IMATH::Vector &Translation, float Size, bool Wire)
{
	DS->D3DDevice->SetRenderState(D3DRS_AMBIENT, 0x80808080);
	// world matrix
	static float EarthRotation = 0;
	EarthRotation += 0.4f * DS->Sp->Delta;
	I3D::Matrix Mtx;
	Mtx.SetRotationYMatrix(EarthRotation);
	Mtx *= I3D::Matrix::ScaleMatrix(IMATH::Vector(Size * 0.94f, Size * 0.94f, Size * 0.94f));
	Mtx *= I3D::Matrix::TranslationMatrix(Translation);
	DS->D3DDevice->SetTransform(D3DTS_WORLD, &Mtx);

	// earth
	DS->D3DDevice->SetTexture(0, EarthTexture);
	Earth->Render(DS);

	// light for clouds
	DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
	DS->D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);
	DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
	//DS->D3DDevice->SetRenderState(D3DRS_AMBIENT, 0x20202020);

	// clouds
	// matrix
	Mtx.SetRotationYMatrix(0.85f * EarthRotation);
	Mtx *= I3D::Matrix::ScaleMatrix(IMATH::Vector(Size * 0.98f, Size * 0.98f, Size * 0.98f));
	Mtx *= I3D::Matrix::TranslationMatrix(Translation);
	DS->D3DDevice->SetTransform(D3DTS_WORLD, &Mtx);
	DS->D3DDevice->SetTexture(0, EarthCloudsTexture);
	Earth->Render(DS);

	if(Wire)
	{
		// earth wireframe
		DS->D3DDevice->SetRenderState(D3DRS_AMBIENT, 0);
		DS->D3DDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME);
		DS->D3DDevice->SetTexture(0, NULL);
		// matrix
		Mtx = I3D::Matrix::ScaleMatrix(IMATH::Vector(Size, Size, Size));
		Mtx *= I3D::Matrix::TranslationMatrix(Translation);
		DS->D3DDevice->SetTransform(D3DTS_WORLD, &Mtx);
		// render
		EarthMedium->Render(DS);
		// reset
		DS->D3DDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);
	}
	// reset
	DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
	DS->D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);
	DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ZERO);
	SetLight(DS, 1.0f);
}

void StoryRenderer::SetLight(I3D::D3DStruct *DS, float Intensity)
{
	//DS->D3DDevice->LightEnable(0, false);
	// directional light
	DS->D3DDevice->SetRenderState(D3DRS_AMBIENT, 0x80808080);
	D3DLIGHT9 Light;
	memset(&Light, 0, sizeof(D3DLIGHT9));
	Light.Type = D3DLIGHT_DIRECTIONAL;
	Light.Diffuse.r = 0.85f * Intensity;
	Light.Diffuse.g = 0.85f * Intensity;
	Light.Diffuse.b = 0.01f * Intensity;
	Light.Direction.x = 1;
	Light.Direction.y = -1;
	Light.Direction.z = 0;
	DS->D3DDevice->SetLight(1, &Light);
	DS->D3DDevice->LightEnable(1, true);

	Light.Diffuse.r = 0.01f * Intensity;
	Light.Diffuse.g = 0.01f * Intensity;
	Light.Diffuse.b = 0.85f * Intensity;
	Light.Direction.x = 0;
	Light.Direction.y = 0;
	Light.Direction.z = 1;
	DS->D3DDevice->SetLight(2, &Light);
	DS->D3DDevice->LightEnable(2, true);
}

void StoryRenderer::SetProjView(I3D::D3DStruct *DS)
{
	// projection matrix
	D3DMATRIX PMtx = {1,0,0,0,  0,1,0,0,  0,0,0.1f,0,  0,0,0,1};
	DS->D3DDevice->SetTransform(D3DTS_PROJECTION, &PMtx);
	// adjust to monitor size and position
	// screen 3:4 = 0.75f
	D3DMATRIX TMtx = {1,0,0,0,  0,1 / 0.75f,0,0,  0,0,1,0,  0,0,0,1};
	DS->D3DDevice->MultiplyTransform(D3DTS_PROJECTION, &TMtx);

	// view matrix
	I3D::Matrix VMtx(1, 0, 0, 0,  0, 1, 0, 0,  0, 0, 1, 0,  0, 0, 0, 1);
	DS->D3DDevice->SetTransform(D3DTS_VIEW, &VMtx);
}

void StoryRenderer::ResetProjView(I3D::D3DStruct *DS)
{
	// remove directional light
	DS->D3DDevice->LightEnable(1, false);

	DS->D3DDevice->SetTransform(D3DTS_VIEW, &DS->ViewMtx);
	DS->D3DDevice->SetTransform(D3DTS_PROJECTION, &DS->ProjMtx);
}

void StoryRenderer::RenderEarthWireframe(I3D::D3DStruct *DS, const IMATH::Vector &Translation, float Size)
{
	DS->D3DDevice->SetRenderState(D3DRS_AMBIENT, 0x20202020);
	// renderstates
	DS->D3DDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME);
	DS->D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);
	DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
	// rotation
	static float EarthRotation2 = 0;
	EarthRotation2 -= 0.1f * DS->Sp->Delta;
	I3D::Matrix Mtx = I3D::Matrix::RotationYMatrix(EarthRotation2);
	Mtx *= I3D::Matrix::ScaleMatrix(IMATH::Vector(Size, Size, Size));
	Mtx *= I3D::Matrix::TranslationMatrix(Translation);
	DS->D3DDevice->SetTransform(D3DTS_WORLD, &Mtx);
	// render
	EarthMedium->Render(DS);
	// reset
	DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
	DS->D3DDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);
	DS->D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);
	DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ZERO);
}

void StoryRenderer::RenderCNNBreakingNews(I3D::D3DStruct *DS)
{
	SetProjView(DS);
	RenderEarth(DS, IMATH::Vector(0.5f, 0.2f, 1), 0.2f);
	// big wireframe
	SetLight(DS, 0.2f);
	RenderEarthWireframe(DS, IMATH::Vector(0, -0.8f, 2), 1.6f);
	SetLight(DS, 1.0f);
	I3D::Matrix Mtx = I3D::Matrix::ScaleTranslationMatrix(IMATH::Vector(0.5f,0.35f,0.5f), IMATH::Vector(-0.2f, 0.19f, 0));
	DS->D3DDevice->SetTransform(D3DTS_WORLD, &Mtx);

	DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
	DS->D3DDevice->SetTexture(0, CNNTexture);
	DS->D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);
	DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
	CNN.RenderVertex(DS->D3DDevice);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
	DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, TRUE);
	// reset matrices
	ResetProjView(DS);
	// text
	//I3D::Text::DrawString(DS, "CNN", -0.8f, 0.5f, 0.35f, 0.6f);
	I3D::Text::DrawString(DS, "Breaking News", -0.8f, -0.2f, 0.12f, 0.16f);
}

void StoryRenderer::RenderAlJazeeraBreakingNews(I3D::D3DStruct *DS)
{
	SetProjView(DS);
	RenderEarth(DS, IMATH::Vector(0.0f, -4.0f, 5), 4.5f, false);
	// big wireframe
	//RenderEarthWireframe(DS, IMATH::Vector(0, -0.8f, 2), 1.6f);
	I3D::Matrix Mtx = I3D::Matrix::ScaleTranslationMatrix(IMATH::Vector(0.5f,0.5f,1.0f), IMATH::Vector(0.0f, 0.0f, 5.0f));
	Mtx *= I3D::Matrix::ProjectionMatrix();
	DS->D3DDevice->SetTransform(D3DTS_WORLD, &Mtx);
	DS->D3DDevice->SetTexture(0, AlJazeeraTexture);
	AlJazeera->Render(DS);
	// reset matrices
	ResetProjView(DS);
	// text
	//I3D::Text::DrawString(DS, "CNN", -0.8f, 0.5f, 0.35f, 0.6f);
	I3D::Text::DrawString(DS, "Breaking News", -0.8f, -0.2f, 0.12f, 0.16f);
}

void StoryRenderer::RenderCNNLogo(I3D::D3DStruct *DS)
{
	SetProjView(DS);
	RenderEarth(DS, IMATH::Vector(0.65f, -0.45f, 1), 0.05f);
	ResetProjView(DS);
	I3D::Text::DrawString(DS, "GNN", 0.75f, -0.55f, 0.06f, 0.08f);
}

void StoryRenderer::RenderAlJazeeraLogo(I3D::D3DStruct *DS)
{
	SetProjView(DS);
	//DS->D3DDevice->SetRenderState(D3DRS_AMBIENT, 0x20202020);
	I3D::Matrix Mtx = I3D::Matrix::ScaleTranslationMatrix(IMATH::Vector(0.02f,0.02f,0.04f), IMATH::Vector(1.8f, -0.8f, 1.0f));
	Mtx *= I3D::Matrix::ProjectionMatrix();
	DS->D3DDevice->SetTransform(D3DTS_WORLD, &Mtx);
	DS->D3DDevice->SetTexture(0, AlJazeeraTexture);
	DS->D3DDevice->SetRenderState(D3DRS_AMBIENT, 0);
	AlJazeera->Render(DS);
	SetLight(DS, 1.0f);
	ResetProjView(DS);
	I3D::Text::DrawString(DS, "AlGazeera", 0.45f, -0.50f, 0.04f, 0.05f);
}

void StoryRenderer::RenderTicker(I3D::D3DStruct *DS)
{
	SetProjView(DS);
	/*DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
	DS->D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);
	DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ZERO);
	DS->D3DDevice->SetRenderState(D3DRS_AMBIENT, 0xFFFFFFFF);*/
	DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, FALSE);

	I3D::Matrix Mtx = I3D::Matrix::IdentityMatrix();
	DS->D3DDevice->SetTransform(D3DTS_WORLD, &Mtx);
	DS->D3DDevice->SetTexture(0, 0);
	if(TickerStyle)
		Bars.RenderVertex(DS->D3DDevice, 6, 2);
	else
		Bars.RenderVertex(DS->D3DDevice, 0, 2);
	ResetProjView(DS);

	DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, TRUE);
	// text
	static float Scroll = 0;
	Scroll += 1.5f * DS->Sp->Delta;
	if(Scroll > 2.0f + strlen(Ticker) * 0.05f)
		Scroll = 0;
	I3D::Text::DrawString(DS, Ticker, 1.0f - Scroll, -0.86f, 0.05f, 0.07f);
}

void StoryRenderer::RenderTopic(I3D::D3DStruct *DS)
{
	SetProjView(DS);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
	DS->D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
	DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, FALSE);

	I3D::Matrix Mtx = I3D::Matrix::IdentityMatrix();
	DS->D3DDevice->SetTransform(D3DTS_WORLD, &Mtx);
	DS->D3DDevice->SetTexture(0, 0);
	if(TickerStyle)
		Bars.RenderVertex(DS->D3DDevice, 12 + 6, 2);
	else
		Bars.RenderVertex(DS->D3DDevice, 12, 2);
	ResetProjView(DS);

	DS->D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);
	DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ZERO);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
	DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, TRUE);
	// text
	I3D::Text::DrawString(DS, Topic, -0.95f, -0.71f, 0.03f, 0.05f);
}

void StoryRenderer::RenderTextFade(I3D::D3DStruct *DS)
{
	float Transparency, FadeTime, Time = DS->Sp->GameTime - TimeStart;
	if(TimeDelta > 8.0f)
		FadeTime = 4.0f;
	else
		FadeTime = TimeDelta * 0.5f;
	if(FadeTime == 0)
		FadeTime = 10000.0f;
	if(Time < FadeTime)
		Transparency = 1.0f - Time / FadeTime;
	else if(Time > TimeDelta)
		Transparency = 1.0f;
	else if(Time > TimeDelta - FadeTime)
		Transparency = (Time - (TimeDelta - FadeTime)) / FadeTime;
	else
		Transparency = 0.0f;
	int lines = 1;
	char *it = Text;
	// count lines
	while(*it)
		if(*it++ == '\n')
			lines++;
	// offset
	float y = 0.04f + 0.04f * lines;
	char *Text2 = new char[strlen(Text) + 1];
	strcpy(Text2, Text);
	char *str = Text2;
	it = Text2;
	bool stop = false;
	for(;;)
	{
		// find \n
		while(*it != '\n' && *it)
			it++;
		if(!*it)
			stop = true;
		*it = 0;
		I3D::Text::DrawString(DS, str, -0.03f * strlen(str), y -= 0.12f, 0.06f, 0.08f);
		if(stop)
			break;
		// set new str
		str = ++it;
	}
	delete[] Text2;
	I3D::Fader::Fade(DS, Transparency);
}

void StoryRenderer::RenderCNNHeadlines(I3D::D3DStruct *DS)
{
	SetProjView(DS);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
	DS->D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
	DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, FALSE);

	I3D::Matrix Mtx = I3D::Matrix::IdentityMatrix();
	DS->D3DDevice->SetTransform(D3DTS_WORLD, &Mtx);
	DS->D3DDevice->SetTexture(0, 0);
	Bars.RenderVertex(DS->D3DDevice, 24, 2);
	DS->D3DDevice->SetTexture(0, IsraelTexture);
	Bars.RenderVertex(DS->D3DDevice, 24, 2);
	ResetProjView(DS);

	DS->D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);
	DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ZERO);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
	DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, TRUE);
	// text
	I3D::Text::DrawString(DS, Text, -0.95f, 0.95f, 0.03f, 0.05f);
}

void StoryRenderer::RenderAlJazeeraHeadlines(I3D::D3DStruct *DS)
{
	SetProjView(DS);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
	DS->D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
	DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, FALSE);

	I3D::Matrix Mtx = I3D::Matrix::IdentityMatrix();
	DS->D3DDevice->SetTransform(D3DTS_WORLD, &Mtx);
	DS->D3DDevice->SetTexture(0, 0);
	Bars.RenderVertex(DS->D3DDevice, 24 + 6, 2);
	DS->D3DDevice->SetTexture(0, IsraelTexture);
	Bars.RenderVertex(DS->D3DDevice, 24 + 6, 2);
	ResetProjView(DS);

	DS->D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);
	DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ZERO);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
	DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, TRUE);
	// text
	I3D::Text::DrawString(DS, Text, -0.95f, 0.95f, 0.03f, 0.05f);
}

void StoryRenderer::Init3D(I3D::D3DStruct *DS, bool recurse)
{
	EarthTexture = I3D::TextureManager::Load(DS->D3DDevice, "Textures\\Earth2.bmp", 0, 0);
	EarthCloudsTexture = I3D::TextureManager::Load(DS->D3DDevice, "Textures\\EarthClouds.bmp", 0, 0);
	CNNTexture = I3D::TextureManager::Load(DS->D3DDevice, "Textures\\CNN.bmp", 0, 0);
	AlJazeeraTexture = I3D::TextureManager::Load(DS->D3DDevice, "Textures\\logotexture.bmp", 0, 0);
	IsraelTexture = I3D::TextureManager::Load(DS->D3DDevice, "Textures\\israel.bmp", 0, 0);
	Bars.CreateVBuf(DS->D3DDevice);
	CNN.CreateVBuf(DS->D3DDevice);
	if(recurse)
		NextButton.Init3D(DS, true);
}

void StoryRenderer::Render(I3D::D3DStruct *DS)
{
	SetLight(DS, 1.0f);
	switch(BumperID)
	{
	case BUMPER_CNN:
		RenderCNNBreakingNews(DS);
		break;
	case BUMPER_CNN_BREAKING:
		RenderCNNBreakingNews(DS);
		break;
	case BUMPER_ALJAZEERA:
		RenderAlJazeeraBreakingNews(DS);
		break;
	case BUMPER_ALJAZEERA_BREAKING:
		RenderAlJazeeraBreakingNews(DS);
		break;
	case BUMPER_CNN_HEADLINES: // "today's headlines"
		RenderCNNHeadlines(DS);
		break;
	case BUMPER_ALJAZEERA_HEADLINES:
		RenderAlJazeeraHeadlines(DS);
		break;
	case BUMPER_TEXT_FADE: // fade text in and out, like the date and time
		RenderTextFade(DS);
		break;
	}
	if(*Ticker)
		RenderTicker(DS);
	if(*Topic)
		RenderTopic(DS);
	if((*Ticker || *Topic) && (BumperID == BUMPER_NONE || BumperID == BUMPER_CNN_HEADLINES || BumperID == BUMPER_ALJAZEERA_HEADLINES))
	{
		if(TickerStyle == TICKER_CNN)
			RenderCNNLogo(DS);
		else
			RenderAlJazeeraLogo(DS);
	}
	NextButton.Render(DS);
}

}
