/***********************************************************/
/*                                                         */
/* StoryRenderer.h                                         */
/*                                                         */
/* created 06.04.2003                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef STORYRENDERER_H
#define STORYRENDERER_H

#include "D3DStruct.h"
#include "Model.h"
#include "RenderStruct.h"
#include "Button.h"

#include <d3d9.h>

// www.infinitecode.com game library
namespace IGAME
{

class StoryRenderer
{
	#define STORY_RND_STRMEM 1024

	I3D::RenderStruct<I3D::LVertex, I3D_FVFLVERTEX> Bars;
	I3D::RenderStruct<I3D::LVertex, I3D_FVFLVERTEX> CNN;
	I3D::Model *Earth, *EarthMedium, *AlJazeera;
	LPDIRECT3DTEXTURE9 EarthTexture, EarthCloudsTexture, CNNTexture, AlJazeeraTexture, IsraelTexture;
	int BumperID, TickerStyle;
	float TimeDelta, TimeStart;
	char Ticker[STORY_RND_STRMEM], Topic[STORY_RND_STRMEM], Text[STORY_RND_STRMEM];
	I3D::D3DStruct *DS;
	I3D::Button NextButton;

	void CreateBars(I3D::D3DStruct *DS);
	void RenderEarth(I3D::D3DStruct *DS, const IMATH::Vector &Translation, float Size, bool Wire = true);
	void RenderEarthWireframe(I3D::D3DStruct *DS, const IMATH::Vector &Translation, float Size);
	void SetProjView(I3D::D3DStruct *DS);
	void ResetProjView(I3D::D3DStruct *DS);
	void RenderCNNBreakingNews(I3D::D3DStruct *DS);
	void RenderAlJazeeraBreakingNews(I3D::D3DStruct *DS);
	void RenderCNNLogo(I3D::D3DStruct *DS);
	void RenderAlJazeeraLogo(I3D::D3DStruct *DS);
	void RenderCNNHeadlines(I3D::D3DStruct *DS);
	void RenderAlJazeeraHeadlines(I3D::D3DStruct *DS);
	void RenderTicker(I3D::D3DStruct *DS);
	void RenderTopic(I3D::D3DStruct *DS);
	void RenderTextFade(I3D::D3DStruct *DS);
	void SetLight(I3D::D3DStruct *DS, float Intensity);

public:
	enum Bumpers
	{
		BUMPER_NONE = 0,
		BUMPER_CNN = 1, // world news
		BUMPER_CNN_BREAKING = 2,
		BUMPER_ALJAZEERA = 3, // world news
		BUMPER_ALJAZEERA_BREAKING = 4,
		BUMPER_CNN_HEADLINES = 5, // "today's headlines"
		BUMPER_ALJAZEERA_HEADLINES = 6,
		BUMPER_TEXT_FADE = 7 // fade text in and out, like the date and time
	};

	enum TickerStyle
	{
		TICKER_CNN = 0,
		TICKER_ALJAZEERA = 1
	};

	float GetTimeDelta() { return TimeDelta; }

	void SetBumper(int ID);
	void SetTicker(const char *Text);
	void SetTickerStyle(int Style);
	void SetTopic(const char *Text);
	void SetText(const char *Text);
	void SetTimeDelta(float Time);
	void Create(I3D::D3DStruct *DS);
	void Render(I3D::D3DStruct *DS);
	void Init3D(I3D::D3DStruct *DS, bool recurse);
	bool Click(IMATH::Vector2D &Pos);
};

}

#endif
