/***********************************************************/
/*                                                         */
/* Tank.cpp                                                */
/*                                                         */
/* created 27.08.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "Tank.h"
#include "IMath.h"
#include "TextureManager.h"
#include "IDType.h"
#include "IDTank.h"
#include "IDProjectile.h"
#include "IDParticleFX.h"
#include "IDOwner.h"
#include "Physics.h"
#include "Color.h"
#include "Waterlevel.h"
#include "Tree.h"

// www.infinitecode.com game library
namespace IGAME
{

// *** Tank::CIPosMgr ***

void Tank::CIPosMgr::Create(Tank *Base)
{
	CIPosMgr::Base = Base;
}

void *Tank::CIPosMgr::GetBase(void) const
{
	return Base;
}

void *Tank::CIPosMgr::GetType(void) const
{
	return &Base->TypeID;
}

// *** Tank ***

int Tank::TypeID;

void Tank::Create(Tank::TankSpecs *Specs)
{
	Tank::Specs = *Specs;
	AIOrder = 0;
	Speed = 0;
	dAngle = 0;
	TurretAngle = 0;
	dTurretAngle = 0;
	TurretPitch = 0;
	dTurretPitch = 0;
	LastTrack = 0;
	LastDust = 0;
	LastSec = 0;
	HitQuality = 1.0f;
	TurretTarget = 0;
	BurstSequence = 0;
	Lib = &Tank::Specs.TLib->Tanks[Tank::Specs.ID];
	LibPrj = &Tank::Specs.PLib->Projectiles[Lib->Projectile];
	RM.SetBase(this);
	RM2.SetBase(this);
	IPosMgr.Create(this);
	HitPoints = Lib->HitPoints;
	// create selection frame with owner-based color
	SetOwner(Tank::Specs.Owner);
	HomePos = Specs->Pos;
	try
	{
		if(*Lib->Model)
			RM.Mdl = I3D::Model::GetModel(Specs->DS, Lib->Model);
		if(Lib->HasTurret && Lib->TurretModel && *Lib->TurretModel)
			RM2.Mdl = I3D::Model::GetModel(Specs->DS, Lib->TurretModel);
	}
	catch(char *s)
	{
		throw(s);
	}
	catch(...)
	{
		throw("Tank::Create#loading models");
	}
	char TexName[1024];
	try
	{
		if(RM.Mdl->GetStandardTexture() && *RM.Mdl->GetStandardTexture())
		{
			strcpy(TexName, "Textures\\");
			strcat(TexName, RM.Mdl->GetStandardTexture());
			RM.Texture = I3D::TextureManager::GetTexture(Specs->DS, TexName);
		}
		if(RM2.Mdl && RM2.Mdl->GetStandardTexture() && *RM2.Mdl->GetStandardTexture())
		{
			strcpy(TexName, "Textures\\");
			strcat(TexName, RM2.Mdl->GetStandardTexture());
			RM2.Texture = I3D::TextureManager::GetTexture(Specs->DS, TexName);
		}
	}
	catch(char *s)
	{
		throw(s);
	}
	catch(...)
	{
		try
		{
			if(RM.Mdl->GetStandardTexture() && *RM.Mdl->GetStandardTexture())
			{
				strcpy(TexName, "Tank::Create#loading texture: Textures\\");
				strcat(TexName, RM.Mdl->GetStandardTexture());
				throw TexName;
			}
			if(RM2.Mdl && RM2.Mdl->GetStandardTexture() && *RM2.Mdl->GetStandardTexture())
			{
				strcpy(TexName, "Tank::Create#loading texture: Textures\\");
				strcat(TexName, RM2.Mdl->GetStandardTexture());
				throw TexName;
			}
		}
		catch(char *TexName)
		{
			char str[1024];
			sprintf(str, "Tank::Create#loading textures: %s", TexName);
			throw(str);
		}
		catch(...)
		{
			throw "Tank::Create#loading textures";
		}
	}
	LastShot = Specs->DS->Sp->GameTime;
	if(*Tank::Specs.TLib->Tanks[Tank::Specs.ID].TurretModel)
		Flags |= FLAG_TURRET;
	if(Lib->CanMove)
		Flags |= FLAG_VEHICLE;
/*	Actor::Create(&Specs->AS);*/
	RM.SetPos(Specs->Pos);
	ScreenPos = Specs->DS->ProjViewMtx * RM.GetPos();
	Tank::Specs.PosMgr->AddManagedObject(&IPosMgr);
	IMATH::Vector Normal;
	if(Lib->CanMove && !Lib->IsShip)
		Normal = Tank::Specs.QT->Specs.Surface->GetNormal(RM.GetPos().x, RM.GetPos().z);
	else
		Normal = IMATH::Vector(0, 1, 0);
	if(RM.Mdl)
		Specs->SM->AddManagedObject(&RM);
	((I3D::Matrix *)RM.Mtx)->SetTangentMatrix(RM.GetPos(), Tank::Specs.Angle, Normal);
	if(Flags & FLAG_TURRET)
	{
		RM2.SetPos(Specs->Pos);
		ScreenPos = Specs->DS->ProjViewMtx * RM2.GetPos();
		if(RM2.Mdl)
			Specs->SM->AddManagedObject(&RM2);
		((I3D::Matrix *)RM2.Mtx)->SetTangentMatrix(RM2.GetPos(), Tank::Specs.Angle, Normal);
	}
	// overlay frame
	OverlayRS.AddIndex(0);
	OverlayRS.AddIndex(1);
	OverlayRS.AddIndex(3);
	OverlayRS.AddIndex(1);
	OverlayRS.AddIndex(2);
	OverlayRS.AddIndex(3);
	OverlayRS.CreateIBuf(Specs->DS->D3DDevice);
	SelectionTexture = I3D::TextureManager::GetTexture(Specs->DS, "Textures\\SelectBox.bmp");
	CreateHealthRS();
}

void Tank::RelinkQTree()
{
	if(RM.Mdl)
		Specs.SM->AddManagedObject(&RM);
	if(RM2.Mdl)
		Specs.SM->AddManagedObject(&RM2);
}

void Tank::Init3D(I3D::D3DStruct *DS, bool recurse)
{
	/*RM.Mdl->Init3D(DS);
	RM2.Mdl->Init3D(DS);*/
	char TexName[1024];
	if(RM.Mdl->GetStandardTexture() && *RM.Mdl->GetStandardTexture())
	{
		strcpy(TexName, "Textures\\");
		strcat(TexName, RM.Mdl->GetStandardTexture());
		RM.Texture = I3D::TextureManager::GetTexture(DS, TexName);
	}
	if(RM2.Mdl && RM2.Mdl->GetStandardTexture() && *RM2.Mdl->GetStandardTexture())
	{
		strcpy(TexName, "Textures\\");
		strcat(TexName, RM2.Mdl->GetStandardTexture());
		RM2.Texture = I3D::TextureManager::GetTexture(DS, TexName);
	}
	OverlayRS.CreateVBuf(DS->D3DDevice);
	OverlayRS.CreateIBuf(DS->D3DDevice);
	HealthRS.CreateVBuf(DS->D3DDevice);
}

void Tank::CreateHealthRS(void)
{
	float Health;
	if(Lib->HitPoints)
		Health = HitPoints / Lib->HitPoints;
	else
		Health = 1.0f;
	int Color;
	if(Health > 0.5f)
		Color = 0xFF00FF00 +  (int)(0xFF * (1 - 2 * (Health - 0.5f))) * 0x00010000;
	else
		Color = 0xFFFF0000 +  (int)(0xFF * 2 * Health) * 0x00000100;
	// Health Line
	HealthRS.ClearVertex();
	HealthRS.AddVertex(I3D::LVertex(IMATH::Vector(-1, 1.4f, 0), Color, IMATH::Vector2D(0, 0)));
	HealthRS.AddVertex(I3D::LVertex(IMATH::Vector(-1, 1.2f, 0), Color, IMATH::Vector2D(0, 0)));
	HealthRS.AddVertex(I3D::LVertex(IMATH::Vector(-1 + 2 * Health, 1.4f, 0), Color, IMATH::Vector2D(0, 0)));
	HealthRS.AddVertex(I3D::LVertex(IMATH::Vector(-1 + 2 * Health, 1.4f, 0), Color, IMATH::Vector2D(0, 0)));
	HealthRS.AddVertex(I3D::LVertex(IMATH::Vector(-1, 1.2f, 0), Color, IMATH::Vector2D(0, 0)));
	HealthRS.AddVertex(I3D::LVertex(IMATH::Vector(-1 + 2 * Health, 1.2f, 0), Color, IMATH::Vector2D(0, 0)));
	HealthRS.CreateVBuf(Specs.DS->D3DDevice);
}

Tank::~Tank(void)
{
	if(!UnLinked)
		UnLink();
}

void Tank::UnLink(void)
{
	// draw on ground
	for(int i = 0; i < 8; i++)
	{
		float Angle = 2 * PI * (rand() % 100) / 100.0f;
		float Length = 1.4f * Lib->Size * (rand() % 100) / 100.0f;
		float cx = Length * (float)cos(Angle);
		float cz = Length * (float)sin(Angle);
		IMATH::Vector Pos = GetPos() + IMATH::Vector(cx, 0, cz);
		LPDIRECT3DTEXTURE9 Texture = Specs.QT->GetTerrainTexture(Pos);
		if(Texture)
		{
			int x = Specs.QT->GetTexturePosX(Pos.x);
			int z = Specs.QT->GetTexturePosZ(Pos.z);
			I3D::TextureManager::Spot(Texture, x, z, 2, -0x10);
		}
	}
	// delete from managers
	Tank::Specs.PosMgr->DeleteManagedObject(&IPosMgr);
	Specs.SM->DeleteManagedObject(&RM);
	if(Flags & FLAG_TURRET)
		Specs.SM->DeleteManagedObject(&RM2);
	// for debugging:
	UnLinked = true;
	RM.UnLinked = true;
	RM2.UnLinked = true;
}

void Tank::UpdateMatrices()
{
	IMATH::Vector Normal;
	if(Lib->CanMove && !Lib->IsShip && !Lib->IsAirplane)
		Normal = Specs.QT->Specs.Surface->GetNormal(RM.GetPos().x, RM.GetPos().z);
	else
		Normal = IMATH::Vector(0, 1, 0);
	if(Moved || dAngle)
	{
		((I3D::Matrix *)RM.Mtx)->SetTangentMatrix(RM.GetPos(), Tank::Specs.Angle
			, Normal, Lib->PitchRoll, Lib->Turn);
	}
	if((Flags & FLAG_TURRET) && Moved)
		RM2.SetPos(RM.GetPos());
	if(Flags & FLAG_TURRET)
	{
		if(Moved || dAngle || dTurretAngle || dTurretPitch)
			((I3D::Matrix *)RM2.Mtx)->SetAnglePitchMatrix(RM.GetPos(), Tank::Specs.Angle, TurretAngle
			, 0, TurretPitch, Normal
			, Lib->PitchRoll, Lib->Turn, Lib->TurretPitch, Lib->TurretTurn);
	}
}

void Tank::Physics(I3D::D3DStruct *DS)
{
	const float MOVETARGET_EPSILON = 12.0f;
	int State = 0;
	try
	{
		// Sec timer
		bool SecTimer = false;
		if(DS->Sp->GameTime - LastSec > 1.0f)
		{
			SecTimer = true;
			LastSec = DS->Sp->GameTime;
		}
		// keep angle between -PI and PI
		Specs.Angle = ((Specs.Angle + PI) / (2 * PI) - floor((Specs.Angle + PI) / (2 * PI))) * 2 * PI - PI;
		if(Specs.TLib->Tanks[Specs.ID].AI)
		{
			// AI
			State = 1;
			if(!AIOrder)
			{
				if(!TurretTarget && SecTimer && Lib->CanShoot)
				{
					// find target
					I3D::ManagedObject::MgdObjArray Objects;
					Specs.ISpawn->GetNearObjects(RM.GetPos(), Lib->ShootRange, &Objects, GetType());
					if(Objects.Count())
						for(int i = 0; i < Objects.Count(); i++)
						{
							Tank *Tnk = (Tank *)Objects[i]->GetBase();
							if(Tnk->GetOwner() != GetOwner() && Tnk->GetOwner())
							{
								Attack(Tnk);
								NearAttack(Tnk);
								break;
							}
						}
				}
				State = 10;
				if(!Target)
				{
					// back home
					float DistanceFinal = (GetPos() - HomePos).Length();
					if(Specs.Owner == OWNER_ENEMY && DistanceFinal > MOVETARGET_EPSILON)
					{
						Speed = Lib->MaxSpeed;
						Speed = 0.5f * DistanceFinal;
						if(Speed > TNK_MAXSPEED) Speed = TNK_MAXSPEED;
						if(Speed > Specs.MaxSpeed) Speed = Specs.MaxSpeed;
						if(Speed > Lib->MaxSpeed) Speed = Lib->MaxSpeed;
						MoveTo(HomePos);
					}
					else
					{
						// leave turret in position
						if(TurretTarget)
							MoveTo(GetPos());
						Speed = 0;
						dAngle = 0;
					}
				}
				if(!TurretTarget)
				{
					dTurretAngle = -TurretAngle;
					dTurretPitch = -TurretPitch;
				}
				State = 102;
			}
			else
			{
				State = 11;
				bool AttackWithPath = (AIOrder == TNK_AI_ATTACK) && ((RM.GetPos() - Target->GetPos()).Length() > Lib->ShootRange || HitQuality < 0.2f);
				State = 20;
				if(AIOrder == TNK_AI_ATTACK)
				{
					State = 12;
					if(AttackWithPath)
					{
						Speed = 16;
						if(Specs.DS->Sp->GameTime - LastWay > 10)
						{
							SetWay(IMATH::Vector2DXZ(Target->GetPos().x, Target->GetPos().z));
							LastWay = Specs.DS->Sp->GameTime;
						}
						//MoveTo(Target->GetPos());
					}
					else
					{
						Speed = 0;
						// set Angle
						MoveTo(Target->GetPos());
						TargetPosFinal = Target->GetPos();
					}
				}
				if(AIOrder == TNK_AI_MOVE || AIOrder == TNK_AI_MOVE_PATH
					|| AttackWithPath)
				{
					//std::ofstream log("path.txt", ios::app);
					State = 13;
					IMATH::Vector Diff1 = RM.GetPos() - TargetPos;
					State = 130;
					float Distance = IMATH::Vector2DXZ(Diff1.x, Diff1.z).Length();
					IMATH::Vector Diff2 = RM.GetPos() - TargetPosFinal;
					float DistanceFinal = IMATH::Vector2DXZ(Diff2.x, Diff2.z).Length();
					State = 131;
					if(Distance < MOVETARGET_EPSILON)
					{
						State = 14;
						if(AIOrder == TNK_AI_MOVE || DistanceFinal < MOVETARGET_EPSILON)
						{
							if(AIOrder != TNK_AI_ATTACK)
							{
								AIOrder = 0;
								Speed = 0;
								dAngle = 0;
								if(!TurretTarget)
								{
									dTurretAngle = 0;
									dTurretPitch = 0;
								}
								//log << "reached target" << endl;
							}
						}
						else
						{
							// continue with pathfinding
							IMATH::Vector2DXZ TargetPosXZ = AIPath.GetWaypoint(++AINextWaypoint);
							TargetPos = IMATH::Vector(TargetPosXZ.x, Specs.QT->Specs.Surface->GetY(TargetPosXZ.x, TargetPosXZ.z), TargetPosXZ.z);
							// debug
							/*log << "next wp: " << AINextWaypoint << endl;
							log << "TargetPos: " << '(' << TargetPosXZ.x << ", " << TargetPosXZ.z << ')' << endl;
							log << "TargetPosFinal: " << '(' << TargetPosFinal.x << ", " << TargetPosFinal.z << ')' << endl;
							log << "Distance: " << Distance << endl;
							log << "DistanceFinal: " << DistanceFinal << endl;*/
						}
					}
					else
					{
						State = 132;
						Speed = 0.5f * DistanceFinal;
						if(Speed > TNK_MAXSPEED) Speed = TNK_MAXSPEED;
						if(Speed > Specs.MaxSpeed) Speed = Specs.MaxSpeed;
						if(Speed > Lib->MaxSpeed) Speed = Lib->MaxSpeed;
						State = 133;
						MoveTo(TargetPos);
						State = 134;
					}
					State = 135;
				}
				State = 136;
				const float RangeSpeed = 20.0f; // 8.0f;
				const float SlowRangeSpeed = 2.8f; // 8.0f;
				if(AIOrder == TNK_AI_LAND || AIOrder == TNK_AI_START)
				{
					float delta;
					if(AIOrder == TNK_AI_LAND)
						delta = -(RM.GetPos().x - TargetPos.x) / 2;
					else
						delta = -(RM.GetPos().x - (TargetPos.x + 20)) / 1;
					Speed = max(delta, RangeSpeed);
					if(RM.GetPos().x > TargetPos.x && AIOrder == TNK_AI_LAND)
					{
						AIOrder = TNK_AI_UNLOAD;
						AINextWaypoint = 0;
					}
					if(Speed > Specs.MaxSpeed) Speed = Specs.MaxSpeed;
					if(Speed > Lib->MaxSpeed) Speed = Lib->MaxSpeed;
				}
				if(AIOrder == TNK_AI_UNLOAD)
				{
					Speed = 0;
					switch(AINextWaypoint)
					{
					case 0:
						TargetPos += IMATH::Vector(-80, 0, 48);
						TargetPos = IMATH::Vector(TargetPos.x, Specs.QT->Specs.Surface->GetY(TargetPos.x, TargetPos.z), TargetPos.z);
						AINextWaypoint = 1;
						break;
					case 2:
						TargetPos += IMATH::Vector(0, 0, -48);
						TargetPos = IMATH::Vector(TargetPos.x, Specs.QT->Specs.Surface->GetY(TargetPos.x, TargetPos.z), TargetPos.z);
						AINextWaypoint = 3;
						// spawn reinforcements
						{
							IGAME::SpawnInterface::SpawnInterfaceSpecs SS;
							SS.Type = IGAME::IDTYPE_TANK;
							SS.ID = Specs.ID == IDT_TANK_US_C160 ? IDT_TANK_US_TANK : IDT_TANK_PAL_TANK;
							SS.Pos = IMATH::Vector(RM.GetPos().x + 8, RM.GetPos().y, RM.GetPos().z);
							SS.Owner = Specs.Owner;
							SS.Angle = 0.5f * PI;
							SS.AI = IGAME::Tank::TNK_AI_MOVE;
							SS.TargetX = RM.GetPos().x + 60;
							SS.TargetZ = RM.GetPos().z + 60;
							strcpy(SS.Name, "runtime-spawned tank");
							Specs.ISpawn->Spawn(&SS);
						}
						break;
					case 4:
						TargetPos += IMATH::Vector(80, 0, 1.2f * RangeSpeed / Lib->MaxTurnSpeed);
						TargetPos = IMATH::Vector(TargetPos.x, Specs.QT->Specs.Surface->GetY(TargetPos.x, TargetPos.z), TargetPos.z);
						AINextWaypoint = 5;
						break;
					case 6:
						TargetPos += IMATH::Vector(-20, 0, 1.1f * -RangeSpeed / Lib->MaxTurnSpeed);
						TargetPos = IMATH::Vector(TargetPos.x, Specs.QT->Specs.Surface->GetY(TargetPos.x, TargetPos.z), TargetPos.z);
						AINextWaypoint = 7;
						break;
					case 8:
						Specs.Angle = -PI / 2;
						dAngle = 0;
						AIOrder = TNK_AI_START;
						break;
					case 1:
					case 3:
					case 5:
					case 7:
						if(AINextWaypoint == 7)
						{
							float delta = -(RM.GetPos().x - (TargetPos.x + 20)) / 1;
							Speed = max(delta, RangeSpeed);
						}
						else if(AINextWaypoint == 3)
							Speed = SlowRangeSpeed;
						else
							Speed = RangeSpeed;
						MoveTo(IMATH::Vector(TargetPos.x, Specs.QT->Specs.Surface->GetY(TargetPos.x, TargetPos.z), TargetPos.z));
						if((RM.GetPos() - TargetPos).Length() < 1)
							AINextWaypoint++;
						break;
					}
				}
				State = 137;
			}
		}
		State = 2;
		if(Lib->CanMove)
		{
			// TODO: optimize for speed
			if(Speed)
				Moved = true;
			if(Moved)
			{
				// position and angle change
				RM.SetPos(RM.GetPos() + I3D::Vector(Speed * (float)sin(Tank::Specs.Angle) * DS->Sp->Delta, 0
					, Speed * (float)cos(Tank::Specs.Angle) * DS->Sp->Delta));
				LastTrack += Speed * DS->Sp->Delta;
				LastDust += Speed * DS->Sp->Delta;
				Specs.PosMgr->MoveManagedObject(&IPosMgr);
				Specs.SM->MoveManagedObject(&RM);
				if(!Lib->IsAirplane)
					CollisionTest();
				if(Flags & FLAG_TURRET)
					Specs.SM->MoveManagedObject(&RM2);
				// surface position
				float y, delta;
				if(Lib->IsAirplane)
				{
					if(AIOrder == TNK_AI_LAND || AIOrder == TNK_AI_START)
					{
						delta = RM.GetPos().x - (TargetPos.x - ((AIOrder == TNK_AI_LAND) ? 176.0f : 154.0f));
						float alt, h;
						if(delta < -90)
						{
							alt = (AIOrder == TNK_AI_LAND) ? 30.0f : 50.0f;
							h = TargetPos.y;
						}
						else if(delta > 10)
						{
							alt = 0;
							h = Specs.QT->Specs.Surface->GetY(RM.GetPos().x, RM.GetPos().z);
						}
						else
						{
							float c = 0.01f * delta + 0.9f;
							alt = ((AIOrder == TNK_AI_LAND) ? 15 : 25) * (1 + (float)cos(c * PI));
							h =  (1 - c) * TargetPos.y + c * Specs.QT->Specs.Surface->GetY(RM.GetPos().x, RM.GetPos().z);
						}
						y = max(Specs.QT->Specs.Surface->GetY(RM.GetPos().x, RM.GetPos().z), h + alt);
						//y = max(Specs.QT->Specs.Surface->GetY(RM.GetPos().x, RM.GetPos().z), TargetPos.y + min(delta, 30));
					}
					else
						y = Specs.QT->Specs.Surface->GetY(RM.GetPos().x, RM.GetPos().z);
				}
				else
					y = Lib->IsShip ? WATERLEVEL : Specs.QT->Specs.Surface->GetY(RM.GetPos().x, RM.GetPos().z);
				RM.SetPos(IMATH::Vector(RM.GetPos().x, y, RM.GetPos().z));
				if(!Lib->IsShip && (!Lib->IsAirplane || (AIOrder == TNK_AI_LAND && (delta >= -0.5f && delta <= 3.5f))))
				{
				// tracks
					if(LastTrack >= (Lib->IsAirplane ? 0.3f : 0.7f))
					{
						LastTrack = 0;
						DrawTracks();
					}
				}
				if(!Lib->IsAirplane && !Lib->IsShip && LastDust >= 2.0f)
				{
					LastDust = 0;
					Dust();
				}
				if(((Lib->IsAirplane || Lib->IsShip) && HitPoints < Lib->HitPoints) || (Lib->IsAirplane && AIOrder == TNK_AI_LAND && (delta >= -0.5f && delta <= 3.5f)))
				{
					if(LastDust >= 1.0f)
					{
						LastDust = 0;
						Dust();
					}
				}
			}
		}
		Tank::Specs.Angle += dAngle * DS->Sp->Delta;
		TurretAngle += dTurretAngle * DS->Sp->Delta;
		TurretPitch += dTurretPitch * DS->Sp->Delta;
		if(TurretPitch > TNK_MAXTPITCH)
			TurretPitch = TNK_MAXTPITCH;
		if(TurretPitch < TNK_MINTPITCH)
			TurretPitch = TNK_MINTPITCH;
		State = 3;
		// calculate matrices
		UpdateMatrices();
		State = 4;
		// find target for turret
		if(Specs.TLib->Tanks[Specs.ID].CanShoot
			&& (Specs.DS->Sp->GameTime - LastShot > LibPrj->ShotDelay || (Lib->Burst && Specs.DS->Sp->GameTime - LastShot > Lib->BurstDelay && BurstSequence != 0)))
		{
			if(SecTimer && (AIOrder != TNK_AI_ATTACK || (RM.GetPos() - Target->GetPos()).Length() > Lib->ShootRange))
			{
				TurretTarget = 0;
				I3D::ManagedObject::MgdObjArray Objects;
				Specs.ISpawn->GetNearObjects(RM.GetPos(), Lib->ShootRange, &Objects, GetType());
				if(Objects.Count())
					for(int i = 0; i < Objects.Count(); i++)
					{
						Tank *Tnk = (Tank *)Objects[i]->GetBase();
						if(Tnk->GetOwner() != GetOwner() && Tnk->GetOwner())
						{
							TurretTarget = Tnk;
							// suggest other near tanks to attack it as well
							NearAttack(Tnk);
							break;
						}
					}
			}
			if(AIOrder == TNK_AI_ATTACK && (RM.GetPos() - Target->GetPos()).Length() <= Lib->ShootRange)
				TurretTarget = Target;
			// shoot
			if(TurretTarget)
			{
				State = 40;
				LastShot = Specs.DS->Sp->GameTime;
				if(!Lib->Burst)
					LastShot -= (rand() % 500) / 1000.0f * LibPrj->ShotDelay;
				// spawn shot
				SpawnInterface::SpawnInterfaceSpecs SS;
				SS.Type = IDTYPE_PROJECTILE;
				SS.ID = Specs.TLib->Tanks[Specs.ID].Projectile;
				SS.Target = TurretTarget;
				SS.Spawner = this;
				SS.Owner = Specs.Owner;
				// direction and speed
				I3D::Matrix LaunchMtx;
				LaunchMtx.SetAnglePitchMatrix(IMATH::Vector(0, 0, 0), Tank::Specs.Angle, TurretAngle
					, 0, TurretPitch, Specs.QT->Specs.Surface->GetNormal(RM.GetPos().x, RM.GetPos().z)
					, Lib->PitchRoll, Lib->Turn, Lib->TurretPitch, Lib->TurretTurn);
				/*LaunchMtx.SetAnglePitchMatrix(IMATH::Vector(0, 0, 0), Tank::Specs.Angle + TurretAngle
					, TurretPitch, Specs.QT->Specs.Surface->GetNormal(RM.GetPos().x, RM.GetPos().z));*/
				IMATH::Vector Disp(0, 0, 0);
				if(Lib->Burst && Lib->BurstNumber)
				{
					Disp = ((float)BurstSequence - (float)(Lib->BurstNumber - 1) / 2.0f) * Lib->BurstDisplacement;
					BurstSequence = (BurstSequence + 1) % Lib->BurstNumber;
				}
				SS.Pos = RM.GetPos() + LaunchMtx * (Specs.TLib->Tanks[Specs.ID].ProjectileSpawnPos + Disp);
				SS.dPos = IMATH::Vector(0, 0, LibPrj->LaunchSpeed);
				SS.dPos = LaunchMtx * SS.dPos;
				State = 41;
				Specs.ISpawn->Spawn(&SS);
				State = 42;
				if(LibPrj->LaunchPFX)
				{
					// spawn particle effect
					SS.Type = IDTYPE_PARTICLEFX;
					SS.ID = LibPrj->LaunchPFX;
					SS.Delay = 0;
					SS.dPos = IMATH::Vector(0, 0, 0);
					State = 5;
					Specs.ISpawn->Spawn(&SS);
					State = 6;
				}
			}
		}
		State = 7;
		Moved = false; // at the end
	}
	catch(char *s)
	{
		throw(s);
	}
	catch(...)
	{
		char Str[1024];
		sprintf(Str, "exception in Tank::Physics#%d", State);
		throw(Str);
	}
}

void Tank::SetWay(const IMATH::Vector2DXZ &TargetPos)
{
	// pathfinding
	Specs.QT->Specs.Surface->GetPath(&IMATH::Vector2DXZ(GetPos().x, GetPos().z), &TargetPos, &AIPath, Lib->IsShip);
	IMATH::Vector2DXZ TargetPosFinalXZ = AIPath.GetWaypoint(AIPath.CountWaypoints());
	TargetPosFinal = IMATH::Vector(TargetPosFinalXZ.x, Specs.QT->Specs.Surface->GetY(TargetPosFinalXZ.x, TargetPosFinalXZ.z), TargetPosFinalXZ.z);
	AINextWaypoint = 1;
	IMATH::Vector2DXZ TargetPosXZ = AIPath.GetWaypoint(AINextWaypoint);
	Tank::TargetPos = IMATH::Vector(TargetPosXZ.x, Specs.QT->Specs.Surface->GetY(TargetPosXZ.x, TargetPosXZ.z), TargetPosXZ.z);
}

void Tank::SetTargetPos(const IMATH::Vector2DXZ &TargetPos)
{
	if((TargetPos - IMATH::Vector2DXZ(GetPos().x, GetPos().z)).Length() < 20.0f)
	{
		Tank::TargetPos = IMATH::Vector(TargetPos.x, Specs.QT->Specs.Surface->GetY(TargetPos.x, TargetPos.z), TargetPos.z);
		TargetPosFinal = Tank::TargetPos;
		AIOrder = TNK_AI_MOVE;
	}
	else
	{
		SetWay(TargetPos);
		AIOrder = TNK_AI_MOVE_PATH;
	}
	Target = 0;
}

void Tank::Land(const IMATH::Vector2DXZ &TargetPos)
{
	Tank::TargetPos = IMATH::Vector(TargetPos.x, Specs.QT->Specs.Surface->GetY(TargetPos.x, TargetPos.z), TargetPos.z);
	AIOrder = TNK_AI_LAND;
}

void Tank::Attack(Tank *Target)
{
	LastWay = Specs.DS->Sp->GameTime;
	SetWay(IMATH::Vector2DXZ(Target->GetPos().x, Target->GetPos().z));
	Tank::Target = Target;
	AIOrder = TNK_AI_ATTACK;
}

void Tank::ClearTank(Tank *Target)
{
	if(Tank::Target == Target)
	{
		Tank::Target = 0;
		if(AIOrder == TNK_AI_ATTACK)
			AIOrder = TNK_AI_IDLE;
	}
	if(Tank::TurretTarget == Target)
		Tank::TurretTarget = 0;
}

void Tank::Idle(void)
{
	Tank::Target = 0;
	AIOrder = 0;
}

void Tank::Hit(Tank *Attacker, float Amount)
{
	HitPoints -= Amount;
	if(HitPoints < 0)
	{
		// play sound
		if(Specs.SndMgr && *Lib->ExplodeSound)
		{
			IAU::DSSound *Snd = Specs.SndMgr->Get(Lib->ExplodeSound, 5);
			Snd->Play(ScreenPos.x);
		}
		Specs.ISpawn->DeleteTank(this);
		return;
	}
	// let all near friendly units without orders attack my attacker
	NearAttack(Attacker);
	CreateHealthRS();
}

// let all near friendly units without orders attack T, if T is an enemy
void Tank::NearAttack(Tank *T)
{
	if(T && T->GetOwner() != Specs.Owner)
	{
		I3D::ManagedObject::MgdObjArray Objects;
		Specs.ISpawn->GetNearObjects(RM.GetPos(), 200, &Objects, GetType());
		for(int i = 0; i < Objects.Count(); i++)
		{
			Tank *Tnk = (Tank *)Objects[i]->GetBase();
			if(!Tnk->AIOrder && Tnk->GetOwner() == Specs.Owner)
				Tnk->Attack(T);
		}
	}
}

void Tank::CollisionTest(void)
{
	I3D::ManagedObject::MgdObjArray Objects;
	float CollisionSize = Lib->Size * 1.5f;
	Specs.ISpawn->GetNearObjects(RM.GetPos(), 2 * CollisionSize, &Objects, GetType());
	for(int i = 0; i < Objects.Count(); i++)
	{
		Tank *Tnk = (Tank *)Objects[i]->GetBase();
		if(Tnk != this)
		{
			IMATH::Vector Dir = Tnk->GetPos() - RM.GetPos();
			float Distance = Dir.Length();
			float MinDist = Specs.TLib->Tanks[Specs.ID].Size + CollisionSize;
			if(Distance < MinDist)
			{
				float ClipLength = (Distance - MinDist) / 2.0f;
				RM.SetPos(RM.GetPos() + Dir.Normalize() * ClipLength);
				if(Specs.TLib->Tanks[Tnk->GetID()].CanMove && !Specs.TLib->Tanks[Tnk->GetID()].IsAirplane)
				{
					Tnk->SetPos(Tnk->GetPos() - Dir.Normalize() * ClipLength);
				}
			}
		}
	}
	// trees
	Objects.Clear();
	Specs.ISpawn->GetNearObjects(RM.GetPos(), Lib->Size, &Objects, Tree::GetClassType());
	for(int i = 0; i < Objects.Count(); i++)
	{
		Tree *Tr = (Tree *)Objects[i]->GetBase();
		Tr->Collision(IMATH::Vector(Speed * (float)sin(Tank::Specs.Angle), 0, Speed * (float)cos(Tank::Specs.Angle)));
	}
}

void Tank::DrawTracks(void)
{
	LPDIRECT3DTEXTURE9 Texture = Specs.QT->GetTerrainTexture(GetPos());
	if(Texture)
	{
		const float cx = (float)cos(-Specs.Angle) * 0.5f * Lib->Size;
		const float cz = (float)sin(-Specs.Angle) * 0.5f * Lib->Size;
		// tracks
		int x = Specs.QT->GetTexturePosX(GetPos().x + cx);
		int z = Specs.QT->GetTexturePosZ(GetPos().z + cz);
		I3D::TextureManager::Brightness(Texture, x, z, -0x10);
		x = Specs.QT->GetTexturePosX(GetPos().x - cx);
		z = Specs.QT->GetTexturePosZ(GetPos().z - cz);
		I3D::TextureManager::Brightness(Texture, x, z, -0x10);
	}
}

void Tank::Dust(void)
{
	// smoke
	if(Lib->Dust || Lib->IsAirplane || Lib->IsShip)
	{
		const float cx = (float)cos(-Specs.Angle) * 0.5f * Lib->Size;
		const float cz = (float)sin(-Specs.Angle) * 0.5f * Lib->Size;
		if(Lib->DustPFX)
		{
			// spawn particle effect
			SpawnInterface::SpawnInterfaceSpecs SS;
			SS.Type = IDTYPE_PARTICLEFX;
			SS.ID = Lib->DustPFX;
			SS.Pos = IMATH::Vector(GetPos().x + cx, GetPos().y, GetPos().z + cz);
			SS.Delay = 0;
			Specs.ISpawn->Spawn(&SS);
			SS.Pos = IMATH::Vector(GetPos().x - cx, GetPos().y, GetPos().z - cz);
			Specs.ISpawn->Spawn(&SS);
		}
	}
}

void Tank::MoveTo(const IMATH::Vector &Target)
{
	int State = 0;
	try
	{
		State = 1;
		// relative position
		IMATH::Vector RelTarget = Target - RM.GetPos();
		// direction
		IMATH::Vector2DXZ Dir = IMATH::Vector2DXZ(RelTarget.x, RelTarget.z);
		float TargetAngle;
		if(Dir.z)
		{
			// target angle
			TargetAngle = (float)atan(Dir.x / Dir.z);
			if(Dir.z < 0) TargetAngle += PI;
		}
		else if(Dir.x > 0)
			TargetAngle = PI / 2.0f;
		else
			TargetAngle = -PI / 2.0f;
		// vehicle angle
		float MinAngle = TargetAngle - Specs.Angle;
		for(int i = -2; i <= 2; i++)
		{
			if(absf(TargetAngle - Specs.Angle + i * 2 * PI) < absf(MinAngle))
				MinAngle = TargetAngle - Specs.Angle + i * 2 * PI;
		}
		dAngle = 16.0f * MinAngle;
		if(dAngle >  TNK_MAXDANGLE) dAngle =  TNK_MAXDANGLE;
		if(dAngle < -TNK_MAXDANGLE) dAngle = -TNK_MAXDANGLE;
		if(dAngle >  Lib->MaxTurnSpeed) dAngle =  Lib->MaxTurnSpeed;
		if(dAngle < -Lib->MaxTurnSpeed) dAngle = -Lib->MaxTurnSpeed;
		if(absf(MinAngle) > PI / 4.0f)
			Speed = 0;
		State = 5;
		if(Flags & FLAG_TURRET)
		{
			// get relative target position from launch position
			I3D::Matrix LaunchMtx;
			LaunchMtx.SetAnglePitchMatrix(IMATH::Vector(0, 0, 0), Tank::Specs.Angle, TurretAngle
				, 0, TurretPitch, Specs.QT->Specs.Surface->GetNormal(RM.GetPos().x, RM.GetPos().z)
				, Lib->PitchRoll, Lib->Turn, Lib->TurretPitch, Lib->TurretTurn);
			IMATH::Vector LaunchPos = GetPos() + LaunchMtx * Specs.TLib->Tanks[Specs.ID].ProjectileSpawnPos;
			if(Tank::Target)
			{
				State = 10;
				RelTarget = Tank::Target->GetPos() - LaunchPos;
				State = 11;
			}
			else if(Tank::TurretTarget)
				RelTarget = Tank::TurretTarget->GetPos() - LaunchPos;
			State = 12;
			// antigravity with RelTarget (world coordinates)
			float Distance = (float)sqrt(RelTarget.x * RelTarget.x + RelTarget.z * RelTarget.z);
			float Height = RelTarget.y;
			//float ShotSpeed = LibPrj->LaunchSpeed;
			// get real shotspeed (why doesn't this equal LibPrj->LaunchSpeed?)
			float ShotSpeed = (LaunchMtx * IMATH::Vector(0, 0, LibPrj->LaunchSpeed)).Length();
			float Root = ShotSpeed * ShotSpeed * ShotSpeed * ShotSpeed / (PHYS_GRAVITY * PHYS_GRAVITY) + 2 * Height * ShotSpeed * ShotSpeed / PHYS_GRAVITY - Distance * Distance;
			float AntiGravity;
			if(Root >= 0)
			{
				if(LibPrj->ShootHigh)
					AntiGravity = -ShotSpeed * ShotSpeed / PHYS_GRAVITY - Height + (float)sqrt(Root);
				else
					AntiGravity = -ShotSpeed * ShotSpeed / PHYS_GRAVITY - Height - (float)sqrt(Root);
			}
			else
				AntiGravity = -ShotSpeed * ShotSpeed / PHYS_GRAVITY - Height;
			State = 15;
			// attack the tank, not the grass under it
			//RelTarget.y += 1.0f;
			// eliminate gravity
			RelTarget.y += AntiGravity;
			// correct surface angles
			I3D::Matrix TargetAdjustMtx;
			TargetAdjustMtx = LaunchMtx.Invert();
			// correct surface angles
			IMATH::Vector VirtualTarget = TargetAdjustMtx * RelTarget;
			// angle
			Dir = IMATH::Vector2DXZ(VirtualTarget.x, VirtualTarget.z);
			if(Dir.z)
			{
				// target angle
				TargetAngle = (float)atan(Dir.x / Dir.z);
				if(Dir.z < 0) TargetAngle += PI;
			}
			else if(Dir.x > 0)
				TargetAngle = PI / 2.0f;
			else
				TargetAngle = -PI / 2.0f;
			// calculate dTurretAngle
			float MinAngle = TargetAngle;
			for(int i = -2; i <= 2 ; i++)
			{
				if(absf(TargetAngle + i * 2 * PI) < absf(MinAngle))
					MinAngle = TargetAngle + i * 2 * PI;
			}
			dTurretAngle = 16.0f * MinAngle;
			if(dTurretAngle >  TNK_MAXDTANGLE) dTurretAngle =  TNK_MAXDTANGLE;
			if(dTurretAngle < -TNK_MAXDTANGLE) dTurretAngle = -TNK_MAXDTANGLE;

			State = 20;
			// pitch
			float TargetPitch;
			if(Distance)
			{
				float Dist = (float)sqrt(VirtualTarget.x * VirtualTarget.x + VirtualTarget.z * VirtualTarget.z);
				TargetPitch = (float)atan(VirtualTarget.y / Dist);
				if(Dist < 0) TargetPitch -= PI;
			}
			else if(Height > 0)
				TargetPitch = PI / 2.0f;
			else
				TargetPitch = -PI / 2.0f;
			dTurretPitch = 10 * TargetPitch;
			if(dTurretPitch > TNK_MAXDTPITCH)
				dTurretPitch = TNK_MAXDTPITCH;
			if(dTurretPitch < -TNK_MAXDTPITCH)
				dTurretPitch = -TNK_MAXDTPITCH;
			State = 25;
		}
		State = 26;
	}
	catch(char *s)
	{
		throw(s);
	}
	catch(...)
	{
		char Str[1024];
		sprintf(Str, "exception in Tank::MoveTo#%d", State);
		throw(Str);
	}
}

bool Tank::Select(const I3D::XZPos &Pos)
{
	//ScreenPos = DS->ProjViewMtx * RM.Pos;
	return absf(Pos.Pos.x - RM.GetPos().x) < 4 * Lib->Size && absf(Pos.Pos.z - RM.GetPos().z) < 4 * Lib->Size;
}

float Tank::SelectDist(const IMATH::Vector2D &Pos)
{
	IMATH::Vector ViewPos = Specs.DS->ViewMtx * RM.GetPos();
	if(ViewPos.z >= 0)
	{
		ScreenPos = Specs.DS->ProjMtx * ViewPos;
		IMATH::Vector ScreenPosX = Specs.DS->ProjViewMtx * (RM.GetPos() + IMATH::Vector(1, 0, 0));
		IMATH::Vector ScreenPosY = Specs.DS->ProjViewMtx * (RM.GetPos() + IMATH::Vector(0, 1, 0));
		IMATH::Vector ScreenPosZ = Specs.DS->ProjViewMtx * (RM.GetPos() + IMATH::Vector(0, 0, 1));
		float Size = max((ScreenPosX - ScreenPos).Length(), max((ScreenPosY - ScreenPos).Length(), (ScreenPosZ - ScreenPos).Length()));
		if(Size == 0)
			Size = 0.001f;
		return (Pos - IMATH::Vector2D(ScreenPos.x, ScreenPos.y)).Length() / Size;
	}
	else
		return 10000;
}

bool Tank::Select(const IMATH::Vector2D &Pos)
{
	return SelectDist(Pos) < 2 * Lib->Size;
}

bool Tank::Select(I3D::BoundingBox2D *BBox)
{
	if(UnLinked) throw("Actor::GetPos#Access to unlinked Actor");
	ScreenPos = Specs.DS->ProjViewMtx * RM.GetPos();
	IMATH::Vector2D Scr(ScreenPos.x, ScreenPos.y);
	return BBox->IsIn(&Scr);
}

void Tank::RenderSelection(I3D::D3DStruct *DS)
{
	int State = 0;
	try
	{
		State = 1;
		if(UnLinked) throw("Actor::GetPos#Access to unlinked Actor");
		State = 2;
		// alpha blending
		DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
		DS->D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);
		DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATER);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHAREF, 0x10);
		DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
		// view and projection matrices
		D3DMATRIX Matrix = {1,0,0,0,  0,1,0,0,  0,0,1,0,  0,0,0,1};
		DS->D3DDevice->SetTransform(D3DTS_VIEW, &Matrix);
		DS->D3DDevice->SetTransform(D3DTS_PROJECTION, &Matrix);
		// get position on screen
		IMATH::Vector ViewPos = DS->ViewMtx * RM.GetPos();
		ScreenPos = DS->ProjMtx * ViewPos;
	//	if(ScreenPos.z <= 1 && ScreenPos.z >= 0)
		State = 3;
		if(ViewPos.z >= 4)
		{
			State = 4;
			// world matrix
			D3DMATRIX WMatrix = {4.0f/ViewPos.z * Lib->Size,0,0,0,  0,4.0f/ViewPos.z * Lib->Size,0,0,  0,0,1,0,  ScreenPos.x,ScreenPos.y,0,1};
			DS->D3DDevice->SetTransform(D3DTS_WORLD, &WMatrix);
			// render Selection
			State = 5;
			DS->D3DDevice->SetTexture(0, SelectionTexture);
			//DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
			DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCCOLOR);
			State = 6;
			OverlayRS.RenderIndex(DS->D3DDevice);
			State = 7;
			// render Health
			DS->D3DDevice->SetTexture(0, NULL);
			DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ZERO);
			State = 8;
			HealthRS.RenderVertex(DS->D3DDevice);
			State = 9;
			// reset world matrix
			DS->D3DDevice->SetTransform(D3DTS_WORLD, &Matrix);
			State = 10;
		}
		State = 11;
		// reset matrices
		DS->D3DDevice->SetTransform(D3DTS_VIEW, &DS->ViewMtx);
		DS->D3DDevice->SetTransform(D3DTS_PROJECTION, &DS->ProjMtx);
		// reset alpha blending
		DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, TRUE);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, FALSE);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
		State = 12;
	}
	catch(...)
	{
		char Str[1024];
		sprintf(Str, "Tank::RenderSelection#%d", State);
		throw(Str);
	}
}

void Tank::ProjectileHit(bool Hit)
{
	try
	{
		if(Hit)
			HitQuality = 1.0f;
		else
			HitQuality = 0.8f * HitQuality;
	}
	catch(char *s)
	{
		throw(s);
	}
	catch(...)
	{
		throw("exception in Tank::ProjectileHit");
	}
}

}
