/***********************************************************/
/*                                                         */
/* TankLib.cpp                                             */
/*                                                         */
/* created 04.03.2003                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "TankLib.h"
#include "D3DStruct.h"
#include <cstdio>
#include <fstream>

// www.infinitecode.com game library
namespace IGAME
{

void TankLib::Create(void)
{
	ITL::IniLoader TankData;
	std::ofstream of("tlib.txt");
	TankData = ITL::IniLoader("Libs\\Tanks.ini");
	for(int i = 0; i < TankData.CountGroups(); i++)
	{
		Tank Tnk;
		of << "tank" << std::endl;
		of << (Tnk.IsSpecialBuilding?"true":"false") << std::endl;
		for(int j = 0; j < TankData.CountKeys(i);j++)
		{
			const char *Key = TankData.GetKey(i, j);
			const char *Value = TankData.GetValue(i, j);
			// ID
			if(!stricmp(Key, "ID"))
				Tnk.ID = atoi(Value);
			if(!stricmp(Key, "Side"))
			{
				if(!stricmp(Value, "Israel"))
					Tnk.Side = SIDE_ISRAEL;
				else if(!stricmp(Value, "Palestine"))
					Tnk.Side = SIDE_PALESTINE;
				else if(!stricmp(Value, "Neutral"))
					Tnk.Side = SIDE_NEUTRAL;
				else if(!stricmp(Value, "Biest"))
					Tnk.Side = SIDE_BIEST;
			}
			// Projectile
			if(!stricmp(Key, "Projectile"))
				Tnk.Projectile = atoi(Value);
			if(!stricmp(Key, "Size"))
				Tnk.Size = (float)atof(Value);
			if(!stricmp(Key, "Model"))
				sprintf(Tnk.Model, "Models\\%s", Value);
			if(!stricmp(Key, "TurretModel"))
				sprintf(Tnk.TurretModel, "Models\\%s", Value);
			if(!stricmp(Key, "ExplodeSound"))
				sprintf(Tnk.ExplodeSound, "Sounds\\%s", Value);
			if(!stricmp(Key, "Name"))
				strcpy(Tnk.Name, Value);
			if(!stricmp(Key, "HitPoints"))
				Tnk.HitPoints = (float)atof(Value);
			if(!stricmp(Key, "MaxSpeed"))
				Tnk.MaxSpeed = (float)atof(Value);
			if(!stricmp(Key, "MaxTurnSpeed"))
				Tnk.MaxTurnSpeed = (float)atof(Value);
			if(!stricmp(Key, "ShootRange"))
				Tnk.ShootRange = (float)atof(Value);
			if(!stricmp(Key, "HasTurret"))
				Tnk.HasTurret = !stricmp(Value, "true");
			if(!stricmp(Key, "CanShoot"))
				Tnk.CanShoot = !stricmp(Value, "true");
			if(!stricmp(Key, "CanMove"))
				Tnk.CanMove = !stricmp(Value, "true");
			if(!stricmp(Key, "IsShip"))
				Tnk.IsShip = !stricmp(Value, "true");
			if(!stricmp(Key, "IsAirplane"))
				Tnk.IsAirplane = !stricmp(Value, "true");
			if(!stricmp(Key, "IsCivilBuilding"))
				Tnk.IsCivilBuilding = !stricmp(Value, "true");
			if(!stricmp(Key, "IsSpecialBuilding"))
			{
				Tnk.IsSpecialBuilding = !stricmp(Value, "true");
				of << "setting " << (Tnk.IsSpecialBuilding?"true":"false") << std::endl;
			}
			of << Key << " " << (Tnk.IsSpecialBuilding?"true":"false") << std::endl;
			if(!stricmp(Key, "IsBaseBuilding"))
				Tnk.IsBaseBuilding = !stricmp(Value, "true");
			if(!stricmp(Key, "IsAirbaseBuilding"))
				Tnk.IsAirbaseBuilding = !stricmp(Value, "true");
			if(!stricmp(Key, "IsBaseVehicle"))
				Tnk.IsBaseVehicle = !stricmp(Value, "true");
			if(!stricmp(Key, "NeutralInBase"))
				Tnk.NeutralInBase = !stricmp(Value, "true");
			if(!stricmp(Key, "AI"))
				Tnk.AI = !stricmp(Value, "true");
			if(!stricmp(Key, "ProjectileSpawnPosX"))
				Tnk.ProjectileSpawnPos.x = (float)atof(Value);
			if(!stricmp(Key, "ProjectileSpawnPosY"))
				Tnk.ProjectileSpawnPos.y = (float)atof(Value);
			if(!stricmp(Key, "ProjectileSpawnPosZ"))
				Tnk.ProjectileSpawnPos.z = (float)atof(Value);
			if(!stricmp(Key, "Dust"))
				Tnk.Dust = !stricmp(Value, "true");
			if(!stricmp(Key, "DustPFX"))
				Tnk.DustPFX = atoi(Value);
			if(!stricmp(Key, "TurnX"))
				Tnk.Turn.x = (float)atof(Value);
			if(!stricmp(Key, "TurnY"))
				Tnk.Turn.y = (float)atof(Value);
			if(!stricmp(Key, "TurnZ"))
				Tnk.Turn.z = (float)atof(Value);
			if(!stricmp(Key, "PitchRollX"))
				Tnk.PitchRoll.x = (float)atof(Value);
			if(!stricmp(Key, "PitchRollY"))
				Tnk.PitchRoll.y = (float)atof(Value);
			if(!stricmp(Key, "PitchRollZ"))
				Tnk.PitchRoll.z = (float)atof(Value);
			if(!stricmp(Key, "TurretTurnX"))
				Tnk.TurretTurn.x = (float)atof(Value);
			if(!stricmp(Key, "TurretTurnY"))
				Tnk.TurretTurn.y = (float)atof(Value);
			if(!stricmp(Key, "TurretTurnZ"))
				Tnk.TurretTurn.z = (float)atof(Value);
			if(!stricmp(Key, "TurretPitchX"))
				Tnk.TurretPitch.x = (float)atof(Value);
			if(!stricmp(Key, "TurretPitchY"))
				Tnk.TurretPitch.y = (float)atof(Value);
			if(!stricmp(Key, "TurretPitchZ"))
				Tnk.TurretPitch.z = (float)atof(Value);
			if(!stricmp(Key, "Burst"))
				Tnk.Burst = !stricmp(Value, "true");
			if(!stricmp(Key, "BurstNumber"))
				Tnk.BurstNumber = atoi(Value);
			if(!stricmp(Key, "BurstDelay"))
				Tnk.BurstDelay = (float)atof(Value);
			if(!stricmp(Key, "BurstDisplacementX"))
				Tnk.BurstDisplacement.x = (float)atof(Value);
			if(!stricmp(Key, "BurstDisplacementY"))
				Tnk.BurstDisplacement.y = (float)atof(Value);
			if(!stricmp(Key, "BurstDisplacementZ"))
				Tnk.BurstDisplacement.z = (float)atof(Value);
		}
		Tanks[Tnk.ID] = Tnk;
	}
}

}
