/***********************************************************/
/*                                                         */
/* TankLib.h                                               */
/*                                                         */
/* created 04.03.2003                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef TANKLIB_H
#define TANKLIB_H

#include "IniLoader.h"
#include "DynamicArray.h"
#include "MathStructures.h"
#include "IDSide.h"

#include <fstream>
#include <windows.h>

// www.infinitecode.com game library
namespace IGAME
{

class TankLib
{
public:
	#define TNKL_STRMEM 1024

	class Tank
	{
	public:
		Tank()
			: Projectile(0), HasTurret(false), ProjectileSpawnPos(0, 0, 0), Dust(false), DustPFX(0), AI(true)
			, Turn(0, 0, 0), PitchRoll(0, 0, 0), TurretTurn(0, 0, 0), TurretPitch(0, 0, 0), CanMove(true)
			, CanShoot(true), ShootRange(100), IsShip(false), IsAirplane(false), MaxSpeed(10000.0f)
			, MaxTurnSpeed(10000.0f), IsCivilBuilding(false), IsSpecialBuilding(false), IsBaseBuilding(false)
			, IsAirbaseBuilding(false), IsBaseVehicle(false), NeutralInBase(false), Side(SIDE_NEUTRAL), Burst(false), BurstNumber(1)
		{
			*Model = 0;
			*TurretModel = 0;
			*Name = 0;
			*ExplodeSound = 0;
			//MessageBox(0, "foo", "bar", MB_OK);
			std::ofstream of("tlib_constr.txt");
			of << std::endl;
			of << "constr " << (IsSpecialBuilding?"true":"false") << std::endl;
			of.flush();
			of.close();
		}

		int ID, Projectile, DustPFX, Side, BurstNumber;
		float Size, HitPoints, ShootRange, MaxSpeed, MaxTurnSpeed, BurstDelay;
		bool HasTurret, CanShoot, CanMove, AI, Dust, IsShip, IsAirplane, IsCivilBuilding, IsBaseBuilding, IsAirbaseBuilding, IsBaseVehicle, IsSpecialBuilding, NeutralInBase, Burst;
		IMATH::Vector ProjectileSpawnPos, Turn, PitchRoll, TurretTurn, TurretPitch, BurstDisplacement;
		char Model[TNKL_STRMEM], TurretModel[TNKL_STRMEM], Name[TNKL_STRMEM], ExplodeSound[TNKL_STRMEM];
	};

	ISL::DynamicArray<Tank, 0, 1> Tanks;

	void Create(void);
};

}

#endif
