/***********************************************************/
/*                                                         */
/* TerrainLine.cpp                                         */
/*                                                         */
/* created 14.04.2003                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "TerrainLine.h"

// www.infinitecode.com 3d library
namespace I3D
{

TerrainLine::TerrainLineSpecs::TerrainLineSpecs(void)
	:Width(1.0f)
{
	Color = 0xFFFFFFFF;
}

TerrainLine::TerrainLine(void)
	: used(false)
{
}

TerrainLine::~TerrainLine(void)
{
	Destroy();
}

void TerrainLine::Create(const TerrainLine::TerrainLineSpecs *Specs)
{
	try
	{
		if(used)
			TerrainLine::Specs.TLR->Delete(this);
		used = true;
		TerrainLine::Specs = *Specs;
		// calculate RS
		const IMATH::Vector2DXZ X = TerrainLine::Specs.Pos[1] - TerrainLine::Specs.Pos[0];
		const IMATH::Vector2DXZ Z = TerrainLine::Specs.Width * IMATH::Vector2DXZ(X.z, -X.x).Normalize();
		const IMATH::Vector Y = IMATH::Vector(0, 1, 0);
		float Len = (float)floor(X.Length() / 32);
		if(Len < 1)
			Len = 1;
		RS.ClearVertex();
		if(Len)
		{
			IMATH::Vector2DXZ Pos1 = TerrainLine::Specs.Pos[0];
			IMATH::Vector2DXZ Pos2 = TerrainLine::Specs.Pos[0] + X / Len;
			for(int j = 0; j < Len; j++)
			{
				IMATH::Vector2DXZ PointXZ = Pos1 - Z;
				IMATH::Vector Point = IMATH::Vector(PointXZ.x, TerrainLine::Specs.TI->GetY(PointXZ.x, PointXZ.z), PointXZ.z) + Y;
				RS.AddVertex(LVertex(Point, TerrainLine::Specs.Color, IMATH::Vector2D(0, 0)));
				PointXZ = Pos1 + Z;
				Point = IMATH::Vector(PointXZ.x, TerrainLine::Specs.TI->GetY(PointXZ.x, PointXZ.z), PointXZ.z) + Y;
				RS.AddVertex(LVertex(Point, TerrainLine::Specs.Color, IMATH::Vector2D(0, 1)));
				PointXZ = Pos2 - Z;
				Point = IMATH::Vector(PointXZ.x, TerrainLine::Specs.TI->GetY(PointXZ.x, PointXZ.z), PointXZ.z) + Y;
				RS.AddVertex(LVertex(Point, TerrainLine::Specs.Color, IMATH::Vector2D(1, 0)));
				PointXZ = Pos2 - Z;
				Point = IMATH::Vector(PointXZ.x, TerrainLine::Specs.TI->GetY(PointXZ.x, PointXZ.z), PointXZ.z) + Y;
				RS.AddVertex(LVertex(Point, TerrainLine::Specs.Color, IMATH::Vector2D(1, 0)));
				PointXZ = Pos1 + Z;
				Point = IMATH::Vector(PointXZ.x, TerrainLine::Specs.TI->GetY(PointXZ.x, PointXZ.z), PointXZ.z) + Y;
				RS.AddVertex(LVertex(Point, TerrainLine::Specs.Color, IMATH::Vector2D(0, 1)));
				PointXZ = Pos2 + Z;
				Point = IMATH::Vector(PointXZ.x, TerrainLine::Specs.TI->GetY(PointXZ.x, PointXZ.z), PointXZ.z) + Y;
				RS.AddVertex(LVertex(Point, TerrainLine::Specs.Color, IMATH::Vector2D(1, 1)));
				if(j < Len - 1)
				{
					Pos1 += X / Len;
					Pos2 += X / Len;
				}
			}
		}
		RS.CreateVBuf(TerrainLine::Specs.DS->D3DDevice);
		// add to TLR
		TerrainLine::Specs.TLR->Add(this);
	}
	catch(char *s)
	{
		throw s;
	}
	catch(...)
	{
		throw "Error in TerrainLine::Create";
	}
}

void TerrainLine::Init3D(D3DStruct *DS, bool recurse)
{
	RS.CreateVBuf(TerrainLine::Specs.DS->D3DDevice);
}

void TerrainLine::Destroy(void)
{
	if(used)
	{
		UnLink();
		used = false;
	}
}

void TerrainLine::UnLink(void)
{
	Specs.TLR->Delete(this);
}

void TerrainLine::Render(D3DStruct *DS)
{
	DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, false);
	DS->D3DDevice->SetTexture(0, 0);
	RS.RenderVertex(DS->D3DDevice);
	DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, true);
}

}
