/***********************************************************/
/*                                                         */
/* TerrainLineRenderer.cpp                                 */
/*                                                         */
/* created 14.04.2003                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "TerrainLineRenderer.h"
#include "RenderStruct.h"
#include "TerrainLine.h"

// www.infinitecode.com 3d interface
namespace I3D
{

TerrainLineRenderer::~TerrainLineRenderer(void)
{
	Destroy();
}

void TerrainLineRenderer::Create(D3DStruct *DS)
{
}

void TerrainLineRenderer::Init3D(D3DStruct *DS, bool recurse)
{
	if(recurse)
		for(int i = 0; i < Lines.Count(); i++)
			Lines[i]->Init3D(DS, true);
}

void TerrainLineRenderer::Destroy(void)
{
	if(Lines.Count())
		for(int i = 0; i < Lines.Count(); i++)
			Lines[i]->UnLink();
}

void TerrainLineRenderer::Add(TerrainLine *TL)
{
	Lines.Add(TL);
}

void TerrainLineRenderer::Delete(TerrainLine *TL)
{
	Lines.Delete(TL);
}

void TerrainLineRenderer::Render(D3DStruct *DS)
{
	try
	{
		Matrix M = Matrix::IdentityMatrix();
		DS->D3DDevice->SetTransform(D3DTS_WORLD, &M);
		RenderStruct<LVertex, I3D_FVFLVERTEX, 0, 32> RS;
		for(int i = 0; i < Lines.Count(); i++)
		{
			Lines[i]->Render(DS);
		}
	}
	catch(char *s)
	{
		throw(s);
	}
	catch(...)
	{
		throw("TerrainLineRenderer::Render");
	}
}

}
