/***********************************************************/
/*                                                         */
/* Text.h                                                  */
/*                                                         */
/* created 29.08.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef TEXT_H
#define TEXT_H

#include "D3DStruct.h"
#include "RenderStruct.h"
#include "TextureManager.h"

#include <d3d9.h>

// www.infinitecode.com 3d library
namespace I3D
{

class Text
{
	static LPDIRECT3DTEXTURE9 LucidaConsole[4];
	static RenderStruct<Vertex, I3D_FVFVERTEX, 256, 256> RS;

public:

	static void Text::Create(D3DStruct *DS)
	{
		int i;
		for(i = 0; i < 4; i++)
		{
			char Str[1024];
			sprintf(Str, "Textures\\TypeB%d_white.bmp", i + 1);
			LucidaConsole[i] = TextureManager::GetTexture(DS, Str);
		}
		for(i = 0; i < 256; i++)
		{
			float XOfs = (i & 0x7) / 8.0f;
			float YOfs = ((i >> 3) & 0x7) / 8.0f;
			float Delta = 1 / 8.0f;
			RS.AddVertex(Vertex(Vector(0, 0, 0), Vector(0, 0, -1), Vector2D(XOfs, YOfs + Delta)));
			RS.AddVertex(Vertex(Vector(1, 0, 0), Vector(0, 0, -1), Vector2D(XOfs + Delta, YOfs + Delta)));
			RS.AddVertex(Vertex(Vector(1, 1, 0), Vector(0, 0, -1), Vector2D(XOfs + Delta, YOfs)));
			RS.AddVertex(Vertex(Vector(0, 1, 0), Vector(0, 0, -1), Vector2D(XOfs, YOfs)));
		}
		RS.CreateVBuf(DS->D3DDevice);
		RS.AddIndex(0);
		RS.AddIndex(1);
		RS.AddIndex(2);
		RS.AddIndex(0);
		RS.AddIndex(2);
		RS.AddIndex(3);
		RS.CreateIBuf(DS->D3DDevice);
	}

	static void Text::Init3D(D3DStruct *DS, bool recurse)
	{
		for(int i = 0; i < 4; i++)
		{
			char Str[1024];
			sprintf(Str, "Textures\\TypeB%d_white.bmp", i + 1);
			LucidaConsole[i] = TextureManager::GetTexture(DS, Str);
		}
		RS.CreateVBuf(DS->D3DDevice);
		RS.CreateIBuf(DS->D3DDevice);
	}

	static void Text::DrawChar(D3DStruct *DS, char c, float x, float y, float dx, float dy)
	{
		if(!LucidaConsole[0])
			Create(DS);
		Matrix Mtx, WMtx;
		Mtx.SetIdentityMatrix();
		// optimize antialiasing
		LPDIRECT3DSURFACE9 Sur;
		D3DSURFACE_DESC Desc;
		DS->D3DDevice->GetRenderTarget(0, &Sur);
		Sur->GetDesc(&Desc);
		Sur->Release();
		float tx = x - 0.2f * dx;
		float ty = y - 0.2f * dy;
		float tdx = 1.4f * dx;
		float tdy = 1.4f * dy;
		if(Desc.Width && Desc.Height)
		{
			int ix = (int)(((tx + 1) * (int)Desc.Width) / 2);
			tx = (float)((ix + 0.5f) * 2.0f / (int)Desc.Width - 1);
			int iy = (int)(((ty + 1) * (int)Desc.Height) / 2);
			ty = (float)((iy + 0.5f) * 2.0f / (int)Desc.Height - 1);
			int idx = (int)(((tdx + 1) * (int)Desc.Width) / 2);
			tdx = (float)((idx + 0.5f) * 2.0f / (int)Desc.Width - 1);
			int idy = (int)(((tdy + 1) * (int)Desc.Height) / 2);
			tdy = (float)((idy + 0.5f) * 2.0f / (int)Desc.Height - 1);
		}
		WMtx.SetScaleTranslationMatrix(Vector(tdx, tdy, 0), Vector(tx, ty, 0));
		DS->D3DDevice->SetTransform(D3DTS_VIEW, &Mtx);
		DS->D3DDevice->SetTransform(D3DTS_PROJECTION, &Mtx);
		DS->D3DDevice->SetTransform(D3DTS_WORLD, &WMtx);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
		DS->D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);
		DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATER);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHAREF, 0x10);
		DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
		DS->D3DDevice->SetTexture(0, LucidaConsole[c >> 6]);
		RS.RenderIndex(DS->D3DDevice, 4 * (unsigned char)c, 4, 0, 6);
		DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, TRUE);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, FALSE);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
		DS->D3DDevice->SetTransform(D3DTS_VIEW, &DS->ProjMtx);
		DS->D3DDevice->SetTransform(D3DTS_PROJECTION, &DS->ViewMtx);
	}

	static void Text::DrawString(D3DStruct *DS, const char *Str, float x, float y, float dx, float dy, float LineSpacing = 0.5f)
	{
		if(!LucidaConsole[0])
			Create(DS);
		Matrix Mtx, WMtx;
		Mtx.SetIdentityMatrix();
		DS->D3DDevice->SetTransform(D3DTS_VIEW, &Mtx);
		DS->D3DDevice->SetTransform(D3DTS_PROJECTION, &Mtx);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
		DS->D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);
		DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATER);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHAREF, 0x10);
		DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
		DS->D3DDevice->SetTexture(0, 0);
		float nx = x;
		float ny = y - dy;
		for(int i = 0; i < (int)strlen(Str); i++)
		{
			DS->D3DDevice->SetTexture(0, LucidaConsole[(unsigned const char)Str[i] >> 6]);
			// optimize antialiasing
			LPDIRECT3DSURFACE9 Sur;
			D3DSURFACE_DESC Desc;
			DS->D3DDevice->GetRenderTarget(0, &Sur);
			Sur->GetDesc(&Desc);
			Sur->Release();
			float tx = nx - 0.2f * dx;
			float ty = ny - 0.2f * dy;
			float tdx = 1.4f * dx;
			float tdy = 1.4f * dy;
			if(Desc.Width && Desc.Height)
			{
				int ix = (int)(((tx + 1) * (int)Desc.Width) / 2);
				tx = (float)((ix + 0.5f) * 2.0f / (int)Desc.Width - 1);
				int iy = (int)(((ty + 1) * (int)Desc.Height) / 2);
				ty = (float)((iy + 0.5f) * 2.0f / (int)Desc.Height - 1);
				int idx = (int)(((tdx + 1) * (int)Desc.Width) / 2);
				tdx = (float)((idx + 0.5f) * 2.0f / (int)Desc.Width - 1);
				int idy = (int)(((tdy + 1) * (int)Desc.Height) / 2);
				tdy = (float)((idy + 0.5f) * 2.0f / (int)Desc.Height - 1);
				WMtx.SetScaleTranslationMatrix(Vector(tdx, tdy, 0), Vector(tx, ty, 0));
				DS->D3DDevice->SetTransform(D3DTS_WORLD, &WMtx);
			}

			nx += dx;
			if(Str[i] == '\n')
			{
				nx = x;
				ny -= (1 + LineSpacing) * dy;
			}
			else
				RS.RenderIndex(DS->D3DDevice, 4 * (unsigned const char)Str[i], 4, 0, 6);
		}
		DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, TRUE);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, FALSE);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
		DS->D3DDevice->SetTransform(D3DTS_VIEW, &DS->ProjMtx);
		DS->D3DDevice->SetTransform(D3DTS_PROJECTION, &DS->ViewMtx);
	}

};

}

#endif
