/***********************************************************/
/*                                                         */
/* TextField.cpp                                           */
/*                                                         */
/* created 08.03.2003                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "TextField.h"
#include "Text.h"

#include <fstream>

// www.infinitecode.com 3d library
namespace I3D
{

LPDIRECT3DTEXTURE9 TextField::Background = 0;

void TextField::Create(D3DStruct *DS)
{
	std::ofstream os("textfield.txt");
	os << "1" << std::endl;
	//Text.str(TextString);
	os << "2" << std::endl;
	Clear();
	os << "3" << std::endl;
	RS.AddVertex(Vertex(Vector(0, 0, 0), Vector(0, 0, -1), Vector2D(0, 0)));
	RS.AddVertex(Vertex(Vector(1, 0, 0), Vector(0, 0, -1), Vector2D(1, 0)));
	RS.AddVertex(Vertex(Vector(1, 1, 0), Vector(0, 0, -1), Vector2D(1, 1)));
	RS.AddVertex(Vertex(Vector(0, 1, 0), Vector(0, 0, -1), Vector2D(0, 1)));
	RS.AddIndex(0);
	RS.AddIndex(1);
	RS.AddIndex(2);
	RS.AddIndex(0);
	RS.AddIndex(2);
	RS.AddIndex(3);
	os << "4" << std::endl;
	Init3D(DS, false);
	os << "5" << std::endl;
}

void TextField::Clear()
{
	TextString = "";
	//Text.str("");
}

void TextField::Init3D(D3DStruct *DS, bool recurse)
{
	Background = TextureManager::GetTexture(DS, "Textures\\TextField.bmp", 0, TM_FLAG_ALPHA80);
	RS.CreateVBuf(DS->D3DDevice);
	RS.CreateIBuf(DS->D3DDevice);
}

void TextField::Render(D3DStruct *DS, float x, float y, float dx, float dy)
{
	Matrix Mtx, WMtx;
	Mtx.SetIdentityMatrix();
	WMtx.SetScaleTranslationMatrix(Vector(dx, dy, 0), Vector(x, y, 0));
	DS->D3DDevice->SetTransform(D3DTS_VIEW, &Mtx);
	DS->D3DDevice->SetTransform(D3DTS_PROJECTION, &Mtx);
	DS->D3DDevice->SetTransform(D3DTS_WORLD, &WMtx);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
	DS->D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATER);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHAREF, 0x10);
	DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
	DS->D3DDevice->SetTexture(0, Background);
	// Background
	RS.RenderIndex(DS->D3DDevice, 0, 4, 0, 6);
	// Text
	float Width = dx / 24.0f;
	float Height = dy / 24.0f;
	//Text::DrawString(DS, Text.str().c_str(), x, y + dy - 0.5f * Height, Width, Height, 0.5f);
	Text::DrawString(DS, TextString.c_str(), x, y + dy - 0.5f * Height, Width, Height, 0.5f);
	DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, TRUE);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, FALSE);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
	DS->D3DDevice->SetTransform(D3DTS_VIEW, &DS->ProjMtx);
	DS->D3DDevice->SetTransform(D3DTS_PROJECTION, &DS->ViewMtx);
}

}
