/***********************************************************/
/*                                                         */
/* Tree.cpp                                                */
/*                                                         */
/* created 16.07.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "Tree.h"
#include "QuadTree.h"
#include "TextureManager.h"
#include "IMath.h"

// www.infinitecode.com game library
namespace IGAME
{

int Tree::Type;

void Tree::Create(Tree::TreeSpecs *Specs)
{
	if(Valid)
		Destroy();
	using namespace I3D;
	Tree::Specs = *Specs;
	// position
	RM.SetBase(this);
	// matrix
	// texture
	TreeType = rand() % 4;
	switch(TreeType)
	{
	case 0:
		RM.Mdl = TreeModel::GetModel(Specs->DS, "Models\\tree1.x");
		RM.Texture = TextureManager::GetTexture(Specs->DS, "Textures\\trees.bmp", "Textures\\trees_alpha.bmp", TM_FLAG_MIPALPHAIGN | TM_FLAG_ACCBILOPT | TM_FLAG_MIPALPHA60);
		break;
	case 1:
		RM.Mdl = TreeModel::GetModel(Specs->DS, "Models\\palm1.x");
		RM.Texture = TextureManager::GetTexture(Specs->DS, "Textures\\trees.bmp", "Textures\\trees_alpha.bmp", TM_FLAG_MIPALPHAIGN | TM_FLAG_ACCBILOPT | TM_FLAG_MIPALPHA60);
		break;
	case 2:
		RM.Mdl = TreeModel::GetModel(Specs->DS, "Models\\palm2.x");
		RM.Texture = TextureManager::GetTexture(Specs->DS, "Textures\\trees.bmp", "Textures\\trees_alpha.bmp", TM_FLAG_MIPALPHAIGN | TM_FLAG_ACCBILOPT | TM_FLAG_MIPALPHA60);
		break;
	case 3:
		RM.Mdl = TreeModel::GetModel(Specs->DS, "Models\\olivetree.x");
		RM.Texture = TextureManager::GetTexture(Specs->DS, "Textures\\olivetree.bmp", "Textures\\olivetree_alpha.bmp", TM_FLAG_MIPALPHAIGN | TM_FLAG_ACCBILOPT | TM_FLAG_MIPALPHA60);
		break;
	}
	// model
	//RM.Mdl = TreeModel::GetModel(Specs->DS, "Models\\Tree.t4m");
	//RM.Mtx = new Matrix();
	((Matrix *)RM.Mtx)->SetTranslationMatrix(RM.GetPos());
	//Specs->QT->AddRenderableModel(&RM);
	Tree::Specs.Angle = (rand() % 16384) / 16384.0f * 2 * PI;
	RM.SetPos(Tree::Specs.Pos);
	FallAngle = 0;
	Falling = false;
	//ScreenPos = Tree::Specs.DS->ProjViewMtx * RM.Pos;
	Tree::Specs.SM->AddManagedObject(&RM);
	Tree::Specs.PosMgr->AddManagedObject(this);
	((I3D::Matrix *)RM.Mtx)->SetTangentMatrix(RM.GetPos(), Tree::Specs.Angle, IMATH::Vector(0, 1, 0));
	Valid = true;
}

void Tree::Init3D(I3D::D3DStruct *DS, bool recurse)
{
	using namespace I3D;
	switch(TreeType)
	{
	case 0:
		RM.Texture = TextureManager::GetTexture(DS, "Textures\\trees.bmp", "Textures\\trees_alpha.bmp", TM_FLAG_MIPALPHAIGN | TM_FLAG_ACCBILOPT | TM_FLAG_MIPALPHA60);
		break;
	case 1:
		RM.Texture = TextureManager::GetTexture(DS, "Textures\\trees.bmp", "Textures\\trees_alpha.bmp", TM_FLAG_MIPALPHAIGN | TM_FLAG_ACCBILOPT | TM_FLAG_MIPALPHA60);
		break;
	case 2:
		RM.Texture = TextureManager::GetTexture(DS, "Textures\\trees.bmp", "Textures\\trees_alpha.bmp", TM_FLAG_MIPALPHAIGN | TM_FLAG_ACCBILOPT | TM_FLAG_MIPALPHA60);
		break;
	case 3:
		RM.Texture = TextureManager::GetTexture(DS, "Textures\\olivetree.bmp", "Textures\\olivetree_alpha.bmp", TM_FLAG_MIPALPHAIGN | TM_FLAG_ACCBILOPT | TM_FLAG_MIPALPHA60);
		break;
	}
}

Tree::~Tree(void)
{
	if(Valid)
		Destroy();
}

void Tree::Destroy()
{
	Tree::Specs.SM->DeleteManagedObject(&RM);
	Tree::Specs.PosMgr->DeleteManagedObject(this);
	Valid = false;
}

void Tree::Collision(const IMATH::Vector &V)
{
	if(!Falling)
	{
		Falling = true;
		if(V.z > 0)
			CollisionAngle = (float)atan((double)(V.x / V.z));
		else if(V.z < 0)
			CollisionAngle = PI + (float)atan((double)(V.x / V.z));
		else
			CollisionAngle = PI / 2;
		CollisionSpeed = 0.125f * V.Length();
		if(CollisionSpeed < 0.5f)
			CollisionSpeed = 0.5f;
	}
}

void Tree::Physics(I3D::D3DStruct *DS)
{
	if(Falling)
	{
		FallAngle += CollisionSpeed * DS->Sp->Delta;
		if(FallAngle >= PI / 2)
			FallAngle = PI / 2;
		((I3D::Matrix *)RM.Mtx)->SetTangentMatrix(RM.GetPos(), Tree::Specs.Angle,
			IMATH::Vector((float)sin((double)CollisionAngle) * (float)sin((double)FallAngle),
			(float)cos((double)FallAngle),
			(float)cos((double)CollisionAngle) * (float)sin((double)FallAngle)));
		Tree::Specs.SM->MoveManagedObject(&RM);
		Tree::Specs.PosMgr->MoveManagedObject(this);
	}
}

// ****** TreeModel ******

void TreeModel::PreRender(I3D::D3DStruct *DS)
{
	DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
	DS->D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATER);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHAREF, 0xC0);
	DS->D3DDevice->SetRenderState(D3DRS_CULLMODE,D3DCULL_NONE);
/*	DS->D3DDevice->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_POINT);
	DS->D3DDevice->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_POINT);
	DS->D3DDevice->SetTextureStageState(0, D3DTSS_MIPFILTER, D3DTEXF_POINT);*/
	//DS->D3DDevice->SetRenderState(D3DRS_LIGHTING,FALSE);
}

void TreeModel::PostRender(I3D::D3DStruct *DS)
{
/*	DS->D3DDevice->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
	DS->D3DDevice->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
	DS->D3DDevice->SetTextureStageState(0, D3DTSS_MIPFILTER, D3DTEXF_POINT);*/
	DS->D3DDevice->SetRenderState(D3DRS_CULLMODE,D3DCULL_CW);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, FALSE);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
	D3DMATRIX WMatrix = {1,0,0,0,  0,1,0,0,  0,0,1,0,  0,0,0,1};
    DS->D3DDevice->SetTransform(D3DTS_WORLD, &WMatrix);
}

}
