/***********************************************************/
/*                                                         */
/* Tree.h                                                  */
/*                                                         */
/* created 16.07.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef TREE_H
#define TREE_H

#include "RenderableModel.h"
#include "MathStructures.h"
#include "Model.h"
#include "QuadTree.h"
#include "SpaceManagerInterface.h"

// www.infinitecode.com game library
namespace IGAME
{

class Tree: public I3D::ManagedObject
{
public:

	struct TreeSpecs
	{
		I3D::D3DStruct *DS;
		IMATH::Vector Pos;
		I3D::QuadTree *QT;
		I3D::SpaceManagerInterface *SM;
		I3D::SpaceManagerInterface *PosMgr;
		float Angle;
	};

private:
	TreeSpecs Specs;
	I3D::RenderableModel RM;
	static int Type;
	float FallAngle;
	bool Falling, Valid;
	float CollisionAngle, CollisionSpeed;
	int TreeType;

public:
	Tree(): Valid(false) {}
	~Tree(void);

	static void *GetClassType()
	{
		return &Type;
	}

	virtual void *GetType(void) const
	{
		return &Type;
	}

	virtual void *GetBase(void) const
	{
		return (void *)this;
	}

	virtual void SetPos(const IMATH::Vector &Value)
	{
		RM.SetPos(Value);
	}

	virtual const IMATH::Vector &GetPos(void) const
	{
		return RM.GetPos();
	}

	virtual void Create(TreeSpecs *Specs);
	void Init3D(I3D::D3DStruct *DS, bool recurse);
	virtual void Destroy();
	virtual void Physics(I3D::D3DStruct *DS);
	void Collision(const IMATH::Vector &V);
};

// ****** TreeModel ******

class TreeModel: public I3D::Model
{

public:
	TreeModel(void) {}

	static I3D::Model *GetModel(I3D::D3DStruct *DS, char *Name)
	{
		for(int i = 0; i < LoadedModels.Count(); i++)
			if(!strcmpi(Name, LoadedModels[i].Name))
				return LoadedModels[i].Mdl;
		I3D::Model *NewModel = new TreeModel();
		NewModel->Create(DS, Name);
		LoadedModels[LoadedModels.Count()] = LoadedModel(Name, NewModel);
		return NewModel;
	}

	virtual void PreRender(I3D::D3DStruct *DS);
	virtual void PostRender(I3D::D3DStruct *DS);
};

}

#endif
