/***********************************************************/
/*                                                         */
/* WaterPlane.cpp                                          */
/*                                                         */
/* created 16.06.2003                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "WaterPlane.h"
#include "TextureManager.h"
#include "IMath.h"

#include <d3dx9.h>

// www.infinitecode.com 3d library
namespace I3D
{

unsigned *WaterPlane::PShader = 0;

void WaterPlane::Create(D3DStruct *DS, BoundingBox *BBox, float WaterLevel)
{
	WaterPlane::BBox = *BBox;
	WaterPlane::WaterLevel = WaterLevel;
	Tex = TextureManager::GetTexture(DS, "Textures\\Water.bmp", 0, TM_FLAG_ALPHA60);
	Bump = TextureManager::GetTexture(DS, "Textures\\Water.bmp", 0, TM_FLAG_ALPHA60);
	Env = TextureManager::GetTexture(DS, "Textures\\Water.bmp", 0, TM_FLAG_ALPHA60);
	// create model
	Vertices[0] = LVertex(Vector(WaterPlane::BBox.Data[BB_LEFT], WaterLevel, WaterPlane::BBox.Data[BB_FAR]), 0xFFFFFFFF, Vector2D(0, 0));
	Vertices[1] = LVertex(Vector(WaterPlane::BBox.Data[BB_LEFT], WaterLevel, WaterPlane::BBox.Data[BB_NEAR]), 0xFFFFFFFF, Vector2D(0, 1));
	Vertices[2] = LVertex(Vector(WaterPlane::BBox.Data[BB_RIGHT], WaterLevel, WaterPlane::BBox.Data[BB_NEAR]), 0xFFFFFFFF, Vector2D(1, 1));
	Vertices[3] = LVertex(Vector(WaterPlane::BBox.Data[BB_RIGHT], WaterLevel, WaterPlane::BBox.Data[BB_FAR]), 0xFFFFFFFF, Vector2D(1, 0));
	RS.CreateVBuf(DS->D3DDevice);
	RS.AddIndex(0);
	RS.AddIndex(1);
	RS.AddIndex(2);
	RS.AddIndex(0);
	RS.AddIndex(2);
	RS.AddIndex(3);
	RS.CreateIBuf(DS->D3DDevice);
	// create pixel shader
	/*if(!PShader)
	{
		try
		{
			D3DCAPS8 caps;
			DS->D3DDevice->GetDeviceCaps(&caps);
			if(caps.PixelShaderVersion >= D3DPS_VERSION(1, 1))
			{
				//const char *Code = "ps.1.0\nmov r0, v0\n";
				LPD3DXBUFFER Compiled, Constants, Errors;
				//if(D3DXAssembleShader(Code, strlen(Code), 0, 0, &Compiled, &Errors) != D3D_OK)
				if(D3DXAssembleShaderFromFileA("FX\\Water.tps", 0, &Constants, &Compiled, &Errors) != D3D_OK)
					throw((char *)Errors->GetBufferPointer());
				DWORD *Buffer = (DWORD *)Compiled->GetBufferPointer();
				DS->D3DDevice->CreatePixelShader(Buffer, (DWORD *)&PShader);
			}
		}
		catch(char *s)
		{
			throw(s);
		}
		catch(...)
		{
			throw("WaterPlane::Create#pixel shader");
		}
	}*/
}

void WaterPlane::Init3D(D3DStruct *DS, bool recurse)
{
	Tex = TextureManager::GetTexture(DS, "Textures\\Water.bmp", 0, TM_FLAG_ALPHA60);
	Bump = TextureManager::GetTexture(DS, "Textures\\Water.bmp", 0, TM_FLAG_ALPHA60);
	Env = TextureManager::GetTexture(DS, "Textures\\Water.bmp", 0, TM_FLAG_ALPHA60);
}

void WaterPlane::CreateView(D3DStruct *DS)
{
	RS.CreateVBuf(DS->D3DDevice);
	RS.CreateIBuf(DS->D3DDevice);
}

void WaterPlane::Render(D3DStruct *DS)
{
	// set renderstates
	DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
	DS->D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
	DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
	Matrix Mtx;
	Mtx.SetIdentityMatrix();
//	DS->D3DDevice->SetTransform(D3DTS_VIEW, &Mtx);
//	DS->D3DDevice->SetTransform(D3DTS_PROJECTION, &Mtx);
	DS->D3DDevice->SetTransform(D3DTS_WORLD, &Mtx);

	//if(PShader)
	//	DS->D3DDevice->SetPixelShader((DWORD)PShader);
	// update flow
	Vertices[0].y = WaterLevel + cos(DS->Sp->GameTime + (2 * PI / 512.0f) * (Vertices[0].x + Vertices[0].z));
	Vertices[1].y = WaterLevel + cos(DS->Sp->GameTime + (2 * PI / 512.0f) * (Vertices[1].x + Vertices[1].z));
	Vertices[2].y = WaterLevel + cos(DS->Sp->GameTime + (2 * PI / 512.0f) * (Vertices[2].x + Vertices[2].z));
	Vertices[3].y = WaterLevel + cos(DS->Sp->GameTime + (2 * PI / 512.0f) * (Vertices[3].x + Vertices[3].z));

	RS.ClearVertexElements();
	for(int i = 0; i < 4; ++i)
		RS.AddVertex(Vertices[i]);
	RS.CreateVBuf(DS->D3DDevice);
	// render
	DS->D3DDevice->SetTexture(0, Tex);
	RS.RenderIndex(DS->D3DDevice, 0, 4, 0, 6);
	// reset renderstates
	if(PShader)
		DS->D3DDevice->SetPixelShader(0);
//	DS->D3DDevice->SetTransform(D3DTS_VIEW, &DS->ProjMtx);
//	DS->D3DDevice->SetTransform(D3DTS_PROJECTION, &DS->ViewMtx);
	DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, TRUE);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
}

}
