/***********************************************************/
/*                                                         */
/* Way.h                                                   */
/*                                                         */
/* created 22.03.2003                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef WAY_H
#define WAY_H

#include "MathStructures.h"
#include "IDWay.h"

// www.infinitecode.com 3d library
namespace I3D
{

	class Way
	{
	public:
		struct WaySpecs
		{
			int ID;
			IMATH::Vector2DXZ Pos1, Pos2;
			char Name[1024];
		} Specs;

		Way(void) {}

		Way(WaySpecs *Specs)
		{
			Way::Specs = *Specs;
		}

		const IMATH::Vector2DXZ &GetPos(int n) const
		{
			if(!n)
				return Specs.Pos1;
			else
				return Specs.Pos2;
		}

		void SetPos(int n, const IMATH::Vector2DXZ &Pos)
		{
			if(!n)
				Specs.Pos1 = Pos;
			else
				Specs.Pos2 = Pos;
		}

		const char *GetName()
		{
			return Specs.Name;
		}

		int GetID()
		{
			return Specs.ID;
		}

		float GetSize()
		{
			return GetSizeOfID(Specs.ID);
		}

		static float GetSizeOfID(int ID)
		{
			switch(ID)
			{
			case IDT_WAY_WAY:
				return 5;
			case IDT_WAY_STREET:
				return 7;
			case IDT_WAY_HIGHWAY:
				return IDT_MAX_WAYSIZE;
			default:
				return 5;
			}
		}
	};

}

#endif
