/***********************************************************/
/*                                                         */
/* World.cpp                                               */
/*                                                         */
/* created 16.07.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "World.h"

#include "IMath.h"
#include "DataStructures.h"
#include "QuadTree.h"
#include "D3DStructures.h"
#include "Tank.h"
#include "IDType.h"
#include "IDTank.h"
#include "IDParticleFX.h"
#include "ParticleEffect.h"
#include "WaterLevel.h"

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

// www.infinitecode.com game library
namespace IGAME
{

// spawn objects
void World::CISpawn::Spawn(SpawnInterfaceSpecs *SS)
{
	int State = 0;
	try
	{
		State = 1;
		// tank
		if(SS->Type == IDTYPE_TANK)
		{
			Tank::TankSpecs Specs;
			Specs.ID = SS->ID;
			Specs.TLib = &Wrld->TLib;
			Specs.PLib = &Wrld->ProjLib;
			strcpy(Specs.Name, SS->Name);
			Specs.DS = Wrld->Specs.DS;
			Specs.Pos = SS->Pos;
			Specs.QT = &Wrld->QTree;
			Specs.SM = &Wrld->QTree.ISpaceMgr;
			Specs.PosMgr = &Wrld->LUT;
			Specs.ISpawn = this;
			Specs.Owner = SS->Owner;
			Specs.Angle = SS->Angle;
			Specs.SndMgr = Wrld->GetSoundManager();
			Specs.Commandable = SS->Commandable;
			Specs.Highlight = SS->Highlight;
			Specs.MaxSpeed = SS->MaxSpeed;
			Wrld->Tanks[Wrld->Tanks.Count()] = new Tank();
			Wrld->Tanks.Last()->Create(&Specs);
			if(SS->Target)
				Wrld->Tanks.Last()->Attack(SS->Target);
			if(SS->AI == Tank::TNK_AI_MOVE)
			{
				Wrld->Tanks.Last()->SetTargetPos(IMATH::Vector2DXZ(SS->TargetX, SS->TargetZ));
				Wrld->Tanks.Last()->SetHomePos(IMATH::Vector(SS->TargetX, Wrld->GetSurfaceY(SS->TargetX, SS->TargetZ), SS->TargetZ));
			}
			if(SS->AI == Tank::TNK_AI_LAND)
				Wrld->Tanks.Last()->Land(IMATH::Vector2DXZ(SS->TargetX, SS->TargetZ));
			// add to minimap
			Wrld->MMap.AddObject(Wrld->Tanks.Last());
		}
		// projectile
		if(SS->Type == IDTYPE_PROJECTILE)
		{
			State = 10;
			Projectile::ProjectileSpecs Specs;
			Specs.DS = Wrld->Specs.DS;
			Specs.Pos = SS->Pos;
			Specs.QT = &Wrld->QTree;
			Specs.SM = &Wrld->QTree.ISpaceMgr;
			Specs.PosMgr = &Wrld->LUT;
			Specs.ISpawn = this;
			Specs.Shooter = SS->Spawner;
			Specs.dPos = SS->dPos;
			Specs.ID = SS->ID;
			Specs.Owner = SS->Owner;
			Specs.PLib = &Wrld->ProjLib;
			Specs.SndMgr = Wrld->GetSoundManager();
			Wrld->Projectiles.Add(new Projectile());
			State = 11;
			Wrld->Projectiles.Last()->Create(&Specs);
			State = 12;
			if(SS->Target)
				Wrld->Projectiles.Last()->Attack(SS->Target);
			State = 13;
		}
		// particle
		if(SS->Type == IDTYPE_PARTICLEFX)
		{
			I3D::ParticleEffect::ParticleEffectSpecs Specs;
			Specs.Pos = SS->Pos;
			Specs.SM = &Wrld->QTree.ISpaceMgr;
			Specs.Time = Wrld->Specs.DS->Sp->GameTime;
			Specs.Delay = SS->Delay;
			Specs.ID = SS->ID;
			Specs.PFXRenderer = &Wrld->PFXRenderer;
			Specs.PFXLib = &Wrld->PFXLib;
			Specs.IWorldPFX = &Wrld->IWorldPFX;
			Specs.DS = Wrld->Specs.DS;
			Wrld->PFX[Wrld->PFX.Count()] = new I3D::ParticleEffect();
			Wrld->PFX[Wrld->PFX.Count() - 1]->Create(&Specs);
		}
		// update lines
		if(Wrld->GetDrawMELines() && SS->Type == IDTYPE_TANK)
			Wrld->CreateLines();
	}
	catch(char *s)
	{
		throw(s);
	}
	catch(...)
	{
		char Str[1024];
		sprintf(Str, "exception in World::CISpawn::ISpawn#%d", State);
		throw(Str);
	}
}

/*void World::CISpawn::Display(SpawnInterfaceSpecs *SS)
{

}*/

void World::CIWorldPFX::Delete(I3D::ParticleEffect *PFX)
{
	Wrld->PFXDeleteList[Wrld->PFXDeleteList.Count()] = PFX;
}

void World::CISpawn::GetNearObjects(const IMATH::Vector &Pos, const float Area, I3D::ManagedObject::MgdObjArray *Objects) const
{
	Wrld->LUT.GetNearObjects(&Pos, Area, Objects);
}

void World::CISpawn::GetNearObjects(const IMATH::Vector &Pos, const float Area, I3D::ManagedObject::MgdObjArray *Objects, const void *Type) const
{
	Wrld->LUT.GetNearObjects(&Pos, Area, Objects, Type);
}

/*void World::CISpawn::GetNearTanks(const IMATH::Vector &Pos, const float Area, ISL::DynamicArray<Tank *> *Tanks)
{
	for(int i = 0; i < Wrld->Tanks.Count(); i++)
	{
		if((Wrld->Tanks[i]->GetPos() - Pos).Length() < Area)
			Tanks->Add(Wrld->Tanks[i]);
	}
}*/

void World::CISpawn::DeleteTank(Tank *Tnk)
{
	Wrld->TankDeleteList.Add(Tnk);
}

void World::CISpawn::DeleteProjectile(Projectile *Prj)
{
	Wrld->ProjectileDeleteList.Add(Prj);
}

void World::CreateLines()
{
	Lines.ClearPtr();
	if(DrawMELines)
	{
		ISL::DynamicArray<I3D::TerrainDsc> *Dscs = Surface.GetTerrainDscs();
		int i;
		for(i = 0; i < Dscs->Count(); ++i)
		{
			for(int j = 0; j < 3; ++j)
			{
				I3D::TerrainLine *Line = new I3D::TerrainLine();
				I3D::TerrainLine::TerrainLineSpecs Specs;
				Specs.DS = World::Specs.DS;
				Specs.Color = 0xFFFFFFFF;
				Specs.TI = &Surface;
				Specs.TLR = &TLR;
				Specs.Pos[0] = (*Dscs)[i].GetPos(j);
				Specs.Pos[1] = (*Dscs)[i].GetPos((j + 1) % 3);
				Line->Create(&Specs);
				Lines.Add(Line);
			}
		}
		ISL::DynamicArray<I3D::Way> *Ways = Surface.GetWays();
		for(i = 0; i < Ways->Count(); ++i)
		{
			for(int j = 0; j < 2; ++j)
			{
				I3D::TerrainLine *Line = new I3D::TerrainLine();
				I3D::TerrainLine::TerrainLineSpecs Specs;
				Specs.DS = World::Specs.DS;
				Specs.Color = 0xFFFFFFFF;
				Specs.TI = &Surface;
				Specs.TLR = &TLR;
				Specs.Pos[0] = (*Ways)[i].Specs.Pos1;
				Specs.Pos[1] = (*Ways)[i].Specs.Pos2;
				Line->Create(&Specs);
				Lines.Add(Line);
			}
		}
	}
	

	return;
	// draw ai lines for selected tanks
	for(int i = 0; i < Selection.Count(); i++)
	{
		bool move = Selection[i]->GetAI() == Tank::TNK_AI_MOVE_PATH || Selection[i]->GetAI() == Tank::TNK_AI_MOVE;
		bool attack = Selection[i]->GetAI() == Tank::TNK_AI_ATTACK;
		if(move || attack)
		{
			I3D::TerrainLine *Line = new I3D::TerrainLine();
			I3D::TerrainLine::TerrainLineSpecs Specs;
			Specs.DS = World::Specs.DS;
			Specs.Color = 0xFFFFFF00;
			Specs.TI = &Surface;
			Specs.TLR = &TLR;
			Specs.Pos[0] = IMATH::Vector2DXZ(Selection[i]->GetPos().x, Selection[i]->GetPos().z);
			Specs.Pos[1] = Selection[i]->GetDirectAITarget();
			Specs.Width = 0.2f;
			Line->Create(&Specs);
			Lines.Add(Line);

			Line = new I3D::TerrainLine();
			if(move)
				Specs.Color = 0xFF00FF00;
			else
				Specs.Color = 0xFFFF0000;
			Specs.Pos[1] = Selection[i]->GetAITarget();
			Line->Create(&Specs);
			Lines.Add(Line);
		}
	}

}

void World::AddAirfieldPre(Airfield *AF)
{
	Specs.Airfields.Add(AF);
	Surface.AddPlain(AF->GetPos() + IMATH::Vector2DXZ(-64, 0), 10);
	Surface.AddPlain(AF->GetPos() + IMATH::Vector2DXZ(0, 0), 10);
	Surface.AddPlain(AF->GetPos() + IMATH::Vector2DXZ(64, 0), 10);
	// kill trees
	KillTrees(AF->GetPos() + IMATH::Vector2DXZ(-128, -43), AF->GetPos() + IMATH::Vector2DXZ(128, 43));
}

void World::AddAirfield(Airfield *AF)
{
	int State = 0;
	try
	{
		State = 1;
		AddAirfieldPre(AF);
		State = 2;
		CompileDecals();
		State = 3;
	}
	catch(char *s)
	{
		throw s;
	}
	catch(...)
	{
		char Str[1024];
		sprintf(Str, "World::AddAirfield#%d (%f, %f)", State, AF->GetPos().x, AF->GetPos().z);
		throw Str;
	}
}

void World::AddDecalPre(Decal *D)
{
	Surface.AddPlain(D->GetPos(), (D->GetSizeX() + D->GetSizeZ()) / 4);
	Specs.Decals.Add(D);
	// kill trees
	KillTrees(D->GetPos() + IMATH::Vector2DXZ(-D->GetSizeX() / 2, -D->GetSizeZ() / 2), D->GetPos() + IMATH::Vector2DXZ(D->GetSizeX() / 2, D->GetSizeZ() / 2));
}

void World::CompileDecalsAdditive()
{
/*	QTree.UpdateSurface();
	for(int i = 0; i < Specs.Decals.Count(); ++i)
		QTree.AddTexture(IMATH::Vector2DXZ(Specs.Decals[i]->GetPos().x - 128, Specs.Decals[i]->GetPos().z - 128), Specs.Decals[i]->GetTexture(World::Specs.DS));*/
	//QTree.AddTexture(IMATH::Vector2DXZ(D->GetPos().x - D->GetCenterX(), D->GetPos().z - D->GetCenterZ()), IMATH::Vector2DXZ(D->GetCenterX() - D->GetSizeX()/2, D->GetCenterZ() - D->GetSizeZ()/2), IMATH::Vector2DXZ(D->GetSizeX(), D->GetSizeZ()), D->GetTexture(World::Specs.DS));
	for(int i = 0; i < Specs.Decals.Count(); ++i)
		QTree.AddTexture(IMATH::Vector2DXZ(Specs.Decals[i]->GetPos().x - Specs.Decals[i]->GetCenterX(), Specs.Decals[i]->GetPos().z - Specs.Decals[i]->GetCenterZ())
			, IMATH::Vector2DXZ(Specs.Decals[i]->GetCenterX() - Specs.Decals[i]->GetSizeX()/2, Specs.Decals[i]->GetCenterZ() - Specs.Decals[i]->GetSizeZ()/2)
			, IMATH::Vector2DXZ(Specs.Decals[i]->GetSizeX(), Specs.Decals[i]->GetSizeZ()), Specs.Decals[i]->GetTexture(World::Specs.DS), false);
	for(int i = 0; i < Specs.Airfields.Count(); ++i)
		QTree.AddTexture(IMATH::Vector2DXZ(Specs.Airfields[i]->GetPos().x - 128, Specs.Airfields[i]->GetPos().z - 43)
			, IMATH::Vector2DXZ(0, 0)
			, IMATH::Vector2DXZ(256, 86), Airfield::GetTexture(World::Specs.DS), false);
	QTree.ReconstructMipMaps();
}

void World::CompileDecals()
{
	QTree.UpdateSurface();
	CompileDecalsAdditive();
}

void World::UpdateSurfaceTextures()
{
	QTree.CreateTextures(Specs.DS);
}

void World::AddDecal(Decal *D)
{
	AddDecalPre(D);
	CompileDecals();
}

void World::KillTrees(const IMATH::Vector2DXZ &From, const IMATH::Vector2DXZ &To)
{
	for(int i = 0; i < Trees.Count(); ++i)
	{
		if(Trees[i]->GetPos().x > From.x && Trees[i]->GetPos().x < To.x
			&& Trees[i]->GetPos().z > From.z && Trees[i]->GetPos().z < To.z)
		{
			Tree *T = Trees[i];
			Trees.Delete(i);
			delete T;
			--i;
		}
	}
}

void World::CreateQuadTree(World::WorldSpecs *WS)
{
	I3D::QuadTreeSpecs QTSpecs;
	QTSpecs.TexDepth = 3;
	QTSpecs.Depth = 7;
	QTSpecs.Size = Size;
	QTSpecs.Surface = &Surface;
	QTSpecs.DS = Specs.DS;
	SendEvent("World: creating quadtree");
	QTree.Create(&QTSpecs);

	SendEvent("World: adding tanks to quadtree");
	for(int i = 0; i < Tanks.Count(); ++i)
		Tanks[i]->RelinkQTree();

	SendEvent("World: adding decals to quadtree");
	CompileDecalsAdditive();
/*	// airfield textures (after QuadTree creation!)
	for(int i = 0; i < WS->Airfields.Count(); ++i)
		QTree.AddTexture(IMATH::Vector2DXZ(WS->Airfields[i]->GetPos().x - 128, WS->Airfields[i]->GetPos().z - 43)
			, IMATH::Vector2DXZ(0, 0)
			, IMATH::Vector2DXZ(256, 86), WS->Airfields[i]->GetTexture(Specs.DS));

	// decals (after QuadTree creation!)
	for(int i = 0; i < WS->Decals.Count(); ++i)
		QTree.AddTexture(IMATH::Vector2DXZ(WS->Decals[i]->GetPos().x - WS->Decals[i]->GetCenterX(), WS->Decals[i]->GetPos().z - WS->Decals[i]->GetCenterZ())
			, IMATH::Vector2DXZ(WS->Decals[i]->GetCenterX() - WS->Decals[i]->GetSizeX()/2, WS->Decals[i]->GetCenterZ() - WS->Decals[i]->GetSizeZ()/2)
			, IMATH::Vector2DXZ(WS->Decals[i]->GetSizeX(), WS->Decals[i]->GetSizeZ()), WS->Decals[i]->GetTexture(Specs.DS));*/
}

void World::CreateTrees(World::WorldSpecs *WS)
{
	srand(WS->Seed);
	for(int i = 0; i < GAME_TREES; i++)
	{
		float x, z;
		int timeout = 100;
		do
		{
			x = (rand() % 16384 - 8192) / 16384.0f * Size;
			z = (rand() % 16384 - 8192) / 16384.0f * Size;
		} while(!(Surface.TreePropability(IMATH::Vector2DXZ(x, z)) > ((float)(rand()%256) / 256.0f)) && --timeout);
		if(timeout)
		{
			IGAME::Tree::TreeSpecs TSpecs;
			TSpecs.Pos = IMATH::Vector(x, Surface.GetY(x, z), z);
			TSpecs.DS = Specs.DS;
			TSpecs.QT = &QTree;
			TSpecs.SM = &QTree.ISpaceMgr;
			TSpecs.PosMgr = &LUT;
			Trees.Add(new Tree());
			Trees.Last()->Create(&TSpecs);
		}
	}

	// kill trees at airfields
	for(int i = 0; i < WS->Airfields.Count(); ++i)
		KillTrees(WS->Airfields[i]->GetPos() + IMATH::Vector2DXZ(-128, -32), WS->Airfields[i]->GetPos() + IMATH::Vector2DXZ(128, 32));

	// kill trees at ways
	for(int i = 0; i < WS->Ways.Count(); i++)
	{
		IMATH::Vector2DXZ Delta = WS->Ways[i]->Specs.Pos2 - WS->Ways[i]->Specs.Pos1;
		IMATH::Vector2DXZ NDelta = Delta.Normalize();
		for(float j = 0; j <= Delta.Length(); j += 5)
			KillTrees(WS->Ways[i]->Specs.Pos1 + IMATH::Vector2DXZ(j * NDelta.x, j * NDelta.z) + IMATH::Vector2DXZ(-8, -8), WS->Ways[i]->Specs.Pos1 + IMATH::Vector2DXZ(j * NDelta.x, j * NDelta.z) + IMATH::Vector2DXZ(8, 8));
	}
}

void World::CreateMinimap()
{
		Minimap::Specs Sp;
		Sp.DS = Specs.DS;
		Sp.Surface = &Surface;
		MMap.Create(Sp);
		for(int i = 0; i < Tanks.Count(); i++)
			MMap.AddObject(Tanks[i]);
}

void World::Init3D(I3D::D3DStruct *DS, bool recurse)
{
	int State = 0;
	try
	{
		if(Initialized)
		{
			if(recurse)
			{
				SendEvent("World: init 3d: tanks");
				for(int i = 0; i < Tanks.Count(); ++i)
				{
					Tanks[i]->Init3D(DS, true);
				}
				SendEvent("World: init 3d: projectiles");
				for(int i = 0; i < Projectiles.Count(); ++i)
				{
					Projectiles[i]->Init3D(DS, true);
				}
				SendEvent("World: init 3d: particle effects");
				for(int i = 0; i < PFX.Count(); ++i)
				{
					PFX[i]->Init3D(DS, true);
				}
				SendEvent("World: init 3d: trees");
				for(int i = 0; i < Trees.Count(); ++i)
				{
					Trees[i]->Init3D(DS, true);
				}
				// quadtree after tanks!
				State = 1;
				SendEvent("World: init 3d: quadtree");
				QTree.Init3D(DS, true);
				// airfield textures (after QuadTree!)
				SendEvent("World: init 3d: airfields");
				for(int i = 0; i < Specs.Airfields.Count(); ++i)
					QTree.AddTexture(IMATH::Vector2DXZ(Specs.Airfields[i]->GetPos().x - 128, Specs.Airfields[i]->GetPos().z - 43)
						, IMATH::Vector2DXZ(0, 0)
						, IMATH::Vector2DXZ(256, 86), Specs.Airfields[i]->GetTexture(Specs.DS));
				// decals (after QuadTree!)
				SendEvent("World: init 3d: decals");
				for(int i = 0; i < Specs.Decals.Count(); ++i)
					QTree.AddTexture(IMATH::Vector2DXZ(Specs.Decals[i]->GetPos().x - Specs.Decals[i]->GetCenterX(), Specs.Decals[i]->GetPos().z - Specs.Decals[i]->GetCenterZ()/2)
						, IMATH::Vector2DXZ(Specs.Decals[i]->GetCenterX() - Specs.Decals[i]->GetSizeX()/2, Specs.Decals[i]->GetCenterZ() - Specs.Decals[i]->GetSizeZ()/2)
						, IMATH::Vector2DXZ(Specs.Decals[i]->GetSizeX(), Specs.Decals[i]->GetSizeZ()), Specs.Decals[i]->GetTexture(Specs.DS));
				SendEvent("World: init 3d: pfx renderer");
				PFXRenderer.Init3D(DS, true);
				State = 2;
				SendEvent("World: init 3d: minimap");
				MMap.Init3D(DS, true);
				State = 5;
				SendEvent("World: init 3d: terrain line renderer");
				TLR.Init3D(DS, true);
			}
			State = 3;
			SendEvent("World: init 3d: mouse");
			DragRS.CreateVBuf(Specs.DS->D3DDevice);
			DragRS.CreateIBuf(Specs.DS->D3DDevice);
			State = 4;
			DragTexture = I3D::TextureManager::GetTexture(Specs.DS, "Textures\\Drag.bmp", 0, I3D::TM_FLAG_ALPHACC0);
		}
	}
	catch(char *s)
	{
		throw(s);
	}
	catch(...)
	{
		char Str[1024];
		sprintf(Str, "World::Init3D#%d", State);
		throw(Str);
	}
}

void World::Create(World::WorldSpecs *WS)
{
	int State = 0;
	try
	{
		SendEvent("World: initializing");
		Initialized = true;
		State = 1;
		Specs = *WS;
		ISpawn.Wrld = this;
		IWorldPFX.Wrld = this;
		CameraPos = I3D::Vector(0, 400.0f, -400.0f);
		CameraAngle = I3D::Vector(0.20f * PI, 0, 0);
		DrawAccMap = false;
		Dragging.Type = DRAGGING_NOTHING;
		InitLight(Specs.DS);
		int i;

		SendEvent("World: collecting garbage");
		State = 2;
		for(i = 0; i < Trees.Count(); i++)
			delete Trees[i];
		Trees.Clear();
		for(i = 0; i < Projectiles.Count(); i++)
			delete Projectiles[i];
		Projectiles.Clear();
		for(i = 0; i < Tanks.Count(); i++)
			delete Tanks[i];
		Tanks.Clear();
		Selection.Clear();

		Size = 2 * 1024.0f; // 2 kilometers ;-)

		// profiler
		State = 21;
		Prf.Create(Specs.DS);
		State = 22;
		Prf.SetName(0, "quadtree");
		Prf.SetName(1, "terrainlines");
		Prf.SetName(2, "particle effects");
		Prf.SetName(3, "minimap");

		/*State = 23;
		PhysicsPrf.Create(Specs.DS);
		State = 24;
		PhysicsPrf.SetName(0, "tank");
		PhysicsPrf.SetName(1, "lines");
		PhysicsPrf.SetName(2, "projectiles");
		PhysicsPrf.SetName(3, "collisions");
		PhysicsPrf.SetName(4, "trees");
		PhysicsPrf.SetName(5, "pfx");
		PhysicsPrf.SetName(6, "delete");*/

		State = 3;
		// surface
		SendEvent("World: creating surface");
		Surface.Create(256, 256, 64, 64, Size, WS->Ways.Count() ? &WS->Ways[0] : 0, WS->Ways.Count(), WS->TerrainDscs.Count() ? &WS->TerrainDscs[0] : 0, WS->TerrainDscs.Count(), Specs.Seed);
	/*	for(i = 0; i < WS->Ways.Count(); i++)
			Surface.AddWay(WS->Ways[i]);
		Surface.AddTerrainDscs(&WS->TerrainDscs[0], WS->TerrainDscs.Count());*/
		// libraries
		SendEvent("World: loading libraries");
		State = 4;
		PFXLib.Create();
		State = 5;
		TLib.Create();
		State = 6;
		DLib.Create();
		State = 7;
		ProjLib.Create();
		State = 8;
		// PFX Renderer
		I3D::ParticleFXRenderer::ParticleFXRendererSpecs PFXSpecs;
		PFXSpecs.DS = Specs.DS;
		PFXSpecs.PFXLib = &PFXLib;
		PFXSpecs.ForceFeedback = this;
		PFXRenderer.Create(&PFXSpecs);

		SendEvent("World: creating airfields and plains");
		State = 9;
		// airfield plains
		for(i = 0; i < WS->Airfields.Count(); ++i)
		{
			Surface.AddPlain(WS->Airfields[i]->GetPos() + IMATH::Vector2DXZ(-64, 0), 10);
			Surface.AddPlain(WS->Airfields[i]->GetPos() + IMATH::Vector2DXZ(0, 0), 10);
			Surface.AddPlain(WS->Airfields[i]->GetPos() + IMATH::Vector2DXZ(64, 0), 10);
		}
		State = 10;
		// plains
		for(i = 0; i < WS->Plains.Count(); ++i)
			Surface.AddPlain(WS->Plains[i].Pos, WS->Plains[i].Size);

		State = 11;
		// QuadTree
		SendEvent("World: creating quadtree");
		CreateQuadTree(WS);

		State = 12;
		// LUT2D
		I3D::LUT2D::LUT2DSpecs LSpecs;
		LSpecs.ResX = 64;
		LSpecs.ResZ = 64;
		LSpecs.SizeX = Size;
		LSpecs.SizeZ = Size;
		SendEvent("World: creating lookup table");
		LUT.Create(&LSpecs);

		State = 13;
		// TerrainLineRenderer
		SendEvent("World: tool renderers");
		TLR.Create(Specs.DS);

		State = 14;
		// Accessibility Map Renderer
		I3D::AccMapRenderer::AMRSpecs AMRSpecs;
		AMRSpecs.TI = &Surface;
		AMRSpecs.TLR = &TLRAcc;
		AMRSpecs.DS = Specs.DS;
		AccRnd.Create(&AMRSpecs);

		State = 15;
		// Lines
		DrawMELines = false;

		// Line
	/*	I3D::TerrainLine::TerrainLineSpecs TLSpecs;
		TLSpecs.TLR = &TLR;
		TLSpecs.TI = &Surface;
		TLSpecs.Pos[0] = IMATH::Vector2DXZ(-Size / 2, -Size / 2);
		TLSpecs.Pos[1] = IMATH::Vector2DXZ( Size / 2,  Size / 2);
		I3D::TerrainLine *Line = new I3D::TerrainLine();
		Line->Create(&TLSpecs);
		TerrainLines.Add(Line);*/

		State = 16;
		// trees
		SendEvent("World: creating trees");
		CreateTrees(WS);

		State = 17;
		// minimap
		SendEvent("World: creating minimap");
		CreateMinimap();

		State = 18;

		State = 19;
		// drag box
		using namespace I3D;
		DragRS.ClearVertex();
		DragRS.AddVertex(Vertex(Vector(0, 0, 0), Vector(0, 0, -1), Vector2D(0, 0)));
		DragRS.AddVertex(Vertex(Vector(1, 0, 0), Vector(0, 0, -1), Vector2D(1, 0)));
		DragRS.AddVertex(Vertex(Vector(1, 1, 0), Vector(0, 0, -1), Vector2D(1, 1)));
		DragRS.AddVertex(Vertex(Vector(0, 1, 0), Vector(0, 0, -1), Vector2D(0, 1)));
		DragRS.ClearIndex();
		DragRS.AddIndex(0);
		DragRS.AddIndex(1);
		DragRS.AddIndex(3);
		DragRS.AddIndex(1);
		DragRS.AddIndex(2);
		DragRS.AddIndex(3);
		SendEvent("World: initializing 3d stuff");
		Init3D(Specs.DS, false);
		SendEvent("World: done");
		MMapDrag = false;
		State = 20;
	}
	catch(char *s)
	{
		throw(s);
	}
	catch(...)
	{
		char Str[1024];
		sprintf(Str, "World::Create#%d", State);
		throw(Str);
	}
}

void World::Destroy(void)
{
	Lines.ClearPtr();
	int i;
	for(i = 0; i < Trees.Count(); i++)
		delete Trees[i];
	Trees.Clear();
	for(i = 0; i < Tanks.Count(); i++)
		delete Tanks[i];
	Tanks.Clear();
	for(i = 0; i < Projectiles.Count(); i++)
		delete Projectiles[i];
	Projectiles.Clear();
	for(i = 0; i < TerrainLines.Count(); i++)
		delete TerrainLines[i];
	Projectiles.Clear();
	Initialized = false;
}

// called when the camera moves
void World::OnViewChange(void)
{
	RenderFlags |= RENDERABLE_FLAG_VIEWCHANGE;
}

int World::GetTankNumber(int Owner)
{
	int Num = 0;
	for(int i = 0; i < Tanks.Count(); i++)
		if(Tanks[i]->GetOwner() == Owner)
			Num++;
	return Num;
}

int World::GetTankNumber(int Owner, const IMATH::Vector2DXZ *Area)
{
	int Num = 0;
	for(int i = 0; i < Tanks.Count(); i++)
		if(Tanks[i]->GetOwner() == Owner && Tanks[i]->GetPos().x > Area[0].x && Tanks[i]->GetPos().x < Area[1].x && Tanks[i]->GetPos().z > Area[0].z && Tanks[i]->GetPos().z < Area[1].z)
			Num++;
	return Num;
}

void World::AddWayNoHMap(I3D::Way::WaySpecs *WS)
{
	AddWayNoHMap(I3D::Way(WS));
}

void World::AddWayNoHMap(const I3D::Way &W)
{
	int State = 0;
	try
	{
		State = 2;
		Surface.AddWayNoHMap(&W);
		State = 3;
	}
	catch(char *s)
	{
		throw(s);
	}
	catch(...)
	{
		char Str[1024];
		sprintf(Str, "World::AddWayNoHMap#%d", State);
		throw(Str);
	}
}

void World::AddWay(I3D::Way::WaySpecs *WS)
{
	int State = 0;
	try
	{
		State = 1;
		I3D::Way NewWay(WS);
		State = 2;
		Surface.AddWay(&NewWay);
		State = 3;
		QTree.UpdateSurface();
		QTree.CreateTextures(Specs.DS);
		State = 4;
	}
	catch(char *s)
	{
		throw(s);
	}
	catch(...)
	{
		char Str[1024];
		sprintf(Str, "World::AddWay#%d", State);
		throw(Str);
	}
}

void World::AddTerrainNoHMap(I3D::TerrainDsc::TerrainDscSpecs *TS)
{
	I3D::TerrainDsc NewTerrain(TS);
	Surface.AddTerrainDscNoHMap(&NewTerrain);
	// update accessibility map
	I3D::AccMapRenderer::AMRSpecs AMRSpecs;
	AMRSpecs.TI = &Surface;
	AMRSpecs.TLR = &TLRAcc;
	AMRSpecs.DS = Specs.DS;
	//AccRnd.Create(&AMRSpecs);
	CreateLines();
}

void World::AddTerrain(I3D::TerrainDsc::TerrainDscSpecs *TS)
{
	I3D::TerrainDsc NewTerrain(TS);
	Surface.AddTerrainDsc(&NewTerrain);
	QTree.UpdateSurface();
	QTree.CreateTextures(Specs.DS);
	// update accessibility map
	I3D::AccMapRenderer::AMRSpecs AMRSpecs;
	AMRSpecs.TI = &Surface;
	AMRSpecs.TLR = &TLRAcc;
	AMRSpecs.DS = Specs.DS;
	AccRnd.Create(&AMRSpecs);
	CreateLines();
}

Tank *World::GetTank(const char *Name)
{
	for(int i = 0; i < Tanks.Count(); i++)
		if(!stricmp(Tanks[i]->GetName(), Name))
			return Tanks[i];
	return 0;
}

void World::ManageDeletionLists(void)
{
	// Delete Actors on TankDeleteList
	if(TankDeleteList.Count())
	{
		for(int i = 0; i < TankDeleteList.Count(); i++)
		{
			// ignore double entries
			bool ignore = false;
			if(i)
			{
				for(int j = 0; j < i; j++)
				{
					if(TankDeleteList[j] == TankDeleteList[i])
					{
						ignore = true;
						break;
					}
				}
			}
			if(!ignore)
			{
				// clear references
				int j;
				for(j = 0; j < Tanks.Count(); j++)
					Tanks[j]->ClearTank(TankDeleteList[i]);
				for(j = 0; j < Projectiles.Count(); j++)
					Projectiles[j]->ClearTank(TankDeleteList[i]);
				// delete in Selection
				Selection.Delete(TankDeleteList[i]);
				Tank *Tnk = TankDeleteList[i];
				// delete in minimap
				MMap.DeleteObject(Tnk);
				// delete in Tanks
				Tanks.Delete(Tnk);
				// delete the Tank
				delete Tnk;
			}
		}
		TankDeleteList.Clear();
	}
	// Delete Projectiles on ProjectileDeleteList
	//TODO: simplify?
	if(ProjectileDeleteList.Count())
	{
		for(int i = 0; i < ProjectileDeleteList.Count(); i++)
		{
			// ignore double entries
			bool ignore = false;
			if(i)
			{
				for(int j = 0; j < i; j++)
				{
					if(ProjectileDeleteList[j] == ProjectileDeleteList[i])
					{
						ignore = true;
						break;
					}
				}
			}
			if(!ignore)
			{
				// clear references
				Projectile *Prj = ProjectileDeleteList[i];
				// delete in Actors
				Projectiles.Delete(Prj);
				// delete the Actor
				delete Prj;
			}
		}
		ProjectileDeleteList.Clear();
	}
	// Delete PFX on PFXDeleteList
	if(PFXDeleteList.Count())
	{
		for(int i = 0; i < PFXDeleteList.Count(); i++)
		{
			// ignore double entries
			bool ignore = false;
			if(i)
			{
				for(int j = 0; j < i; j++)
				{
					if(PFXDeleteList[j] == PFXDeleteList[i])
					{
						ignore = true;
						break;
					}
				}
			}
			if(!ignore)
			{
				I3D::ParticleEffect *FX = PFXDeleteList[i];
				PFX.Delete(FX);
				// delete the PFX
				delete FX;
			}
		}
		PFXDeleteList.Clear();
	}
}

void World::PhysicsPFX(I3D::D3DStruct *DS)
{
	for(int i = 0; i < PFX.Count(); i++)
		PFX[i]->Physics(DS);
}

void World::Physics(I3D::D3DStruct *DS)
{
	int Stage = 0;
	try
	{
		static bool inited = false;
		if(!inited)
		{
			PhysicsPrf.Create(Specs.DS);
			PhysicsPrf.SetName(0, "tank");
			PhysicsPrf.SetName(1, "lines");
			PhysicsPrf.SetName(2, "projectiles");
			PhysicsPrf.SetName(3, "collisions");
			PhysicsPrf.SetName(4, "trees");
			PhysicsPrf.SetName(5, "pfx");
			PhysicsPrf.SetName(6, "delete");
			inited = true;
		}

		PhysicsPrf.Start(0);
		int i;
		for(i = 0; i < Tanks.Count(); i++)
			Tanks[i]->Physics(DS);
		PhysicsPrf.Stop(0);
		PhysicsPrf.Start(1);
		// update ai lines
		CreateLines();
		PhysicsPrf.Stop(1);
		PhysicsPrf.Start(2);
		Stage = 2;
		for(i = 0; i < Projectiles.Count(); i++)
			Projectiles[i]->Physics(DS);
		PhysicsPrf.Stop(2);
		PhysicsPrf.Start(3);
		Stage = 3;
		// protect near friendly tanks
		float SafeFriendlyDistanceSq = 32 * 32;
		// tank-projectile collisions
		I3D::ManagedObject::MgdObjArray Objects;
		for(i = 0; i < Tanks.Count(); i++)
		{
			Objects.ClearElements();
			ISpawn.GetNearObjects(Tanks[i]->GetPos(), 1.0f * TLib.Tanks[Tanks[i]->GetID()].Size, &Objects, Projectile::GetClassType());
			if(Objects.Count())
			{
				for(int j = 0; j < Objects.Count(); j++)
				{
					Projectile *Prj = (Projectile *)Objects[j]; 
					Tank *Shooter = Prj->GetShooter();
					float Size = TLib.Tanks[Tanks[i]->GetID()].Size;
					if(Shooter != Tanks[i] && (Prj->GetPos() - Tanks[i]->GetPos()).LengthSq() < Size * Size)
					{
						if(Shooter == 0 || Shooter->GetOwner() != Tanks[i]->GetOwner() || (Shooter->GetPos() - Tanks[i]->GetPos()).LengthSq() > SafeFriendlyDistanceSq)
						{
							Prj->CollidedWith(Tanks[i]);
						}
					}
				}
			}
		}
		Stage = 4;
		PhysicsPrf.Stop(3);
		PhysicsPrf.Start(4);
		for(i = 0; i < Trees.Count(); i++)
			Trees[i]->Physics(DS);
		PhysicsPrf.Stop(4);
		PhysicsPrf.Start(5);
		// particle physics
		PhysicsPFX(DS);
		PhysicsPrf.Stop(5);
		PhysicsPrf.Start(6);
		Stage = 5;
		ManageDeletionLists();
		PhysicsPrf.Stop(6);
		//std::ofstream prf("world_physics.txt");
		//prf << Prf;
	}
	catch(char *s)
	{
		throw(s);
	}
	catch(...)
	{
		char Str[1024];
		sprintf(Str, "exception in World::Physics#%d", Stage);
		throw(Str);
	}
}

void World::CreateView(I3D::D3DStruct *DS)
{
	QTree.OnViewChange();
	//For.OnViewChange();
}

void World::Render(I3D::D3DStruct *DS)
{
	int Stage = 0;
	try
	{
		Stage = -10;
		if(RenderFlags & RENDERABLE_FLAG_VIEWCHANGE)
		{
			RenderFlags &= ~RENDERABLE_FLAG_VIEWCHANGE;
			// create new View
			CreateView(DS);
		}
		// set camera
	//	Angle += 0.02f;
		Stage = -9;
		D3DMATRIX MatrixZ = {(float)cos(-CameraAngle.z),(float)sin(-CameraAngle.z),0,0,  -(float)sin(-CameraAngle.z),(float)cos(-CameraAngle.z),0,0,  0,0,1,0,  0,0,0,1};
		DS->D3DDevice->MultiplyTransform(D3DTS_VIEW, &MatrixZ);
		D3DMATRIX MatrixX = {1,0,0,0,  0,(float)cos(-CameraAngle.x),(float)sin(-CameraAngle.x),0,  0,-(float)sin(-CameraAngle.x),(float)cos(-CameraAngle.x),0,  0,0,0,1};
		DS->D3DDevice->MultiplyTransform(D3DTS_VIEW, &MatrixX);
		D3DMATRIX MatrixY = {(float)cos(-CameraAngle.y),0,-(float)sin(-CameraAngle.y),0,  0,1,0,0,  (float)sin(-CameraAngle.y),0,(float)cos(-CameraAngle.y),0,  0,0,0,1};
		DS->D3DDevice->MultiplyTransform(D3DTS_VIEW, &MatrixY);
		D3DMATRIX MatrixT = {1,0,0,0,  0,1,0,0,  0,0,1,0,  -CameraPos.x,-CameraPos.y,-CameraPos.z,1};
		DS->D3DDevice->MultiplyTransform(D3DTS_VIEW, &MatrixT);
		// force feedback
		D3DMATRIX MatrixS = {1,0,0,0,  0,1,0,0,  0,0,1,0,  ShakeVector.x,ShakeVector.y,ShakeVector.z,1};
		ShakeVector = IMATH::Vector(0, 0, 0);
		DS->D3DDevice->MultiplyTransform(D3DTS_VIEW, &MatrixS);
		DS->D3DDevice->GetTransform(D3DTS_VIEW, &DS->ViewMtx);
		DS->CreateProjViewMtx();
		Stage = -8;
		// debug: screen clipping
		/*D3DMATRIX TMatrix2 = {1,0,0,0,  0,1,0,0,  0,0,1,0,  0,0,800.0f,1};
		DS->D3DDevice->SetTransform(D3DTS_VIEW, &TMatrix2);
		DS->D3DDevice->MultiplyTransform(D3DTS_VIEW, &DS->ViewMtx);*/
		// render
		Stage = 1;
		Prf.Start(0);
		// quadtree
		QTree.Render(DS);
		Prf.Stop(0);
		Stage = 2;
		Prf.Start(1);
		// TerrainLines
		TLR.Render(DS);
		if(DrawAccMap)
			TLRAcc.Render(DS);
		Prf.Stop(1);
		// particle effects
		Prf.Start(2);
		PFXRenderer.Render(DS);
		Prf.Stop(2);
		// minimap
		Prf.Start(3);
		IMATH::Vector2DXZ Pos = GetSurfaceXZ(IMATH::Vector2D(0, -0.5f));
/*		IMATH::Vector2DXZ Vec = GetSurfaceXZ(IMATH::Vector2D(0, 0.1f));
		if(Pos == IMATH::Vector2DXZ(0, 0) || Vec == IMATH::Vector2DXZ(0, 0))
			MMap.SetViewingVector(IMATH::Vector2DXZ(0, 0));
		else
		{
			Vec = Vec - Pos;
			MMap.SetViewingVector(Vec.Normalize());
		}*/
		MMap.SetViewingVector(IMATH::Vector2DXZ(sin(CameraAngle.y), cos(CameraAngle.y)));
		//MMap.SetViewingPos(IMATH::Vector2DXZ(CameraPos.x, CameraPos.z));
		MMap.SetViewingPos(Pos);
		MMap.SetAngle(CameraAngle.y);
		MMap.Render();
		Prf.Stop(3);
		Stage = 3;
		// overlay: current selection
		for(int i = 0; i < Selection.Count(); i++)
		{
			Selection[i]->RenderSelection(DS);
		}
		Stage = 4;
		// profiler
		Prf.Render(DS);
		PhysicsPrf.Render(DS);
		Stage = 5;
		// mouse
		if(Dragging.Type == DRAGGING_MARK && !MMapDrag)
		{
			Stage = 6;
			// draw drag box
			Specs.DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
			Specs.DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE);
			Specs.DS->D3DDevice->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATER);
			Specs.DS->D3DDevice->SetRenderState(D3DRS_ALPHAREF, 0x10);
			Specs.DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
			Specs.DS->D3DDevice->SetSamplerState(0,D3DSAMP_ADDRESSU,D3DTADDRESS_CLAMP); 
			Specs.DS->D3DDevice->SetSamplerState(0,D3DSAMP_ADDRESSV,D3DTADDRESS_CLAMP); 
			//Specs.DS->D3DDevice->SetTextureStageState(0, D3DTSS_ADDRESSU, D3DTADDRESS_CLAMP);
			//Specs.DS->D3DDevice->SetTextureStageState(0, D3DTSS_ADDRESSV, D3DTADDRESS_CLAMP);
			D3DMATRIX Matrix = {1,0,0,0,  0,1,0,0,  0,0,1,0,  0,0,0,1};
			Specs.DS->D3DDevice->SetTransform(D3DTS_VIEW, &Matrix);
			Specs.DS->D3DDevice->SetTransform(D3DTS_PROJECTION, &Matrix);
			Specs.DS->D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);
			Specs.DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
			Specs.DS->D3DDevice->SetTexture(0, DragTexture);
			D3DMATRIX WMatrix = {DragPos[1].x - DragPos[0].x,0,0,0,  0,DragPos[1].y - DragPos[0].y,0,0,  0,0,1,0,  DragPos[0].x,DragPos[0].y,0,1};
			Specs.DS->D3DDevice->SetTransform(D3DTS_WORLD, &WMatrix);
			Specs.DS->D3DDevice->SetRenderState(D3DRS_CULLMODE,D3DCULL_NONE);
			DragRS.RenderIndex(Specs.DS->D3DDevice);
			Specs.DS->D3DDevice->SetRenderState(D3DRS_CULLMODE,D3DCULL_CW);
			Specs.DS->D3DDevice->SetTransform(D3DTS_VIEW, &Specs.DS->ViewMtx);
			Specs.DS->D3DDevice->SetTransform(D3DTS_PROJECTION, &Specs.DS->ProjMtx);
			Specs.DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, TRUE);
			Specs.DS->D3DDevice->SetSamplerState(0,D3DSAMP_ADDRESSU,D3DTADDRESS_WRAP); 
			Specs.DS->D3DDevice->SetSamplerState(0,D3DSAMP_ADDRESSV,D3DTADDRESS_WRAP); 
//			Specs.DS->D3DDevice->SetTextureStageState(0, D3DTSS_ADDRESSU, D3DTADDRESS_WRAP);
//			Specs.DS->D3DDevice->SetTextureStageState(0, D3DTSS_ADDRESSV, D3DTADDRESS_WRAP);
			Specs.DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, FALSE);
			Specs.DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
			Stage = 7;
		}
		Stage = 8;
	}
	catch(char *s)
	{
		throw(s);
	}
	catch(...)
	{
		char Str[1024];
		sprintf(Str, "Unknown exception in World::Render#%d", Stage);
		throw(Str);
	}
}

void World::DestroyLight(I3D::D3DStruct *DS)
{
	DS->D3DDevice->SetRenderState(D3DRS_AMBIENT, 0);
	DS->D3DDevice->LightEnable(0, false);
	DS->D3DDevice->LightEnable(1, false);
	DS->D3DDevice->LightEnable(2, false);
	DS->D3DDevice->LightEnable(3, false);
}

void World::InitLight(I3D::D3DStruct *DS)
{
	// lighting
	DS->D3DDevice->SetRenderState(D3DRS_AMBIENT, 0x50505050);

	D3DLIGHT9 Light;
	memset(&Light, 0, sizeof(D3DLIGHT9));
	Light.Type = D3DLIGHT_DIRECTIONAL;
	Light.Diffuse.r = 0.7f;
	Light.Diffuse.g = 0.6f;
	Light.Diffuse.b = 0.3f;
	Light.Specular.r=Light.Diffuse.r;
	Light.Specular.g=Light.Diffuse.g;
	Light.Specular.b=Light.Diffuse.b;
	Light.Attenuation0 = 0.0;
	Light.Attenuation1 = 0.0;
	Light.Attenuation2 = 0.0;
	Light.Range = 0;
	Light.Position.x = 0;
	Light.Position.y = 0;
	Light.Position.z = 0;
	Light.Direction.x = 1.0f;
	Light.Direction.y = -0.5f;
	Light.Direction.z = 1.0f;
	DS->D3DDevice->SetLight(1, &Light);
	DS->D3DDevice->LightEnable(1, true);

	memset(&Light, 0, sizeof(D3DLIGHT9));
	Light.Type = D3DLIGHT_DIRECTIONAL;
	Light.Diffuse.r = 0.3f;
	Light.Diffuse.g = 0.6f;
	Light.Diffuse.b = 0.7f;
	Light.Specular.r=Light.Diffuse.r;
	Light.Specular.g=Light.Diffuse.g;
	Light.Specular.b=Light.Diffuse.b;
	Light.Attenuation0 = 0.0;
	Light.Attenuation1 = 0.0;
	Light.Attenuation2 = 0.0;
	Light.Range = 0;
	Light.Position.x = 0;
	Light.Position.y = 0;
	Light.Position.z = 0;
	Light.Direction.x = -1.0f;
	Light.Direction.y = -5.5f;
	Light.Direction.z = -1.0f;
	DS->D3DDevice->SetLight(2, &Light);
	DS->D3DDevice->LightEnable(2, true);

	/*memset(&Light, 0, sizeof(D3DLIGHT9));
	Light.Type = D3DLIGHT_DIRECTIONAL;
	Light.Diffuse.r = 0.25f;
	Light.Diffuse.g = 0.75f;
	Light.Diffuse.b = 0.25f;
	Light.Specular.r=Light.Diffuse.r;
	Light.Specular.g=Light.Diffuse.g;
	Light.Specular.b=Light.Diffuse.b;
	Light.Attenuation0 = 0.0;
	Light.Attenuation1 = 0.0;
	Light.Attenuation2 = 0.0;
	Light.Range = 0;
	Light.Position.x = 0;
	Light.Position.y = 0;
	Light.Position.z = 0;
	Light.Direction.x = 0.0f;
	Light.Direction.y = -1.0f;
	Light.Direction.z = 0.0f;
	DS->D3DDevice->SetLight(3, &Light);
	DS->D3DDevice->LightEnable(3, true);*/
}

void World::MoveCamera(const I3D::Vector &V)
{
	I3D::Matrix Rot;
	Rot.SetRotationYMatrix(CameraAngle.y);
	I3D::Vector Vr = Rot * V;
	CameraPos += Vr;
	CameraConstraints();
	OnViewChange();
}

void World::MoveCameraDirect(const I3D::Vector &V)
{
	I3D::Matrix Rot;
	Rot.SetRotationZMatrix(CameraAngle.z);
	I3D::Vector Vr = Rot * V;
	Rot.SetRotationXMatrix(CameraAngle.x);
	Vr = Rot * Vr;
	Rot.SetRotationYMatrix(CameraAngle.y);
	Vr = Rot * Vr;
	CameraPos += Vr;
	CameraConstraints();
	OnViewChange();
}

void World::RotateCamera(const I3D::Vector &V)
{
	CameraAngle += V;
	CameraConstraints();
	OnViewChange();
}

void World::CameraConstraints()
{
	if(!Specs.EditMode)
	{
		CameraPos.x = min(max(CameraPos.x, -0.9f * 0.5f * Surface.GetSize()), 0.9f * 0.5f * Surface.GetSize());
		CameraPos.z = min(max(CameraPos.z, -0.9f * 0.5f * Surface.GetSize()), 0.9f * 0.5f * Surface.GetSize());
		float SurfaceY = max(Surface.GetY(CameraPos.x, CameraPos.z), WATERLEVEL);
		CameraPos.y = min(max(CameraPos.y, SurfaceY + 20), SurfaceY + 200);
		CameraAngle.x = min(max(CameraAngle.x, 0), 0.5f * PI);
	}
}

void World::Select(I3D::BoundingBox2D *BBox)
{
	Selection.Clear();
	for(int i = 0; i < Tanks.Count(); i++)
	{
		if(Tanks[i]->Select(BBox))
		{
			// add to select list
			Selection[Selection.Count()] = Tanks[i];
		}
	}
	if(Selection.Count() > 1)
		for(int i = 0; i < Selection.Count(); i++)
			if((Selection[i]->GetOwner() != 2 || !Selection[i]->IsVehicle()) && !Specs.EditMode)
			{
				Selection.Delete(i);
				i--;
			}
	CreateLines();
}

bool World::GetStreetAccessible(const IMATH::Vector2DXZ &Pos)
{
	float Radius = 32;
	for(int i = 0; i < Specs.Decals.Count(); i++)
	{
		if(absf(Specs.Decals[i]->GetPos().x - Pos.x) <= Radius && absf(Specs.Decals[i]->GetPos().z - Pos.z) <= Radius)
			if(Specs.Decals[i]->GetID() == 0 || Specs.Decals[i]->GetID() == 1 || Specs.Decals[i]->GetID() == 2)
				return true;
	}
	ISL::DynamicArray<I3D::Way> *Ways = Surface.GetWays();
	for(int i = 0; i < Ways->Count(); i++)
	{
		if((*Ways)[i].GetID() == I3D::IDT_WAY_STREET || (*Ways)[i].GetID() == I3D::IDT_WAY_WAY)
		{
			I3D::Terrain::WayDistanceInfo Info = Surface.GetWayDistance((*Ways)[i], Pos);
			if(Info.SideDistance > -Radius && Info.SideDistance < Radius && Info.LengthCoefficient > 0 && Info.LengthCoefficient < 1)
				return true;
		}
	}
	return false;
}

Decal *World::GetDecal(const IMATH::Vector2DXZ &Pos)
{
	for(int i = 0; i < Specs.Decals.Count(); i++)
		if(absf(Specs.Decals[i]->GetPos().x - Pos.x) <= Specs.Decals[i]->GetSizeX() / 2 && absf(Specs.Decals[i]->GetPos().z - Pos.z) <= Specs.Decals[i]->GetSizeZ() / 2)
			return Specs.Decals[i];
	return 0;
}

int World::GetLandscape(const IMATH::Vector2DXZ &Pos)
{
	Tank *Vehicle = 0;
	for(int i = 0; i < Tanks.Count(); i++)
	{
		if(Tanks[i]->Select(I3D::XZPos(Pos)))
		{
			Vehicle = Tanks[i];
		}
	}
	if(Vehicle)
		return VEHICLE;
	for(int i = 0; i < Specs.Airfields.Count(); i++)
	{
		if(absf(Specs.Airfields[i]->GetPos().x - Pos.x) <= Specs.Airfields[i]->GetSizeX() / 2 && absf(Specs.Airfields[i]->GetPos().z - Pos.z) <= Specs.Airfields[i]->GetSizeZ() / 2)
			return AIRFIELD;
	}
	for(int i = 0; i < Specs.Decals.Count(); i++)
	{
		if(absf(Specs.Decals[i]->GetPos().x - Pos.x) <= Specs.Decals[i]->GetSizeX() / 2 && absf(Specs.Decals[i]->GetPos().z - Pos.z) <= Specs.Decals[i]->GetSizeZ() / 2)
			if(Specs.Decals[i]->IsStreet())
				switch(Specs.Decals[i]->GetID())
				{
				case 0: return STREET_HORZ; break;
				case 1: return STREET_VERT; break;
				case 2: return STREET_CROSS; break;
				}
			else
				return DECAL;
	}
	if(Surface.GetTerrainDefined(Pos.x, Pos.z))
		return TERRAIN;
	ISL::DynamicArray<I3D::Way> *Ways = Surface.GetWays();
	for(int i = 0 ; i < Ways->Count(); i++)
	{
		I3D::Terrain::WayDistanceInfo Info = Surface.GetWayDistance((*Ways)[i], Pos);
		if(Info.LengthCoefficient >= 0 && Info.LengthCoefficient <= 1 && Info.SideDistance > -(*Ways)[i].GetSize() && Info.SideDistance < (*Ways)[i].GetSize())
			return WAY;
	}
	return NOTHING;
}

int World::GetMouseTarget(const IMATH::Vector2D &Pos)
{
	IMATH::Vector2DXZ SurfacePos = GetSurfaceXZ(Pos);
	Tank *Vehicle = GetTankAtPos(SurfacePos);
	//Tank *Vehicle = 0;
	/*I3D::XZPos Target = QTree.GetXZPos(Pos);
	for(int i = 0; i < Tanks.Count(); i++)
	{
		if(Tanks[i]->Select(Pos))
		{
			Vehicle = Tanks[i];
		}
	}*/
	//ofstream o("cdebug.txt", ios::app);
	//o << Pos.x << ", " << Pos.y << "; " << Target.Pos.x << ", " << Target.Pos.z << "; " << Surface.GetAccNormed(Surface.WorldToAMapX(Target.Pos.x), Surface.WorldToAMapZ(Target.Pos.z)) << endl;
	if(!Vehicle)
	{
		bool Acc = (Surface.GetAccNormed(SurfacePos.x, SurfacePos.z) < 0.8f);
		return Acc ? MOUSE_TARGET_NOTHING : MOUSE_TARGET_IMPASSABLE;
	}
	switch(Vehicle->GetOwner())
	{
	case 2:
		return MOUSE_TARGET_FRIEND;
	case 0:
	case 1:
	default:
		return MOUSE_TARGET_ENEMY;
	}
	return MOUSE_TARGET_NOTHING;
}

Tank *World::GetTankAtPos(const IMATH::Vector2DXZ &Pos)
{
	/*Tank *Vehicle = 0;
	//I3D::XZPos Target = QTree.GetXZPos(Pos);
	float MinDist;
	for(int i = 0; i < Tanks.Count(); i++)
	{
		float Dist = Tanks[i]->SelectDist(Pos);
		if(i == 0 || Dist < MinDist)
		{
			MinDist = Dist;
			Vehicle = Tanks[i];
		}
	}
	if(Vehicle && MinDist < 2 * TLib.Tanks[Vehicle->GetID()].Size)
		return Vehicle;
	else
		return 0;*/
	//I3D::XZPos Target = QTree.GetXZPos(Pos);
	//I3D::XZPos Target;
	I3D::ManagedObject::MgdObjArray Objects;
	//Objects.Add(Tanks[0]->GetManagedObjectIf());
	ISpawn.GetNearObjects(IMATH::Vector(Pos.x, GetSurfaceY(Pos.x, Pos.z), Pos.z), 10.0f, &Objects, Tank::GetClassType());
	Tank *Vehicle = 0;
	float MinDistSq;
	if(Objects.Count())
	{
		for(int j = 0; j < Objects.Count(); j++)
		{
			//float Dist = ((Tank *)Objects[j]->GetBase())->SelectDist(Pos);
			IMATH::Vector2DXZ TankPos(((Tank *)Objects[j]->GetBase())->GetPos().x, ((Tank *)Objects[j]->GetBase())->GetPos().z);
			float DistSq = (TankPos - Pos).LengthSq();
			if(j == 0 || DistSq < MinDistSq)
			{
				MinDistSq = DistSq;
				Vehicle = (Tank *)Objects[j]->GetBase();
			}
		}
	}
	//return Tanks[0];
	if(Vehicle && MinDistSq < 2 * TLib.Tanks[Vehicle->GetID()].Size * 2 * TLib.Tanks[Vehicle->GetID()].Size)
		return Vehicle;
	else
		return 0;
}

void World::RightClick(const IMATH::Vector2D &Pos)
{
	Selection.Clear();
	CreateLines();
}

void World::Drag(const IMATH::Vector2D &Pos, const IMATH::Vector2D &Pos2)
{
	// minimap
	if(MMapDrag && MMap.ClickTest(Pos))
	{
		float y = GetCameraPos().y;
		SetCameraPos(MMap.Click(Pos));
		MoveCameraDirect(IMATH::Vector(0, 0, -1.5f * y));
		CameraPos.y = y;//max(Surface.GetY(CameraPos.x, CameraPos.z) + 40, CameraPos.y);
		OnViewChange();
	}
	if(Dragging.Type == DRAGGING_VEHICLE)
	{
		I3D::XZPos Target = QTree.GetXZPos(Pos);
		Dragging.Vehicle->SetPos(IMATH::Vector(Target.Pos.x, max(Surface.GetY(Target.Pos.x, Target.Pos.z), WATERLEVEL), Target.Pos.z));
		Dragging.Vehicle->UpdateMatrices();
		Specs.MOPropIf->SetPos(Dragging.Vehicle->GetName(), Dragging.Vehicle->GetPos());
		if(Dragging.Vehicle->GetAI() != Tank::TNK_AI_IDLE)
			CreateLines();
		/*std::ofstream log("drag.txt", std::ios::ios_base::app);
		log << Dragging.Vehicle->GetName() << std::endl;
		log << Specs.MOPropIf << std::endl;
		log << "x: " << Dragging.Vehicle->GetPos().x << ", y: " << Dragging.Vehicle->GetPos().y << ", z: " << Dragging.Vehicle->GetPos().z << std::endl;*/
	}
	else if(Dragging.Type == DRAGGING_WAY)
	{
		I3D::XZPos Target = QTree.GetXZPos(Pos);
		if(Dragging.ID == 1)
		{
			Dragging.Way->Specs.Pos1 = Target.Pos;
		}
		else
			Dragging.Way->Specs.Pos2 = Target.Pos;
		Specs.MOPropIf->SetWayPos(Dragging.Way->Specs.Name, Dragging.ID, Target.Pos);
		CreateLines();
	}
	else if(Dragging.Type == DRAGGING_TERRAIN)
	{
		I3D::XZPos Target = QTree.GetXZPos(Pos);
		Dragging.Terrain->SetPos(Dragging.ID, Target.Pos);
		Specs.MOPropIf->SetTerrainPos(Dragging.Terrain->GetName(), Dragging.ID, Target.Pos);
		CreateLines();
	}
	if(!MMapDrag)
	{
		DragPos[0] = Pos;
		DragPos[1] = Pos2;
	}
}

void World::DragStart(const IMATH::Vector2D &Pos)
{
	// vehicles
	Dragging.Type = DRAGGING_NOTHING;
	//I3D::XZPos Target = QTree.GetXZPos(Pos);
	if(Specs.EditMode)
	{
		IMATH::Vector2DXZ Target = GetSurfaceXZ(Pos);
		Dragging.Vehicle = GetTankAtPos(Target);
		if(Dragging.Vehicle)
			Dragging.Type = DRAGGING_VEHICLE;
		/*for(int i = 0; i < Tanks.Count(); i++)
		{
			if(Tanks[i]->Select(Target))
			{
				Dragging.Type = DRAGGING_VEHICLE;
				Dragging.Vehicle = Tanks[i];
				break;
			}
		}*/
		if(Dragging.Type == DRAGGING_NOTHING && GetDrawMELines())
		{
			ISL::DynamicArray<I3D::Way> *Ways = Surface.GetWays();
			for(int i = 0; i < Ways->Count(); i++)
			{
				if((Target - (*Ways)[i].Specs.Pos1).Length() < 5)
				{
					Dragging.Type = DRAGGING_WAY;
					Dragging.Way = &(*Ways)[i];
					Dragging.ID = 1;
					break;
				}
				else if((Target - (*Ways)[i].Specs.Pos2).Length() < 5)
				{
					Dragging.Type = DRAGGING_WAY;
					Dragging.Way = &(*Ways)[i];
					Dragging.ID = 2;
					break;
				}
			}
		}
		if(Dragging.Type == DRAGGING_NOTHING && GetDrawMELines())
		{
			ISL::DynamicArray<I3D::TerrainDsc> *Dscs = Surface.GetTerrainDscs();
			for(int i = 0; i < Dscs->Count(); i++)
				for(int k = 0; k < 3; k++)
					if((Target - (*Dscs)[i].Specs.Pos[k]).Length() < 5)
					{
						Dragging.Type = DRAGGING_TERRAIN;
						Dragging.Terrain = &(*Dscs)[i];
						Dragging.ID = k;
						break;
					}
		}
	}
	DragPos[0] = Pos;
	DragPos[1] = Pos;
	MMapDrag = MMap.ClickTest(Pos) || Dragging.Type != DRAGGING_NOTHING;
	if(!MMapDrag)
		Dragging.Type = DRAGGING_MARK;
	//Mouse.SetDragDrop(!MMapDrag);
}

void World::DragEnd(const IMATH::Vector2D &Pos, const IMATH::Vector2D &Pos2)
{
	Dragging.Type = DRAGGING_NOTHING;
	if(!MMapDrag)
	{
		I3D::BoundingBox2D BBox;
		BBox.Data[I3D::BB_LEFT] = minf(Pos.x, Pos2.x);
		BBox.Data[I3D::BB_RIGHT] = maxf(Pos.x, Pos2.x);
		BBox.Data[I3D::BB_BOTTOM] = minf(Pos.y, Pos2.y);
		BBox.Data[I3D::BB_TOP] = maxf(Pos.y, Pos2.y);
		Select(&BBox);
	}
	MMapDrag = false;
	//Mouse.SetDragDrop(true);
}

void World::Click(const IMATH::Vector2D &Pos)
{
	// minimap
	if(MMap.ClickTest(Pos))
	{
		float y = GetCameraPos().y;
		SetCameraPos(MMap.Click(Pos));
		MoveCameraDirect(IMATH::Vector(0, 0, -1.5f * y));
		CameraPos.y = y;//max(Surface.GetY(CameraPos.x, CameraPos.z) + 40, CameraPos.y);
		CameraConstraints();
		OnViewChange();
	}
	else
	{
		IMATH::Vector2DXZ Target = GetSurfaceXZ(Pos);
		// vehicles
		Tank *Vehicle = GetTankAtPos(Target);
		//I3D::XZPos Target = QTree.GetXZPos(Pos);
		/*for(int i = 0; i < Tanks.Count(); i++)
		{
			if(Tanks[i]->Select(Target))
			{
				Vehicle = Tanks[i];
			}
		}*/
		//if(Target.Level)
		{
			// select vehicle
			if(Vehicle && (Vehicle->GetOwner() == 2 || Specs.EditMode))
			{
				Selection.Clear();
				Selection[0] = Vehicle;
				CreateLines();
				return;
			}
			// commands
			if(Selection.Count() && (Selection[0]->GetOwner() == 2 || Specs.EditMode))
			{
				// attack vehicle
				if(Vehicle && Vehicle->GetOwner() != 1)
				{
					for(int i = 0; i < Selection.Count(); i++)
						if(Vehicle && Vehicle->GetOwner() != 1)
						{
							if(Selection[i]->GetCommandable())
								Selection[i]->Attack(Vehicle);
							if(Specs.MOPropIf)
							{
								Specs.MOPropIf->AIAttack(Selection[i]->GetName(), Vehicle->GetName());
								CreateLines();
							}
						}
					return;
				}
				// move
				int Num = Selection.Count();
				if(Num)
				{
					// get average position
					IMATH::Vector2DXZ Mid(0, 0);
					int i;
					for(i = 0; i < Num; i++)
						Mid += IMATH::Vector2DXZ(Selection[i]->GetPos().x, Selection[i]->GetPos().z);
					Mid = IMATH::Vector2DXZ(Mid.x / Num, Mid.z / Num);
					// move
					for(i = 0; i < Num; i++)
					{
						// move
						if(Selection[i]->GetCommandable() || Specs.EditMode)
						{
							Selection[i]->SetTargetPos(Target + (IMATH::Vector2DXZ(Selection[i]->GetPos().x, Selection[i]->GetPos().z) - Mid));
							// update MissionLib for missioneditor
							if(Specs.MOPropIf)
							{
								Specs.MOPropIf->AIMove(Selection[i]->GetName(), Target + (IMATH::Vector2DXZ(Selection[i]->GetPos().x, Selection[i]->GetPos().z) - Mid));
								CreateLines();
							}
						}
					}
				}
			}
		}
	}
}

IMATH::Vector2DXZ World::GetXZMousePos(const IMATH::Vector2D &Pos)
{
	return QTree.GetXZPos(Pos).Pos;
}

}
