/***********************************************************/
/*                                                         */
/* LUT2D.cpp                                               */
/*                                                         */
/* created 26.03.2003                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "LUT2D.h"

// www.infinitecode.com 3d library
namespace I3D
{

LUT2D::~LUT2D(void)
{
	if(Table)
		delete[] Table;
}

void LUT2D::Create(LUT2D::LUT2DSpecs *Specs)
{
	LUT2D::Specs = *Specs;
	ResOverSizeX = LUT2D::Specs.ResX / LUT2D::Specs.SizeX;
	ResOverSizeZ = LUT2D::Specs.ResZ / LUT2D::Specs.SizeZ;
	if(Table)
		delete[] Table;
	Table = new Node[LUT2D::Specs.ResX * LUT2D::Specs.ResZ];
}

int LUT2D::GetNodeX(float x) const
{
	return (int)((x + Specs.SizeX * 0.5f) * ResOverSizeX);
}

int LUT2D::GetNodeZ(float z) const
{
	return (int)((z + Specs.SizeZ * 0.5f) * ResOverSizeZ);
}

void LUT2D::GetNearObjects(const IMATH::Vector *Pos, const float Area, I3D::ManagedObject::MgdObjArray *Objects) const
{
	const int Left = GetNodeX(Pos->x - Area);
	const int Right = GetNodeX(Pos->x + Area);
	const int Near = GetNodeZ(Pos->z - Area);
	const int Far = GetNodeZ(Pos->z + Area);
	for(int x = Left; x <= Right; x++)
		if(x >= 0 && x < Specs.ResX)
			for(int z = Near; z <= Far; z++)
				if(z >= 0 && z < Specs.ResZ)
				{
					const int Index = x + Specs.ResX * z;
					const int Num = Table[Index].Objects.Count();
					for(int k = 0; k < Num; k++)
					{
						ManagedObject *Obj = Table[Index].Objects[k];
						const IMATH::Vector ObjPos = Obj->GetPos();
						const float DstX = ObjPos.x - Pos->x;
						const float DstZ = ObjPos.z - Pos->z;
						if(DstX * DstX + DstZ * DstZ < Area * Area)
							Objects->Add(Obj);
					}
				}
}

void LUT2D::GetNearObjects(const IMATH::Vector *Pos, const float Area, I3D::ManagedObject::MgdObjArray *Objects, const void *Type) const
{
	const int Left = GetNodeX(Pos->x - Area);
	const int Right = GetNodeX(Pos->x + Area);
	const int Near = GetNodeZ(Pos->z - Area);
	const int Far = GetNodeZ(Pos->z + Area);
	for(int x = Left; x <= Right; x++)
		if(x >= 0 && x < Specs.ResX)
			for(int z = Near; z <= Far; z++)
				if(z >= 0 && z < Specs.ResZ)
				{
					const int Index = x + Specs.ResX * z;
					const int Num = Table[Index].Objects.Count();
					for(int k = 0; k < Num; k++)
					{
						ManagedObject *Obj = Table[Index].Objects[k];
						if(Obj->GetType() == Type)
						{
							const IMATH::Vector ObjPos = Obj->GetPos();
							const float DstX = ObjPos.x - Pos->x;
							const float DstZ = ObjPos.z - Pos->z;
							if(DstX * DstX + DstZ * DstZ < Area * Area)
								Objects->Add(Obj);
						}
					}
				}
}

void LUT2D::AddManagedObject(ManagedObject *Obj)
{
	int x = GetNodeX(Obj->GetPos().x);
	int z = GetNodeZ(Obj->GetPos().z);
	if(x >= 0 && x < Specs.ResX && z >= 0 && z < Specs.ResZ)
	{
		Node *Nd = &Table[x + z * Specs.ResX];
		((Node *)Nd)->Objects.Add(Obj);
		Obj->Node = Nd;
	}
	else
		Obj->Node = 0;
}

void LUT2D::DeleteManagedObject(ManagedObject *Obj)
{
	if(Obj->Node)
		((Node *)Obj->Node)->Objects.Delete(Obj);
}

void LUT2D::MoveManagedObject(ManagedObject *Obj)
{
	/*// position
	int x = GetNodeX(Obj->GetPos().x);
	int z = GetNodeZ(Obj->GetPos().z);
	// node
	Node *Nd;
	if(x >= 0 && x < Specs.ResX && z >= 0 && z < Specs.ResZ)
		Nd = &Table[x + z * Specs.ResX];
	else
		Nd = 0;
	// changed?
	if((Node *)Obj->Node == Nd)
		return;
	else
	{
		// change node
		// delete
		if(Obj->Node)
			((Node *)Obj->Node)->Objects.Delete(Obj);
		// add
		if(Nd)
			((Node *)Nd)->Objects.Add(Obj);
		Obj->Node = Nd;
	}*/
	// faster (why?):
	DeleteManagedObject(Obj);
	AddManagedObject(Obj);
}

}
