/***********************************************************/
/*                                                         */
/* LUT2D.h                                                 */
/*                                                         */
/* created 26.03.2003                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef LUT2D_H
#define LUT2D_H

#include "ManagedObject.h"
#include "DynamicArray.h"
#include "SpaceManagerInterface.h"

// www.infinitecode.com 3d library
namespace I3D
{

class LUT2D: public SpaceManagerInterface
{
public:
	struct LUT2DSpecs
	{
		int ResX, ResZ;
		float SizeX, SizeZ;
	};

private:
	struct Node
	{
		ISL::DynamicArray<ManagedObject *> Objects;
	};

	LUT2DSpecs Specs;
	Node *Table;
	float ResOverSizeX, ResOverSizeZ;

	int GetNodeX(float x) const;
	int GetNodeZ(float z) const;

public:
	LUT2D(void): Table(0) {}
	~LUT2D(void);

	void Create(LUT2DSpecs *Specs);
	// from SpaceManagerInterface
	virtual void AddManagedObject(ManagedObject *Obj);
	virtual void DeleteManagedObject(ManagedObject *Obj);
	virtual void MoveManagedObject(ManagedObject *Obj);
	void GetNearObjects(const IMATH::Vector *Pos, const float Area, I3D::ManagedObject::MgdObjArray *Objects) const;
	void GetNearObjects(const IMATH::Vector *Pos, const float Area, I3D::ManagedObject::MgdObjArray *Objects, const void *Type) const;
};

}

#endif
